#line 1 "ad_x86.cpp"
//
// Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_x86.hpp"
#include "adfiles/ad_x86.hpp"
#include "memory/allocation.inline.hpp"
#include "code/compiledIC.hpp"
#include "code/nativeInst.hpp"
#include "code/vmreg.inline.hpp"
#include "gc/shared/collectedHeap.inline.hpp"
#include "oops/compiledICHolder.hpp"
#include "oops/compressedOops.hpp"
#include "oops/markWord.hpp"
#include "oops/method.hpp"
#include "oops/oop.inline.hpp"
#include "opto/c2_MacroAssembler.hpp"
#include "opto/cfgnode.hpp"
#include "opto/intrinsicnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/safepointMechanism.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "utilities/powerOfTwo.hpp"

//SourceForm

#line 253 "src/hotspot/cpu/x86/x86_32.ad"

#define   RELOC_IMM32    Assembler::imm_operand
#define   RELOC_DISP32   Assembler::disp32_operand

#define __ _masm.

// How to find the high register of a Long pair, given the low register
#define   HIGH_FROM_LOW(x) ((x)+2)

// These masks are used to provide 128-bit aligned bitmasks to the XMM
// instructions, to allow sign-masking or sign-bit flipping.  They allow
// fast versions of NegF/NegD and AbsF/AbsD.

// Note: 'double' and 'long long' have 32-bits alignment on x86.
static jlong* double_quadword(jlong *adr, jlong lo, jlong hi) {
  // Use the expression (adr)&(~0xF) to provide 128-bits aligned address
  // of 128-bits operands for SSE instructions.
  jlong *operand = (jlong*)(((uintptr_t)adr)&((uintptr_t)(~0xF)));
  // Store the value to a 128-bits operand.
  operand[0] = lo;
  operand[1] = hi;
  return operand;
}

// Buffer for 128-bits masks used by SSE instructions.
static jlong fp_signmask_pool[(4+1)*2]; // 4*128bits(data) + 128bits(alignment)

// Static initialization during VM startup.
static jlong *float_signmask_pool  = double_quadword(&fp_signmask_pool[1*2], CONST64(0x7FFFFFFF7FFFFFFF), CONST64(0x7FFFFFFF7FFFFFFF));
static jlong *double_signmask_pool = double_quadword(&fp_signmask_pool[2*2], CONST64(0x7FFFFFFFFFFFFFFF), CONST64(0x7FFFFFFFFFFFFFFF));
static jlong *float_signflip_pool  = double_quadword(&fp_signmask_pool[3*2], CONST64(0x8000000080000000), CONST64(0x8000000080000000));
static jlong *double_signflip_pool = double_quadword(&fp_signmask_pool[4*2], CONST64(0x8000000000000000), CONST64(0x8000000000000000));

// Offset hacking within calls.
static int pre_call_resets_size() {
  int size = 0;
  Compile* C = Compile::current();
  if (C->in_24_bit_fp_mode()) {
    size += 6; // fldcw
  }
  if (VM_Version::supports_vzeroupper()) {
    size += 3; // vzeroupper
  }
  return size;
}

// !!!!! Special hack to get all type of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
int MachCallStaticJavaNode::ret_addr_offset() {
  return 5 + pre_call_resets_size();  // 5 bytes from start of call to where return address points
}

int MachCallDynamicJavaNode::ret_addr_offset() {
  return 10 + pre_call_resets_size();  // 10 bytes from start of call to where return address points
}

static int sizeof_FFree_Float_Stack_All = -1;

int MachCallRuntimeNode::ret_addr_offset() {
  assert(sizeof_FFree_Float_Stack_All != -1, "must have been emitted already");
  return sizeof_FFree_Float_Stack_All + 5 + pre_call_resets_size();
}

//
// Compute padding required for nodes which need alignment
//

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const {
  current_offset += pre_call_resets_size();  // skip fldcw, if any
  current_offset += 1;      // skip call opcode byte
  return align_up(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const {
  current_offset += pre_call_resets_size();  // skip fldcw, if any
  current_offset += 5;      // skip MOV instruction
  current_offset += 1;      // skip call opcode byte
  return align_up(current_offset, alignment_required()) - current_offset;
}

// EMIT_RM()
void emit_rm(CodeBuffer &cbuf, int f1, int f2, int f3) {
  unsigned char c = (unsigned char)((f1 << 6) | (f2 << 3) | f3);
  cbuf.insts()->emit_int8(c);
}

// EMIT_CC()
void emit_cc(CodeBuffer &cbuf, int f1, int f2) {
  unsigned char c = (unsigned char)( f1 | f2 );
  cbuf.insts()->emit_int8(c);
}

// EMIT_OPCODE()
void emit_opcode(CodeBuffer &cbuf, int code) {
  cbuf.insts()->emit_int8((unsigned char) code);
}

// EMIT_OPCODE() w/ relocation information
void emit_opcode(CodeBuffer &cbuf, int code, relocInfo::relocType reloc, int offset = 0) {
  cbuf.relocate(cbuf.insts_mark() + offset, reloc);
  emit_opcode(cbuf, code);
}

// EMIT_D8()
void emit_d8(CodeBuffer &cbuf, int d8) {
  cbuf.insts()->emit_int8((unsigned char) d8);
}

// EMIT_D16()
void emit_d16(CodeBuffer &cbuf, int d16) {
  cbuf.insts()->emit_int16(d16);
}

// EMIT_D32()
void emit_d32(CodeBuffer &cbuf, int d32) {
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from relocInfo::relocType
void emit_d32_reloc(CodeBuffer &cbuf, int d32, relocInfo::relocType reloc,
        int format) {
  cbuf.relocate(cbuf.insts_mark(), reloc, format);
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from RelocationHolder
void emit_d32_reloc(CodeBuffer &cbuf, int d32, RelocationHolder const& rspec,
        int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type && d32 != 0 && d32 != (int)Universe::non_oop_word()) {
    assert(oopDesc::is_oop(cast_to_oop(d32)), "cannot embed broken oops in code");
  }
#endif
  cbuf.relocate(cbuf.insts_mark(), rspec, format);
  cbuf.insts()->emit_int32(d32);
}

// Access stack slot for load or store
void store_to_stackslot(CodeBuffer &cbuf, int opcode, int rm_field, int disp) {
  emit_opcode( cbuf, opcode );               // (e.g., FILD   [ESP+src])
  if( -128 <= disp && disp <= 127 ) {
    emit_rm( cbuf, 0x01, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d8 (cbuf, disp);     // Displacement  // R/M byte
  } else {
    emit_rm( cbuf, 0x02, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d32(cbuf, disp);     // Displacement  // R/M byte
  }
}

   // rRegI ereg, memory mem) %{    // emit_reg_mem
void encode_RegMem( CodeBuffer &cbuf, int reg_encoding, int base, int index, int scale, int displace, relocInfo::relocType disp_reloc ) {
  // There is no index & no scale, use form without SIB byte
  if ((index == 0x4) &&
      (scale == 0) && (base != ESP_enc)) {
    // If no displacement, mode is 0x0; unless base is [EBP]
    if ( (displace == 0) && (base != EBP_enc) ) {
      emit_rm(cbuf, 0x0, reg_encoding, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && (disp_reloc == relocInfo::none) ) {
        emit_rm(cbuf, 0x1, reg_encoding, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == -1) { // Special flag for absolute address
          emit_rm(cbuf, 0x0, reg_encoding, 0x5);
          // (manual lies; no SIB needed here)
          if ( disp_reloc != relocInfo::none ) {
            emit_d32_reloc(cbuf, displace, disp_reloc, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
        else {                // Normal base + offset
          emit_rm(cbuf, 0x2, reg_encoding, base);
          if ( disp_reloc != relocInfo::none ) {
            emit_d32_reloc(cbuf, displace, disp_reloc, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
      }
    }
  }
  else {                      // Else, encode with the SIB byte
    // If no displacement, mode is 0x0; unless base is [EBP]
    if (displace == 0 && (base != EBP_enc)) {  // If no displacement
      emit_rm(cbuf, 0x0, reg_encoding, 0x4);
      emit_rm(cbuf, scale, index, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && (disp_reloc == relocInfo::none) ) {
        emit_rm(cbuf, 0x1, reg_encoding, 0x4);
        emit_rm(cbuf, scale, index, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == 0x04 ) {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, 0x04);
        } else {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, base);
        }
        if ( disp_reloc != relocInfo::none ) {
          emit_d32_reloc(cbuf, displace, disp_reloc, 1);
        } else {
          emit_d32      (cbuf, displace);
        }
      }
    }
  }
}


void encode_Copy( CodeBuffer &cbuf, int dst_encoding, int src_encoding ) {
  if( dst_encoding == src_encoding ) {
    // reg-reg copy, use an empty encoding
  } else {
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, dst_encoding, src_encoding );
  }
}

void emit_cmpfp_fixup(MacroAssembler& _masm) {
  Label exit;
  __ jccb(Assembler::noParity, exit);
  __ pushf();
  //
  // comiss/ucomiss instructions set ZF,PF,CF flags and
  // zero OF,AF,SF for NaN values.
  // Fixup flags by zeroing ZF,PF so that compare of NaN
  // values returns 'less than' result (CF is set).
  // Leave the rest of flags unchanged.
  //
  //    7 6 5 4 3 2 1 0
  //   |S|Z|r|A|r|P|r|C|  (r - reserved bit)
  //    0 0 1 0 1 0 1 1   (0x2B)
  //
  __ andl(Address(rsp, 0), 0xffffff2b);
  __ popf();
  __ bind(exit);
}

void emit_cmpfp3(MacroAssembler& _masm, Register dst) {
  Label done;
  __ movl(dst, -1);
  __ jcc(Assembler::parity, done);
  __ jcc(Assembler::below, done);
  __ setb(Assembler::notEqual, dst);
  __ movzbl(dst, dst);
  __ bind(done);
}


//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_postalloc_expand() const { return false; }
void MachConstantBaseNode::postalloc_expand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("# MachConstantBaseNode (empty encoding)");
}
#endif


//=============================================================================
#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  Compile* C = ra_->C;

  int framesize = C->output()->frame_size_in_bytes();
  int bangsize = C->output()->bang_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return addr which is already pushed.
  framesize -= wordSize;

  if (C->output()->need_stack_bang(bangsize)) {
    framesize -= wordSize;
    st->print("# stack bang (%d bytes)", bangsize);
    st->print("\n\t");
    st->print("PUSH   EBP\t# Save EBP");
    if (PreserveFramePointer) {
      st->print("\n\t");
      st->print("MOV    EBP, ESP\t# Save the caller's SP into EBP");
    }
    if (framesize) {
      st->print("\n\t");
      st->print("SUB    ESP, #%d\t# Create frame",framesize);
    }
  } else {
    st->print("SUB    ESP, #%d\t# Create frame",framesize);
    st->print("\n\t");
    framesize -= wordSize;
    st->print("MOV    [ESP + #%d], EBP\t# Save EBP",framesize);
    if (PreserveFramePointer) {
      st->print("\n\t");
      st->print("MOV    EBP, ESP\t# Save the caller's SP into EBP");
      if (framesize > 0) {
        st->print("\n\t");
        st->print("ADD    EBP, #%d", framesize);
      }
    }
  }

  if (VerifyStackAtCalls) {
    st->print("\n\t");
    framesize -= wordSize;
    st->print("MOV    [ESP + #%d], 0xBADB100D\t# Majik cookie for stack depth check",framesize);
  }

  if( C->in_24_bit_fp_mode() ) {
    st->print("\n\t");
    st->print("FLDCW  \t# load 24 bit fpu control word");
  }
  if (UseSSE >= 2 && VerifyFPU) {
    st->print("\n\t");
    st->print("# verify FPU stack (must be clean on entry)");
  }

#ifdef ASSERT
  if (VerifyStackAtCalls) {
    st->print("\n\t");
    st->print("# stack alignment check");
  }
#endif
  st->cr();
}
#endif


void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  int framesize = C->output()->frame_size_in_bytes();
  int bangsize = C->output()->bang_size_in_bytes();

  __ verified_entry(framesize, C->output()->need_stack_bang(bangsize)?bangsize:0, C->in_24_bit_fp_mode(), C->stub_function() != NULL);

  C->output()->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    ConstantTable& constant_table = C->output()->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_); // too many variables; just compute it the hard way
}

int MachPrologNode::reloc() const {
  return 0; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  Compile *C = ra_->C;
  int framesize = C->output()->frame_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  if (C->max_vector_size() > 16) {
    st->print("VZEROUPPER");
    st->cr(); st->print("\t");
  }
  if (C->in_24_bit_fp_mode()) {
    st->print("FLDCW  standard control word");
    st->cr(); st->print("\t");
  }
  if (framesize) {
    st->print("ADD    ESP,%d\t# Destroy frame",framesize);
    st->cr(); st->print("\t");
  }
  st->print_cr("POPL   EBP"); st->print("\t");
  if (do_polling() && C->is_method_compilation()) {
    st->print("TEST   PollPage,EAX\t! Poll Safepoint");
    st->cr(); st->print("\t");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile *C = ra_->C;
  MacroAssembler _masm(&cbuf);

  if (C->max_vector_size() > 16) {
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    _masm.vzeroupper();
  }
  // If method set FPU control word, restore to standard control word
  if (C->in_24_bit_fp_mode()) {
    _masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
  }

  int framesize = C->output()->frame_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  // Note that VerifyStackAtCalls' Majik cookie does not change the frame size popped here

  if (framesize >= 128) {
    emit_opcode(cbuf, 0x81); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d32(cbuf, framesize);
  } else if (framesize) {
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, framesize);
  }

  emit_opcode(cbuf, 0x58 | EBP_enc);

  if (StackReservedPages > 0 && C->has_reserved_stack_access()) {
    __ reserved_stack_check();
  }

  if (do_polling() && C->is_method_compilation()) {
    Register pollReg = as_Register(EBX_enc);
    MacroAssembler masm(&cbuf);
    masm.get_thread(pollReg);
    masm.movl(pollReg, Address(pollReg, in_bytes(Thread::polling_page_offset())));
    masm.relocate(relocInfo::poll_return_type);
    masm.testl(rax, Address(pollReg, 0));
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachEpilogNode::reloc() const {
  return 0; // a large enough number
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

//=============================================================================

enum RC { rc_bad, rc_int, rc_float, rc_xmm, rc_stack };
static enum RC rc_class( OptoReg::Name reg ) {

  if( !OptoReg::is_valid(reg)  ) return rc_bad;
  if (OptoReg::is_stack(reg)) return rc_stack;

  VMReg r = OptoReg::as_VMReg(reg);
  if (r->is_Register()) return rc_int;
  if (r->is_FloatRegister()) {
    assert(UseSSE < 2, "shouldn't be used in SSE2+ mode");
    return rc_float;
  }
  assert(r->is_XMMRegister(), "must be");
  return rc_xmm;
}

static int impl_helper( CodeBuffer *cbuf, bool do_size, bool is_load, int offset, int reg,
                        int opcode, const char *op_str, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode  (*cbuf, opcode );
    encode_RegMem(*cbuf, Matcher::_regEncode[reg], ESP_enc, 0x4, 0, offset, relocInfo::none);
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    if( opcode == 0x8B || opcode == 0x89 ) { // MOV
      if( is_load ) st->print("%s   %s,[ESP + #%d]",op_str,Matcher::regName[reg],offset);
      else          st->print("%s   [ESP + #%d],%s",op_str,offset,Matcher::regName[reg]);
    } else { // FLD, FST, PUSH, POP
      st->print("%s [ESP + #%d]",op_str,offset);
    }
#endif
  }
  int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  return size+3+offset_size;
}

// Helper for XMM registers.  Extra opcode bits, limited syntax.
static int impl_x_helper( CodeBuffer *cbuf, bool do_size, bool is_load,
                         int offset, int reg_lo, int reg_hi, int size, outputStream* st ) {
  int in_size_in_bits = Assembler::EVEX_32bit;
  int evex_encoding = 0;
  if (reg_lo+1 == reg_hi) {
    in_size_in_bits = Assembler::EVEX_64bit;
    evex_encoding = Assembler::VEX_W;
  }
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: Compressed displacemement is better than AVX on spill mem operations,
    //                          it maps more cases to single byte displacement
    _masm.set_managed();
    if (reg_lo+1 == reg_hi) { // double move?
      if (is_load) {
        __ movdbl(as_XMMRegister(Matcher::_regEncode[reg_lo]), Address(rsp, offset));
      } else {
        __ movdbl(Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[reg_lo]));
      }
    } else {
      if (is_load) {
        __ movflt(as_XMMRegister(Matcher::_regEncode[reg_lo]), Address(rsp, offset));
      } else {
        __ movflt(Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[reg_lo]));
      }
    }
#ifndef PRODUCT
  } else if (!do_size) {
    if (size != 0) st->print("\n\t");
    if (reg_lo+1 == reg_hi) { // double move?
      if (is_load) st->print("%s %s,[ESP + #%d]",
                              UseXmmLoadAndClearUpper ? "MOVSD " : "MOVLPD",
                              Matcher::regName[reg_lo], offset);
      else         st->print("MOVSD  [ESP + #%d],%s",
                              offset, Matcher::regName[reg_lo]);
    } else {
      if (is_load) st->print("MOVSS  %s,[ESP + #%d]",
                              Matcher::regName[reg_lo], offset);
      else         st->print("MOVSS  [ESP + #%d],%s",
                              offset, Matcher::regName[reg_lo]);
    }
#endif
  }
  bool is_single_byte = false;
  if ((UseAVX > 2) && (offset != 0)) {
    is_single_byte = Assembler::query_compressed_disp_byte(offset, true, 0, Assembler::EVEX_T1S, in_size_in_bits, evex_encoding);
  }
  int offset_size = 0;
  if (UseAVX > 2 ) {
    offset_size = (offset == 0) ? 0 : ((is_single_byte) ? 1 : 4);
  } else {
    offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  }
  size += (UseAVX > 2) ? 2 : 0; // Need an additional two bytes for EVEX
  // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
  return size+5+offset_size;
}


static int impl_movx_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                            int src_hi, int dst_hi, int size, outputStream* st ) {
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    if (src_lo+1 == src_hi && dst_lo+1 == dst_hi) { // double move?
      __ movdbl(as_XMMRegister(Matcher::_regEncode[dst_lo]),
                as_XMMRegister(Matcher::_regEncode[src_lo]));
    } else {
      __ movflt(as_XMMRegister(Matcher::_regEncode[dst_lo]),
                as_XMMRegister(Matcher::_regEncode[src_lo]));
    }
#ifndef PRODUCT
  } else if (!do_size) {
    if (size != 0) st->print("\n\t");
    if (UseXmmRegToRegMoveAll) {//Use movaps,movapd to move between xmm registers
      if (src_lo+1 == src_hi && dst_lo+1 == dst_hi) { // double move?
        st->print("MOVAPD %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVAPS %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
    } else {
      if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double move?
        st->print("MOVSD  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVSS  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
    }
#endif
  }
  // VEX_2bytes prefix is used if UseAVX > 0, and it takes the same 2 bytes as SIMD prefix.
  // Only MOVAPS SSE prefix uses 1 byte.  EVEX uses an additional 2 bytes.
  int sz = (UseAVX > 2) ? 6 : 4;
  if (!(src_lo+1 == src_hi && dst_lo+1 == dst_hi) &&
      UseXmmRegToRegMoveAll && (UseAVX == 0)) sz = 3;
  return size + sz;
}

static int impl_movgpr2x_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                            int src_hi, int dst_hi, int size, outputStream* st ) {
  // 32-bit
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    __ movdl(as_XMMRegister(Matcher::_regEncode[dst_lo]),
             as_Register(Matcher::_regEncode[src_lo]));
#ifndef PRODUCT
  } else if (!do_size) {
    st->print("movdl   %s, %s\t# spill", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
#endif
  }
  return (UseAVX> 2) ? 6 : 4;
}


static int impl_movx2gpr_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                                 int src_hi, int dst_hi, int size, outputStream* st ) {
  // 32-bit
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    __ movdl(as_Register(Matcher::_regEncode[dst_lo]),
             as_XMMRegister(Matcher::_regEncode[src_lo]));
#ifndef PRODUCT
  } else if (!do_size) {
    st->print("movdl   %s, %s\t# spill", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
#endif
  }
  return (UseAVX> 2) ? 6 : 4;
}

static int impl_mov_helper( CodeBuffer *cbuf, bool do_size, int src, int dst, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode(*cbuf, 0x8B );
    emit_rm    (*cbuf, 0x3, Matcher::_regEncode[dst], Matcher::_regEncode[src] );
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    st->print("MOV    %s,%s",Matcher::regName[dst],Matcher::regName[src]);
#endif
  }
  return size+2;
}

static int impl_fp_store_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int src_hi, int dst_lo, int dst_hi,
                                 int offset, int size, outputStream* st ) {
  if( src_lo != FPR1L_num ) {      // Move value to top of FP stack, if not already there
    if( cbuf ) {
      emit_opcode( *cbuf, 0xD9 );  // FLD (i.e., push it)
      emit_d8( *cbuf, 0xC0-1+Matcher::_regEncode[src_lo] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("FLD    %s",Matcher::regName[src_lo]);
#endif
    }
    size += 2;
  }

  int st_op = (src_lo != FPR1L_num) ? EBX_num /*store & pop*/ : EDX_num /*store no pop*/;
  const char *op_str;
  int op;
  if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double store?
    op_str = (src_lo != FPR1L_num) ? "FSTP_D" : "FST_D ";
    op = 0xDD;
  } else {                   // 32-bit store
    op_str = (src_lo != FPR1L_num) ? "FSTP_S" : "FST_S ";
    op = 0xD9;
    assert( !OptoReg::is_valid(src_hi) && !OptoReg::is_valid(dst_hi), "no non-adjacent float-stores" );
  }

  return impl_helper(cbuf,do_size,false,offset,st_op,op,op_str,size, st);
}

// Next two methods are shared by 32- and 64-bit VM. They are defined in x86.ad.
static int vec_mov_helper(CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st);

static int vec_spill_helper(CodeBuffer *cbuf, bool do_size, bool is_load,
                            int stack_offset, int reg, uint ireg, outputStream* st);

static int vec_stack_to_stack_helper(CodeBuffer *cbuf, bool do_size, int src_offset,
                                     int dst_offset, uint ireg, outputStream* st) {
  int calc_size = 0;
  int src_offset_size = (src_offset == 0) ? 0 : ((src_offset < 0x80) ? 1 : 4);
  int dst_offset_size = (dst_offset == 0) ? 0 : ((dst_offset < 0x80) ? 1 : 4);
  switch (ireg) {
  case Op_VecS:
    calc_size = 3+src_offset_size + 3+dst_offset_size;
    break;
  case Op_VecD: {
    calc_size = 3+src_offset_size + 3+dst_offset_size;
    int tmp_src_offset = src_offset + 4;
    int tmp_dst_offset = dst_offset + 4;
    src_offset_size = (tmp_src_offset == 0) ? 0 : ((tmp_src_offset < 0x80) ? 1 : 4);
    dst_offset_size = (tmp_dst_offset == 0) ? 0 : ((tmp_dst_offset < 0x80) ? 1 : 4);
    calc_size += 3+src_offset_size + 3+dst_offset_size;
    break;
  }
  case Op_VecX:
  case Op_VecY:
  case Op_VecZ:
    calc_size = 6 + 6 + 5+src_offset_size + 5+dst_offset_size;
    break;
  default:
    ShouldNotReachHere();
  }
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    int offset = __ offset();
    switch (ireg) {
    case Op_VecS:
      __ pushl(Address(rsp, src_offset));
      __ popl (Address(rsp, dst_offset));
      break;
    case Op_VecD:
      __ pushl(Address(rsp, src_offset));
      __ popl (Address(rsp, dst_offset));
      __ pushl(Address(rsp, src_offset+4));
      __ popl (Address(rsp, dst_offset+4));
      break;
    case Op_VecX:
      __ movdqu(Address(rsp, -16), xmm0);
      __ movdqu(xmm0, Address(rsp, src_offset));
      __ movdqu(Address(rsp, dst_offset), xmm0);
      __ movdqu(xmm0, Address(rsp, -16));
      break;
    case Op_VecY:
      __ vmovdqu(Address(rsp, -32), xmm0);
      __ vmovdqu(xmm0, Address(rsp, src_offset));
      __ vmovdqu(Address(rsp, dst_offset), xmm0);
      __ vmovdqu(xmm0, Address(rsp, -32));
      break;
    case Op_VecZ:
      __ evmovdquq(Address(rsp, -64), xmm0, 2);
      __ evmovdquq(xmm0, Address(rsp, src_offset), 2);
      __ evmovdquq(Address(rsp, dst_offset), xmm0, 2);
      __ evmovdquq(xmm0, Address(rsp, -64), 2);
      break;
    default:
      ShouldNotReachHere();
    }
    int size = __ offset() - offset;
    assert(size == calc_size, "incorrect size calculation");
    return size;
#ifndef PRODUCT
  } else if (!do_size) {
    switch (ireg) {
    case Op_VecS:
      st->print("pushl   [rsp + #%d]\t# 32-bit mem-mem spill\n\t"
                "popl    [rsp + #%d]",
                src_offset, dst_offset);
      break;
    case Op_VecD:
      st->print("pushl   [rsp + #%d]\t# 64-bit mem-mem spill\n\t"
                "popq    [rsp + #%d]\n\t"
                "pushl   [rsp + #%d]\n\t"
                "popq    [rsp + #%d]",
                src_offset, dst_offset, src_offset+4, dst_offset+4);
      break;
     case Op_VecX:
      st->print("movdqu  [rsp - #16], xmm0\t# 128-bit mem-mem spill\n\t"
                "movdqu  xmm0, [rsp + #%d]\n\t"
                "movdqu  [rsp + #%d], xmm0\n\t"
                "movdqu  xmm0, [rsp - #16]",
                src_offset, dst_offset);
      break;
    case Op_VecY:
      st->print("vmovdqu [rsp - #32], xmm0\t# 256-bit mem-mem spill\n\t"
                "vmovdqu xmm0, [rsp + #%d]\n\t"
                "vmovdqu [rsp + #%d], xmm0\n\t"
                "vmovdqu xmm0, [rsp - #32]",
                src_offset, dst_offset);
      break;
    case Op_VecZ:
      st->print("vmovdqu [rsp - #64], xmm0\t# 512-bit mem-mem spill\n\t"
                "vmovdqu xmm0, [rsp + #%d]\n\t"
                "vmovdqu [rsp + #%d], xmm0\n\t"
                "vmovdqu xmm0, [rsp - #64]",
                src_offset, dst_offset);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
  return calc_size;
}

uint MachSpillCopyNode::implementation( CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream* st ) const {
  // Get registers to move
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this );
  OptoReg::Name dst_first = ra_->get_reg_first(this );

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert( OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first), "must move at least 1 register" );

  // Generate spill code!
  int size = 0;

  if( src_first == dst_first && src_second == dst_second )
    return size;            // Self copy, no move

  if (bottom_type()->isa_vect() != NULL) {
    uint ireg = ideal_reg();
    assert((src_first_rc != rc_int && dst_first_rc != rc_int), "sanity");
    assert((src_first_rc != rc_float && dst_first_rc != rc_float), "sanity");
    assert((ireg == Op_VecS || ireg == Op_VecD || ireg == Op_VecX || ireg == Op_VecY || ireg == Op_VecZ ), "sanity");
    if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
      // mem -> mem
      int src_offset = ra_->reg2offset(src_first);
      int dst_offset = ra_->reg2offset(dst_first);
      return vec_stack_to_stack_helper(cbuf, do_size, src_offset, dst_offset, ireg, st);
    } else if (src_first_rc == rc_xmm && dst_first_rc == rc_xmm ) {
      return vec_mov_helper(cbuf, do_size, src_first, dst_first, src_second, dst_second, ireg, st);
    } else if (src_first_rc == rc_xmm && dst_first_rc == rc_stack ) {
      int stack_offset = ra_->reg2offset(dst_first);
      return vec_spill_helper(cbuf, do_size, false, stack_offset, src_first, ireg, st);
    } else if (src_first_rc == rc_stack && dst_first_rc == rc_xmm ) {
      int stack_offset = ra_->reg2offset(src_first);
      return vec_spill_helper(cbuf, do_size, true,  stack_offset, dst_first, ireg, st);
    } else {
      ShouldNotReachHere();
    }
  }

  // --------------------------------------
  // Check for mem-mem move.  push/pop to move.
  if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
    if( src_second == dst_first ) { // overlapping stack copy ranges
      assert( src_second_rc == rc_stack && dst_second_rc == rc_stack, "we only expect a stk-stk copy here" );
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
      src_second_rc = dst_second_rc = rc_bad;  // flag as already moved the second bits
    }
    // move low bits
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),ESI_num,0xFF,"PUSH  ",size, st);
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),EAX_num,0x8F,"POP   ",size, st);
    if( src_second_rc == rc_stack && dst_second_rc == rc_stack ) { // mov second bits
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
    }
    return size;
  }

  // --------------------------------------
  // Check for integer reg-reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_int )
    size = impl_mov_helper(cbuf,do_size,src_first,dst_first,size, st);

  // Check for integer store
  if( src_first_rc == rc_int && dst_first_rc == rc_stack )
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),src_first,0x89,"MOV ",size, st);

  // Check for integer load
  if( dst_first_rc == rc_int && src_first_rc == rc_stack )
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first,0x8B,"MOV ",size, st);

  // Check for integer reg-xmm reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_xmm ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad),
            "no 64 bit integer-float reg moves" );
    return impl_movgpr2x_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }
  // --------------------------------------
  // Check for float reg-reg copy
  if( src_first_rc == rc_float && dst_first_rc == rc_float ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second), "no non-adjacent float-moves" );
    if( cbuf ) {

      // Note the mucking with the register encode to compensate for the 0/1
      // indexing issue mentioned in a comment in the reg_def sections
      // for FPR registers many lines above here.

      if( src_first != FPR1L_num ) {
        emit_opcode  (*cbuf, 0xD9 );           // FLD    ST(i)
        emit_d8      (*cbuf, 0xC0+Matcher::_regEncode[src_first]-1 );
        emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
        emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
     } else {
        emit_opcode  (*cbuf, 0xDD );           // FST    ST(i)
        emit_d8      (*cbuf, 0xD0+Matcher::_regEncode[dst_first]-1 );
     }
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      if( src_first != FPR1L_num ) st->print("FLD    %s\n\tFSTP   %s",Matcher::regName[src_first],Matcher::regName[dst_first]);
      else                      st->print(             "FST    %s",                            Matcher::regName[dst_first]);
#endif
    }
    return size + ((src_first != FPR1L_num) ? 2+2 : 2);
  }

  // Check for float store
  if( src_first_rc == rc_float && dst_first_rc == rc_stack ) {
    return impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,ra_->reg2offset(dst_first),size, st);
  }

  // Check for float load
  if( dst_first_rc == rc_float && src_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_first);
    const char *op_str;
    int op;
    if( src_first+1 == src_second && dst_first+1 == dst_second ) { // double load?
      op_str = "FLD_D";
      op = 0xDD;
    } else {                   // 32-bit load
      op_str = "FLD_S";
      op = 0xD9;
      assert( src_second_rc == rc_bad && dst_second_rc == rc_bad, "no non-adjacent float-loads" );
    }
    if( cbuf ) {
      emit_opcode  (*cbuf, op );
      encode_RegMem(*cbuf, 0x0, ESP_enc, 0x4, 0, offset, relocInfo::none);
      emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
      emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("%s  ST,[ESP + #%d]\n\tFSTP   %s",op_str, offset,Matcher::regName[dst_first]);
#endif
    }
    int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
    return size + 3+offset_size+2;
  }

  // Check for xmm reg-reg copy
  if( src_first_rc == rc_xmm && dst_first_rc == rc_xmm ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second),
            "no non-adjacent float-moves" );
    return impl_movx_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }

  // Check for xmm reg-integer reg copy
  if( src_first_rc == rc_xmm && dst_first_rc == rc_int ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad),
            "no 64 bit float-integer reg moves" );
    return impl_movx2gpr_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }

  // Check for xmm store
  if( src_first_rc == rc_xmm && dst_first_rc == rc_stack ) {
    return impl_x_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),src_first, src_second, size, st);
  }

  // Check for float xmm load
  if( dst_first_rc == rc_xmm && src_first_rc == rc_stack ) {
    return impl_x_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first, dst_second, size, st);
  }

  // Copy from float reg to xmm reg
  if( dst_first_rc == rc_xmm && src_first_rc == rc_float ) {
    // copy to the top of stack from floating point reg
    // and use LEA to preserve flags
    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP-8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0xF8);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP-8]");
#endif
    }
    size += 4;

    size = impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,0,size, st);

    // Copy from the temp memory to the xmm reg.
    size = impl_x_helper(cbuf,do_size,true ,0,dst_first, dst_second, size, st);

    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP+8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0x08);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP+8]");
#endif
    }
    size += 4;
    return size;
  }

  assert( size > 0, "missed a case" );

  // --------------------------------------------------------------------
  // Check for second bits still needing moving.
  if( src_second == dst_second )
    return size;               // Self copy; no move
  assert( src_second_rc != rc_bad && dst_second_rc != rc_bad, "src_second & dst_second cannot be Bad" );

  // Check for second word int-int move
  if( src_second_rc == rc_int && dst_second_rc == rc_int )
    return impl_mov_helper(cbuf,do_size,src_second,dst_second,size, st);

  // Check for second word integer store
  if( src_second_rc == rc_int && dst_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),src_second,0x89,"MOV ",size, st);

  // Check for second word integer load
  if( dst_second_rc == rc_int && src_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),dst_second,0x8B,"MOV ",size, st);


  Unimplemented();
  return 0; // Mute compiler
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream* st) const {
  implementation( NULL, ra_, false, st );
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation( &cbuf, ra_, false, NULL );
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}


//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("LEA    %s,[ESP + #%d]",Matcher::regName[reg],offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_encode(this);
  if( offset >= 128 ) {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x2, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d32(cbuf, offset);
  }
  else {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x1, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d8(cbuf, offset);
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  if( offset >= 128 ) {
    return 7;
  }
  else {
    return 4;
  }
}

//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  st->print_cr(  "CMP    EAX,[ECX+4]\t# Inline cache check");
  st->print_cr("\tJNE    SharedRuntime::handle_ic_miss_stub");
  st->print_cr("\tNOP");
  st->print_cr("\tNOP");
  if( !OptoBreakpoint )
    st->print_cr("\tNOP");
}
#endif

void MachUEPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler masm(&cbuf);
#ifdef ASSERT
  uint insts_size = cbuf.insts_size();
#endif
  masm.cmpptr(rax, Address(rcx, oopDesc::klass_offset_in_bytes()));
  masm.jump_cc(Assembler::notEqual,
               RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  /* WARNING these NOPs are critical so that verified entry point is properly
     aligned for patching by NativeJump::patch_verified_entry() */
  int nops_cnt = 2;
  if( !OptoBreakpoint ) // Leave space for int3
     nops_cnt += 1;
  masm.nop(nops_cnt);

  assert(cbuf.insts_size() - insts_size == size(ra_), "checking code size of inline cache node");
}

uint MachUEPNode::size(PhaseRegAlloc *ra_) const {
  return OptoBreakpoint ? 11 : 12;
}


//=============================================================================

int Matcher::regnum_to_fpu_offset(int regnum) {
  return regnum - 32; // The FP registers are in the second chunk
}

// This is UltraSparc specific, true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.
  // On 86 a branch displacement is calculated relative to address
  // of a next instruction.
  offset -= br_size;

  // the short version of jmpConUCF2 contains multiple branches,
  // making the reach slightly less
  if (rule == jmpConUCF2_rule)
    return (-126 <= offset && offset <= 125);
  return (-128 <= offset && offset <= 127);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  return false;
}

// The ecx parameter to rep stos for the ClearArray node is in dwords.
const bool Matcher::init_array_count_is_in_bytes = false;

// Needs 2 CMOV's for longs.
const int Matcher::long_cmove_cost() { return 1; }

// No CMOVF/CMOVD with SSE/SSE2
const int Matcher::float_cmove_cost() { return (UseSSE>=1) ? ConditionalMoveLimit : 0; }

// Does the CPU require late expand (see block.cpp for description of late expand)?
const bool Matcher::require_postalloc_expand = false;

// Do we need to mask the count passed to shift instructions or does
// the cpu only look at the lower 5/6 bits anyway?
const bool Matcher::need_masked_shift_count = false;

bool Matcher::narrow_oop_use_complex_address() {
  ShouldNotCallThis();
  return true;
}

bool Matcher::narrow_klass_use_complex_address() {
  ShouldNotCallThis();
  return true;
}

bool Matcher::const_oop_prefer_decode() {
  ShouldNotCallThis();
  return true;
}

bool Matcher::const_klass_prefer_decode() {
  ShouldNotCallThis();
  return true;
}

// Is it better to copy float constants, or load them directly from memory?
// Intel can load a float constant from a direct address, requiring no
// extra registers.  Most RISCs will have to materialize an address into a
// register first, so they would do better to copy the constant from stack.
const bool Matcher::rematerialize_float_constants = true;

// If CPU can load and store mis-aligned doubles directly then no fixup is
// needed.  Else we split the double into 2 integer pieces and move it
// piece-by-piece.  Only happens when passing doubles into C code as the
// Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;


void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {
  // Get the memory operand from the node
  uint numopnds = node->num_opnds();        // Virtual call for number of operands
  uint skipped  = node->oper_input_base();  // Sum of leaves skipped so far
  assert( idx >= skipped, "idx too low in pd_implicit_null_fixup" );
  uint opcnt     = 1;                 // First operand
  uint num_edges = node->_opnds[1]->num_edges(); // leaves for first operand
  while( idx >= skipped+num_edges ) {
    skipped += num_edges;
    opcnt++;                          // Bump operand count
    assert( opcnt < numopnds, "Accessing non-existent operand" );
    num_edges = node->_opnds[opcnt]->num_edges(); // leaves for next operand
  }

  MachOper *memory = node->_opnds[opcnt];
  MachOper *new_memory = NULL;
  switch (memory->opcode()) {
  case DIRECT:
  case INDOFFSET32X:
    // No transformation necessary.
    return;
  case INDIRECT:
    new_memory = new indirect_win95_safeOper( );
    break;
  case INDOFFSET8:
    new_memory = new indOffset8_win95_safeOper(memory->disp(NULL, NULL, 0));
    break;
  case INDOFFSET32:
    new_memory = new indOffset32_win95_safeOper(memory->disp(NULL, NULL, 0));
    break;
  case INDINDEXOFFSET:
    new_memory = new indIndexOffset_win95_safeOper(memory->disp(NULL, NULL, 0));
    break;
  case INDINDEXSCALE:
    new_memory = new indIndexScale_win95_safeOper(memory->scale());
    break;
  case INDINDEXSCALEOFFSET:
    new_memory = new indIndexScaleOffset_win95_safeOper(memory->scale(), memory->disp(NULL, NULL, 0));
    break;
  case LOAD_LONG_INDIRECT:
  case LOAD_LONG_INDOFFSET32:
    // Does not use EBP as address register, use { EDX, EBX, EDI, ESI}
    return;
  default:
    assert(false, "unexpected memory operand in pd_implicit_null_fixup()");
    return;
  }
  node->_opnds[opcnt] = new_memory;
}

// Advertise here if the CPU requires explicit rounding operations to implement strictfp mode.
const bool Matcher::strict_fp_requires_explicit_rounding = true;

// Are floats conerted to double when stored to stack during deoptimization?
// On x32 it is stored with convertion only when FPU is used for floats.
bool Matcher::float_in_double() { return (UseSSE == 0); }

// Do ints take an entire long register or just half?
const bool Matcher::int_in_long = false;

// Return whether or not this register is ever used as an argument.  This
// function is used on startup to build the trampoline stubs in generateOptoStub.
// Registers not mentioned will be killed by the VM call in the trampoline, and
// arguments in those registers not be available to the callee.
bool Matcher::can_be_java_arg( int reg ) {
  if(  reg == ECX_num   || reg == EDX_num   ) return true;
  if( (reg == XMM0_num  || reg == XMM1_num ) && UseSSE>=1 ) return true;
  if( (reg == XMM0b_num || reg == XMM1b_num) && UseSSE>=2 ) return true;
  return false;
}

bool Matcher::is_spillable_arg( int reg ) {
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con( jlong divisor ) {
  // Use hardware integer DIV instruction when
  // it is faster than a code which use multiply.
  // Only when constant divisor fits into 32 bit
  // (min_jint is excluded to get only correct
  // positive 32 bit values from negative).
  return VM_Version::has_fast_idiv() &&
         (divisor == (int)divisor && divisor != min_jint);
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  return EAX_REG_mask();
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  return EDX_REG_mask();
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return NO_REG_mask();
}

// Returns true if the high 32 bits of the value is known to be zero.
bool is_operand_hi32_zero(Node* n) {
  int opc = n->Opcode();
  if (opc == Op_AndL) {
    Node* o2 = n->in(2);
    if (o2->is_Con() && (o2->get_long() & 0xFFFFFFFF00000000LL) == 0LL) {
      return true;
    }
  }
  if (opc == Op_ConL && (n->get_long() & 0xFFFFFFFF00000000LL) == 0LL) {
    return true;
  }
  return false;
}


#line 1387 "ad_x86.cpp"


//SourceForm

#line 1215 "src/hotspot/cpu/x86/x86.ad"


#include "opto/addnode.hpp"
#include "c2_intelJccErratum_x86.hpp"

void PhaseOutput::pd_perform_mach_node_analysis() {
  if (VM_Version::has_intel_jcc_erratum()) {
    int extra_padding = IntelJccErratum::tag_affected_machnodes(C, C->cfg(), C->regalloc());
    _buf_sizes._code += extra_padding;
  }
}

int MachNode::pd_alignment_required() const {
  if (VM_Version::has_intel_jcc_erratum() && IntelJccErratum::is_jcc_erratum_branch(this)) {
    // Conservatively add worst case padding. We assume that relocInfo::addr_unit() is 1 on x86.
    return IntelJccErratum::largest_jcc_size() + 1;
  } else {
    return 1;
  }
}

int MachNode::compute_padding(int current_offset) const {
  if (flags() & Node::PD::Flag_intel_jcc_erratum) {
    Compile* C = Compile::current();
    PhaseOutput* output = C->output();
    Block* block = output->block();
    int index = output->index();
    return IntelJccErratum::compute_padding(current_offset, this, block, index, C->regalloc());
  } else {
    return 0;
  }
}

// Emit exception handler code.
// Stuff framesize into a register and call a VM stub routine.
int HandlerImpl::emit_exception_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();
  __ jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// Emit deopt handler code.
int HandlerImpl::emit_deopt_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();

#ifdef _LP64
  address the_pc = (address) __ pc();
  Label next;
  // push a "the_pc" on the stack without destroying any registers
  // as they all may be live.

  // push address of "next"
  __ call(next, relocInfo::none); // reloc none is fine since it is a disp32
  __ bind(next);
  // adjust it so it matches "the_pc"
  __ subptr(Address(rsp, 0), __ offset() - offset);
#else
  InternalAddress here(__ pc());
  __ pushptr(here.addr());
#endif

  __ jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));
  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow %d", (__ offset() - offset));
  __ end_a_stub();
  return offset;
}


//=============================================================================

  // Float masks come from different places depending on platform.
#ifdef _LP64
  static address float_signmask()  { return StubRoutines::x86::float_sign_mask(); }
  static address float_signflip()  { return StubRoutines::x86::float_sign_flip(); }
  static address double_signmask() { return StubRoutines::x86::double_sign_mask(); }
  static address double_signflip() { return StubRoutines::x86::double_sign_flip(); }
#else
  static address float_signmask()  { return (address)float_signmask_pool; }
  static address float_signflip()  { return (address)float_signflip_pool; }
  static address double_signmask() { return (address)double_signmask_pool; }
  static address double_signflip() { return (address)double_signflip_pool; }
#endif
  static address vector_short_to_byte_mask() { return StubRoutines::x86::vector_short_to_byte_mask(); }
  static address vector_byte_perm_mask() { return StubRoutines::x86::vector_byte_perm_mask(); }
  static address vector_long_sign_mask() { return StubRoutines::x86::vector_long_sign_mask(); }

//=============================================================================
const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode)) {
    return false; // no match rule present
  }
  switch (opcode) {
    case Op_AbsVL:
      if (UseAVX < 3) {
        return false;
      }
      break;
    case Op_PopCountI:
    case Op_PopCountL:
      if (!UsePopCountInstruction) {
        return false;
      }
      break;
    case Op_PopCountVI:
      if (!UsePopCountInstruction || !VM_Version::supports_avx512_vpopcntdq()) {
        return false;
      }
      break;
    case Op_MulVI:
      if ((UseSSE < 4) && (UseAVX < 1)) { // only with SSE4_1 or AVX
        return false;
      }
      break;
    case Op_MulVL:
    case Op_MulReductionVL:
      if (VM_Version::supports_avx512dq() == false) {
        return false;
      }
      break;
    case Op_AbsVB:
    case Op_AbsVS:
    case Op_AbsVI:
    case Op_AddReductionVI:
    case Op_AndReductionV:
    case Op_OrReductionV:
    case Op_XorReductionV:
      if (UseSSE < 3) { // requires at least SSSE3
        return false;
      }
      break;
    case Op_MulReductionVI:
      if (UseSSE < 4) { // requires at least SSE4
        return false;
      }
      break;
    case Op_SqrtVD:
    case Op_SqrtVF:
      if (UseAVX < 1) { // enabled for AVX only
        return false;
      }
      break;
    case Op_CompareAndSwapL:
#ifdef _LP64
    case Op_CompareAndSwapP:
#endif
      if (!VM_Version::supports_cx8()) {
        return false;
      }
      break;
    case Op_CMoveVF:
    case Op_CMoveVD:
      if (UseAVX < 1 || UseAVX > 2) {
        return false;
      }
      break;
    case Op_StrIndexOf:
      if (!UseSSE42Intrinsics) {
        return false;
      }
      break;
    case Op_StrIndexOfChar:
      if (!UseSSE42Intrinsics) {
        return false;
      }
      break;
    case Op_OnSpinWait:
      if (VM_Version::supports_on_spin_wait() == false) {
        return false;
      }
      break;
    case Op_MulVB:
    case Op_LShiftVB:
    case Op_RShiftVB:
    case Op_URShiftVB:
      if (UseSSE < 4) {
        return false;
      }
      break;
#ifdef _LP64
    case Op_MaxD:
    case Op_MaxF:
    case Op_MinD:
    case Op_MinF:
      if (UseAVX < 1) { // enabled for AVX only
        return false;
      }
      break;
#endif
    case Op_CacheWB:
    case Op_CacheWBPreSync:
    case Op_CacheWBPostSync:
      if (!VM_Version::supports_data_cache_line_flush()) {
        return false;
      }
      break;
    case Op_RoundDoubleMode:
      if (UseSSE < 4) {
        return false;
      }
      break;
    case Op_RoundDoubleModeV:
      if (VM_Version::supports_avx() == false) {
        return false; // 128bit vroundpd is not available
      }
      break;
    case Op_MacroLogicV:
      if (UseAVX < 3 || !UseVectorMacroLogic) {
        return false;
      }
      break;
#ifndef _LP64
    case Op_AddReductionVF:
    case Op_AddReductionVD:
    case Op_MulReductionVF:
    case Op_MulReductionVD:
      if (UseSSE < 1) { // requires at least SSE
        return false;
      }
      break;
    case Op_MulAddVS2VI:
    case Op_RShiftVL:
    case Op_AbsVD:
    case Op_NegVD:
      if (UseSSE < 2) {
        return false;
      }
      break;
#endif // !LP64
  }
  return true;  // Match rules are supported by default.
}

//------------------------------------------------------------------------

// Identify extra cases that we might want to provide match rules for vector nodes and
// other intrinsics guarded with vector length (vlen) and element type (bt).
const bool Matcher::match_rule_supported_vector(int opcode, int vlen, BasicType bt) {
  if (!match_rule_supported(opcode)) {
    return false;
  }
  // Matcher::vector_size_supported() restricts vector sizes in the following way (see Matcher::vector_width_in_bytes):
  //   * SSE2 supports 128bit vectors for all types;
  //   * AVX1 supports 256bit vectors only for FLOAT and DOUBLE types;
  //   * AVX2 supports 256bit vectors for all types;
  //   * AVX512F supports 512bit vectors only for INT, FLOAT, and DOUBLE types;
  //   * AVX512BW supports 512bit vectors for BYTE, SHORT, and CHAR types.
  // There's also a limit on minimum vector size supported: 2 elements (or 4 bytes for BYTE).
  // And MaxVectorSize is taken into account as well.
  if (!vector_size_supported(bt, vlen)) {
    return false;
  }
  // Special cases which require vector length follow:
  //   * implementation limitations
  //   * some 512bit vector operations on FLOAT and DOUBLE types require AVX512DQ
  //   * 128bit vroundpd instruction is present only in AVX1
  int size_in_bits = vlen * type2aelembytes(bt) * BitsPerByte;
  switch (opcode) {
    case Op_AbsVF:
    case Op_NegVF:
      if ((vlen == 16) && (VM_Version::supports_avx512dq() == false)) {
        return false; // 512bit vandps and vxorps are not available
      }
      break;
    case Op_AbsVD:
    case Op_NegVD:
      if ((vlen == 8) && (VM_Version::supports_avx512dq() == false)) {
        return false; // 512bit vandpd and vxorpd are not available
      }
      break;
    case Op_CMoveVF:
      if (vlen != 8) {
        return false; // implementation limitation (only vcmov8F_reg is present)
      }
      break;
    case Op_MacroLogicV:
      if (!VM_Version::supports_evex() ||
          ((size_in_bits != 512) && !VM_Version::supports_avx512vl())) {
        return false;
      }
      break;
    case Op_CMoveVD:
      if (vlen != 4) {
        return false; // implementation limitation (only vcmov4D_reg is present)
      }
      break;
  }
  return true;  // Per default match rules are supported.
}

// x86 supports generic vector operands: vec and legVec.
const bool Matcher::supports_generic_vector_operands = true;

MachOper* Matcher::pd_specialize_generic_vector_operand(MachOper* generic_opnd, uint ideal_reg, bool is_temp) {
  assert(Matcher::is_generic_vector(generic_opnd), "not generic");
  bool legacy = (generic_opnd->opcode() == LEGVEC);
  if (!VM_Version::supports_avx512vlbwdq() && // KNL
      is_temp && !legacy && (ideal_reg == Op_VecZ)) {
    // Conservatively specialize 512bit vec TEMP operands to legVecZ (zmm0-15) on KNL.
    return new legVecZOper();
  }
  if (legacy) {
    switch (ideal_reg) {
      case Op_VecS: return new legVecSOper();
      case Op_VecD: return new legVecDOper();
      case Op_VecX: return new legVecXOper();
      case Op_VecY: return new legVecYOper();
      case Op_VecZ: return new legVecZOper();
    }
  } else {
    switch (ideal_reg) {
      case Op_VecS: return new vecSOper();
      case Op_VecD: return new vecDOper();
      case Op_VecX: return new vecXOper();
      case Op_VecY: return new vecYOper();
      case Op_VecZ: return new vecZOper();
    }
  }
  ShouldNotReachHere();
  return NULL;
}

bool Matcher::is_generic_reg2reg_move(MachNode* m) {
  switch (m->rule()) {
    case MoveVec2Leg_rule:
    case MoveLeg2Vec_rule:
      return true;
    default:
      return false;
  }
}

bool Matcher::is_generic_vector(MachOper* opnd) {
  switch (opnd->opcode()) {
    case VEC:
    case LEGVEC:
      return true;
    default:
      return false;
  }
}

//------------------------------------------------------------------------

const bool Matcher::has_predicated_vectors(void) {
  bool ret_value = false;
  if (UseAVX > 2) {
    ret_value = VM_Version::supports_avx512vl();
  }

  return ret_value;
}

const int Matcher::float_pressure(int default_pressure_threshold) {
  int float_pressure_threshold = default_pressure_threshold;
#ifdef _LP64
  if (UseAVX > 2) {
    // Increase pressure threshold on machines with AVX3 which have
    // 2x more XMM registers.
    float_pressure_threshold = default_pressure_threshold * 2;
  }
#endif
  return float_pressure_threshold;
}

// Max vector size in bytes. 0 if not supported.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  assert(is_java_primitive(bt), "only primitive type vectors");
  if (UseSSE < 2) return 0;
  // SSE2 supports 128bit vectors for all types.
  // AVX2 supports 256bit vectors for all types.
  // AVX2/EVEX supports 512bit vectors for all types.
  int size = (UseAVX > 1) ? (1 << UseAVX) * 8 : 16;
  // AVX1 supports 256bit vectors only for FLOAT and DOUBLE.
  if (UseAVX > 0 && (bt == T_FLOAT || bt == T_DOUBLE))
    size = (UseAVX > 2) ? 64 : 32;
  if (UseAVX > 2 && (bt == T_BYTE || bt == T_SHORT || bt == T_CHAR))
    size = (VM_Version::supports_avx512bw()) ? 64 : 32;
  // Use flag to limit vector size.
  size = MIN2(size,(int)MaxVectorSize);
  // Minimum 2 values in vector (or 4 for bytes).
  switch (bt) {
  case T_DOUBLE:
  case T_LONG:
    if (size < 16) return 0;
    break;
  case T_FLOAT:
  case T_INT:
    if (size < 8) return 0;
    break;
  case T_BOOLEAN:
    if (size < 4) return 0;
    break;
  case T_CHAR:
    if (size < 4) return 0;
    break;
  case T_BYTE:
    if (size < 4) return 0;
    break;
  case T_SHORT:
    if (size < 4) return 0;
    break;
  default:
    ShouldNotReachHere();
  }
  return size;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt)/type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
  int max_size = max_vector_size(bt);
  // Min size which can be loaded into vector is 4 bytes.
  int size = (type2aelembytes(bt) == 1) ? 4 : 2;
  return MIN2(size,max_size);
}

// Vector ideal reg corresponding to specified size in bytes
const uint Matcher::vector_ideal_reg(int size) {
  assert(MaxVectorSize >= size, "");
  switch(size) {
    case  4: return Op_VecS;
    case  8: return Op_VecD;
    case 16: return Op_VecX;
    case 32: return Op_VecY;
    case 64: return Op_VecZ;
  }
  ShouldNotReachHere();
  return 0;
}

// x86 supports misaligned vectors store/load.
const bool Matcher::misaligned_vectors_ok() {
  return true;
}

// x86 AES instructions are compatible with SunJCE expanded
// keys, hence we do not need to pass the original key to stubs
const bool Matcher::pass_original_key_for_aes() {
  return false;
}


const bool Matcher::convi2l_type_required = true;

// Check for shift by small constant as well
static bool clone_shift(Node* shift, Matcher* matcher, Matcher::MStack& mstack, VectorSet& address_visited) {
  if (shift->Opcode() == Op_LShiftX && shift->in(2)->is_Con() &&
      shift->in(2)->get_int() <= 3 &&
      // Are there other uses besides address expressions?
      !matcher->is_visited(shift)) {
    address_visited.set(shift->_idx); // Flag as address_visited
    mstack.push(shift->in(2), Matcher::Visit);
    Node *conv = shift->in(1);
#ifdef _LP64
    // Allow Matcher to match the rule which bypass
    // ConvI2L operation for an array index on LP64
    // if the index value is positive.
    if (conv->Opcode() == Op_ConvI2L &&
        conv->as_Type()->type()->is_long()->_lo >= 0 &&
        // Are there other uses besides address expressions?
        !matcher->is_visited(conv)) {
      address_visited.set(conv->_idx); // Flag as address_visited
      mstack.push(conv->in(1), Matcher::Pre_Visit);
    } else
#endif
      mstack.push(conv, Matcher::Pre_Visit);
    return true;
  }
  return false;
}

// This function identifies sub-graphs in which a 'load' node is
// input to two different nodes, and such that it can be matched
// with BMI instructions like blsi, blsr, etc.
// Example : for b = -a[i] & a[i] can be matched to blsi r32, m32.
// The graph is (AndL (SubL Con0 LoadL*) LoadL*), where LoadL*
// refers to the same node.
//
// Match the generic fused operations pattern (op1 (op2 Con{ConType} mop) mop)
// This is a temporary solution until we make DAGs expressible in ADL.
template<typename ConType>
class FusedPatternMatcher {
  Node* _op1_node;
  Node* _mop_node;
  int _con_op;

  static int match_next(Node* n, int next_op, int next_op_idx) {
    if (n->in(1) == NULL || n->in(2) == NULL) {
      return -1;
    }

    if (next_op_idx == -1) { // n is commutative, try rotations
      if (n->in(1)->Opcode() == next_op) {
        return 1;
      } else if (n->in(2)->Opcode() == next_op) {
        return 2;
      }
    } else {
      assert(next_op_idx > 0 && next_op_idx <= 2, "Bad argument index");
      if (n->in(next_op_idx)->Opcode() == next_op) {
        return next_op_idx;
      }
    }
    return -1;
  }

 public:
  FusedPatternMatcher(Node* op1_node, Node* mop_node, int con_op) :
    _op1_node(op1_node), _mop_node(mop_node), _con_op(con_op) { }

  bool match(int op1, int op1_op2_idx,  // op1 and the index of the op1->op2 edge, -1 if op1 is commutative
             int op2, int op2_con_idx,  // op2 and the index of the op2->con edge, -1 if op2 is commutative
             typename ConType::NativeType con_value) {
    if (_op1_node->Opcode() != op1) {
      return false;
    }
    if (_mop_node->outcnt() > 2) {
      return false;
    }
    op1_op2_idx = match_next(_op1_node, op2, op1_op2_idx);
    if (op1_op2_idx == -1) {
      return false;
    }
    // Memory operation must be the other edge
    int op1_mop_idx = (op1_op2_idx & 1) + 1;

    // Check that the mop node is really what we want
    if (_op1_node->in(op1_mop_idx) == _mop_node) {
      Node* op2_node = _op1_node->in(op1_op2_idx);
      if (op2_node->outcnt() > 1) {
        return false;
      }
      assert(op2_node->Opcode() == op2, "Should be");
      op2_con_idx = match_next(op2_node, _con_op, op2_con_idx);
      if (op2_con_idx == -1) {
        return false;
      }
      // Memory operation must be the other edge
      int op2_mop_idx = (op2_con_idx & 1) + 1;
      // Check that the memory operation is the same node
      if (op2_node->in(op2_mop_idx) == _mop_node) {
        // Now check the constant
        const Type* con_type = op2_node->in(op2_con_idx)->bottom_type();
        if (con_type != Type::TOP && ConType::as_self(con_type)->get_con() == con_value) {
          return true;
        }
      }
    }
    return false;
  }
};

static bool is_bmi_pattern(Node* n, Node* m) {
  assert(UseBMI1Instructions, "sanity");
  if (n != NULL && m != NULL) {
    if (m->Opcode() == Op_LoadI) {
      FusedPatternMatcher<TypeInt> bmii(n, m, Op_ConI);
      return bmii.match(Op_AndI, -1, Op_SubI,  1,  0)  ||
             bmii.match(Op_AndI, -1, Op_AddI, -1, -1)  ||
             bmii.match(Op_XorI, -1, Op_AddI, -1, -1);
    } else if (m->Opcode() == Op_LoadL) {
      FusedPatternMatcher<TypeLong> bmil(n, m, Op_ConL);
      return bmil.match(Op_AndL, -1, Op_SubL,  1,  0) ||
             bmil.match(Op_AndL, -1, Op_AddL, -1, -1) ||
             bmil.match(Op_XorL, -1, Op_AddL, -1, -1);
    }
  }
  return false;
}

// Should the matcher clone input 'm' of node 'n'?
bool Matcher::pd_clone_node(Node* n, Node* m, Matcher::MStack& mstack) {
  // If 'n' and 'm' are part of a graph for BMI instruction, clone the input 'm'.
  if (UseBMI1Instructions && is_bmi_pattern(n, m)) {
    mstack.push(m, Visit);
    return true;
  }
  return false;
}

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?
bool Matcher::pd_clone_address_expressions(AddPNode* m, Matcher::MStack& mstack, VectorSet& address_visited) {
  Node *off = m->in(AddPNode::Offset);
  if (off->is_Con()) {
    address_visited.test_set(m->_idx); // Flag as address_visited
    Node *adr = m->in(AddPNode::Address);

    // Intel can handle 2 adds in addressing mode
    // AtomicAdd is not an addressing expression.
    // Cheap to find it by looking for screwy base.
    if (adr->is_AddP() &&
        !adr->in(AddPNode::Base)->is_top() &&
        LP64_ONLY( off->get_long() == (int) (off->get_long()) && ) // immL32
        // Are there other uses besides address expressions?
        !is_visited(adr)) {
      address_visited.set(adr->_idx); // Flag as address_visited
      Node *shift = adr->in(AddPNode::Offset);
      if (!clone_shift(shift, this, mstack, address_visited)) {
        mstack.push(shift, Pre_Visit);
      }
      mstack.push(adr->in(AddPNode::Address), Pre_Visit);
      mstack.push(adr->in(AddPNode::Base), Pre_Visit);
    } else {
      mstack.push(adr, Pre_Visit);
    }

    // Clone X+offset as it also folds into most addressing expressions
    mstack.push(off, Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  } else if (clone_shift(off, this, mstack, address_visited)) {
    address_visited.test_set(m->_idx); // Flag as address_visited
    mstack.push(m->in(AddPNode::Address), Pre_Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  }
  return false;
}

void Compile::reshape_address(AddPNode* addp) {
}

// Helper methods for MachSpillCopyNode::implementation().
static int vec_mov_helper(CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st) {
  // In 64-bit VM size calculation is very complex. Emitting instructions
  // into scratch buffer is used to get size in 64-bit VM.
  LP64_ONLY( assert(!do_size, "this method calculates size only for 32-bit VM"); )
  assert(ireg == Op_VecS || // 32bit vector
         (src_lo & 1) == 0 && (src_lo + 1) == src_hi &&
         (dst_lo & 1) == 0 && (dst_lo + 1) == dst_hi,
         "no non-adjacent vector moves" );
  if (cbuf) {
    C2_MacroAssembler _masm(cbuf);
    int offset = __ offset();
    switch (ireg) {
    case Op_VecS: // copy whole register
    case Op_VecD:
    case Op_VecX:
#ifndef _LP64
      __ movdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
#else
      if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
        __ movdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      } else {
        __ vextractf32x4(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 0x0);
     }
#endif
      break;
    case Op_VecY:
#ifndef _LP64
      __ vmovdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
#else
      if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
        __ vmovdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      } else {
        __ vextractf64x4(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 0x0);
     }
#endif
      break;
    case Op_VecZ:
      __ evmovdquq(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 2);
      break;
    default:
      ShouldNotReachHere();
    }
    int size = __ offset() - offset;
#ifdef ASSERT
    // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
    assert(!do_size || size == 4, "incorrect size calculattion");
#endif
    return size;
#ifndef PRODUCT
  } else if (!do_size) {
    switch (ireg) {
    case Op_VecS:
    case Op_VecD:
    case Op_VecX:
      st->print("movdqu  %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    case Op_VecY:
    case Op_VecZ:
      st->print("vmovdqu %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
  // VEX_2bytes prefix is used if UseAVX > 0, and it takes the same 2 bytes as SIMD prefix.
  return (UseAVX > 2) ? 6 : 4;
}

int vec_spill_helper(CodeBuffer *cbuf, bool do_size, bool is_load,
                     int stack_offset, int reg, uint ireg, outputStream* st) {
  // In 64-bit VM size calculation is very complex. Emitting instructions
  // into scratch buffer is used to get size in 64-bit VM.
  LP64_ONLY( assert(!do_size, "this method calculates size only for 32-bit VM"); )
  if (cbuf) {
    C2_MacroAssembler _masm(cbuf);
    int offset = __ offset();
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        __ movdl(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecD:
        __ movq(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecX:
#ifndef _LP64
        __ movdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ movdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        } else {
          __ vpxor(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), 2);
          __ vinsertf32x4(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset),0x0);
        }
#endif
        break;
      case Op_VecY:
#ifndef _LP64
        __ vmovdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ vmovdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        } else {
          __ vpxor(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), 2);
          __ vinsertf64x4(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset),0x0);
        }
#endif
        break;
      case Op_VecZ:
        __ evmovdquq(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset), 2);
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        __ movdl(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecD:
        __ movq(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecX:
#ifndef _LP64
        __ movdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ movdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        }
        else {
          __ vextractf32x4(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 0x0);
        }
#endif
        break;
      case Op_VecY:
#ifndef _LP64
        __ vmovdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ vmovdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        }
        else {
          __ vextractf64x4(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 0x0);
        }
#endif
        break;
      case Op_VecZ:
        __ evmovdquq(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 2);
        break;
      default:
        ShouldNotReachHere();
      }
    }
    int size = __ offset() - offset;
#ifdef ASSERT
    int offset_size = (stack_offset == 0) ? 0 : ((stack_offset < 0x80) ? 1 : (UseAVX > 2) ? 6 : 4);
    // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
    assert(!do_size || size == (5+offset_size), "incorrect size calculattion");
#endif
    return size;
#ifndef PRODUCT
  } else if (!do_size) {
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        st->print("movd    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecD:
        st->print("movq    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
       case Op_VecX:
        st->print("movdqu  %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecY:
      case Op_VecZ:
        st->print("vmovdqu %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        st->print("movd    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecD:
        st->print("movq    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
       case Op_VecX:
        st->print("movdqu  [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecY:
      case Op_VecZ:
        st->print("vmovdqu [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      default:
        ShouldNotReachHere();
      }
    }
#endif
  }
  bool is_single_byte = false;
  int vec_len = 0;
  if ((UseAVX > 2) && (stack_offset != 0)) {
    int tuple_type = Assembler::EVEX_FVM;
    int input_size = Assembler::EVEX_32bit;
    switch (ireg) {
    case Op_VecS:
      tuple_type = Assembler::EVEX_T1S;
      break;
    case Op_VecD:
      tuple_type = Assembler::EVEX_T1S;
      input_size = Assembler::EVEX_64bit;
      break;
    case Op_VecX:
      break;
    case Op_VecY:
      vec_len = 1;
      break;
    case Op_VecZ:
      vec_len = 2;
      break;
    }
    is_single_byte = Assembler::query_compressed_disp_byte(stack_offset, true, vec_len, tuple_type, input_size, 0);
  }
  int offset_size = 0;
  int size = 5;
  if (UseAVX > 2 ) {
    if (VM_Version::supports_avx512novl() && (vec_len == 2)) {
      offset_size = (stack_offset == 0) ? 0 : ((is_single_byte) ? 1 : 4);
      size += 2; // Need an additional two bytes for EVEX encoding
    } else if (VM_Version::supports_avx512novl() && (vec_len < 2)) {
      offset_size = (stack_offset == 0) ? 0 : ((stack_offset <= 127) ? 1 : 4);
    } else {
      offset_size = (stack_offset == 0) ? 0 : ((is_single_byte) ? 1 : 4);
      size += 2; // Need an additional two bytes for EVEX encodding
    }
  } else {
    offset_size = (stack_offset == 0) ? 0 : ((stack_offset <= 127) ? 1 : 4);
  }
  // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
  return size+offset_size;
}

static inline jint replicate4_imm(int con, int width) {
  // Load a constant of "width" (in bytes) and replicate it to fill 32bit.
  assert(width == 1 || width == 2, "only byte or short types here");
  int bit_width = width * 8;
  jint val = con;
  val &= (1 << bit_width) - 1;  // mask off sign bits
  while(bit_width < 32) {
    val |= (val << bit_width);
    bit_width <<= 1;
  }
  return val;
}

static inline jlong replicate8_imm(int con, int width) {
  // Load a constant of "width" (in bytes) and replicate it to fill 64bit.
  assert(width == 1 || width == 2 || width == 4, "only byte, short or int types here");
  int bit_width = width * 8;
  jlong val = con;
  val &= (((jlong) 1) << bit_width) - 1;  // mask off sign bits
  while(bit_width < 64) {
    val |= (val << bit_width);
    bit_width <<= 1;
  }
  return val;
}

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    C2_MacroAssembler _masm(&cbuf);
    __ nop(_count);
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count;
  }

#ifndef PRODUCT
  void MachBreakpointNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("# breakpoint");
  }
#endif

  void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc* ra_) const {
    C2_MacroAssembler _masm(&cbuf);
    __ int3();
  }

  uint MachBreakpointNode::size(PhaseRegAlloc* ra_) const {
    return MachNode::size(ra_);
  }


#line 2343 "ad_x86.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // ECX
  'C', // EBX
  'N', // EBP
  'C', // EDI
  'C', // EAX
  'C', // EDX
  'C', // ESI
  'N', // ESP
  'C', // FPR0L
  'C', // FPR0H
  'C', // FPR1L
  'C', // FPR1H
  'C', // FPR2L
  'C', // FPR2H
  'C', // FPR3L
  'C', // FPR3H
  'C', // FPR4L
  'C', // FPR4H
  'C', // FPR5L
  'C', // FPR5H
  'C', // FPR6L
  'C', // FPR6H
  'C', // FPR7L
  'C', // FPR7H
  'C', // FILL0
  'C', // FILL1
  'C', // FILL2
  'C', // FILL3
  'C', // FILL4
  'C', // FILL5
  'C', // FILL6
  'C', // FILL7
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM0i
  'C', // XMM0j
  'C', // XMM0k
  'C', // XMM0l
  'C', // XMM0m
  'C', // XMM0n
  'C', // XMM0o
  'C', // XMM0p
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM1i
  'C', // XMM1j
  'C', // XMM1k
  'C', // XMM1l
  'C', // XMM1m
  'C', // XMM1n
  'C', // XMM1o
  'C', // XMM1p
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM2i
  'C', // XMM2j
  'C', // XMM2k
  'C', // XMM2l
  'C', // XMM2m
  'C', // XMM2n
  'C', // XMM2o
  'C', // XMM2p
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM3i
  'C', // XMM3j
  'C', // XMM3k
  'C', // XMM3l
  'C', // XMM3m
  'C', // XMM3n
  'C', // XMM3o
  'C', // XMM3p
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM4i
  'C', // XMM4j
  'C', // XMM4k
  'C', // XMM4l
  'C', // XMM4m
  'C', // XMM4n
  'C', // XMM4o
  'C', // XMM4p
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM5i
  'C', // XMM5j
  'C', // XMM5k
  'C', // XMM5l
  'C', // XMM5m
  'C', // XMM5n
  'C', // XMM5o
  'C', // XMM5p
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM6i
  'C', // XMM6j
  'C', // XMM6k
  'C', // XMM6l
  'C', // XMM6m
  'C', // XMM6n
  'C', // XMM6o
  'C', // XMM6p
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM7i
  'C', // XMM7j
  'C', // XMM7k
  'C', // XMM7l
  'C', // XMM7m
  'C', // XMM7n
  'C', // XMM7o
  'C', // XMM7p
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // ECX
  'E', // EBX
  'E', // EBP
  'E', // EDI
  'C', // EAX
  'C', // EDX
  'E', // ESI
  'N', // ESP
  'C', // FPR0L
  'C', // FPR0H
  'C', // FPR1L
  'C', // FPR1H
  'C', // FPR2L
  'C', // FPR2H
  'C', // FPR3L
  'C', // FPR3H
  'C', // FPR4L
  'C', // FPR4H
  'C', // FPR5L
  'C', // FPR5H
  'C', // FPR6L
  'C', // FPR6H
  'C', // FPR7L
  'C', // FPR7H
  'C', // FILL0
  'C', // FILL1
  'C', // FILL2
  'C', // FILL3
  'C', // FILL4
  'C', // FILL5
  'C', // FILL6
  'C', // FILL7
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM0i
  'C', // XMM0j
  'C', // XMM0k
  'C', // XMM0l
  'C', // XMM0m
  'C', // XMM0n
  'C', // XMM0o
  'C', // XMM0p
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM1i
  'C', // XMM1j
  'C', // XMM1k
  'C', // XMM1l
  'C', // XMM1m
  'C', // XMM1n
  'C', // XMM1o
  'C', // XMM1p
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM2i
  'C', // XMM2j
  'C', // XMM2k
  'C', // XMM2l
  'C', // XMM2m
  'C', // XMM2n
  'C', // XMM2o
  'C', // XMM2p
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM3i
  'C', // XMM3j
  'C', // XMM3k
  'C', // XMM3l
  'C', // XMM3m
  'C', // XMM3n
  'C', // XMM3o
  'C', // XMM3p
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM4i
  'C', // XMM4j
  'C', // XMM4k
  'C', // XMM4l
  'C', // XMM4m
  'C', // XMM4n
  'C', // XMM4o
  'C', // XMM4p
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM5i
  'C', // XMM5j
  'C', // XMM5k
  'C', // XMM5l
  'C', // XMM5m
  'C', // XMM5n
  'C', // XMM5o
  'C', // XMM5p
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM6i
  'C', // XMM6j
  'C', // XMM6k
  'C', // XMM6l
  'C', // XMM6m
  'C', // XMM6n
  'C', // XMM6o
  'C', // XMM6p
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM7i
  'C', // XMM7j
  'C', // XMM7k
  'C', // XMM7l
  'C', // XMM7m
  'C', // XMM7n
  'C', // XMM7o
  'C', // XMM7p
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI0_rule,
  /*   10 */  immI1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  immI2_rule,
  /*   13 */  immI8_rule,
  /*   14 */  immU8_rule,
  /*   15 */  immI16_rule,
  /*   16 */  immU31_rule,
  /*   17 */  immI_32_rule,
  /*   18 */  immI_1_31_rule,
  /*   19 */  immI_32_63_rule,
  /*   20 */  immI_1_rule,
  /*   21 */  immI_2_rule,
  /*   22 */  immI_3_rule,
  /*   23 */  immP_rule,
  /*   24 */  immP0_rule,
  /*   25 */  immL_rule,
  /*   26 */  immL0_rule,
  /*   27 */  immL_M1_rule,
  /*   28 */  immL_127_rule,
  /*   29 */  immL_32bits_rule,
  /*   30 */  immL32_rule,
  /*   31 */  immDPR0_rule,
  /*   32 */  immDPR1_rule,
  /*   33 */  immDPR_rule,
  /*   34 */  immD_rule,
  /*   35 */  immD0_rule,
  /*   36 */  immFPR0_rule,
  /*   37 */  immFPR1_rule,
  /*   38 */  immFPR_rule,
  /*   39 */  immF_rule,
  /*   40 */  immF0_rule,
  /*   41 */  immI_16_rule,
  /*   42 */  immI_24_rule,
  /*   43 */  immI_255_rule,
  /*   44 */  immI_65535_rule,
  /*   45 */  rRegI_rule,
  /*   46 */  xRegI_rule,
  /*   47 */  eAXRegI_rule,
  /*   48 */  eBXRegI_rule,
  /*   49 */  eCXRegI_rule,
  /*   50 */  eDXRegI_rule,
  /*   51 */  eDIRegI_rule,
  /*   52 */  naxRegI_rule,
  /*   53 */  nadxRegI_rule,
  /*   54 */  ncxRegI_rule,
  /*   55 */  eSIRegI_rule,
  /*   56 */  anyRegP_rule,
  /*   57 */  eRegP_rule,
  /*   58 */  eRegP_no_EBP_rule,
  /*   59 */  naxRegP_rule,
  /*   60 */  nabxRegP_rule,
  /*   61 */  pRegP_rule,
  /*   62 */  eAXRegP_rule,
  /*   63 */  eBXRegP_rule,
  /*   64 */  eCXRegP_rule,
  /*   65 */  eDXRegP_rule,
  /*   66 */  eSIRegP_rule,
  /*   67 */  eDIRegP_rule,
  /*   68 */  eRegL_rule,
  /*   69 */  eADXRegL_rule,
  /*   70 */  eBCXRegL_rule,
  /*   71 */  eADXRegL_low_only_rule,
  /*   72 */  eFlagsReg_rule,
  /*   73 */  eFlagsRegU_rule,
  /*   74 */  eFlagsRegUCF_rule,
  /*   75 */  flagsReg_long_LTGE_rule,
  /*   76 */  flagsReg_long_EQNE_rule,
  /*   77 */  flagsReg_long_LEGT_rule,
  /*   78 */  flagsReg_ulong_LTGE_rule,
  /*   79 */  flagsReg_ulong_EQNE_rule,
  /*   80 */  flagsReg_ulong_LEGT_rule,
  /*   81 */  regDPR_rule,
  /*   82 */  regDPR1_rule,
  /*   83 */  regDPR2_rule,
  /*   84 */  regnotDPR1_rule,
  /*   85 */  regFPR_rule,
  /*   86 */  regFPR1_rule,
  /*   87 */  regF_rule,
  /*   88 */  vlRegF_rule,
  /*   89 */  regD_rule,
  /*   90 */  vlRegD_rule,
  /*   91 */  direct_rule,
  /*   92 */  indirect_rule,
  /*   93 */  indOffset8_rule,
  /*   94 */  indOffset32_rule,
  /*   95 */  indOffset32X_rule,
  /*   96 */  indIndexOffset_rule,
  /*   97 */  indIndex_rule,
  /*   98 */  indIndexScale_rule,
  /*   99 */  indIndexScaleOffset_rule,
  /*  100 */  load_long_RegP_rule,
  /*  101 */  load_long_indirect_rule,
  /*  102 */  load_long_indOffset32_rule,
  /*  103 */  stackSlotP_rule,
  /*  104 */  stackSlotI_rule,
  /*  105 */  stackSlotF_rule,
  /*  106 */  stackSlotD_rule,
  /*  107 */  stackSlotL_rule,
  /*  108 */  indirect_win95_safe_rule,
  /*  109 */  indOffset8_win95_safe_rule,
  /*  110 */  indOffset32_win95_safe_rule,
  /*  111 */  indIndexOffset_win95_safe_rule,
  /*  112 */  indIndexScale_win95_safe_rule,
  /*  113 */  indIndexScaleOffset_win95_safe_rule,
  /*  114 */  cmpOp_rule,
  /*  115 */  cmpOpU_rule,
  /*  116 */  cmpOpUCF_rule,
  /*  117 */  cmpOpUCF2_rule,
  /*  118 */  cmpOp_fcmov_rule,
  /*  119 */  cmpOp_commute_rule,
  /*  120 */  cmpOpU_commute_rule,
  /*  121 */  vec_rule,
  /*  122 */  legVec_rule,
  /*  123 */  vecS_rule,
  /*  124 */  legVecS_rule,
  /*  125 */  vecD_rule,
  /*  126 */  legVecD_rule,
  /*  127 */  vecX_rule,
  /*  128 */  legVecX_rule,
  /*  129 */  vecY_rule,
  /*  130 */  legVecY_rule,
  /*  131 */  vecZ_rule,
  /*  132 */  legVecZ_rule,
  /*  133 */  cmpOp_vcmppd_rule,
  // last operand
  /*  134 */  load_long_memory_rule,
  /*  135 */  memory_rule,
  /*  136 */  long_memory_rule,
  // last operand class
  /*  137 */  _AddP_eRegP_rRegI_rule,
  /*  138 */  _LShiftI_rRegI_immI2_rule,
  /*  139 */  _AddP_eRegP__LShiftI_rRegI_immI2_rule,
  /*  140 */  _AddP_eRegP_no_EBP_rRegI_rule,
  /*  141 */  _AddP_eRegP_no_EBP__LShiftI_rRegI_immI2_rule,
  /*  142 */  _LoadI_memory__rule,
  /*  143 */  _LoadL_memory__rule,
  /*  144 */  _LoadB_memory__rule,
  /*  145 */  _LoadUB_memory__rule,
  /*  146 */  _AndI__LoadUB_memory__immI_rule,
  /*  147 */  _LoadS_memory__rule,
  /*  148 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  149 */  _LoadUS_memory__rule,
  /*  150 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  151 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  152 */  _AndI__LoadUS_memory__immI_rule,
  /*  153 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  154 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  155 */  _AndI__LoadI_memory__immI_255_rule,
  /*  156 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  157 */  _AndI__LoadI_memory__immU31_rule,
  /*  158 */  _ConvI2L__LoadI_memory___rule,
  /*  159 */  _ConvL2I_eRegL__rule,
  /*  160 */  _RoundDouble_regDPR1__rule,
  /*  161 */  _RoundFloat_regFPR1__rule,
  /*  162 */  _ConvD2F_regDPR1__rule,
  /*  163 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  164 */  _Binary_rRegI_rRegI_rule,
  /*  165 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  166 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  167 */  _Binary_rRegI__LoadI_memory__rule,
  /*  168 */  _Binary_eRegP_eRegP_rule,
  /*  169 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  170 */  _Binary_regDPR1_regDPR_rule,
  /*  171 */  _Binary_regFPR1_regFPR_rule,
  /*  172 */  _Binary_regDPR_regDPR_rule,
  /*  173 */  _Binary_regFPR_regFPR_rule,
  /*  174 */  _Binary_regF_regF_rule,
  /*  175 */  _Binary_regD_regD_rule,
  /*  176 */  _Binary_eRegL_eRegL_rule,
  /*  177 */  _AddI__LoadI_memory__rRegI_rule,
  /*  178 */  _AddI_rRegI__LoadI_memory__rule,
  /*  179 */  _AddI__LoadI_memory__immI_rule,
  /*  180 */  _AddI__LoadI_memory__immI1_rule,
  /*  181 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  182 */  _Binary_eAXRegP_eRegP_rule,
  /*  183 */  _Binary_eAXRegI_rRegI_rule,
  /*  184 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  185 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  186 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  187 */  _SubI__LoadI_memory__rRegI_rule,
  /*  188 */  _SubI_immI0_rRegI_rule,
  /*  189 */  _ConvI2L_nadxRegI__rule,
  /*  190 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  191 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule,
  /*  192 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  193 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule,
  /*  194 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule,
  /*  195 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule,
  /*  196 */  _ConvI2L_eAXRegI__rule,
  /*  197 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  198 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  199 */  _RShiftI__LoadI_memory__immI1_rule,
  /*  200 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  201 */  _LShiftI_xRegI_immI_24_rule,
  /*  202 */  _LShiftI_xRegI_immI_16_rule,
  /*  203 */  _AndI__LoadI_memory__rRegI_rule,
  /*  204 */  _AndI_rRegI__LoadI_memory__rule,
  /*  205 */  _AndI__LoadI_memory__immI_rule,
  /*  206 */  _XorI_rRegI_immI_M1_rule,
  /*  207 */  _SubI_immI0__LoadI_memory__rule,
  /*  208 */  _AddI_rRegI_immI_M1_rule,
  /*  209 */  _CastP2X_eRegP__rule,
  /*  210 */  _OrI__LoadI_memory__rRegI_rule,
  /*  211 */  _OrI_rRegI__LoadI_memory__rule,
  /*  212 */  _OrI__LoadI_memory__immI_rule,
  /*  213 */  _LShiftI_rRegI_immI1_rule,
  /*  214 */  _URShiftI_rRegI_immI_M1_rule,
  /*  215 */  _LShiftI_rRegI_immI8_rule,
  /*  216 */  _URShiftI_rRegI_immI8_rule,
  /*  217 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  218 */  _SubI_immI0_eCXRegI_rule,
  /*  219 */  _URShiftI_ncxRegI__SubI_immI0_eCXRegI_rule,
  /*  220 */  _SubI_immI_32_eCXRegI_rule,
  /*  221 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  222 */  _URShiftI_rRegI_immI1_rule,
  /*  223 */  _LShiftI_rRegI_immI_M1_rule,
  /*  224 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  225 */  _LShiftI_ncxRegI__SubI_immI0_eCXRegI_rule,
  /*  226 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  227 */  _XorI__LoadI_memory__rRegI_rule,
  /*  228 */  _XorI_rRegI__LoadI_memory__rule,
  /*  229 */  _XorI__LoadI_memory__immI_rule,
  /*  230 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  231 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  232 */  _SubI_rRegI_rRegI_rule,
  /*  233 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  234 */  _LoadL_load_long_memory__rule,
  /*  235 */  _XorL_eRegL_immL_M1_rule,
  /*  236 */  _SubL_immL0_eRegL_rule,
  /*  237 */  _SubL_immL0__LoadL_memory__rule,
  /*  238 */  _AddL_eRegL_immL_M1_rule,
  /*  239 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  240 */  _LoadD_memory__rule,
  /*  241 */  _SubD_regDPR_regDPR_rule,
  /*  242 */  _AddD_regDPR_regDPR_rule,
  /*  243 */  _AddD__LoadD_memory__regDPR_rule,
  /*  244 */  _RoundDouble__AddD__LoadD_memory__regDPR__rule,
  /*  245 */  _AddD_regDPR__LoadD_memory__rule,
  /*  246 */  _RoundDouble__AddD_regDPR__LoadD_memory___rule,
  /*  247 */  _AddD_regDPR_immDPR_rule,
  /*  248 */  _MulD_regDPR_regDPR_rule,
  /*  249 */  _DivD_regDPR_regDPR_rule,
  /*  250 */  _LoadF_memory__rule,
  /*  251 */  _MulF__LoadF_memory__regFPR_rule,
  /*  252 */  _MulF_regFPR__LoadF_memory__rule,
  /*  253 */  _MulF_regFPR_regFPR_rule,
  /*  254 */  _SubF_regFPR_regFPR_rule,
  /*  255 */  _ConvI2L_rRegI__rule,
  /*  256 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  257 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  258 */  _Binary_eDIRegP_eSIRegP_rule,
  /*  259 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  260 */  _Binary_eSIRegP_immI_rule,
  /*  261 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  262 */  _AndI_rRegI_immI_rule,
  /*  263 */  _AndI_rRegI_memory_rule,
  /*  264 */  _LoadP_memory__rule,
  /*  265 */  _Binary_nadxRegI_eAXRegI_rule,
  /*  266 */  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule,
  /*  267 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  268 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  269 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  270 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  271 */  _LoadVector_memory__rule,
  /*  272 */  _Binary_cmpOp_vcmppd_immI8_rule,
  /*  273 */  _Binary_legVec_legVec_rule,
  /*  274 */  _Binary_vec_vec_rule,
  /*  275 */  _Binary_vec__LoadVector_memory__rule,
  /*  276 */  _MulAddVS2VI_vec_vec_rule,
  /*  277 */  _Binary_vec_immU8_rule,
  /*  278 */  _Binary__LoadVector_memory__immU8_rule,
  // last internally defined operand
  /*  279 */  stackSlotI_rule,
  /*  280 */  stackSlotP_rule,
  /*  281 */  stackSlotL_rule,
  /*  282 */  eRegP_rule,
  /*  283 */  eRegP_rule,
  /*  284 */  eRegP_rule,
  /*  285 */  eRegP_rule,
  /*  286 */  eRegP_rule,
  /*  287 */  rRegI_rule,
  /*  288 */  rRegI_rule,
  /*  289 */  eRegP_rule,
  /*  290 */  eRegL_rule,
  /*  291 */  eRegL_rule,
  /*  292 */  regFPR_rule,
  /*  293 */  regFPR_rule,
  /*  294 */  regFPR_rule,
  /*  295 */  regF_rule,
  /*  296 */  regF_rule,
  /*  297 */  regDPR_rule,
  /*  298 */  regDPR_rule,
  /*  299 */  regDPR_rule,
  /*  300 */  regD_rule,
  /*  301 */  regD_rule,
  /*  302 */  rRegI_rule,
  /*  303 */  eRegL_rule,
  /*  304 */  eRegP_rule,
  /*  305 */  regFPR_rule,
  /*  306 */  regDPR_rule,
  /*  307 */  vlRegD_rule,
  /*  308 */  regD_rule,
  /*  309 */  vlRegF_rule,
  /*  310 */  regF_rule,
  /*  311 */  eADXRegL_low_only_rule,
  /*  312 */  legVec_rule,
  /*  313 */  vec_rule,
  /*  314 */  Universe_rule,
  /*  315 */  eAXRegP_rule,
  /*  316 */  rRegI_rule,
  /*  317 */  eRegP_rule,
  /*  318 */  eRegP_rule,
  /*  319 */  rRegI_rule,
  /*  320 */  eFlagsReg_rule,
  /*  321 */  eFlagsReg_rule,
  /*  322 */  eFlagsReg_rule,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  eFlagsReg_rule,
  /*  326 */  eFlagsReg_rule,
  /*  327 */  eFlagsReg_rule,
  /*  328 */  eFlagsReg_rule,
  /*  329 */  eFlagsReg_rule,
  /*  330 */  eFlagsReg_rule,
  /*  331 */  eFlagsReg_rule,
  /*  332 */  eFlagsRegU_rule,
  /*  333 */  eFlagsRegUCF_rule,
  /*  334 */  eFlagsRegU_rule,
  /*  335 */  eFlagsRegU_rule,
  /*  336 */  eFlagsRegUCF_rule,
  /*  337 */  eFlagsRegU_rule,
  /*  338 */  eFlagsRegUCF_rule,
  /*  339 */  regDPR1_rule,
  /*  340 */  regDPR1_rule,
  /*  341 */  eFlagsRegU_rule,
  /*  342 */  eFlagsRegUCF_rule,
  /*  343 */  eFlagsRegU_rule,
  /*  344 */  eFlagsRegU_rule,
  /*  345 */  eFlagsRegUCF_rule,
  /*  346 */  eFlagsRegU_rule,
  /*  347 */  eFlagsRegUCF_rule,
  /*  348 */  regFPR1_rule,
  /*  349 */  regFPR1_rule,
  /*  350 */  rRegI_rule,
  /*  351 */  rRegI_rule,
  /*  352 */  regFPR_rule,
  /*  353 */  regF_rule,
  /*  354 */  regF_rule,
  /*  355 */  eRegL_rule,
  /*  356 */  regDPR_rule,
  /*  357 */  regD_rule,
  /*  358 */  regD_rule,
  /*  359 */  eFlagsReg_rule,
  /*  360 */  eFlagsReg_rule,
  /*  361 */  eFlagsReg_rule,
  /*  362 */  eFlagsReg_rule,
  /*  363 */  eFlagsReg_rule,
  /*  364 */  eFlagsReg_rule,
  /*  365 */  eFlagsRegU_rule,
  /*  366 */  eFlagsRegU_rule,
  /*  367 */  eFlagsRegU_rule,
  /*  368 */  eFlagsRegU_rule,
  /*  369 */  eFlagsRegU_rule,
  /*  370 */  eFlagsRegU_rule,
  /*  371 */  eFlagsRegU_rule,
  /*  372 */  eFlagsRegU_rule,
  /*  373 */  eFlagsReg_rule,
  /*  374 */  eFlagsReg_rule,
  /*  375 */  eFlagsReg_rule,
  /*  376 */  flagsReg_long_LTGE_rule,
  /*  377 */  flagsReg_long_LTGE_rule,
  /*  378 */  flagsReg_ulong_LTGE_rule,
  /*  379 */  flagsReg_ulong_LTGE_rule,
  /*  380 */  flagsReg_long_EQNE_rule,
  /*  381 */  flagsReg_long_EQNE_rule,
  /*  382 */  flagsReg_ulong_EQNE_rule,
  /*  383 */  flagsReg_ulong_EQNE_rule,
  /*  384 */  flagsReg_long_LEGT_rule,
  /*  385 */  flagsReg_long_LEGT_rule,
  /*  386 */  flagsReg_ulong_LEGT_rule,
  /*  387 */  flagsReg_ulong_LEGT_rule,
  /*  388 */  eFlagsReg_rule,
  /*  389 */  eFlagsReg_rule,
  /*  390 */  eFlagsReg_rule,
  /*  391 */  regF_rule,
  /*  392 */  vlRegF_rule,
  /*  393 */  regD_rule,
  /*  394 */  vlRegD_rule,
  /*  395 */  regF_rule,
  /*  396 */  vlRegF_rule,
  /*  397 */  regD_rule,
  /*  398 */  vlRegD_rule,
  /*  399 */  vec_rule,
  /*  400 */  vec_rule,
  /*  401 */  vec_rule,
  /*  402 */  vec_rule,
  /*  403 */  vec_rule,
  /*  404 */  vec_rule,
  /*  405 */  vec_rule,
  /*  406 */  vec_rule,
  /*  407 */  vec_rule,
  /*  408 */  vec_rule,
  /*  409 */  vec_rule,
  /*  410 */  rRegI_rule,
  /*  411 */  eRegL_rule,
  /*  412 */  rRegI_rule,
  /*  413 */  rRegI_rule,
  /*  414 */  rRegI_rule,
  /*  415 */  rRegI_rule,
  /*  416 */  rRegI_rule,
  /*  417 */  rRegI_rule,
  /*  418 */  rRegI_rule,
  /*  419 */  rRegI_rule,
  /*  420 */  rRegI_rule,
  /*  421 */  rRegI_rule,
  /*  422 */  rRegI_rule,
  /*  423 */  rRegI_rule,
  /*  424 */  rRegI_rule,
  /*  425 */  rRegI_rule,
  /*  426 */  xRegI_rule,
  /*  427 */  eRegL_rule,
  /*  428 */  xRegI_rule,
  /*  429 */  eRegL_rule,
  /*  430 */  eRegL_rule,
  /*  431 */  rRegI_rule,
  /*  432 */  rRegI_rule,
  /*  433 */  eRegL_rule,
  /*  434 */  rRegI_rule,
  /*  435 */  rRegI_rule,
  /*  436 */  eRegL_rule,
  /*  437 */  eRegL_rule,
  /*  438 */  eRegL_rule,
  /*  439 */  rRegI_rule,
  /*  440 */  rRegI_rule,
  /*  441 */  rRegI_rule,
  /*  442 */  rRegI_rule,
  /*  443 */  rRegI_rule,
  /*  444 */  eRegL_rule,
  /*  445 */  eRegL_rule,
  /*  446 */  eRegL_rule,
  /*  447 */  eRegL_rule,
  /*  448 */  eRegL_rule,
  /*  449 */  eRegL_rule,
  /*  450 */  stackSlotL_rule,
  /*  451 */  stackSlotL_rule,
  /*  452 */  eRegL_rule,
  /*  453 */  rRegI_rule,
  /*  454 */  eRegP_rule,
  /*  455 */  eRegP_rule,
  /*  456 */  regDPR_rule,
  /*  457 */  regD_rule,
  /*  458 */  regD_rule,
  /*  459 */  regF_rule,
  /*  460 */  regFPR_rule,
  /*  461 */  Universe_rule,
  /*  462 */  Universe_rule,
  /*  463 */  Universe_rule,
  /*  464 */  Universe_rule,
  /*  465 */  Universe_rule,
  /*  466 */  Universe_rule,
  /*  467 */  Universe_rule,
  /*  468 */  Universe_rule,
  /*  469 */  Universe_rule,
  /*  470 */  Universe_rule,
  /*  471 */  Universe_rule,
  /*  472 */  Universe_rule,
  /*  473 */  Universe_rule,
  /*  474 */  Universe_rule,
  /*  475 */  Universe_rule,
  /*  476 */  Universe_rule,
  /*  477 */  Universe_rule,
  /*  478 */  Universe_rule,
  /*  479 */  Universe_rule,
  /*  480 */  Universe_rule,
  /*  481 */  Universe_rule,
  /*  482 */  Universe_rule,
  /*  483 */  Universe_rule,
  /*  484 */  Universe_rule,
  /*  485 */  Universe_rule,
  /*  486 */  Universe_rule,
  /*  487 */  Universe_rule,
  /*  488 */  Universe_rule,
  /*  489 */  Universe_rule,
  /*  490 */  Universe_rule,
  /*  491 */  Universe_rule,
  /*  492 */  Universe_rule,
  /*  493 */  Universe_rule,
  /*  494 */  Universe_rule,
  /*  495 */  Universe_rule,
  /*  496 */  Universe_rule,
  /*  497 */  rRegI_rule,
  /*  498 */  rRegI_rule,
  /*  499 */  rRegI_rule,
  /*  500 */  rRegI_rule,
  /*  501 */  rRegI_rule,
  /*  502 */  rRegI_rule,
  /*  503 */  rRegI_rule,
  /*  504 */  rRegI_rule,
  /*  505 */  eRegP_rule,
  /*  506 */  eRegP_rule,
  /*  507 */  eRegP_rule,
  /*  508 */  eRegP_rule,
  /*  509 */  regDPR1_rule,
  /*  510 */  regFPR1_rule,
  /*  511 */  regDPR_rule,
  /*  512 */  regFPR_rule,
  /*  513 */  regF_rule,
  /*  514 */  regD_rule,
  /*  515 */  regF_rule,
  /*  516 */  regF_rule,
  /*  517 */  regD_rule,
  /*  518 */  regD_rule,
  /*  519 */  eRegL_rule,
  /*  520 */  eRegL_rule,
  /*  521 */  eRegL_rule,
  /*  522 */  rRegI_rule,
  /*  523 */  rRegI_rule,
  /*  524 */  rRegI_rule,
  /*  525 */  rRegI_rule,
  /*  526 */  eRegP_rule,
  /*  527 */  rRegI_rule,
  /*  528 */  eRegP_rule,
  /*  529 */  eRegP_rule,
  /*  530 */  rRegI_rule,
  /*  531 */  rRegI_rule,
  /*  532 */  Universe_rule,
  /*  533 */  Universe_rule,
  /*  534 */  Universe_rule,
  /*  535 */  Universe_rule,
  /*  536 */  Universe_rule,
  /*  537 */  eRegP_rule,
  /*  538 */  rRegI_rule,
  /*  539 */  rRegI_rule,
  /*  540 */  rRegI_rule,
  /*  541 */  rRegI_rule,
  /*  542 */  rRegI_rule,
  /*  543 */  rRegI_rule,
  /*  544 */  rRegI_rule,
  /*  545 */  rRegI_rule,
  /*  546 */  rRegI_rule,
  /*  547 */  rRegI_rule,
  /*  548 */  eADXRegL_rule,
  /*  549 */  eAXRegP_rule,
  /*  550 */  eAXRegI_rule,
  /*  551 */  eAXRegI_rule,
  /*  552 */  eAXRegI_rule,
  /*  553 */  Universe_rule,
  /*  554 */  xRegI_rule,
  /*  555 */  Universe_rule,
  /*  556 */  rRegI_rule,
  /*  557 */  Universe_rule,
  /*  558 */  rRegI_rule,
  /*  559 */  xRegI_rule,
  /*  560 */  rRegI_rule,
  /*  561 */  rRegI_rule,
  /*  562 */  pRegP_rule,
  /*  563 */  rRegI_rule,
  /*  564 */  rRegI_rule,
  /*  565 */  rRegI_rule,
  /*  566 */  Universe_rule,
  /*  567 */  eRegP_rule,
  /*  568 */  rRegI_rule,
  /*  569 */  rRegI_rule,
  /*  570 */  rRegI_rule,
  /*  571 */  eDXRegI_rule,
  /*  572 */  eDXRegI_rule,
  /*  573 */  eDXRegI_rule,
  /*  574 */  eDXRegI_rule,
  /*  575 */  rRegI_rule,
  /*  576 */  rRegI_rule,
  /*  577 */  rRegI_rule,
  /*  578 */  rRegI_rule,
  /*  579 */  eADXRegL_rule,
  /*  580 */  eADXRegL_rule,
  /*  581 */  eADXRegL_rule,
  /*  582 */  eADXRegL_rule,
  /*  583 */  eADXRegL_rule,
  /*  584 */  eADXRegL_rule,
  /*  585 */  eADXRegL_rule,
  /*  586 */  eADXRegL_rule,
  /*  587 */  eADXRegL_rule,
  /*  588 */  eAXRegI_rule,
  /*  589 */  eADXRegL_rule,
  /*  590 */  Universe_rule,
  /*  591 */  eDXRegI_rule,
  /*  592 */  eADXRegL_rule,
  /*  593 */  eADXRegL_rule,
  /*  594 */  eADXRegL_rule,
  /*  595 */  rRegI_rule,
  /*  596 */  rRegI_rule,
  /*  597 */  rRegI_rule,
  /*  598 */  rRegI_rule,
  /*  599 */  Universe_rule,
  /*  600 */  rRegI_rule,
  /*  601 */  Universe_rule,
  /*  602 */  rRegI_rule,
  /*  603 */  rRegI_rule,
  /*  604 */  rRegI_rule,
  /*  605 */  rRegI_rule,
  /*  606 */  rRegI_rule,
  /*  607 */  rRegI_rule,
  /*  608 */  rRegI_rule,
  /*  609 */  rRegI_rule,
  /*  610 */  rRegI_rule,
  /*  611 */  rRegI_rule,
  /*  612 */  Universe_rule,
  /*  613 */  Universe_rule,
  /*  614 */  Universe_rule,
  /*  615 */  rRegI_rule,
  /*  616 */  rRegI_rule,
  /*  617 */  rRegI_rule,
  /*  618 */  rRegI_rule,
  /*  619 */  rRegI_rule,
  /*  620 */  rRegI_rule,
  /*  621 */  rRegI_rule,
  /*  622 */  rRegI_rule,
  /*  623 */  rRegI_rule,
  /*  624 */  rRegI_rule,
  /*  625 */  rRegI_rule,
  /*  626 */  rRegI_rule,
  /*  627 */  rRegI_rule,
  /*  628 */  rRegI_rule,
  /*  629 */  rRegI_rule,
  /*  630 */  rRegI_rule,
  /*  631 */  rRegI_rule,
  /*  632 */  rRegI_rule,
  /*  633 */  rRegI_rule,
  /*  634 */  rRegI_rule,
  /*  635 */  rRegI_rule,
  /*  636 */  rRegI_rule,
  /*  637 */  Universe_rule,
  /*  638 */  Universe_rule,
  /*  639 */  Universe_rule,
  /*  640 */  0,
  /*  641 */  0,
  /*  642 */  0,
  /*  643 */  rRegI_rule,
  /*  644 */  rRegI_rule,
  /*  645 */  rRegI_rule,
  /*  646 */  rRegI_rule,
  /*  647 */  ncxRegI_rule,
  /*  648 */  ncxRegI_rule,
  /*  649 */  ncxRegI_rule,
  /*  650 */  ncxRegI_rule,
  /*  651 */  0,
  /*  652 */  0,
  /*  653 */  0,
  /*  654 */  rRegI_rule,
  /*  655 */  rRegI_rule,
  /*  656 */  rRegI_rule,
  /*  657 */  rRegI_rule,
  /*  658 */  ncxRegI_rule,
  /*  659 */  ncxRegI_rule,
  /*  660 */  ncxRegI_rule,
  /*  661 */  ncxRegI_rule,
  /*  662 */  rRegI_rule,
  /*  663 */  rRegI_rule,
  /*  664 */  rRegI_rule,
  /*  665 */  rRegI_rule,
  /*  666 */  rRegI_rule,
  /*  667 */  Universe_rule,
  /*  668 */  Universe_rule,
  /*  669 */  Universe_rule,
  /*  670 */  0,
  /*  671 */  rRegI_rule,
  /*  672 */  0,
  /*  673 */  rRegI_rule,
  /*  674 */  eCXRegI_rule,
  /*  675 */  rRegI_rule,
  /*  676 */  rRegI_rule,
  /*  677 */  rRegI_rule,
  /*  678 */  rRegI_rule,
  /*  679 */  rRegI_rule,
  /*  680 */  rRegI_rule,
  /*  681 */  rRegI_rule,
  /*  682 */  rRegI_rule,
  /*  683 */  eRegL_rule,
  /*  684 */  eRegL_rule,
  /*  685 */  eRegL_rule,
  /*  686 */  eRegL_rule,
  /*  687 */  eRegL_rule,
  /*  688 */  eRegL_rule,
  /*  689 */  eRegL_rule,
  /*  690 */  eRegL_rule,
  /*  691 */  eRegL_rule,
  /*  692 */  eRegL_rule,
  /*  693 */  eRegL_rule,
  /*  694 */  eRegL_rule,
  /*  695 */  eRegL_rule,
  /*  696 */  eRegL_rule,
  /*  697 */  eRegL_rule,
  /*  698 */  eRegL_rule,
  /*  699 */  eRegL_rule,
  /*  700 */  eRegL_rule,
  /*  701 */  eRegL_rule,
  /*  702 */  eRegL_rule,
  /*  703 */  eRegL_rule,
  /*  704 */  eRegL_rule,
  /*  705 */  eRegL_rule,
  /*  706 */  eRegL_rule,
  /*  707 */  eRegL_rule,
  /*  708 */  eRegL_rule,
  /*  709 */  eRegL_rule,
  /*  710 */  eRegL_rule,
  /*  711 */  eRegL_rule,
  /*  712 */  eRegL_rule,
  /*  713 */  eRegL_rule,
  /*  714 */  eRegL_rule,
  /*  715 */  eRegL_rule,
  /*  716 */  eRegL_rule,
  /*  717 */  eRegL_rule,
  /*  718 */  eRegL_rule,
  /*  719 */  eRegL_rule,
  /*  720 */  eRegL_rule,
  /*  721 */  eRegL_rule,
  /*  722 */  eRegL_rule,
  /*  723 */  eRegL_rule,
  /*  724 */  eRegL_rule,
  /*  725 */  eRegL_rule,
  /*  726 */  eRegL_rule,
  /*  727 */  eRegL_rule,
  /*  728 */  eRegL_rule,
  /*  729 */  eRegL_rule,
  /*  730 */  eRegL_rule,
  /*  731 */  eRegL_rule,
  /*  732 */  rRegI_rule,
  /*  733 */  rRegI_rule,
  /*  734 */  xRegI_rule,
  /*  735 */  xRegI_rule,
  /*  736 */  regDPR_rule,
  /*  737 */  stackSlotD_rule,
  /*  738 */  regDPR_rule,
  /*  739 */  regDPR_rule,
  /*  740 */  stackSlotD_rule,
  /*  741 */  regDPR_rule,
  /*  742 */  regDPR_rule,
  /*  743 */  Universe_rule,
  /*  744 */  Universe_rule,
  /*  745 */  regDPR_rule,
  /*  746 */  regDPR_rule,
  /*  747 */  stackSlotD_rule,
  /*  748 */  regDPR_rule,
  /*  749 */  regDPR1_rule,
  /*  750 */  regDPR_rule,
  /*  751 */  regDPR_rule,
  /*  752 */  regDPR_rule,
  /*  753 */  regDPR_rule,
  /*  754 */  regDPR_rule,
  /*  755 */  regDPR_rule,
  /*  756 */  regDPR_rule,
  /*  757 */  regDPR_rule,
  /*  758 */  regDPR_rule,
  /*  759 */  regDPR1_rule,
  /*  760 */  stackSlotD_rule,
  /*  761 */  regDPR_rule,
  /*  762 */  regD_rule,
  /*  763 */  regDPR_rule,
  /*  764 */  regD_rule,
  /*  765 */  regDPR_rule,
  /*  766 */  rRegI_rule,
  /*  767 */  rRegI_rule,
  /*  768 */  xRegI_rule,
  /*  769 */  xRegI_rule,
  /*  770 */  stackSlotF_rule,
  /*  771 */  regFPR_rule,
  /*  772 */  stackSlotF_rule,
  /*  773 */  regFPR_rule,
  /*  774 */  stackSlotF_rule,
  /*  775 */  stackSlotF_rule,
  /*  776 */  regFPR_rule,
  /*  777 */  regFPR_rule,
  /*  778 */  stackSlotF_rule,
  /*  779 */  stackSlotF_rule,
  /*  780 */  stackSlotF_rule,
  /*  781 */  stackSlotF_rule,
  /*  782 */  stackSlotF_rule,
  /*  783 */  regFPR_rule,
  /*  784 */  stackSlotF_rule,
  /*  785 */  regFPR_rule,
  /*  786 */  stackSlotF_rule,
  /*  787 */  stackSlotF_rule,
  /*  788 */  regFPR_rule,
  /*  789 */  regFPR_rule,
  /*  790 */  stackSlotF_rule,
  /*  791 */  stackSlotF_rule,
  /*  792 */  regFPR_rule,
  /*  793 */  regFPR_rule,
  /*  794 */  regFPR_rule,
  /*  795 */  regFPR_rule,
  /*  796 */  regFPR_rule,
  /*  797 */  regFPR_rule,
  /*  798 */  regFPR_rule,
  /*  799 */  regFPR_rule,
  /*  800 */  regFPR_rule,
  /*  801 */  regFPR_rule,
  /*  802 */  stackSlotF_rule,
  /*  803 */  regFPR_rule,
  /*  804 */  stackSlotF_rule,
  /*  805 */  regFPR_rule,
  /*  806 */  regF_rule,
  /*  807 */  stackSlotF_rule,
  /*  808 */  stackSlotD_rule,
  /*  809 */  stackSlotF_rule,
  /*  810 */  regF_rule,
  /*  811 */  regF_rule,
  /*  812 */  regDPR_rule,
  /*  813 */  stackSlotD_rule,
  /*  814 */  regDPR_rule,
  /*  815 */  regD_rule,
  /*  816 */  eAXRegI_rule,
  /*  817 */  eAXRegI_rule,
  /*  818 */  eADXRegL_rule,
  /*  819 */  eADXRegL_rule,
  /*  820 */  eAXRegI_rule,
  /*  821 */  eAXRegI_rule,
  /*  822 */  eADXRegL_rule,
  /*  823 */  eADXRegL_rule,
  /*  824 */  regDPR_rule,
  /*  825 */  regD_rule,
  /*  826 */  regD_rule,
  /*  827 */  regD_rule,
  /*  828 */  regDPR_rule,
  /*  829 */  regFPR_rule,
  /*  830 */  stackSlotF_rule,
  /*  831 */  stackSlotF_rule,
  /*  832 */  regFPR_rule,
  /*  833 */  regFPR_rule,
  /*  834 */  regF_rule,
  /*  835 */  regF_rule,
  /*  836 */  eRegL_rule,
  /*  837 */  eRegL_rule,
  /*  838 */  eRegL_rule,
  /*  839 */  stackSlotD_rule,
  /*  840 */  regD_rule,
  /*  841 */  regF_rule,
  /*  842 */  stackSlotF_rule,
  /*  843 */  rRegI_rule,
  /*  844 */  stackSlotI_rule,
  /*  845 */  stackSlotI_rule,
  /*  846 */  stackSlotF_rule,
  /*  847 */  stackSlotL_rule,
  /*  848 */  stackSlotL_rule,
  /*  849 */  eRegL_rule,
  /*  850 */  stackSlotD_rule,
  /*  851 */  regD_rule,
  /*  852 */  Universe_rule,
  /*  853 */  Universe_rule,
  /*  854 */  eAXRegI_rule,
  /*  855 */  eAXRegI_rule,
  /*  856 */  eAXRegI_rule,
  /*  857 */  eAXRegI_rule,
  /*  858 */  eAXRegI_rule,
  /*  859 */  eBXRegI_rule,
  /*  860 */  eBXRegI_rule,
  /*  861 */  eBXRegI_rule,
  /*  862 */  eBXRegI_rule,
  /*  863 */  eBXRegI_rule,
  /*  864 */  eBXRegI_rule,
  /*  865 */  eBXRegI_rule,
  /*  866 */  eAXRegI_rule,
  /*  867 */  eAXRegI_rule,
  /*  868 */  eAXRegI_rule,
  /*  869 */  eAXRegI_rule,
  /*  870 */  Universe_rule,
  /*  871 */  eAXRegI_rule,
  /*  872 */  rRegI_rule,
  /*  873 */  rRegI_rule,
  /*  874 */  eAXRegI_rule,
  /*  875 */  Universe_rule,
  /*  876 */  Universe_rule,
  /*  877 */  Universe_rule,
  /*  878 */  Universe_rule,
  /*  879 */  Universe_rule,
  /*  880 */  Universe_rule,
  /*  881 */  Universe_rule,
  /*  882 */  Universe_rule,
  /*  883 */  Universe_rule,
  /*  884 */  Universe_rule,
  /*  885 */  Universe_rule,
  /*  886 */  Universe_rule,
  /*  887 */  eDIRegP_rule,
  /*  888 */  Universe_rule,
  /*  889 */  Universe_rule,
  /*  890 */  Universe_rule,
  /*  891 */  Universe_rule,
  /*  892 */  Universe_rule,
  /*  893 */  Universe_rule,
  /*  894 */  Universe_rule,
  /*  895 */  Universe_rule,
  /*  896 */  eSIRegI_rule,
  /*  897 */  Universe_rule,
  /*  898 */  Universe_rule,
  /*  899 */  eRegL_rule,
  /*  900 */  eRegL_rule,
  /*  901 */  rRegI_rule,
  /*  902 */  rRegI_rule,
  /*  903 */  eRegP_rule,
  /*  904 */  regDPR_rule,
  /*  905 */  regD_rule,
  /*  906 */  regFPR_rule,
  /*  907 */  regF_rule,
  /*  908 */  Universe_rule,
  /*  909 */  Universe_rule,
  /*  910 */  eRegL_rule,
  /*  911 */  eRegL_rule,
  /*  912 */  rRegI_rule,
  /*  913 */  rRegI_rule,
  /*  914 */  eRegP_rule,
  /*  915 */  regDPR_rule,
  /*  916 */  regD_rule,
  /*  917 */  regFPR_rule,
  /*  918 */  regF_rule,
  /*  919 */  Universe_rule,
  /*  920 */  Universe_rule,
  /*  921 */  eRegL_rule,
  /*  922 */  eRegL_rule,
  /*  923 */  rRegI_rule,
  /*  924 */  rRegI_rule,
  /*  925 */  eRegP_rule,
  /*  926 */  regDPR_rule,
  /*  927 */  regD_rule,
  /*  928 */  regFPR_rule,
  /*  929 */  regF_rule,
  /*  930 */  Universe_rule,
  /*  931 */  Universe_rule,
  /*  932 */  Universe_rule,
  /*  933 */  Universe_rule,
  /*  934 */  Universe_rule,
  /*  935 */  Universe_rule,
  /*  936 */  Universe_rule,
  /*  937 */  Universe_rule,
  /*  938 */  eAXRegP_rule,
  /*  939 */  Universe_rule,
  /*  940 */  Universe_rule,
  /*  941 */  eRegP_rule,
  /*  942 */  Universe_rule,
  /*  943 */  rRegI_rule,
  /*  944 */  regF_rule,
  /*  945 */  regF_rule,
  /*  946 */  regF_rule,
  /*  947 */  regF_rule,
  /*  948 */  regF_rule,
  /*  949 */  regF_rule,
  /*  950 */  regF_rule,
  /*  951 */  regF_rule,
  /*  952 */  regD_rule,
  /*  953 */  regD_rule,
  /*  954 */  regD_rule,
  /*  955 */  regD_rule,
  /*  956 */  regD_rule,
  /*  957 */  regD_rule,
  /*  958 */  regD_rule,
  /*  959 */  regD_rule,
  /*  960 */  regF_rule,
  /*  961 */  regF_rule,
  /*  962 */  regF_rule,
  /*  963 */  regF_rule,
  /*  964 */  regF_rule,
  /*  965 */  regF_rule,
  /*  966 */  regD_rule,
  /*  967 */  regD_rule,
  /*  968 */  regD_rule,
  /*  969 */  regD_rule,
  /*  970 */  regD_rule,
  /*  971 */  regD_rule,
  /*  972 */  regF_rule,
  /*  973 */  regF_rule,
  /*  974 */  regF_rule,
  /*  975 */  regF_rule,
  /*  976 */  regF_rule,
  /*  977 */  regF_rule,
  /*  978 */  regF_rule,
  /*  979 */  regF_rule,
  /*  980 */  regD_rule,
  /*  981 */  regD_rule,
  /*  982 */  regD_rule,
  /*  983 */  regD_rule,
  /*  984 */  regD_rule,
  /*  985 */  regD_rule,
  /*  986 */  regD_rule,
  /*  987 */  regD_rule,
  /*  988 */  regF_rule,
  /*  989 */  regF_rule,
  /*  990 */  regF_rule,
  /*  991 */  regF_rule,
  /*  992 */  regF_rule,
  /*  993 */  regF_rule,
  /*  994 */  regD_rule,
  /*  995 */  regD_rule,
  /*  996 */  regD_rule,
  /*  997 */  regD_rule,
  /*  998 */  regD_rule,
  /*  999 */  regD_rule,
  /* 1000 */  regF_rule,
  /* 1001 */  regF_rule,
  /* 1002 */  regF_rule,
  /* 1003 */  regD_rule,
  /* 1004 */  regD_rule,
  /* 1005 */  regD_rule,
  /* 1006 */  Universe_rule,
  /* 1007 */  regD_rule,
  /* 1008 */  regF_rule,
  /* 1009 */  vec_rule,
  /* 1010 */  Universe_rule,
  /* 1011 */  vec_rule,
  /* 1012 */  vec_rule,
  /* 1013 */  vec_rule,
  /* 1014 */  vec_rule,
  /* 1015 */  vec_rule,
  /* 1016 */  vec_rule,
  /* 1017 */  vec_rule,
  /* 1018 */  vec_rule,
  /* 1019 */  vec_rule,
  /* 1020 */  vec_rule,
  /* 1021 */  vec_rule,
  /* 1022 */  vec_rule,
  /* 1023 */  vec_rule,
  /* 1024 */  vec_rule,
  /* 1025 */  vec_rule,
  /* 1026 */  vec_rule,
  /* 1027 */  legVec_rule,
  /* 1028 */  vec_rule,
  /* 1029 */  vec_rule,
  /* 1030 */  vec_rule,
  /* 1031 */  vec_rule,
  /* 1032 */  vec_rule,
  /* 1033 */  vec_rule,
  /* 1034 */  vec_rule,
  /* 1035 */  vec_rule,
  /* 1036 */  vec_rule,
  /* 1037 */  vec_rule,
  /* 1038 */  rRegI_rule,
  /* 1039 */  rRegI_rule,
  /* 1040 */  rRegI_rule,
  /* 1041 */  rRegI_rule,
  /* 1042 */  rRegI_rule,
  /* 1043 */  rRegI_rule,
  /* 1044 */  rRegI_rule,
  /* 1045 */  rRegI_rule,
  /* 1046 */  rRegI_rule,
  /* 1047 */  rRegI_rule,
  /* 1048 */  regF_rule,
  /* 1049 */  regF_rule,
  /* 1050 */  regF_rule,
  /* 1051 */  regF_rule,
  /* 1052 */  regF_rule,
  /* 1053 */  regF_rule,
  /* 1054 */  regD_rule,
  /* 1055 */  regD_rule,
  /* 1056 */  regD_rule,
  /* 1057 */  regD_rule,
  /* 1058 */  regD_rule,
  /* 1059 */  regD_rule,
  /* 1060 */  vec_rule,
  /* 1061 */  vec_rule,
  /* 1062 */  vec_rule,
  /* 1063 */  vec_rule,
  /* 1064 */  vec_rule,
  /* 1065 */  vec_rule,
  /* 1066 */  vec_rule,
  /* 1067 */  vec_rule,
  /* 1068 */  vec_rule,
  /* 1069 */  vec_rule,
  /* 1070 */  vec_rule,
  /* 1071 */  vec_rule,
  /* 1072 */  vec_rule,
  /* 1073 */  vec_rule,
  /* 1074 */  vec_rule,
  /* 1075 */  vec_rule,
  /* 1076 */  vec_rule,
  /* 1077 */  vec_rule,
  /* 1078 */  vec_rule,
  /* 1079 */  vec_rule,
  /* 1080 */  vec_rule,
  /* 1081 */  vec_rule,
  /* 1082 */  vec_rule,
  /* 1083 */  vec_rule,
  /* 1084 */  vec_rule,
  /* 1085 */  vec_rule,
  /* 1086 */  vec_rule,
  /* 1087 */  vec_rule,
  /* 1088 */  vec_rule,
  /* 1089 */  vec_rule,
  /* 1090 */  vec_rule,
  /* 1091 */  vec_rule,
  /* 1092 */  vec_rule,
  /* 1093 */  vec_rule,
  /* 1094 */  vec_rule,
  /* 1095 */  vec_rule,
  /* 1096 */  vec_rule,
  /* 1097 */  vec_rule,
  /* 1098 */  vec_rule,
  /* 1099 */  vec_rule,
  /* 1100 */  vec_rule,
  /* 1101 */  vec_rule,
  /* 1102 */  vec_rule,
  /* 1103 */  vec_rule,
  /* 1104 */  vec_rule,
  /* 1105 */  vec_rule,
  /* 1106 */  vec_rule,
  /* 1107 */  vec_rule,
  /* 1108 */  vec_rule,
  /* 1109 */  vec_rule,
  /* 1110 */  vec_rule,
  /* 1111 */  vec_rule,
  /* 1112 */  vec_rule,
  /* 1113 */  vec_rule,
  /* 1114 */  vec_rule,
  /* 1115 */  vec_rule,
  /* 1116 */  vec_rule,
  /* 1117 */  vec_rule,
  /* 1118 */  vec_rule,
  /* 1119 */  vec_rule,
  /* 1120 */  vec_rule,
  /* 1121 */  vec_rule,
  /* 1122 */  vec_rule,
  /* 1123 */  vec_rule,
  /* 1124 */  vec_rule,
  /* 1125 */  vec_rule,
  /* 1126 */  legVec_rule,
  /* 1127 */  legVec_rule,
  /* 1128 */  vec_rule,
  /* 1129 */  vec_rule,
  /* 1130 */  vec_rule,
  /* 1131 */  vec_rule,
  /* 1132 */  vec_rule,
  /* 1133 */  vec_rule,
  /* 1134 */  vec_rule,
  /* 1135 */  vec_rule,
  /* 1136 */  vec_rule,
  /* 1137 */  vec_rule,
  /* 1138 */  vec_rule,
  /* 1139 */  vec_rule,
  /* 1140 */  vec_rule,
  /* 1141 */  vec_rule,
  /* 1142 */  vec_rule,
  /* 1143 */  vec_rule,
  /* 1144 */  vec_rule,
  /* 1145 */  vec_rule,
  /* 1146 */  vec_rule,
  /* 1147 */  vec_rule,
  /* 1148 */  vec_rule,
  /* 1149 */  vec_rule,
  /* 1150 */  vec_rule,
  /* 1151 */  vec_rule,
  /* 1152 */  vec_rule,
  /* 1153 */  vec_rule,
  /* 1154 */  vec_rule,
  /* 1155 */  vec_rule,
  /* 1156 */  vec_rule,
  /* 1157 */  vec_rule,
  /* 1158 */  vec_rule,
  /* 1159 */  vec_rule,
  /* 1160 */  vec_rule,
  /* 1161 */  vec_rule,
  /* 1162 */  vec_rule,
  /* 1163 */  vec_rule,
  /* 1164 */  vec_rule,
  /* 1165 */  vec_rule,
  /* 1166 */  vec_rule,
  /* 1167 */  vec_rule,
  /* 1168 */  vec_rule,
  /* 1169 */  vec_rule,
  /* 1170 */  vec_rule,
  /* 1171 */  vec_rule,
  /* 1172 */  vec_rule,
  /* 1173 */  vec_rule,
  /* 1174 */  vec_rule,
  /* 1175 */  vec_rule,
  /* 1176 */  vec_rule,
  /* 1177 */  vec_rule,
  /* 1178 */  vec_rule,
  /* 1179 */  vec_rule,
  /* 1180 */  vec_rule,
  /* 1181 */  vec_rule,
  /* 1182 */  vec_rule,
  /* 1183 */  vec_rule,
  /* 1184 */  vec_rule,
  /* 1185 */  vec_rule,
  /* 1186 */  vec_rule,
  /* 1187 */  rRegI_rule,
  /* 1188 */  rRegI_rule,
  /* 1189 */  eAXRegP_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  eRegP_rule,
  /*   94 */  eRegP_rule,
  /*   95 */  immP_rule,
  /*   96 */  _AddP_eRegP_rRegI_rule,
  /*   97 */  eRegP_rule,
  /*   98 */  eRegP_rule,
  /*   99 */  _AddP_eRegP__LShiftI_rRegI_immI2_rule,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  load_long_RegP_rule,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  0,
  /*  108 */  0,
  /*  109 */  eRegP_no_EBP_rule,
  /*  110 */  eRegP_no_EBP_rule,
  /*  111 */  _AddP_eRegP_no_EBP_rRegI_rule,
  /*  112 */  eRegP_no_EBP_rule,
  /*  113 */  _AddP_eRegP_no_EBP__LShiftI_rRegI_immI2_rule,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  0,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  // last operand
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  // last operand class
  /*  137 */  eRegP_rule,
  /*  138 */  rRegI_rule,
  /*  139 */  eRegP_rule,
  /*  140 */  eRegP_no_EBP_rule,
  /*  141 */  eRegP_no_EBP_rule,
  /*  142 */  memory_rule,
  /*  143 */  memory_rule,
  /*  144 */  memory_rule,
  /*  145 */  memory_rule,
  /*  146 */  _LoadUB_memory__rule,
  /*  147 */  memory_rule,
  /*  148 */  _LoadS_memory__rule,
  /*  149 */  memory_rule,
  /*  150 */  _LoadUS_memory__rule,
  /*  151 */  _LoadUS_memory__rule,
  /*  152 */  _LoadUS_memory__rule,
  /*  153 */  _LoadI_memory__rule,
  /*  154 */  _LoadI_memory__rule,
  /*  155 */  _LoadI_memory__rule,
  /*  156 */  _LoadI_memory__rule,
  /*  157 */  _LoadI_memory__rule,
  /*  158 */  _LoadI_memory__rule,
  /*  159 */  eRegL_rule,
  /*  160 */  regDPR1_rule,
  /*  161 */  regFPR1_rule,
  /*  162 */  regDPR1_rule,
  /*  163 */  cmpOp_rule,
  /*  164 */  rRegI_rule,
  /*  165 */  cmpOpU_rule,
  /*  166 */  cmpOpUCF_rule,
  /*  167 */  rRegI_rule,
  /*  168 */  eRegP_rule,
  /*  169 */  cmpOp_fcmov_rule,
  /*  170 */  regDPR1_rule,
  /*  171 */  regFPR1_rule,
  /*  172 */  regDPR_rule,
  /*  173 */  regFPR_rule,
  /*  174 */  regF_rule,
  /*  175 */  regD_rule,
  /*  176 */  eRegL_rule,
  /*  177 */  _LoadI_memory__rule,
  /*  178 */  rRegI_rule,
  /*  179 */  _LoadI_memory__rule,
  /*  180 */  _LoadI_memory__rule,
  /*  181 */  _LoadI_memory__rule,
  /*  182 */  eAXRegP_rule,
  /*  183 */  eAXRegI_rule,
  /*  184 */  eADXRegL_rule,
  /*  185 */  eAXRegP_rule,
  /*  186 */  eAXRegI_rule,
  /*  187 */  _LoadI_memory__rule,
  /*  188 */  immI0_rule,
  /*  189 */  nadxRegI_rule,
  /*  190 */  _ConvI2L_nadxRegI__rule,
  /*  191 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  192 */  eADXRegL_low_only_rule,
  /*  193 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  194 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  195 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  196 */  eAXRegI_rule,
  /*  197 */  _ConvI2L_eAXRegI__rule,
  /*  198 */  _ConvI2L_nadxRegI__rule,
  /*  199 */  _LoadI_memory__rule,
  /*  200 */  _LoadI_memory__rule,
  /*  201 */  xRegI_rule,
  /*  202 */  xRegI_rule,
  /*  203 */  _LoadI_memory__rule,
  /*  204 */  rRegI_rule,
  /*  205 */  _LoadI_memory__rule,
  /*  206 */  rRegI_rule,
  /*  207 */  immI0_rule,
  /*  208 */  rRegI_rule,
  /*  209 */  eRegP_rule,
  /*  210 */  _LoadI_memory__rule,
  /*  211 */  rRegI_rule,
  /*  212 */  _LoadI_memory__rule,
  /*  213 */  rRegI_rule,
  /*  214 */  rRegI_rule,
  /*  215 */  rRegI_rule,
  /*  216 */  rRegI_rule,
  /*  217 */  ncxRegI_rule,
  /*  218 */  immI0_rule,
  /*  219 */  ncxRegI_rule,
  /*  220 */  immI_32_rule,
  /*  221 */  ncxRegI_rule,
  /*  222 */  rRegI_rule,
  /*  223 */  rRegI_rule,
  /*  224 */  ncxRegI_rule,
  /*  225 */  ncxRegI_rule,
  /*  226 */  ncxRegI_rule,
  /*  227 */  _LoadI_memory__rule,
  /*  228 */  rRegI_rule,
  /*  229 */  _LoadI_memory__rule,
  /*  230 */  rRegI_rule,
  /*  231 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  232 */  rRegI_rule,
  /*  233 */  rRegI_rule,
  /*  234 */  load_long_memory_rule,
  /*  235 */  eRegL_rule,
  /*  236 */  immL0_rule,
  /*  237 */  immL0_rule,
  /*  238 */  eRegL_rule,
  /*  239 */  _LoadL_memory__rule,
  /*  240 */  memory_rule,
  /*  241 */  regDPR_rule,
  /*  242 */  regDPR_rule,
  /*  243 */  _LoadD_memory__rule,
  /*  244 */  _AddD__LoadD_memory__regDPR_rule,
  /*  245 */  regDPR_rule,
  /*  246 */  _AddD_regDPR__LoadD_memory__rule,
  /*  247 */  regDPR_rule,
  /*  248 */  regDPR_rule,
  /*  249 */  regDPR_rule,
  /*  250 */  memory_rule,
  /*  251 */  _LoadF_memory__rule,
  /*  252 */  regFPR_rule,
  /*  253 */  regFPR_rule,
  /*  254 */  regFPR_rule,
  /*  255 */  rRegI_rule,
  /*  256 */  eDIRegP_rule,
  /*  257 */  eSIRegP_rule,
  /*  258 */  eDIRegP_rule,
  /*  259 */  eDIRegP_rule,
  /*  260 */  eSIRegP_rule,
  /*  261 */  eSIRegP_rule,
  /*  262 */  rRegI_rule,
  /*  263 */  rRegI_rule,
  /*  264 */  memory_rule,
  /*  265 */  nadxRegI_rule,
  /*  266 */  eSIRegP_rule,
  /*  267 */  cmpOp_rule,
  /*  268 */  eRegL_rule,
  /*  269 */  cmpOp_rule,
  /*  270 */  cmpOp_commute_rule,
  /*  271 */  memory_rule,
  /*  272 */  cmpOp_vcmppd_rule,
  /*  273 */  legVec_rule,
  /*  274 */  vec_rule,
  /*  275 */  vec_rule,
  /*  276 */  vec_rule,
  /*  277 */  vec_rule,
  /*  278 */  _LoadVector_memory__rule,
  // last internally defined operand
  /*  279 */  rRegI_rule,
  /*  280 */  eRegP_rule,
  /*  281 */  eRegL_rule,
  /*  282 */  indOffset8_rule,
  /*  283 */  indOffset32_rule,
  /*  284 */  indIndexOffset_rule,
  /*  285 */  indIndexScale_rule,
  /*  286 */  indIndexScaleOffset_rule,
  /*  287 */  immI_rule,
  /*  288 */  immI0_rule,
  /*  289 */  immP_rule,
  /*  290 */  immL_rule,
  /*  291 */  immL0_rule,
  /*  292 */  immFPR_rule,
  /*  293 */  immFPR0_rule,
  /*  294 */  immFPR1_rule,
  /*  295 */  immF_rule,
  /*  296 */  immF0_rule,
  /*  297 */  immDPR_rule,
  /*  298 */  immDPR0_rule,
  /*  299 */  immDPR1_rule,
  /*  300 */  immD_rule,
  /*  301 */  immD0_rule,
  /*  302 */  stackSlotI_rule,
  /*  303 */  stackSlotL_rule,
  /*  304 */  stackSlotP_rule,
  /*  305 */  stackSlotF_rule,
  /*  306 */  stackSlotD_rule,
  /*  307 */  regD_rule,
  /*  308 */  vlRegD_rule,
  /*  309 */  regF_rule,
  /*  310 */  vlRegF_rule,
  /*  311 */  immL32_rule,
  /*  312 */  vec_rule,
  /*  313 */  legVec_rule,
  /*  314 */  0,
  /*  315 */  eAXRegI_rule,
  /*  316 */  eRegP_rule,
  /*  317 */  eRegP_rule,
  /*  318 */  eRegP_rule,
  /*  319 */  rRegI_rule,
  /*  320 */  memory_rule,
  /*  321 */  memory_rule,
  /*  322 */  memory_rule,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  eAXRegI_rule,
  /*  326 */  eAXRegI_rule,
  /*  327 */  rRegI_rule,
  /*  328 */  rRegI_rule,
  /*  329 */  immI0_rule,
  /*  330 */  eAXRegI_rule,
  /*  331 */  rRegI_rule,
  /*  332 */  regDPR_rule,
  /*  333 */  regDPR_rule,
  /*  334 */  regDPR_rule,
  /*  335 */  regD_rule,
  /*  336 */  regD_rule,
  /*  337 */  regD_rule,
  /*  338 */  regD_rule,
  /*  339 */  regDPR1_rule,
  /*  340 */  regDPR1_rule,
  /*  341 */  regFPR_rule,
  /*  342 */  regFPR_rule,
  /*  343 */  regFPR_rule,
  /*  344 */  regF_rule,
  /*  345 */  regF_rule,
  /*  346 */  regF_rule,
  /*  347 */  regF_rule,
  /*  348 */  regFPR1_rule,
  /*  349 */  regFPR1_rule,
  /*  350 */  stackSlotF_rule,
  /*  351 */  regF_rule,
  /*  352 */  stackSlotI_rule,
  /*  353 */  stackSlotI_rule,
  /*  354 */  rRegI_rule,
  /*  355 */  stackSlotD_rule,
  /*  356 */  stackSlotL_rule,
  /*  357 */  stackSlotL_rule,
  /*  358 */  stackSlotL_rule,
  /*  359 */  rRegI_rule,
  /*  360 */  rRegI_rule,
  /*  361 */  rRegI_rule,
  /*  362 */  rRegI_rule,
  /*  363 */  _AndI_rRegI_immI_rule,
  /*  364 */  _AndI_rRegI_memory_rule,
  /*  365 */  rRegI_rule,
  /*  366 */  rRegI_rule,
  /*  367 */  rRegI_rule,
  /*  368 */  rRegI_rule,
  /*  369 */  eRegP_rule,
  /*  370 */  eRegP_rule,
  /*  371 */  eRegP_rule,
  /*  372 */  eRegP_rule,
  /*  373 */  eRegP_rule,
  /*  374 */  _LoadP_memory__rule,
  /*  375 */  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule,
  /*  376 */  eRegL_rule,
  /*  377 */  eRegL_rule,
  /*  378 */  eRegL_rule,
  /*  379 */  eRegL_rule,
  /*  380 */  eRegL_rule,
  /*  381 */  eRegL_rule,
  /*  382 */  eRegL_rule,
  /*  383 */  eRegL_rule,
  /*  384 */  eRegL_rule,
  /*  385 */  eRegL_rule,
  /*  386 */  eRegL_rule,
  /*  387 */  eRegL_rule,
  /*  388 */  eRegP_rule,
  /*  389 */  eRegP_rule,
  /*  390 */  eRegP_rule,
  /*  391 */  regF_rule,
  /*  392 */  vlRegF_rule,
  /*  393 */  regD_rule,
  /*  394 */  vlRegD_rule,
  /*  395 */  regF_rule,
  /*  396 */  vlRegF_rule,
  /*  397 */  regD_rule,
  /*  398 */  vlRegD_rule,
  /*  399 */  vec_rule,
  /*  400 */  _LoadVector_memory__rule,
  /*  401 */  vec_rule,
  /*  402 */  _LoadVector_memory__rule,
  /*  403 */  rRegI_rule,
  /*  404 */  rRegI_rule,
  /*  405 */  vec_rule,
  /*  406 */  vec_rule,
  /*  407 */  vec_rule,
  /*  408 */  vec_rule,
  /*  409 */  vec_rule,
  /*  410 */  rRegI_rule,
  /*  411 */  eRegL_rule,
  /*  412 */  rRegI_rule,
  /*  413 */  rRegI_rule,
  /*  414 */  rRegI_rule,
  /*  415 */  rRegI_rule,
  /*  416 */  eRegL_rule,
  /*  417 */  eRegL_rule,
  /*  418 */  rRegI_rule,
  /*  419 */  rRegI_rule,
  /*  420 */  eRegL_rule,
  /*  421 */  eRegL_rule,
  /*  422 */  rRegI_rule,
  /*  423 */  _LoadI_memory__rule,
  /*  424 */  eRegL_rule,
  /*  425 */  _LoadL_memory__rule,
  /*  426 */  memory_rule,
  /*  427 */  _LoadB_memory__rule,
  /*  428 */  memory_rule,
  /*  429 */  _LoadUB_memory__rule,
  /*  430 */  _AndI__LoadUB_memory__immI_rule,
  /*  431 */  memory_rule,
  /*  432 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  433 */  _LoadS_memory__rule,
  /*  434 */  memory_rule,
  /*  435 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  436 */  _LoadUS_memory__rule,
  /*  437 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  438 */  _AndI__LoadUS_memory__immI_rule,
  /*  439 */  memory_rule,
  /*  440 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  441 */  _LoadI_memory__rule,
  /*  442 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  443 */  _LoadI_memory__rule,
  /*  444 */  _LoadI_memory__rule,
  /*  445 */  _AndI__LoadI_memory__immI_255_rule,
  /*  446 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  447 */  _AndI__LoadI_memory__immU31_rule,
  /*  448 */  _ConvI2L__LoadI_memory___rule,
  /*  449 */  load_long_memory_rule,
  /*  450 */  memory_rule,
  /*  451 */  memory_rule,
  /*  452 */  memory_rule,
  /*  453 */  memory_rule,
  /*  454 */  memory_rule,
  /*  455 */  memory_rule,
  /*  456 */  memory_rule,
  /*  457 */  memory_rule,
  /*  458 */  memory_rule,
  /*  459 */  memory_rule,
  /*  460 */  memory_rule,
  /*  461 */  memory_rule,
  /*  462 */  memory_rule,
  /*  463 */  memory_rule,
  /*  464 */  memory_rule,
  /*  465 */  memory_rule,
  /*  466 */  memory_rule,
  /*  467 */  memory_rule,
  /*  468 */  memory_rule,
  /*  469 */  long_memory_rule,
  /*  470 */  memory_rule,
  /*  471 */  memory_rule,
  /*  472 */  memory_rule,
  /*  473 */  memory_rule,
  /*  474 */  memory_rule,
  /*  475 */  memory_rule,
  /*  476 */  memory_rule,
  /*  477 */  memory_rule,
  /*  478 */  memory_rule,
  /*  479 */  memory_rule,
  /*  480 */  memory_rule,
  /*  481 */  memory_rule,
  /*  482 */  memory_rule,
  /*  483 */  memory_rule,
  /*  484 */  memory_rule,
  /*  485 */  memory_rule,
  /*  486 */  memory_rule,
  /*  487 */  memory_rule,
  /*  488 */  memory_rule,
  /*  489 */  0,
  /*  490 */  0,
  /*  491 */  0,
  /*  492 */  0,
  /*  493 */  0,
  /*  494 */  0,
  /*  495 */  0,
  /*  496 */  0,
  /*  497 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  498 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  499 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  500 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  501 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  502 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  503 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  504 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  505 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  506 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  507 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  508 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  509 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  510 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  511 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  512 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  513 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  514 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  515 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  516 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  517 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  518 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  519 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  520 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  521 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  522 */  rRegI_rule,
  /*  523 */  rRegI_rule,
  /*  524 */  rRegI_rule,
  /*  525 */  rRegI_rule,
  /*  526 */  eRegP_rule,
  /*  527 */  rRegI_rule,
  /*  528 */  eRegP_rule,
  /*  529 */  eRegP_rule,
  /*  530 */  rRegI_rule,
  /*  531 */  _LoadI_memory__rule,
  /*  532 */  memory_rule,
  /*  533 */  memory_rule,
  /*  534 */  memory_rule,
  /*  535 */  memory_rule,
  /*  536 */  memory_rule,
  /*  537 */  memory_rule,
  /*  538 */  eSIRegP_rule,
  /*  539 */  eSIRegP_rule,
  /*  540 */  pRegP_rule,
  /*  541 */  pRegP_rule,
  /*  542 */  pRegP_rule,
  /*  543 */  pRegP_rule,
  /*  544 */  pRegP_rule,
  /*  545 */  pRegP_rule,
  /*  546 */  pRegP_rule,
  /*  547 */  pRegP_rule,
  /*  548 */  eSIRegP_rule,
  /*  549 */  pRegP_rule,
  /*  550 */  pRegP_rule,
  /*  551 */  pRegP_rule,
  /*  552 */  pRegP_rule,
  /*  553 */  memory_rule,
  /*  554 */  memory_rule,
  /*  555 */  memory_rule,
  /*  556 */  memory_rule,
  /*  557 */  memory_rule,
  /*  558 */  memory_rule,
  /*  559 */  memory_rule,
  /*  560 */  memory_rule,
  /*  561 */  memory_rule,
  /*  562 */  memory_rule,
  /*  563 */  rRegI_rule,
  /*  564 */  rRegI_rule,
  /*  565 */  rRegI_rule,
  /*  566 */  memory_rule,
  /*  567 */  eRegP_rule,
  /*  568 */  immI0_rule,
  /*  569 */  rRegI_rule,
  /*  570 */  rRegI_rule,
  /*  571 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule,
  /*  572 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule,
  /*  573 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule,
  /*  574 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule,
  /*  575 */  _LoadI_memory__rule,
  /*  576 */  rRegI_rule,
  /*  577 */  _LoadI_memory__rule,
  /*  578 */  _Binary_rRegI_rRegI_rule,
  /*  579 */  _ConvI2L_eAXRegI__rule,
  /*  580 */  _ConvI2L_nadxRegI__rule,
  /*  581 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  582 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  583 */  eADXRegL_rule,
  /*  584 */  eADXRegL_rule,
  /*  585 */  eADXRegL_rule,
  /*  586 */  eADXRegL_rule,
  /*  587 */  eADXRegL_rule,
  /*  588 */  eAXRegI_rule,
  /*  589 */  eRegL_rule,
  /*  590 */  eAXRegI_rule,
  /*  591 */  eAXRegI_rule,
  /*  592 */  eRegL_rule,
  /*  593 */  eADXRegL_rule,
  /*  594 */  eADXRegL_rule,
  /*  595 */  rRegI_rule,
  /*  596 */  rRegI_rule,
  /*  597 */  rRegI_rule,
  /*  598 */  rRegI_rule,
  /*  599 */  memory_rule,
  /*  600 */  rRegI_rule,
  /*  601 */  memory_rule,
  /*  602 */  rRegI_rule,
  /*  603 */  rRegI_rule,
  /*  604 */  rRegI_rule,
  /*  605 */  _LShiftI_xRegI_immI_24_rule,
  /*  606 */  _LShiftI_xRegI_immI_16_rule,
  /*  607 */  rRegI_rule,
  /*  608 */  rRegI_rule,
  /*  609 */  rRegI_rule,
  /*  610 */  rRegI_rule,
  /*  611 */  _LoadI_memory__rule,
  /*  612 */  memory_rule,
  /*  613 */  memory_rule,
  /*  614 */  memory_rule,
  /*  615 */  _XorI_rRegI_immI_M1_rule,
  /*  616 */  rRegI_rule,
  /*  617 */  _XorI_rRegI_immI_M1_rule,
  /*  618 */  _LoadI_memory__rule,
  /*  619 */  _SubI_immI0_rRegI_rule,
  /*  620 */  rRegI_rule,
  /*  621 */  _SubI_immI0__LoadI_memory__rule,
  /*  622 */  _LoadI_memory__rule,
  /*  623 */  _AddI_rRegI_immI_M1_rule,
  /*  624 */  rRegI_rule,
  /*  625 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  626 */  _LoadI_memory__rule,
  /*  627 */  _AddI_rRegI_immI_M1_rule,
  /*  628 */  rRegI_rule,
  /*  629 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  630 */  _LoadI_memory__rule,
  /*  631 */  rRegI_rule,
  /*  632 */  rRegI_rule,
  /*  633 */  _CastP2X_eRegP__rule,
  /*  634 */  rRegI_rule,
  /*  635 */  rRegI_rule,
  /*  636 */  _LoadI_memory__rule,
  /*  637 */  memory_rule,
  /*  638 */  memory_rule,
  /*  639 */  memory_rule,
  /*  640 */  0,
  /*  641 */  0,
  /*  642 */  0,
  /*  643 */  _LShiftI_rRegI_immI1_rule,
  /*  644 */  _URShiftI_rRegI_immI_M1_rule,
  /*  645 */  _LShiftI_rRegI_immI8_rule,
  /*  646 */  _URShiftI_rRegI_immI8_rule,
  /*  647 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  648 */  _URShiftI_ncxRegI__SubI_immI0_eCXRegI_rule,
  /*  649 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  650 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  651 */  0,
  /*  652 */  0,
  /*  653 */  0,
  /*  654 */  _URShiftI_rRegI_immI1_rule,
  /*  655 */  _LShiftI_rRegI_immI_M1_rule,
  /*  656 */  _URShiftI_rRegI_immI8_rule,
  /*  657 */  _LShiftI_rRegI_immI8_rule,
  /*  658 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  659 */  _LShiftI_ncxRegI__SubI_immI0_eCXRegI_rule,
  /*  660 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  661 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  662 */  rRegI_rule,
  /*  663 */  rRegI_rule,
  /*  664 */  rRegI_rule,
  /*  665 */  rRegI_rule,
  /*  666 */  _LoadI_memory__rule,
  /*  667 */  memory_rule,
  /*  668 */  memory_rule,
  /*  669 */  memory_rule,
  /*  670 */  0,
  /*  671 */  rRegI_rule,
  /*  672 */  0,
  /*  673 */  eRegP_rule,
  /*  674 */  ncxRegI_rule,
  /*  675 */  rRegI_rule,
  /*  676 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  677 */  _SubI_rRegI_rRegI_rule,
  /*  678 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  679 */  _SubI_rRegI_rRegI_rule,
  /*  680 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  681 */  rRegI_rule,
  /*  682 */  rRegI_rule,
  /*  683 */  eRegL_rule,
  /*  684 */  eRegL_rule,
  /*  685 */  eRegL_rule,
  /*  686 */  _LoadL_load_long_memory__rule,
  /*  687 */  eRegL_rule,
  /*  688 */  eRegL_rule,
  /*  689 */  eRegL_rule,
  /*  690 */  immL0_rule,
  /*  691 */  eRegL_rule,
  /*  692 */  eRegL_rule,
  /*  693 */  eRegL_rule,
  /*  694 */  _LoadL_load_long_memory__rule,
  /*  695 */  _XorL_eRegL_immL_M1_rule,
  /*  696 */  eRegL_rule,
  /*  697 */  _XorL_eRegL_immL_M1_rule,
  /*  698 */  _LoadL_memory__rule,
  /*  699 */  _SubL_immL0_eRegL_rule,
  /*  700 */  eRegL_rule,
  /*  701 */  _SubL_immL0__LoadL_memory__rule,
  /*  702 */  _LoadL_memory__rule,
  /*  703 */  _AddL_eRegL_immL_M1_rule,
  /*  704 */  eRegL_rule,
  /*  705 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  706 */  _LoadL_memory__rule,
  /*  707 */  _AddL_eRegL_immL_M1_rule,
  /*  708 */  eRegL_rule,
  /*  709 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  710 */  _LoadL_memory__rule,
  /*  711 */  eRegL_rule,
  /*  712 */  eRegL_rule,
  /*  713 */  eRegL_rule,
  /*  714 */  _LoadL_load_long_memory__rule,
  /*  715 */  eRegL_rule,
  /*  716 */  eRegL_rule,
  /*  717 */  eRegL_rule,
  /*  718 */  eRegL_rule,
  /*  719 */  _LoadL_load_long_memory__rule,
  /*  720 */  eRegL_rule,
  /*  721 */  eRegL_rule,
  /*  722 */  eRegL_rule,
  /*  723 */  eRegL_rule,
  /*  724 */  eRegL_rule,
  /*  725 */  eRegL_rule,
  /*  726 */  eRegL_rule,
  /*  727 */  eRegL_rule,
  /*  728 */  eRegL_rule,
  /*  729 */  eRegL_rule,
  /*  730 */  eRegL_rule,
  /*  731 */  eRegL_rule,
  /*  732 */  regDPR_rule,
  /*  733 */  regDPR_rule,
  /*  734 */  regD_rule,
  /*  735 */  regD_rule,
  /*  736 */  regDPR_rule,
  /*  737 */  _SubD_regDPR_regDPR_rule,
  /*  738 */  regDPR_rule,
  /*  739 */  regDPR_rule,
  /*  740 */  _AddD_regDPR_regDPR_rule,
  /*  741 */  regDPR_rule,
  /*  742 */  _LoadD_memory__rule,
  /*  743 */  memory_rule,
  /*  744 */  memory_rule,
  /*  745 */  regDPR_rule,
  /*  746 */  regDPR_rule,
  /*  747 */  _AddD_regDPR_immDPR_rule,
  /*  748 */  regDPR_rule,
  /*  749 */  regDPR1_rule,
  /*  750 */  regDPR_rule,
  /*  751 */  regDPR_rule,
  /*  752 */  _LoadD_memory__rule,
  /*  753 */  regDPR_rule,
  /*  754 */  _LoadD_memory__rule,
  /*  755 */  _MulD_regDPR_regDPR_rule,
  /*  756 */  regDPR_rule,
  /*  757 */  _MulD_regDPR_regDPR_rule,
  /*  758 */  regDPR_rule,
  /*  759 */  regDPR1_rule,
  /*  760 */  _DivD_regDPR_regDPR_rule,
  /*  761 */  regDPR_rule,
  /*  762 */  regD_rule,
  /*  763 */  regDPR_rule,
  /*  764 */  regD_rule,
  /*  765 */  regDPR_rule,
  /*  766 */  regFPR_rule,
  /*  767 */  regFPR_rule,
  /*  768 */  regF_rule,
  /*  769 */  regF_rule,
  /*  770 */  regFPR_rule,
  /*  771 */  regFPR_rule,
  /*  772 */  regFPR_rule,
  /*  773 */  regFPR_rule,
  /*  774 */  regFPR_rule,
  /*  775 */  _LoadF_memory__rule,
  /*  776 */  regFPR_rule,
  /*  777 */  _LoadF_memory__rule,
  /*  778 */  memory_rule,
  /*  779 */  memory_rule,
  /*  780 */  _LoadF_memory__rule,
  /*  781 */  memory_rule,
  /*  782 */  regFPR_rule,
  /*  783 */  regFPR_rule,
  /*  784 */  regFPR_rule,
  /*  785 */  regFPR_rule,
  /*  786 */  regFPR_rule,
  /*  787 */  _LoadF_memory__rule,
  /*  788 */  regFPR_rule,
  /*  789 */  _LoadF_memory__rule,
  /*  790 */  memory_rule,
  /*  791 */  regFPR_rule,
  /*  792 */  regFPR_rule,
  /*  793 */  _LoadF_memory__rule,
  /*  794 */  regFPR_rule,
  /*  795 */  _MulF__LoadF_memory__regFPR_rule,
  /*  796 */  regFPR_rule,
  /*  797 */  _MulF_regFPR__LoadF_memory__rule,
  /*  798 */  regFPR_rule,
  /*  799 */  _MulF_regFPR_regFPR_rule,
  /*  800 */  regFPR_rule,
  /*  801 */  _SubF_regFPR_regFPR_rule,
  /*  802 */  regFPR_rule,
  /*  803 */  regFPR_rule,
  /*  804 */  regFPR_rule,
  /*  805 */  regFPR_rule,
  /*  806 */  regF_rule,
  /*  807 */  regFPR_rule,
  /*  808 */  regDPR_rule,
  /*  809 */  regDPR_rule,
  /*  810 */  regDPR_rule,
  /*  811 */  regD_rule,
  /*  812 */  regFPR_rule,
  /*  813 */  regFPR_rule,
  /*  814 */  regF_rule,
  /*  815 */  regF_rule,
  /*  816 */  regDPR_rule,
  /*  817 */  regD_rule,
  /*  818 */  regDPR_rule,
  /*  819 */  regD_rule,
  /*  820 */  regFPR_rule,
  /*  821 */  regF_rule,
  /*  822 */  regFPR_rule,
  /*  823 */  regF_rule,
  /*  824 */  stackSlotI_rule,
  /*  825 */  rRegI_rule,
  /*  826 */  _LoadI_memory__rule,
  /*  827 */  rRegI_rule,
  /*  828 */  _LoadI_memory__rule,
  /*  829 */  stackSlotI_rule,
  /*  830 */  stackSlotI_rule,
  /*  831 */  _LoadI_memory__rule,
  /*  832 */  stackSlotI_rule,
  /*  833 */  _LoadI_memory__rule,
  /*  834 */  rRegI_rule,
  /*  835 */  rRegI_rule,
  /*  836 */  rRegI_rule,
  /*  837 */  _ConvI2L_rRegI__rule,
  /*  838 */  eRegL_rule,
  /*  839 */  eRegL_rule,
  /*  840 */  eRegL_rule,
  /*  841 */  eRegL_rule,
  /*  842 */  eRegL_rule,
  /*  843 */  eRegL_rule,
  /*  844 */  regFPR_rule,
  /*  845 */  regF_rule,
  /*  846 */  rRegI_rule,
  /*  847 */  regDPR_rule,
  /*  848 */  regD_rule,
  /*  849 */  regD_rule,
  /*  850 */  eRegL_rule,
  /*  851 */  eRegL_rule,
  /*  852 */  eCXRegI_rule,
  /*  853 */  eCXRegI_rule,
  /*  854 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  855 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  856 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  857 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  858 */  _Binary_eDIRegP_eSIRegP_rule,
  /*  859 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  860 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  861 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  862 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  863 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  864 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  865 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  866 */  eDIRegP_rule,
  /*  867 */  eDIRegP_rule,
  /*  868 */  eSIRegP_rule,
  /*  869 */  eSIRegP_rule,
  /*  870 */  eSIRegP_rule,
  /*  871 */  eSIRegP_rule,
  /*  872 */  rRegI_rule,
  /*  873 */  rRegI_rule,
  /*  874 */  _Binary_nadxRegI_eAXRegI_rule,
  /*  875 */  rRegI_rule,
  /*  876 */  0,
  /*  877 */  cmpOp_rule,
  /*  878 */  cmpOp_rule,
  /*  879 */  cmpOpU_rule,
  /*  880 */  cmpOpUCF_rule,
  /*  881 */  cmpOp_rule,
  /*  882 */  cmpOpU_rule,
  /*  883 */  cmpOpUCF_rule,
  /*  884 */  cmpOpU_rule,
  /*  885 */  cmpOpUCF_rule,
  /*  886 */  cmpOpUCF2_rule,
  /*  887 */  eSIRegP_rule,
  /*  888 */  0,
  /*  889 */  cmpOp_rule,
  /*  890 */  cmpOp_rule,
  /*  891 */  cmpOpU_rule,
  /*  892 */  cmpOpUCF_rule,
  /*  893 */  cmpOpU_rule,
  /*  894 */  cmpOpUCF_rule,
  /*  895 */  cmpOpUCF2_rule,
  /*  896 */  eRegL_rule,
  /*  897 */  cmpOp_rule,
  /*  898 */  cmpOpU_rule,
  /*  899 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  900 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  901 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  902 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  903 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  904 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  905 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  906 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  907 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  908 */  cmpOp_rule,
  /*  909 */  cmpOpU_rule,
  /*  910 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  911 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  912 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  913 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  914 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  915 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  916 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  917 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  918 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  919 */  cmpOp_commute_rule,
  /*  920 */  cmpOpU_commute_rule,
  /*  921 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  922 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  923 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  924 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  925 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  926 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  927 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  928 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  929 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  930 */  0,
  /*  931 */  0,
  /*  932 */  0,
  /*  933 */  0,
  /*  934 */  0,
  /*  935 */  0,
  /*  936 */  eRegP_no_EBP_rule,
  /*  937 */  eRegP_no_EBP_rule,
  /*  938 */  0,
  /*  939 */  0,
  /*  940 */  eRegP_no_EBP_rule,
  /*  941 */  0,
  /*  942 */  0,
  /*  943 */  rRegI_rule,
  /*  944 */  regF_rule,
  /*  945 */  regF_rule,
  /*  946 */  _LoadF_memory__rule,
  /*  947 */  regF_rule,
  /*  948 */  regF_rule,
  /*  949 */  regF_rule,
  /*  950 */  _LoadF_memory__rule,
  /*  951 */  regF_rule,
  /*  952 */  regD_rule,
  /*  953 */  regD_rule,
  /*  954 */  _LoadD_memory__rule,
  /*  955 */  regD_rule,
  /*  956 */  regD_rule,
  /*  957 */  regD_rule,
  /*  958 */  _LoadD_memory__rule,
  /*  959 */  regD_rule,
  /*  960 */  regF_rule,
  /*  961 */  regF_rule,
  /*  962 */  regF_rule,
  /*  963 */  regF_rule,
  /*  964 */  regF_rule,
  /*  965 */  regF_rule,
  /*  966 */  regD_rule,
  /*  967 */  regD_rule,
  /*  968 */  regD_rule,
  /*  969 */  regD_rule,
  /*  970 */  regD_rule,
  /*  971 */  regD_rule,
  /*  972 */  regF_rule,
  /*  973 */  regF_rule,
  /*  974 */  _LoadF_memory__rule,
  /*  975 */  regF_rule,
  /*  976 */  regF_rule,
  /*  977 */  regF_rule,
  /*  978 */  _LoadF_memory__rule,
  /*  979 */  regF_rule,
  /*  980 */  regD_rule,
  /*  981 */  regD_rule,
  /*  982 */  _LoadD_memory__rule,
  /*  983 */  regD_rule,
  /*  984 */  regD_rule,
  /*  985 */  regD_rule,
  /*  986 */  _LoadD_memory__rule,
  /*  987 */  regD_rule,
  /*  988 */  regF_rule,
  /*  989 */  regF_rule,
  /*  990 */  regF_rule,
  /*  991 */  regF_rule,
  /*  992 */  regF_rule,
  /*  993 */  regF_rule,
  /*  994 */  regD_rule,
  /*  995 */  regD_rule,
  /*  996 */  regD_rule,
  /*  997 */  regD_rule,
  /*  998 */  regD_rule,
  /*  999 */  regD_rule,
  /* 1000 */  regF_rule,
  /* 1001 */  _LoadF_memory__rule,
  /* 1002 */  immF_rule,
  /* 1003 */  regD_rule,
  /* 1004 */  _LoadD_memory__rule,
  /* 1005 */  immD_rule,
  /* 1006 */  0,
  /* 1007 */  regD_rule,
  /* 1008 */  regF_rule,
  /* 1009 */  memory_rule,
  /* 1010 */  memory_rule,
  /* 1011 */  rRegI_rule,
  /* 1012 */  _LoadB_memory__rule,
  /* 1013 */  immI_rule,
  /* 1014 */  immI0_rule,
  /* 1015 */  rRegI_rule,
  /* 1016 */  _LoadS_memory__rule,
  /* 1017 */  immI_rule,
  /* 1018 */  immI0_rule,
  /* 1019 */  rRegI_rule,
  /* 1020 */  _LoadI_memory__rule,
  /* 1021 */  immI_rule,
  /* 1022 */  immI0_rule,
  /* 1023 */  immI_M1_rule,
  /* 1024 */  immI_M1_rule,
  /* 1025 */  immI_M1_rule,
  /* 1026 */  eRegL_rule,
  /* 1027 */  eRegL_rule,
  /* 1028 */  _LoadL_memory__rule,
  /* 1029 */  immL_rule,
  /* 1030 */  immL0_rule,
  /* 1031 */  immL_M1_rule,
  /* 1032 */  vlRegF_rule,
  /* 1033 */  _LoadF_memory__rule,
  /* 1034 */  immF0_rule,
  /* 1035 */  vlRegD_rule,
  /* 1036 */  _LoadD_memory__rule,
  /* 1037 */  immD0_rule,
  /* 1038 */  rRegI_rule,
  /* 1039 */  rRegI_rule,
  /* 1040 */  rRegI_rule,
  /* 1041 */  rRegI_rule,
  /* 1042 */  rRegI_rule,
  /* 1043 */  rRegI_rule,
  /* 1044 */  rRegI_rule,
  /* 1045 */  rRegI_rule,
  /* 1046 */  rRegI_rule,
  /* 1047 */  rRegI_rule,
  /* 1048 */  regF_rule,
  /* 1049 */  regF_rule,
  /* 1050 */  regF_rule,
  /* 1051 */  regF_rule,
  /* 1052 */  regF_rule,
  /* 1053 */  regF_rule,
  /* 1054 */  regD_rule,
  /* 1055 */  regD_rule,
  /* 1056 */  regD_rule,
  /* 1057 */  regD_rule,
  /* 1058 */  regD_rule,
  /* 1059 */  regD_rule,
  /* 1060 */  vec_rule,
  /* 1061 */  vec_rule,
  /* 1062 */  vec_rule,
  /* 1063 */  _LoadVector_memory__rule,
  /* 1064 */  vec_rule,
  /* 1065 */  vec_rule,
  /* 1066 */  vec_rule,
  /* 1067 */  _LoadVector_memory__rule,
  /* 1068 */  vec_rule,
  /* 1069 */  vec_rule,
  /* 1070 */  vec_rule,
  /* 1071 */  _LoadVector_memory__rule,
  /* 1072 */  vec_rule,
  /* 1073 */  vec_rule,
  /* 1074 */  vec_rule,
  /* 1075 */  _LoadVector_memory__rule,
  /* 1076 */  vec_rule,
  /* 1077 */  vec_rule,
  /* 1078 */  vec_rule,
  /* 1079 */  _LoadVector_memory__rule,
  /* 1080 */  vec_rule,
  /* 1081 */  vec_rule,
  /* 1082 */  vec_rule,
  /* 1083 */  _LoadVector_memory__rule,
  /* 1084 */  vec_rule,
  /* 1085 */  vec_rule,
  /* 1086 */  vec_rule,
  /* 1087 */  vec_rule,
  /* 1088 */  vec_rule,
  /* 1089 */  vec_rule,
  /* 1090 */  vec_rule,
  /* 1091 */  vec_rule,
  /* 1092 */  vec_rule,
  /* 1093 */  vec_rule,
  /* 1094 */  vec_rule,
  /* 1095 */  vec_rule,
  /* 1096 */  vec_rule,
  /* 1097 */  vec_rule,
  /* 1098 */  vec_rule,
  /* 1099 */  vec_rule,
  /* 1100 */  vec_rule,
  /* 1101 */  vec_rule,
  /* 1102 */  vec_rule,
  /* 1103 */  vec_rule,
  /* 1104 */  vec_rule,
  /* 1105 */  vec_rule,
  /* 1106 */  vec_rule,
  /* 1107 */  vec_rule,
  /* 1108 */  vec_rule,
  /* 1109 */  vec_rule,
  /* 1110 */  _LoadVector_memory__rule,
  /* 1111 */  vec_rule,
  /* 1112 */  vec_rule,
  /* 1113 */  vec_rule,
  /* 1114 */  _LoadVector_memory__rule,
  /* 1115 */  vec_rule,
  /* 1116 */  vec_rule,
  /* 1117 */  _LoadVector_memory__rule,
  /* 1118 */  vec_rule,
  /* 1119 */  vec_rule,
  /* 1120 */  vec_rule,
  /* 1121 */  _LoadVector_memory__rule,
  /* 1122 */  vec_rule,
  /* 1123 */  vec_rule,
  /* 1124 */  vec_rule,
  /* 1125 */  _LoadVector_memory__rule,
  /* 1126 */  _Binary_cmpOp_vcmppd_immI8_rule,
  /* 1127 */  _Binary_cmpOp_vcmppd_immI8_rule,
  /* 1128 */  vec_rule,
  /* 1129 */  vec_rule,
  /* 1130 */  vec_rule,
  /* 1131 */  vec_rule,
  /* 1132 */  vec_rule,
  /* 1133 */  vec_rule,
  /* 1134 */  vec_rule,
  /* 1135 */  vec_rule,
  /* 1136 */  vec_rule,
  /* 1137 */  vec_rule,
  /* 1138 */  vec_rule,
  /* 1139 */  vec_rule,
  /* 1140 */  vec_rule,
  /* 1141 */  vec_rule,
  /* 1142 */  vec_rule,
  /* 1143 */  vec_rule,
  /* 1144 */  vec_rule,
  /* 1145 */  vec_rule,
  /* 1146 */  vec_rule,
  /* 1147 */  vec_rule,
  /* 1148 */  vec_rule,
  /* 1149 */  vec_rule,
  /* 1150 */  vec_rule,
  /* 1151 */  vec_rule,
  /* 1152 */  vec_rule,
  /* 1153 */  vec_rule,
  /* 1154 */  vec_rule,
  /* 1155 */  vec_rule,
  /* 1156 */  vec_rule,
  /* 1157 */  vec_rule,
  /* 1158 */  vec_rule,
  /* 1159 */  vec_rule,
  /* 1160 */  vec_rule,
  /* 1161 */  vec_rule,
  /* 1162 */  _LoadVector_memory__rule,
  /* 1163 */  vec_rule,
  /* 1164 */  vec_rule,
  /* 1165 */  vec_rule,
  /* 1166 */  _LoadVector_memory__rule,
  /* 1167 */  vec_rule,
  /* 1168 */  vec_rule,
  /* 1169 */  vec_rule,
  /* 1170 */  _LoadVector_memory__rule,
  /* 1171 */  vec_rule,
  /* 1172 */  vec_rule,
  /* 1173 */  vec_rule,
  /* 1174 */  vec_rule,
  /* 1175 */  vec_rule,
  /* 1176 */  vec_rule,
  /* 1177 */  vec_rule,
  /* 1178 */  vec_rule,
  /* 1179 */  vec_rule,
  /* 1180 */  vec_rule,
  /* 1181 */  vec_rule,
  /* 1182 */  vec_rule,
  /* 1183 */  _MulAddVS2VI_vec_vec_rule,
  /* 1184 */  vec_rule,
  /* 1185 */  _Binary_vec_vec_rule,
  /* 1186 */  _Binary_vec_vec_rule,
  /* 1187 */  memory_rule,
  /* 1188 */  memory_rule,
  /* 1189 */  memory_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  immI8_rule,
  /*   94 */  immI_rule,
  /*   95 */  rRegI_rule,
  /*   96 */  immI_rule,
  /*   97 */  rRegI_rule,
  /*   98 */  _LShiftI_rRegI_immI2_rule,
  /*   99 */  immI_rule,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  immI_rule,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  0,
  /*  108 */  0,
  /*  109 */  immI8_rule,
  /*  110 */  immI_rule,
  /*  111 */  immI_rule,
  /*  112 */  _LShiftI_rRegI_immI2_rule,
  /*  113 */  immI_rule,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  0,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  // last operand
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  // last operand class
  /*  137 */  rRegI_rule,
  /*  138 */  immI2_rule,
  /*  139 */  _LShiftI_rRegI_immI2_rule,
  /*  140 */  rRegI_rule,
  /*  141 */  _LShiftI_rRegI_immI2_rule,
  /*  142 */  0,
  /*  143 */  0,
  /*  144 */  0,
  /*  145 */  0,
  /*  146 */  immI_rule,
  /*  147 */  0,
  /*  148 */  immI_24_rule,
  /*  149 */  0,
  /*  150 */  immI_24_rule,
  /*  151 */  immI_255_rule,
  /*  152 */  immI_rule,
  /*  153 */  immI_24_rule,
  /*  154 */  immI_16_rule,
  /*  155 */  immI_255_rule,
  /*  156 */  immI_65535_rule,
  /*  157 */  immU31_rule,
  /*  158 */  0,
  /*  159 */  0,
  /*  160 */  0,
  /*  161 */  0,
  /*  162 */  0,
  /*  163 */  eFlagsReg_rule,
  /*  164 */  rRegI_rule,
  /*  165 */  eFlagsRegU_rule,
  /*  166 */  eFlagsRegUCF_rule,
  /*  167 */  _LoadI_memory__rule,
  /*  168 */  eRegP_rule,
  /*  169 */  eFlagsRegU_rule,
  /*  170 */  regDPR_rule,
  /*  171 */  regFPR_rule,
  /*  172 */  regDPR_rule,
  /*  173 */  regFPR_rule,
  /*  174 */  regF_rule,
  /*  175 */  regD_rule,
  /*  176 */  eRegL_rule,
  /*  177 */  rRegI_rule,
  /*  178 */  _LoadI_memory__rule,
  /*  179 */  immI_rule,
  /*  180 */  immI1_rule,
  /*  181 */  immI_M1_rule,
  /*  182 */  eRegP_rule,
  /*  183 */  rRegI_rule,
  /*  184 */  eBCXRegL_rule,
  /*  185 */  eCXRegP_rule,
  /*  186 */  eCXRegI_rule,
  /*  187 */  rRegI_rule,
  /*  188 */  rRegI_rule,
  /*  189 */  0,
  /*  190 */  eADXRegL_low_only_rule,
  /*  191 */  immI_32_rule,
  /*  192 */  _ConvI2L_nadxRegI__rule,
  /*  193 */  immI_32_rule,
  /*  194 */  immI_32_63_rule,
  /*  195 */  immI_32_63_rule,
  /*  196 */  0,
  /*  197 */  immL_32bits_rule,
  /*  198 */  immL_32bits_rule,
  /*  199 */  immI1_rule,
  /*  200 */  immI8_rule,
  /*  201 */  immI_24_rule,
  /*  202 */  immI_16_rule,
  /*  203 */  rRegI_rule,
  /*  204 */  _LoadI_memory__rule,
  /*  205 */  immI_rule,
  /*  206 */  immI_M1_rule,
  /*  207 */  _LoadI_memory__rule,
  /*  208 */  immI_M1_rule,
  /*  209 */  0,
  /*  210 */  rRegI_rule,
  /*  211 */  _LoadI_memory__rule,
  /*  212 */  immI_rule,
  /*  213 */  immI1_rule,
  /*  214 */  immI_M1_rule,
  /*  215 */  immI8_rule,
  /*  216 */  immI8_rule,
  /*  217 */  eCXRegI_rule,
  /*  218 */  eCXRegI_rule,
  /*  219 */  _SubI_immI0_eCXRegI_rule,
  /*  220 */  eCXRegI_rule,
  /*  221 */  _SubI_immI_32_eCXRegI_rule,
  /*  222 */  immI1_rule,
  /*  223 */  immI_M1_rule,
  /*  224 */  eCXRegI_rule,
  /*  225 */  _SubI_immI0_eCXRegI_rule,
  /*  226 */  _SubI_immI_32_eCXRegI_rule,
  /*  227 */  rRegI_rule,
  /*  228 */  _LoadI_memory__rule,
  /*  229 */  immI_rule,
  /*  230 */  rRegI_rule,
  /*  231 */  rRegI_rule,
  /*  232 */  rRegI_rule,
  /*  233 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  234 */  0,
  /*  235 */  immL_M1_rule,
  /*  236 */  eRegL_rule,
  /*  237 */  _LoadL_memory__rule,
  /*  238 */  immL_M1_rule,
  /*  239 */  immL_M1_rule,
  /*  240 */  0,
  /*  241 */  regDPR_rule,
  /*  242 */  regDPR_rule,
  /*  243 */  regDPR_rule,
  /*  244 */  0,
  /*  245 */  _LoadD_memory__rule,
  /*  246 */  0,
  /*  247 */  immDPR_rule,
  /*  248 */  regDPR_rule,
  /*  249 */  regDPR_rule,
  /*  250 */  0,
  /*  251 */  regFPR_rule,
  /*  252 */  _LoadF_memory__rule,
  /*  253 */  regFPR_rule,
  /*  254 */  regFPR_rule,
  /*  255 */  0,
  /*  256 */  eCXRegI_rule,
  /*  257 */  eDXRegI_rule,
  /*  258 */  eSIRegP_rule,
  /*  259 */  eDXRegI_rule,
  /*  260 */  immI_rule,
  /*  261 */  eAXRegI_rule,
  /*  262 */  immI_rule,
  /*  263 */  memory_rule,
  /*  264 */  0,
  /*  265 */  eAXRegI_rule,
  /*  266 */  eAXRegP_rule,
  /*  267 */  flagsReg_long_LTGE_rule,
  /*  268 */  _LoadL_load_long_memory__rule,
  /*  269 */  flagsReg_long_EQNE_rule,
  /*  270 */  flagsReg_long_LEGT_rule,
  /*  271 */  0,
  /*  272 */  immI8_rule,
  /*  273 */  legVec_rule,
  /*  274 */  vec_rule,
  /*  275 */  _LoadVector_memory__rule,
  /*  276 */  vec_rule,
  /*  277 */  immU8_rule,
  /*  278 */  immU8_rule,
  // last internally defined operand
  /*  279 */  0,
  /*  280 */  0,
  /*  281 */  0,
  /*  282 */  0,
  /*  283 */  0,
  /*  284 */  0,
  /*  285 */  0,
  /*  286 */  0,
  /*  287 */  0,
  /*  288 */  0,
  /*  289 */  0,
  /*  290 */  0,
  /*  291 */  0,
  /*  292 */  0,
  /*  293 */  0,
  /*  294 */  0,
  /*  295 */  0,
  /*  296 */  0,
  /*  297 */  0,
  /*  298 */  0,
  /*  299 */  0,
  /*  300 */  0,
  /*  301 */  0,
  /*  302 */  0,
  /*  303 */  0,
  /*  304 */  0,
  /*  305 */  0,
  /*  306 */  0,
  /*  307 */  0,
  /*  308 */  0,
  /*  309 */  0,
  /*  310 */  0,
  /*  311 */  0,
  /*  312 */  0,
  /*  313 */  0,
  /*  314 */  0,
  /*  315 */  0,
  /*  316 */  0,
  /*  317 */  0,
  /*  318 */  0,
  /*  319 */  0,
  /*  320 */  _Binary_eAXRegP_eRegP_rule,
  /*  321 */  _Binary_eAXRegI_rRegI_rule,
  /*  322 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  rRegI_rule,
  /*  326 */  immI_rule,
  /*  327 */  rRegI_rule,
  /*  328 */  immI_rule,
  /*  329 */  eAXRegI_rule,
  /*  330 */  rRegI_rule,
  /*  331 */  immI_rule,
  /*  332 */  regDPR_rule,
  /*  333 */  regDPR_rule,
  /*  334 */  regDPR_rule,
  /*  335 */  regD_rule,
  /*  336 */  regD_rule,
  /*  337 */  _LoadD_memory__rule,
  /*  338 */  _LoadD_memory__rule,
  /*  339 */  0,
  /*  340 */  0,
  /*  341 */  regFPR_rule,
  /*  342 */  regFPR_rule,
  /*  343 */  regFPR_rule,
  /*  344 */  regF_rule,
  /*  345 */  regF_rule,
  /*  346 */  _LoadF_memory__rule,
  /*  347 */  _LoadF_memory__rule,
  /*  348 */  0,
  /*  349 */  0,
  /*  350 */  0,
  /*  351 */  0,
  /*  352 */  0,
  /*  353 */  0,
  /*  354 */  0,
  /*  355 */  0,
  /*  356 */  0,
  /*  357 */  0,
  /*  358 */  0,
  /*  359 */  rRegI_rule,
  /*  360 */  immI_rule,
  /*  361 */  _LoadI_memory__rule,
  /*  362 */  immI0_rule,
  /*  363 */  immI0_rule,
  /*  364 */  immI0_rule,
  /*  365 */  rRegI_rule,
  /*  366 */  immI_rule,
  /*  367 */  _LoadI_memory__rule,
  /*  368 */  immI0_rule,
  /*  369 */  eRegP_rule,
  /*  370 */  immP_rule,
  /*  371 */  _LoadP_memory__rule,
  /*  372 */  _LoadP_memory__rule,
  /*  373 */  immP0_rule,
  /*  374 */  immI0_rule,
  /*  375 */  immP0_rule,
  /*  376 */  immL0_rule,
  /*  377 */  eRegL_rule,
  /*  378 */  immL0_rule,
  /*  379 */  eRegL_rule,
  /*  380 */  immL0_rule,
  /*  381 */  eRegL_rule,
  /*  382 */  immL0_rule,
  /*  383 */  eRegL_rule,
  /*  384 */  immL0_rule,
  /*  385 */  eRegL_rule,
  /*  386 */  immL0_rule,
  /*  387 */  eRegL_rule,
  /*  388 */  eBXRegP_rule,
  /*  389 */  eBXRegP_rule,
  /*  390 */  eAXRegP_rule,
  /*  391 */  0,
  /*  392 */  0,
  /*  393 */  0,
  /*  394 */  0,
  /*  395 */  0,
  /*  396 */  0,
  /*  397 */  0,
  /*  398 */  0,
  /*  399 */  0,
  /*  400 */  0,
  /*  401 */  0,
  /*  402 */  0,
  /*  403 */  0,
  /*  404 */  0,
  /*  405 */  0,
  /*  406 */  0,
  /*  407 */  0,
  /*  408 */  0,
  /*  409 */  0,
  /*  410 */  0,
  /*  411 */  0,
  /*  412 */  0,
  /*  413 */  0,
  /*  414 */  0,
  /*  415 */  0,
  /*  416 */  0,
  /*  417 */  0,
  /*  418 */  0,
  /*  419 */  0,
  /*  420 */  0,
  /*  421 */  0,
  /*  422 */  0,
  /*  423 */  0,
  /*  424 */  0,
  /*  425 */  0,
  /*  426 */  0,
  /*  427 */  0,
  /*  428 */  0,
  /*  429 */  0,
  /*  430 */  0,
  /*  431 */  0,
  /*  432 */  immI_24_rule,
  /*  433 */  0,
  /*  434 */  0,
  /*  435 */  immI_24_rule,
  /*  436 */  0,
  /*  437 */  0,
  /*  438 */  0,
  /*  439 */  0,
  /*  440 */  immI_24_rule,
  /*  441 */  immI_255_rule,
  /*  442 */  immI_16_rule,
  /*  443 */  immI_65535_rule,
  /*  444 */  0,
  /*  445 */  0,
  /*  446 */  0,
  /*  447 */  0,
  /*  448 */  immL_32bits_rule,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  0,
  /*  452 */  0,
  /*  453 */  0,
  /*  454 */  0,
  /*  455 */  0,
  /*  456 */  0,
  /*  457 */  0,
  /*  458 */  0,
  /*  459 */  0,
  /*  460 */  0,
  /*  461 */  0,
  /*  462 */  0,
  /*  463 */  0,
  /*  464 */  0,
  /*  465 */  0,
  /*  466 */  xRegI_rule,
  /*  467 */  rRegI_rule,
  /*  468 */  rRegI_rule,
  /*  469 */  eRegL_rule,
  /*  470 */  _ConvL2I_eRegL__rule,
  /*  471 */  stackSlotL_rule,
  /*  472 */  stackSlotL_rule,
  /*  473 */  eRegL_rule,
  /*  474 */  anyRegP_rule,
  /*  475 */  immI_rule,
  /*  476 */  immI16_rule,
  /*  477 */  immP_rule,
  /*  478 */  immI8_rule,
  /*  479 */  immI8_rule,
  /*  480 */  regDPR1_rule,
  /*  481 */  _RoundDouble_regDPR1__rule,
  /*  482 */  regD_rule,
  /*  483 */  regF_rule,
  /*  484 */  regFPR1_rule,
  /*  485 */  _RoundFloat_regFPR1__rule,
  /*  486 */  _ConvD2F_regDPR1__rule,
  /*  487 */  immFPR_rule,
  /*  488 */  immF_rule,
  /*  489 */  0,
  /*  490 */  0,
  /*  491 */  0,
  /*  492 */  0,
  /*  493 */  0,
  /*  494 */  0,
  /*  495 */  0,
  /*  496 */  0,
  /*  497 */  _Binary_rRegI_rRegI_rule,
  /*  498 */  _Binary_rRegI_rRegI_rule,
  /*  499 */  _Binary_rRegI_rRegI_rule,
  /*  500 */  _Binary_rRegI_rRegI_rule,
  /*  501 */  _Binary_rRegI_rRegI_rule,
  /*  502 */  _Binary_rRegI__LoadI_memory__rule,
  /*  503 */  _Binary_rRegI__LoadI_memory__rule,
  /*  504 */  _Binary_rRegI__LoadI_memory__rule,
  /*  505 */  _Binary_eRegP_eRegP_rule,
  /*  506 */  _Binary_eRegP_eRegP_rule,
  /*  507 */  _Binary_eRegP_eRegP_rule,
  /*  508 */  _Binary_eRegP_eRegP_rule,
  /*  509 */  _Binary_regDPR1_regDPR_rule,
  /*  510 */  _Binary_regFPR1_regFPR_rule,
  /*  511 */  _Binary_regDPR_regDPR_rule,
  /*  512 */  _Binary_regFPR_regFPR_rule,
  /*  513 */  _Binary_regF_regF_rule,
  /*  514 */  _Binary_regD_regD_rule,
  /*  515 */  _Binary_regF_regF_rule,
  /*  516 */  _Binary_regF_regF_rule,
  /*  517 */  _Binary_regD_regD_rule,
  /*  518 */  _Binary_regD_regD_rule,
  /*  519 */  _Binary_eRegL_eRegL_rule,
  /*  520 */  _Binary_eRegL_eRegL_rule,
  /*  521 */  _Binary_eRegL_eRegL_rule,
  /*  522 */  rRegI_rule,
  /*  523 */  immI_rule,
  /*  524 */  immI1_rule,
  /*  525 */  immI_rule,
  /*  526 */  immI_rule,
  /*  527 */  immI_M1_rule,
  /*  528 */  rRegI_rule,
  /*  529 */  immI_rule,
  /*  530 */  _LoadI_memory__rule,
  /*  531 */  rRegI_rule,
  /*  532 */  _AddI__LoadI_memory__rRegI_rule,
  /*  533 */  _AddI_rRegI__LoadI_memory__rule,
  /*  534 */  _AddI__LoadI_memory__immI_rule,
  /*  535 */  _AddI__LoadI_memory__immI1_rule,
  /*  536 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  537 */  0,
  /*  538 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  539 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  540 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  541 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  542 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  543 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  544 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  545 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  546 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  547 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  548 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  549 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  550 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  551 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  552 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  553 */  immI_rule,
  /*  554 */  xRegI_rule,
  /*  555 */  immI_rule,
  /*  556 */  rRegI_rule,
  /*  557 */  immI_rule,
  /*  558 */  rRegI_rule,
  /*  559 */  xRegI_rule,
  /*  560 */  rRegI_rule,
  /*  561 */  rRegI_rule,
  /*  562 */  pRegP_rule,
  /*  563 */  rRegI_rule,
  /*  564 */  immI_rule,
  /*  565 */  _LoadI_memory__rule,
  /*  566 */  _SubI__LoadI_memory__rRegI_rule,
  /*  567 */  _SubI_immI0_rRegI_rule,
  /*  568 */  rRegI_rule,
  /*  569 */  rRegI_rule,
  /*  570 */  immI_rule,
  /*  571 */  0,
  /*  572 */  0,
  /*  573 */  0,
  /*  574 */  0,
  /*  575 */  immI_rule,
  /*  576 */  _LoadI_memory__rule,
  /*  577 */  rRegI_rule,
  /*  578 */  _Binary_rRegI_rRegI_rule,
  /*  579 */  _ConvI2L_nadxRegI__rule,
  /*  580 */  _ConvI2L_eAXRegI__rule,
  /*  581 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  582 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  583 */  eRegL_rule,
  /*  584 */  eRegL_rule,
  /*  585 */  eRegL_rule,
  /*  586 */  eRegL_rule,
  /*  587 */  immL_127_rule,
  /*  588 */  eCXRegI_rule,
  /*  589 */  eRegL_rule,
  /*  590 */  eCXRegI_rule,
  /*  591 */  eCXRegI_rule,
  /*  592 */  eRegL_rule,
  /*  593 */  immL32_rule,
  /*  594 */  immL32_rule,
  /*  595 */  immI1_rule,
  /*  596 */  immI8_rule,
  /*  597 */  eCXRegI_rule,
  /*  598 */  immI1_rule,
  /*  599 */  _RShiftI__LoadI_memory__immI1_rule,
  /*  600 */  immI8_rule,
  /*  601 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  602 */  eCXRegI_rule,
  /*  603 */  immI1_rule,
  /*  604 */  immI8_rule,
  /*  605 */  immI_24_rule,
  /*  606 */  immI_16_rule,
  /*  607 */  eCXRegI_rule,
  /*  608 */  rRegI_rule,
  /*  609 */  immI_rule,
  /*  610 */  _LoadI_memory__rule,
  /*  611 */  rRegI_rule,
  /*  612 */  _AndI__LoadI_memory__rRegI_rule,
  /*  613 */  _AndI_rRegI__LoadI_memory__rule,
  /*  614 */  _AndI__LoadI_memory__immI_rule,
  /*  615 */  rRegI_rule,
  /*  616 */  _XorI_rRegI_immI_M1_rule,
  /*  617 */  _LoadI_memory__rule,
  /*  618 */  _XorI_rRegI_immI_M1_rule,
  /*  619 */  rRegI_rule,
  /*  620 */  _SubI_immI0_rRegI_rule,
  /*  621 */  _LoadI_memory__rule,
  /*  622 */  _SubI_immI0__LoadI_memory__rule,
  /*  623 */  rRegI_rule,
  /*  624 */  _AddI_rRegI_immI_M1_rule,
  /*  625 */  _LoadI_memory__rule,
  /*  626 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  627 */  rRegI_rule,
  /*  628 */  _AddI_rRegI_immI_M1_rule,
  /*  629 */  _LoadI_memory__rule,
  /*  630 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  631 */  rRegI_rule,
  /*  632 */  _CastP2X_eRegP__rule,
  /*  633 */  rRegI_rule,
  /*  634 */  immI_rule,
  /*  635 */  _LoadI_memory__rule,
  /*  636 */  rRegI_rule,
  /*  637 */  _OrI__LoadI_memory__rRegI_rule,
  /*  638 */  _OrI_rRegI__LoadI_memory__rule,
  /*  639 */  _OrI__LoadI_memory__immI_rule,
  /*  640 */  0,
  /*  641 */  0,
  /*  642 */  0,
  /*  643 */  _URShiftI_rRegI_immI_M1_rule,
  /*  644 */  _LShiftI_rRegI_immI1_rule,
  /*  645 */  _URShiftI_rRegI_immI8_rule,
  /*  646 */  _LShiftI_rRegI_immI8_rule,
  /*  647 */  _URShiftI_ncxRegI__SubI_immI0_eCXRegI_rule,
  /*  648 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  649 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  650 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  651 */  0,
  /*  652 */  0,
  /*  653 */  0,
  /*  654 */  _LShiftI_rRegI_immI_M1_rule,
  /*  655 */  _URShiftI_rRegI_immI1_rule,
  /*  656 */  _LShiftI_rRegI_immI8_rule,
  /*  657 */  _URShiftI_rRegI_immI8_rule,
  /*  658 */  _LShiftI_ncxRegI__SubI_immI0_eCXRegI_rule,
  /*  659 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  660 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  661 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  662 */  rRegI_rule,
  /*  663 */  immI_M1_rule,
  /*  664 */  immI_rule,
  /*  665 */  _LoadI_memory__rule,
  /*  666 */  rRegI_rule,
  /*  667 */  _XorI__LoadI_memory__rRegI_rule,
  /*  668 */  _XorI_rRegI__LoadI_memory__rule,
  /*  669 */  _XorI__LoadI_memory__immI_rule,
  /*  670 */  0,
  /*  671 */  0,
  /*  672 */  0,
  /*  673 */  0,
  /*  674 */  ncxRegI_rule,
  /*  675 */  immI0_rule,
  /*  676 */  _SubI_rRegI_rRegI_rule,
  /*  677 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  678 */  _SubI_rRegI_rRegI_rule,
  /*  679 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  680 */  rRegI_rule,
  /*  681 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  682 */  0,
  /*  683 */  eRegL_rule,
  /*  684 */  immL_rule,
  /*  685 */  _LoadL_load_long_memory__rule,
  /*  686 */  eRegL_rule,
  /*  687 */  eRegL_rule,
  /*  688 */  immL_rule,
  /*  689 */  _LoadL_load_long_memory__rule,
  /*  690 */  eRegL_rule,
  /*  691 */  eRegL_rule,
  /*  692 */  immL_rule,
  /*  693 */  _LoadL_load_long_memory__rule,
  /*  694 */  eRegL_rule,
  /*  695 */  eRegL_rule,
  /*  696 */  _XorL_eRegL_immL_M1_rule,
  /*  697 */  _LoadL_memory__rule,
  /*  698 */  _XorL_eRegL_immL_M1_rule,
  /*  699 */  eRegL_rule,
  /*  700 */  _SubL_immL0_eRegL_rule,
  /*  701 */  _LoadL_memory__rule,
  /*  702 */  _SubL_immL0__LoadL_memory__rule,
  /*  703 */  eRegL_rule,
  /*  704 */  _AddL_eRegL_immL_M1_rule,
  /*  705 */  _LoadL_memory__rule,
  /*  706 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  707 */  eRegL_rule,
  /*  708 */  _AddL_eRegL_immL_M1_rule,
  /*  709 */  _LoadL_memory__rule,
  /*  710 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  711 */  eRegL_rule,
  /*  712 */  immL_rule,
  /*  713 */  _LoadL_load_long_memory__rule,
  /*  714 */  eRegL_rule,
  /*  715 */  eRegL_rule,
  /*  716 */  immL_M1_rule,
  /*  717 */  immL_rule,
  /*  718 */  _LoadL_load_long_memory__rule,
  /*  719 */  eRegL_rule,
  /*  720 */  immI_1_rule,
  /*  721 */  immI_2_rule,
  /*  722 */  immI_3_rule,
  /*  723 */  immI_1_31_rule,
  /*  724 */  immI_32_63_rule,
  /*  725 */  eCXRegI_rule,
  /*  726 */  immI_1_31_rule,
  /*  727 */  immI_32_63_rule,
  /*  728 */  eCXRegI_rule,
  /*  729 */  immI_1_31_rule,
  /*  730 */  immI_32_63_rule,
  /*  731 */  eCXRegI_rule,
  /*  732 */  immDPR0_rule,
  /*  733 */  regDPR_rule,
  /*  734 */  regD_rule,
  /*  735 */  _LoadD_memory__rule,
  /*  736 */  regDPR_rule,
  /*  737 */  0,
  /*  738 */  _LoadD_memory__rule,
  /*  739 */  regDPR_rule,
  /*  740 */  0,
  /*  741 */  _LoadD_memory__rule,
  /*  742 */  regDPR_rule,
  /*  743 */  _RoundDouble__AddD__LoadD_memory__regDPR__rule,
  /*  744 */  _RoundDouble__AddD_regDPR__LoadD_memory___rule,
  /*  745 */  immDPR1_rule,
  /*  746 */  immDPR_rule,
  /*  747 */  0,
  /*  748 */  regDPR_rule,
  /*  749 */  regnotDPR1_rule,
  /*  750 */  immDPR_rule,
  /*  751 */  _LoadD_memory__rule,
  /*  752 */  regDPR_rule,
  /*  753 */  _LoadD_memory__rule,
  /*  754 */  regDPR_rule,
  /*  755 */  regDPR_rule,
  /*  756 */  _MulD_regDPR_regDPR_rule,
  /*  757 */  regDPR_rule,
  /*  758 */  regDPR_rule,
  /*  759 */  regnotDPR1_rule,
  /*  760 */  0,
  /*  761 */  regDPR_rule,
  /*  762 */  regD_rule,
  /*  763 */  regDPR_rule,
  /*  764 */  regD_rule,
  /*  765 */  0,
  /*  766 */  immFPR0_rule,
  /*  767 */  regFPR_rule,
  /*  768 */  regF_rule,
  /*  769 */  _LoadF_memory__rule,
  /*  770 */  regFPR_rule,
  /*  771 */  regFPR_rule,
  /*  772 */  regFPR_rule,
  /*  773 */  regFPR_rule,
  /*  774 */  _LoadF_memory__rule,
  /*  775 */  regFPR_rule,
  /*  776 */  _LoadF_memory__rule,
  /*  777 */  regFPR_rule,
  /*  778 */  regFPR_rule,
  /*  779 */  _LoadF_memory__rule,
  /*  780 */  memory_rule,
  /*  781 */  memory_rule,
  /*  782 */  immFPR_rule,
  /*  783 */  immFPR_rule,
  /*  784 */  regFPR_rule,
  /*  785 */  regFPR_rule,
  /*  786 */  _LoadF_memory__rule,
  /*  787 */  regFPR_rule,
  /*  788 */  _LoadF_memory__rule,
  /*  789 */  regFPR_rule,
  /*  790 */  memory_rule,
  /*  791 */  immFPR_rule,
  /*  792 */  immFPR_rule,
  /*  793 */  regFPR_rule,
  /*  794 */  _LoadF_memory__rule,
  /*  795 */  regFPR_rule,
  /*  796 */  _MulF__LoadF_memory__regFPR_rule,
  /*  797 */  regFPR_rule,
  /*  798 */  _MulF_regFPR__LoadF_memory__rule,
  /*  799 */  regFPR_rule,
  /*  800 */  _MulF_regFPR_regFPR_rule,
  /*  801 */  regFPR_rule,
  /*  802 */  regFPR_rule,
  /*  803 */  regFPR_rule,
  /*  804 */  regFPR_rule,
  /*  805 */  regFPR_rule,
  /*  806 */  regF_rule,
  /*  807 */  0,
  /*  808 */  0,
  /*  809 */  0,
  /*  810 */  0,
  /*  811 */  0,
  /*  812 */  0,
  /*  813 */  0,
  /*  814 */  0,
  /*  815 */  0,
  /*  816 */  0,
  /*  817 */  0,
  /*  818 */  0,
  /*  819 */  0,
  /*  820 */  0,
  /*  821 */  0,
  /*  822 */  0,
  /*  823 */  0,
  /*  824 */  0,
  /*  825 */  0,
  /*  826 */  0,
  /*  827 */  0,
  /*  828 */  0,
  /*  829 */  0,
  /*  830 */  0,
  /*  831 */  0,
  /*  832 */  0,
  /*  833 */  0,
  /*  834 */  0,
  /*  835 */  0,
  /*  836 */  0,
  /*  837 */  immL_32bits_rule,
  /*  838 */  immL_32bits_rule,
  /*  839 */  0,
  /*  840 */  0,
  /*  841 */  0,
  /*  842 */  0,
  /*  843 */  0,
  /*  844 */  0,
  /*  845 */  0,
  /*  846 */  0,
  /*  847 */  0,
  /*  848 */  0,
  /*  849 */  0,
  /*  850 */  0,
  /*  851 */  0,
  /*  852 */  eDIRegP_rule,
  /*  853 */  eDIRegP_rule,
  /*  854 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  855 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  856 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  857 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  858 */  eCXRegI_rule,
  /*  859 */  _Binary_eSIRegP_immI_rule,
  /*  860 */  _Binary_eSIRegP_immI_rule,
  /*  861 */  _Binary_eSIRegP_immI_rule,
  /*  862 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  863 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  864 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  865 */  eAXRegI_rule,
  /*  866 */  eSIRegP_rule,
  /*  867 */  eSIRegP_rule,
  /*  868 */  eCXRegI_rule,
  /*  869 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  870 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  871 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  872 */  rRegI_rule,
  /*  873 */  rRegI_rule,
  /*  874 */  immI_rule,
  /*  875 */  0,
  /*  876 */  0,
  /*  877 */  eFlagsReg_rule,
  /*  878 */  eFlagsReg_rule,
  /*  879 */  eFlagsRegU_rule,
  /*  880 */  eFlagsRegUCF_rule,
  /*  881 */  eFlagsReg_rule,
  /*  882 */  eFlagsRegU_rule,
  /*  883 */  eFlagsRegUCF_rule,
  /*  884 */  eFlagsRegU_rule,
  /*  885 */  eFlagsRegUCF_rule,
  /*  886 */  eFlagsRegUCF_rule,
  /*  887 */  eAXRegP_rule,
  /*  888 */  0,
  /*  889 */  eFlagsReg_rule,
  /*  890 */  eFlagsReg_rule,
  /*  891 */  eFlagsRegU_rule,
  /*  892 */  eFlagsRegUCF_rule,
  /*  893 */  eFlagsRegU_rule,
  /*  894 */  eFlagsRegUCF_rule,
  /*  895 */  eFlagsRegUCF_rule,
  /*  896 */  eRegL_rule,
  /*  897 */  flagsReg_long_LTGE_rule,
  /*  898 */  flagsReg_ulong_LTGE_rule,
  /*  899 */  _Binary_eRegL_eRegL_rule,
  /*  900 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  901 */  _Binary_rRegI_rRegI_rule,
  /*  902 */  _Binary_rRegI__LoadI_memory__rule,
  /*  903 */  _Binary_eRegP_eRegP_rule,
  /*  904 */  _Binary_regDPR_regDPR_rule,
  /*  905 */  _Binary_regD_regD_rule,
  /*  906 */  _Binary_regFPR_regFPR_rule,
  /*  907 */  _Binary_regF_regF_rule,
  /*  908 */  flagsReg_long_EQNE_rule,
  /*  909 */  flagsReg_ulong_EQNE_rule,
  /*  910 */  _Binary_eRegL_eRegL_rule,
  /*  911 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  912 */  _Binary_rRegI_rRegI_rule,
  /*  913 */  _Binary_rRegI__LoadI_memory__rule,
  /*  914 */  _Binary_eRegP_eRegP_rule,
  /*  915 */  _Binary_regDPR_regDPR_rule,
  /*  916 */  _Binary_regD_regD_rule,
  /*  917 */  _Binary_regFPR_regFPR_rule,
  /*  918 */  _Binary_regF_regF_rule,
  /*  919 */  flagsReg_long_LEGT_rule,
  /*  920 */  flagsReg_ulong_LEGT_rule,
  /*  921 */  _Binary_eRegL_eRegL_rule,
  /*  922 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  923 */  _Binary_rRegI_rRegI_rule,
  /*  924 */  _Binary_rRegI__LoadI_memory__rule,
  /*  925 */  _Binary_eRegP_eRegP_rule,
  /*  926 */  _Binary_regDPR_regDPR_rule,
  /*  927 */  _Binary_regD_regD_rule,
  /*  928 */  _Binary_regFPR_regFPR_rule,
  /*  929 */  _Binary_regF_regF_rule,
  /*  930 */  0,
  /*  931 */  0,
  /*  932 */  0,
  /*  933 */  0,
  /*  934 */  0,
  /*  935 */  0,
  /*  936 */  eBXRegP_rule,
  /*  937 */  eAXRegP_rule,
  /*  938 */  0,
  /*  939 */  0,
  /*  940 */  0,
  /*  941 */  0,
  /*  942 */  0,
  /*  943 */  0,
  /*  944 */  regF_rule,
  /*  945 */  _LoadF_memory__rule,
  /*  946 */  regF_rule,
  /*  947 */  immF_rule,
  /*  948 */  regF_rule,
  /*  949 */  _LoadF_memory__rule,
  /*  950 */  regF_rule,
  /*  951 */  immF_rule,
  /*  952 */  regD_rule,
  /*  953 */  _LoadD_memory__rule,
  /*  954 */  regD_rule,
  /*  955 */  immD_rule,
  /*  956 */  regD_rule,
  /*  957 */  _LoadD_memory__rule,
  /*  958 */  regD_rule,
  /*  959 */  immD_rule,
  /*  960 */  regF_rule,
  /*  961 */  _LoadF_memory__rule,
  /*  962 */  immF_rule,
  /*  963 */  regF_rule,
  /*  964 */  _LoadF_memory__rule,
  /*  965 */  immF_rule,
  /*  966 */  regD_rule,
  /*  967 */  _LoadD_memory__rule,
  /*  968 */  immD_rule,
  /*  969 */  regD_rule,
  /*  970 */  _LoadD_memory__rule,
  /*  971 */  immD_rule,
  /*  972 */  regF_rule,
  /*  973 */  _LoadF_memory__rule,
  /*  974 */  regF_rule,
  /*  975 */  immF_rule,
  /*  976 */  regF_rule,
  /*  977 */  _LoadF_memory__rule,
  /*  978 */  regF_rule,
  /*  979 */  immF_rule,
  /*  980 */  regD_rule,
  /*  981 */  _LoadD_memory__rule,
  /*  982 */  regD_rule,
  /*  983 */  immD_rule,
  /*  984 */  regD_rule,
  /*  985 */  _LoadD_memory__rule,
  /*  986 */  regD_rule,
  /*  987 */  immD_rule,
  /*  988 */  regF_rule,
  /*  989 */  _LoadF_memory__rule,
  /*  990 */  immF_rule,
  /*  991 */  regF_rule,
  /*  992 */  _LoadF_memory__rule,
  /*  993 */  immF_rule,
  /*  994 */  regD_rule,
  /*  995 */  _LoadD_memory__rule,
  /*  996 */  immD_rule,
  /*  997 */  regD_rule,
  /*  998 */  _LoadD_memory__rule,
  /*  999 */  immD_rule,
  /* 1000 */  0,
  /* 1001 */  0,
  /* 1002 */  0,
  /* 1003 */  0,
  /* 1004 */  0,
  /* 1005 */  0,
  /* 1006 */  0,
  /* 1007 */  _Binary_regD_regD_rule,
  /* 1008 */  _Binary_regF_regF_rule,
  /* 1009 */  0,
  /* 1010 */  vec_rule,
  /* 1011 */  0,
  /* 1012 */  0,
  /* 1013 */  0,
  /* 1014 */  0,
  /* 1015 */  0,
  /* 1016 */  0,
  /* 1017 */  0,
  /* 1018 */  0,
  /* 1019 */  0,
  /* 1020 */  0,
  /* 1021 */  0,
  /* 1022 */  0,
  /* 1023 */  0,
  /* 1024 */  0,
  /* 1025 */  0,
  /* 1026 */  0,
  /* 1027 */  0,
  /* 1028 */  0,
  /* 1029 */  0,
  /* 1030 */  0,
  /* 1031 */  0,
  /* 1032 */  0,
  /* 1033 */  0,
  /* 1034 */  0,
  /* 1035 */  0,
  /* 1036 */  0,
  /* 1037 */  0,
  /* 1038 */  vec_rule,
  /* 1039 */  vec_rule,
  /* 1040 */  vec_rule,
  /* 1041 */  vec_rule,
  /* 1042 */  vec_rule,
  /* 1043 */  legVec_rule,
  /* 1044 */  legVec_rule,
  /* 1045 */  legVec_rule,
  /* 1046 */  legVec_rule,
  /* 1047 */  legVec_rule,
  /* 1048 */  vec_rule,
  /* 1049 */  vec_rule,
  /* 1050 */  vec_rule,
  /* 1051 */  vec_rule,
  /* 1052 */  legVec_rule,
  /* 1053 */  legVec_rule,
  /* 1054 */  vec_rule,
  /* 1055 */  vec_rule,
  /* 1056 */  vec_rule,
  /* 1057 */  vec_rule,
  /* 1058 */  legVec_rule,
  /* 1059 */  legVec_rule,
  /* 1060 */  vec_rule,
  /* 1061 */  vec_rule,
  /* 1062 */  _LoadVector_memory__rule,
  /* 1063 */  vec_rule,
  /* 1064 */  vec_rule,
  /* 1065 */  vec_rule,
  /* 1066 */  _LoadVector_memory__rule,
  /* 1067 */  vec_rule,
  /* 1068 */  vec_rule,
  /* 1069 */  vec_rule,
  /* 1070 */  _LoadVector_memory__rule,
  /* 1071 */  vec_rule,
  /* 1072 */  vec_rule,
  /* 1073 */  vec_rule,
  /* 1074 */  _LoadVector_memory__rule,
  /* 1075 */  vec_rule,
  /* 1076 */  vec_rule,
  /* 1077 */  vec_rule,
  /* 1078 */  _LoadVector_memory__rule,
  /* 1079 */  vec_rule,
  /* 1080 */  vec_rule,
  /* 1081 */  vec_rule,
  /* 1082 */  _LoadVector_memory__rule,
  /* 1083 */  vec_rule,
  /* 1084 */  vec_rule,
  /* 1085 */  vec_rule,
  /* 1086 */  _LoadVector_memory__rule,
  /* 1087 */  vec_rule,
  /* 1088 */  vec_rule,
  /* 1089 */  _LoadVector_memory__rule,
  /* 1090 */  vec_rule,
  /* 1091 */  vec_rule,
  /* 1092 */  _LoadVector_memory__rule,
  /* 1093 */  vec_rule,
  /* 1094 */  vec_rule,
  /* 1095 */  _LoadVector_memory__rule,
  /* 1096 */  vec_rule,
  /* 1097 */  vec_rule,
  /* 1098 */  _LoadVector_memory__rule,
  /* 1099 */  vec_rule,
  /* 1100 */  vec_rule,
  /* 1101 */  _LoadVector_memory__rule,
  /* 1102 */  vec_rule,
  /* 1103 */  vec_rule,
  /* 1104 */  vec_rule,
  /* 1105 */  vec_rule,
  /* 1106 */  vec_rule,
  /* 1107 */  vec_rule,
  /* 1108 */  vec_rule,
  /* 1109 */  _LoadVector_memory__rule,
  /* 1110 */  vec_rule,
  /* 1111 */  vec_rule,
  /* 1112 */  vec_rule,
  /* 1113 */  _LoadVector_memory__rule,
  /* 1114 */  vec_rule,
  /* 1115 */  vec_rule,
  /* 1116 */  _LoadVector_memory__rule,
  /* 1117 */  vec_rule,
  /* 1118 */  vec_rule,
  /* 1119 */  vec_rule,
  /* 1120 */  _LoadVector_memory__rule,
  /* 1121 */  vec_rule,
  /* 1122 */  vec_rule,
  /* 1123 */  vec_rule,
  /* 1124 */  _LoadVector_memory__rule,
  /* 1125 */  vec_rule,
  /* 1126 */  _Binary_legVec_legVec_rule,
  /* 1127 */  _Binary_legVec_legVec_rule,
  /* 1128 */  vec_rule,
  /* 1129 */  vec_rule,
  /* 1130 */  _LoadVector_memory__rule,
  /* 1131 */  vec_rule,
  /* 1132 */  vec_rule,
  /* 1133 */  _LoadVector_memory__rule,
  /* 1134 */  vec_rule,
  /* 1135 */  vec_rule,
  /* 1136 */  vec_rule,
  /* 1137 */  vec_rule,
  /* 1138 */  vec_rule,
  /* 1139 */  vec_rule,
  /* 1140 */  vec_rule,
  /* 1141 */  vec_rule,
  /* 1142 */  vec_rule,
  /* 1143 */  vec_rule,
  /* 1144 */  vec_rule,
  /* 1145 */  vec_rule,
  /* 1146 */  vec_rule,
  /* 1147 */  vec_rule,
  /* 1148 */  vec_rule,
  /* 1149 */  vec_rule,
  /* 1150 */  vec_rule,
  /* 1151 */  vec_rule,
  /* 1152 */  vec_rule,
  /* 1153 */  vec_rule,
  /* 1154 */  vec_rule,
  /* 1155 */  vec_rule,
  /* 1156 */  vec_rule,
  /* 1157 */  vec_rule,
  /* 1158 */  vec_rule,
  /* 1159 */  vec_rule,
  /* 1160 */  vec_rule,
  /* 1161 */  _LoadVector_memory__rule,
  /* 1162 */  vec_rule,
  /* 1163 */  vec_rule,
  /* 1164 */  vec_rule,
  /* 1165 */  _LoadVector_memory__rule,
  /* 1166 */  vec_rule,
  /* 1167 */  vec_rule,
  /* 1168 */  vec_rule,
  /* 1169 */  _LoadVector_memory__rule,
  /* 1170 */  vec_rule,
  /* 1171 */  0,
  /* 1172 */  0,
  /* 1173 */  0,
  /* 1174 */  0,
  /* 1175 */  0,
  /* 1176 */  0,
  /* 1177 */  _Binary_vec_vec_rule,
  /* 1178 */  _Binary_vec__LoadVector_memory__rule,
  /* 1179 */  _Binary_vec_vec_rule,
  /* 1180 */  _Binary_vec__LoadVector_memory__rule,
  /* 1181 */  vec_rule,
  /* 1182 */  vec_rule,
  /* 1183 */  vec_rule,
  /* 1184 */  _MulAddVS2VI_vec_vec_rule,
  /* 1185 */  _Binary_vec_immU8_rule,
  /* 1186 */  _Binary__LoadVector_memory__immU8_rule,
  /* 1187 */  _Binary_eAXRegP_eRegP_rule,
  /* 1188 */  _Binary_eAXRegP_eRegP_rule,
  /* 1189 */  _Binary_eAXRegP_eRegP_rule,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI0",
  /*   10 */  "IMMI1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "IMMI2",
  /*   13 */  "IMMI8",
  /*   14 */  "IMMU8",
  /*   15 */  "IMMI16",
  /*   16 */  "IMMU31",
  /*   17 */  "IMMI_32",
  /*   18 */  "IMMI_1_31",
  /*   19 */  "IMMI_32_63",
  /*   20 */  "IMMI_1",
  /*   21 */  "IMMI_2",
  /*   22 */  "IMMI_3",
  /*   23 */  "IMMP",
  /*   24 */  "IMMP0",
  /*   25 */  "IMML",
  /*   26 */  "IMML0",
  /*   27 */  "IMML_M1",
  /*   28 */  "IMML_127",
  /*   29 */  "IMML_32BITS",
  /*   30 */  "IMML32",
  /*   31 */  "IMMDPR0",
  /*   32 */  "IMMDPR1",
  /*   33 */  "IMMDPR",
  /*   34 */  "IMMD",
  /*   35 */  "IMMD0",
  /*   36 */  "IMMFPR0",
  /*   37 */  "IMMFPR1",
  /*   38 */  "IMMFPR",
  /*   39 */  "IMMF",
  /*   40 */  "IMMF0",
  /*   41 */  "IMMI_16",
  /*   42 */  "IMMI_24",
  /*   43 */  "IMMI_255",
  /*   44 */  "IMMI_65535",
  /*   45 */  "RREGI",
  /*   46 */  "XREGI",
  /*   47 */  "EAXREGI",
  /*   48 */  "EBXREGI",
  /*   49 */  "ECXREGI",
  /*   50 */  "EDXREGI",
  /*   51 */  "EDIREGI",
  /*   52 */  "NAXREGI",
  /*   53 */  "NADXREGI",
  /*   54 */  "NCXREGI",
  /*   55 */  "ESIREGI",
  /*   56 */  "ANYREGP",
  /*   57 */  "EREGP",
  /*   58 */  "EREGP_NO_EBP",
  /*   59 */  "NAXREGP",
  /*   60 */  "NABXREGP",
  /*   61 */  "PREGP",
  /*   62 */  "EAXREGP",
  /*   63 */  "EBXREGP",
  /*   64 */  "ECXREGP",
  /*   65 */  "EDXREGP",
  /*   66 */  "ESIREGP",
  /*   67 */  "EDIREGP",
  /*   68 */  "EREGL",
  /*   69 */  "EADXREGL",
  /*   70 */  "EBCXREGL",
  /*   71 */  "EADXREGL_LOW_ONLY",
  /*   72 */  "EFLAGSREG",
  /*   73 */  "EFLAGSREGU",
  /*   74 */  "EFLAGSREGUCF",
  /*   75 */  "FLAGSREG_LONG_LTGE",
  /*   76 */  "FLAGSREG_LONG_EQNE",
  /*   77 */  "FLAGSREG_LONG_LEGT",
  /*   78 */  "FLAGSREG_ULONG_LTGE",
  /*   79 */  "FLAGSREG_ULONG_EQNE",
  /*   80 */  "FLAGSREG_ULONG_LEGT",
  /*   81 */  "REGDPR",
  /*   82 */  "REGDPR1",
  /*   83 */  "REGDPR2",
  /*   84 */  "REGNOTDPR1",
  /*   85 */  "REGFPR",
  /*   86 */  "REGFPR1",
  /*   87 */  "REGF",
  /*   88 */  "VLREGF",
  /*   89 */  "REGD",
  /*   90 */  "VLREGD",
  /*   91 */  "DIRECT",
  /*   92 */  "INDIRECT",
  /*   93 */  "INDOFFSET8",
  /*   94 */  "INDOFFSET32",
  /*   95 */  "INDOFFSET32X",
  /*   96 */  "INDINDEXOFFSET",
  /*   97 */  "INDINDEX",
  /*   98 */  "INDINDEXSCALE",
  /*   99 */  "INDINDEXSCALEOFFSET",
  /*  100 */  "LOAD_LONG_REGP",
  /*  101 */  "LOAD_LONG_INDIRECT",
  /*  102 */  "LOAD_LONG_INDOFFSET32",
  /*  103 */  "STACKSLOTP",
  /*  104 */  "STACKSLOTI",
  /*  105 */  "STACKSLOTF",
  /*  106 */  "STACKSLOTD",
  /*  107 */  "STACKSLOTL",
  /*  108 */  "INDIRECT_WIN95_SAFE",
  /*  109 */  "INDOFFSET8_WIN95_SAFE",
  /*  110 */  "INDOFFSET32_WIN95_SAFE",
  /*  111 */  "INDINDEXOFFSET_WIN95_SAFE",
  /*  112 */  "INDINDEXSCALE_WIN95_SAFE",
  /*  113 */  "INDINDEXSCALEOFFSET_WIN95_SAFE",
  /*  114 */  "CMPOP",
  /*  115 */  "CMPOPU",
  /*  116 */  "CMPOPUCF",
  /*  117 */  "CMPOPUCF2",
  /*  118 */  "CMPOP_FCMOV",
  /*  119 */  "CMPOP_COMMUTE",
  /*  120 */  "CMPOPU_COMMUTE",
  /*  121 */  "VEC",
  /*  122 */  "LEGVEC",
  /*  123 */  "VECS",
  /*  124 */  "LEGVECS",
  /*  125 */  "VECD",
  /*  126 */  "LEGVECD",
  /*  127 */  "VECX",
  /*  128 */  "LEGVECX",
  /*  129 */  "VECY",
  /*  130 */  "LEGVECY",
  /*  131 */  "VECZ",
  /*  132 */  "LEGVECZ",
  /*  133 */  "CMPOP_VCMPPD",
  // last operand
  /*  134 */  "LOAD_LONG_MEMORY",
  /*  135 */  "MEMORY",
  /*  136 */  "LONG_MEMORY",
  // last operand class
  /*  137 */  "_AddP_eRegP_rRegI",
  /*  138 */  "_LShiftI_rRegI_immI2",
  /*  139 */  "_AddP_eRegP__LShiftI_rRegI_immI2",
  /*  140 */  "_AddP_eRegP_no_EBP_rRegI",
  /*  141 */  "_AddP_eRegP_no_EBP__LShiftI_rRegI_immI2",
  /*  142 */  "_LoadI_memory_",
  /*  143 */  "_LoadL_memory_",
  /*  144 */  "_LoadB_memory_",
  /*  145 */  "_LoadUB_memory_",
  /*  146 */  "_AndI__LoadUB_memory__immI",
  /*  147 */  "_LoadS_memory_",
  /*  148 */  "_LShiftI__LoadS_memory__immI_24",
  /*  149 */  "_LoadUS_memory_",
  /*  150 */  "_LShiftI__LoadUS_memory__immI_24",
  /*  151 */  "_AndI__LoadUS_memory__immI_255",
  /*  152 */  "_AndI__LoadUS_memory__immI",
  /*  153 */  "_LShiftI__LoadI_memory__immI_24",
  /*  154 */  "_LShiftI__LoadI_memory__immI_16",
  /*  155 */  "_AndI__LoadI_memory__immI_255",
  /*  156 */  "_AndI__LoadI_memory__immI_65535",
  /*  157 */  "_AndI__LoadI_memory__immU31",
  /*  158 */  "_ConvI2L__LoadI_memory__",
  /*  159 */  "_ConvL2I_eRegL_",
  /*  160 */  "_RoundDouble_regDPR1_",
  /*  161 */  "_RoundFloat_regFPR1_",
  /*  162 */  "_ConvD2F_regDPR1_",
  /*  163 */  "_Binary_cmpOp_eFlagsReg",
  /*  164 */  "_Binary_rRegI_rRegI",
  /*  165 */  "_Binary_cmpOpU_eFlagsRegU",
  /*  166 */  "_Binary_cmpOpUCF_eFlagsRegUCF",
  /*  167 */  "_Binary_rRegI__LoadI_memory_",
  /*  168 */  "_Binary_eRegP_eRegP",
  /*  169 */  "_Binary_cmpOp_fcmov_eFlagsRegU",
  /*  170 */  "_Binary_regDPR1_regDPR",
  /*  171 */  "_Binary_regFPR1_regFPR",
  /*  172 */  "_Binary_regDPR_regDPR",
  /*  173 */  "_Binary_regFPR_regFPR",
  /*  174 */  "_Binary_regF_regF",
  /*  175 */  "_Binary_regD_regD",
  /*  176 */  "_Binary_eRegL_eRegL",
  /*  177 */  "_AddI__LoadI_memory__rRegI",
  /*  178 */  "_AddI_rRegI__LoadI_memory_",
  /*  179 */  "_AddI__LoadI_memory__immI",
  /*  180 */  "_AddI__LoadI_memory__immI1",
  /*  181 */  "_AddI__LoadI_memory__immI_M1",
  /*  182 */  "_Binary_eAXRegP_eRegP",
  /*  183 */  "_Binary_eAXRegI_rRegI",
  /*  184 */  "_Binary_eADXRegL_eBCXRegL",
  /*  185 */  "_Binary_eAXRegP_eCXRegP",
  /*  186 */  "_Binary_eAXRegI_eCXRegI",
  /*  187 */  "_SubI__LoadI_memory__rRegI",
  /*  188 */  "_SubI_immI0_rRegI",
  /*  189 */  "_ConvI2L_nadxRegI_",
  /*  190 */  "_MulL__ConvI2L_nadxRegI__eADXRegL_low_only",
  /*  191 */  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32",
  /*  192 */  "_MulL_eADXRegL_low_only__ConvI2L_nadxRegI_",
  /*  193 */  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32",
  /*  194 */  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63",
  /*  195 */  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63",
  /*  196 */  "_ConvI2L_eAXRegI_",
  /*  197 */  "_AndL__ConvI2L_eAXRegI__immL_32bits",
  /*  198 */  "_AndL__ConvI2L_nadxRegI__immL_32bits",
  /*  199 */  "_RShiftI__LoadI_memory__immI1",
  /*  200 */  "_RShiftI__LoadI_memory__immI8",
  /*  201 */  "_LShiftI_xRegI_immI_24",
  /*  202 */  "_LShiftI_xRegI_immI_16",
  /*  203 */  "_AndI__LoadI_memory__rRegI",
  /*  204 */  "_AndI_rRegI__LoadI_memory_",
  /*  205 */  "_AndI__LoadI_memory__immI",
  /*  206 */  "_XorI_rRegI_immI_M1",
  /*  207 */  "_SubI_immI0__LoadI_memory_",
  /*  208 */  "_AddI_rRegI_immI_M1",
  /*  209 */  "_CastP2X_eRegP_",
  /*  210 */  "_OrI__LoadI_memory__rRegI",
  /*  211 */  "_OrI_rRegI__LoadI_memory_",
  /*  212 */  "_OrI__LoadI_memory__immI",
  /*  213 */  "_LShiftI_rRegI_immI1",
  /*  214 */  "_URShiftI_rRegI_immI_M1",
  /*  215 */  "_LShiftI_rRegI_immI8",
  /*  216 */  "_URShiftI_rRegI_immI8",
  /*  217 */  "_LShiftI_ncxRegI_eCXRegI",
  /*  218 */  "_SubI_immI0_eCXRegI",
  /*  219 */  "_URShiftI_ncxRegI__SubI_immI0_eCXRegI",
  /*  220 */  "_SubI_immI_32_eCXRegI",
  /*  221 */  "_URShiftI_ncxRegI__SubI_immI_32_eCXRegI",
  /*  222 */  "_URShiftI_rRegI_immI1",
  /*  223 */  "_LShiftI_rRegI_immI_M1",
  /*  224 */  "_URShiftI_ncxRegI_eCXRegI",
  /*  225 */  "_LShiftI_ncxRegI__SubI_immI0_eCXRegI",
  /*  226 */  "_LShiftI_ncxRegI__SubI_immI_32_eCXRegI",
  /*  227 */  "_XorI__LoadI_memory__rRegI",
  /*  228 */  "_XorI_rRegI__LoadI_memory_",
  /*  229 */  "_XorI__LoadI_memory__immI",
  /*  230 */  "_CmpLTMask_rRegI_rRegI",
  /*  231 */  "_AndI__CmpLTMask_rRegI_rRegI_rRegI",
  /*  232 */  "_SubI_rRegI_rRegI",
  /*  233 */  "_AndI_rRegI__CmpLTMask_rRegI_rRegI",
  /*  234 */  "_LoadL_load_long_memory_",
  /*  235 */  "_XorL_eRegL_immL_M1",
  /*  236 */  "_SubL_immL0_eRegL",
  /*  237 */  "_SubL_immL0__LoadL_memory_",
  /*  238 */  "_AddL_eRegL_immL_M1",
  /*  239 */  "_AddL__LoadL_memory__immL_M1",
  /*  240 */  "_LoadD_memory_",
  /*  241 */  "_SubD_regDPR_regDPR",
  /*  242 */  "_AddD_regDPR_regDPR",
  /*  243 */  "_AddD__LoadD_memory__regDPR",
  /*  244 */  "_RoundDouble__AddD__LoadD_memory__regDPR_",
  /*  245 */  "_AddD_regDPR__LoadD_memory_",
  /*  246 */  "_RoundDouble__AddD_regDPR__LoadD_memory__",
  /*  247 */  "_AddD_regDPR_immDPR",
  /*  248 */  "_MulD_regDPR_regDPR",
  /*  249 */  "_DivD_regDPR_regDPR",
  /*  250 */  "_LoadF_memory_",
  /*  251 */  "_MulF__LoadF_memory__regFPR",
  /*  252 */  "_MulF_regFPR__LoadF_memory_",
  /*  253 */  "_MulF_regFPR_regFPR",
  /*  254 */  "_SubF_regFPR_regFPR",
  /*  255 */  "_ConvI2L_rRegI_",
  /*  256 */  "_Binary_eDIRegP_eCXRegI",
  /*  257 */  "_Binary_eSIRegP_eDXRegI",
  /*  258 */  "_Binary_eDIRegP_eSIRegP",
  /*  259 */  "_Binary_eDIRegP_eDXRegI",
  /*  260 */  "_Binary_eSIRegP_immI",
  /*  261 */  "_Binary_eSIRegP_eAXRegI",
  /*  262 */  "_AndI_rRegI_immI",
  /*  263 */  "_AndI_rRegI_memory",
  /*  264 */  "_LoadP_memory_",
  /*  265 */  "_Binary_nadxRegI_eAXRegI",
  /*  266 */  "_PartialSubtypeCheck_eSIRegP_eAXRegP",
  /*  267 */  "_Binary_cmpOp_flagsReg_long_LTGE",
  /*  268 */  "_Binary_eRegL__LoadL_load_long_memory_",
  /*  269 */  "_Binary_cmpOp_flagsReg_long_EQNE",
  /*  270 */  "_Binary_cmpOp_commute_flagsReg_long_LEGT",
  /*  271 */  "_LoadVector_memory_",
  /*  272 */  "_Binary_cmpOp_vcmppd_immI8",
  /*  273 */  "_Binary_legVec_legVec",
  /*  274 */  "_Binary_vec_vec",
  /*  275 */  "_Binary_vec__LoadVector_memory_",
  /*  276 */  "_MulAddVS2VI_vec_vec",
  /*  277 */  "_Binary_vec_immU8",
  /*  278 */  "_Binary__LoadVector_memory__immU8",
  // last internally defined operand
  /*  279 */  "storeSSI",
  /*  280 */  "storeSSP",
  /*  281 */  "storeSSL",
  /*  282 */  "leaP8",
  /*  283 */  "leaP32",
  /*  284 */  "leaPIdxOff",
  /*  285 */  "leaPIdxScale",
  /*  286 */  "leaPIdxScaleOff",
  /*  287 */  "loadConI",
  /*  288 */  "loadConI0",
  /*  289 */  "loadConP",
  /*  290 */  "loadConL",
  /*  291 */  "loadConL0",
  /*  292 */  "loadConFPR",
  /*  293 */  "loadConFPR0",
  /*  294 */  "loadConFPR1",
  /*  295 */  "loadConF",
  /*  296 */  "loadConF0",
  /*  297 */  "loadConDPR",
  /*  298 */  "loadConDPR0",
  /*  299 */  "loadConDPR1",
  /*  300 */  "loadConD",
  /*  301 */  "loadConD0",
  /*  302 */  "loadSSI",
  /*  303 */  "loadSSL",
  /*  304 */  "loadSSP",
  /*  305 */  "loadSSF",
  /*  306 */  "loadSSD",
  /*  307 */  "MoveD2VL",
  /*  308 */  "MoveVL2D",
  /*  309 */  "MoveF2VL",
  /*  310 */  "MoveVL2F",
  /*  311 */  "loadConL_low_only",
  /*  312 */  "MoveVec2Leg",
  /*  313 */  "MoveLeg2Vec",
  /*  314 */  "membar_volatile",
  /*  315 */  "castX2P",
  /*  316 */  "castP2X",
  /*  317 */  "checkCastPP",
  /*  318 */  "castPP",
  /*  319 */  "castII",
  /*  320 */  "storePConditional",
  /*  321 */  "storeIConditional",
  /*  322 */  "storeLConditional",
  /*  323 */  "movI_nocopy",
  /*  324 */  "movP_nocopy",
  /*  325 */  "overflowAddI_eReg",
  /*  326 */  "overflowAddI_rReg_imm",
  /*  327 */  "overflowSubI_rReg",
  /*  328 */  "overflowSubI_rReg_imm",
  /*  329 */  "overflowNegI_rReg",
  /*  330 */  "overflowMulI_rReg",
  /*  331 */  "overflowMulI_rReg_imm",
  /*  332 */  "cmpDPR_cc_P6",
  /*  333 */  "cmpDPR_cc_P6CF",
  /*  334 */  "cmpDPR_cc",
  /*  335 */  "cmpD_cc",
  /*  336 */  "cmpD_ccCF",
  /*  337 */  "cmpD_ccmem",
  /*  338 */  "cmpD_ccmemCF",
  /*  339 */  "absDPR_reg",
  /*  340 */  "negDPR_reg",
  /*  341 */  "cmpFPR_cc_P6",
  /*  342 */  "cmpFPR_cc_P6CF",
  /*  343 */  "cmpFPR_cc",
  /*  344 */  "cmpF_cc",
  /*  345 */  "cmpF_ccCF",
  /*  346 */  "cmpF_ccmem",
  /*  347 */  "cmpF_ccmemCF",
  /*  348 */  "absFPR_reg",
  /*  349 */  "negFPR_reg",
  /*  350 */  "MoveF2I_stack_reg",
  /*  351 */  "MoveF2I_reg_reg_sse",
  /*  352 */  "MoveI2FPR_stack_reg",
  /*  353 */  "MoveI2F_stack_reg_sse",
  /*  354 */  "MoveI2F_reg_reg_sse",
  /*  355 */  "MoveD2L_stack_reg",
  /*  356 */  "MoveL2DPR_stack_reg",
  /*  357 */  "MoveL2D_stack_reg_sse",
  /*  358 */  "MoveL2D_stack_reg_sse_partial",
  /*  359 */  "compI_eReg",
  /*  360 */  "compI_eReg_imm",
  /*  361 */  "compI_eReg_mem",
  /*  362 */  "testI_reg",
  /*  363 */  "testI_reg_imm",
  /*  364 */  "testI_reg_mem",
  /*  365 */  "compU_eReg",
  /*  366 */  "compU_eReg_imm",
  /*  367 */  "compU_eReg_mem",
  /*  368 */  "testU_reg",
  /*  369 */  "compP_eReg",
  /*  370 */  "compP_eReg_imm",
  /*  371 */  "compP_eReg_mem",
  /*  372 */  "compP_mem_eReg",
  /*  373 */  "testP_reg",
  /*  374 */  "testP_Reg_mem",
  /*  375 */  "partialSubtypeCheck_vs_Zero",
  /*  376 */  "cmpL_zero_flags_LTGE",
  /*  377 */  "cmpL_reg_flags_LTGE",
  /*  378 */  "cmpUL_zero_flags_LTGE",
  /*  379 */  "cmpUL_reg_flags_LTGE",
  /*  380 */  "cmpL_zero_flags_EQNE",
  /*  381 */  "cmpL_reg_flags_EQNE",
  /*  382 */  "cmpUL_zero_flags_EQNE",
  /*  383 */  "cmpUL_reg_flags_EQNE",
  /*  384 */  "cmpL_zero_flags_LEGT",
  /*  385 */  "cmpL_reg_flags_LEGT",
  /*  386 */  "cmpUL_zero_flags_LEGT",
  /*  387 */  "cmpUL_reg_flags_LEGT",
  /*  388 */  "cmpFastLockRTM",
  /*  389 */  "cmpFastLock",
  /*  390 */  "cmpFastUnlock",
  /*  391 */  "absF_reg",
  /*  392 */  "absF_reg_reg",
  /*  393 */  "absD_reg",
  /*  394 */  "absD_reg_reg",
  /*  395 */  "negF_reg",
  /*  396 */  "negF_reg_reg",
  /*  397 */  "negD_reg",
  /*  398 */  "negD_reg_reg",
  /*  399 */  "vsqrtF_reg",
  /*  400 */  "vsqrtF_mem",
  /*  401 */  "vsqrtD_reg",
  /*  402 */  "vsqrtD_mem",
  /*  403 */  "vshiftcnt",
  /*  404 */  "vshiftcnt_0",
  /*  405 */  "vabsB_reg",
  /*  406 */  "vabsS_reg",
  /*  407 */  "vabsI_reg",
  /*  408 */  "vabsL_reg",
  /*  409 */  "vpopcountI",
  /*  410 */  "bytes_reverse_int",
  /*  411 */  "bytes_reverse_long",
  /*  412 */  "bytes_reverse_unsigned_short",
  /*  413 */  "bytes_reverse_short",
  /*  414 */  "countLeadingZerosI",
  /*  415 */  "countLeadingZerosI_bsr",
  /*  416 */  "countLeadingZerosL",
  /*  417 */  "countLeadingZerosL_bsr",
  /*  418 */  "countTrailingZerosI",
  /*  419 */  "countTrailingZerosI_bsf",
  /*  420 */  "countTrailingZerosL",
  /*  421 */  "countTrailingZerosL_bsf",
  /*  422 */  "popCountI",
  /*  423 */  "popCountI_mem",
  /*  424 */  "popCountL",
  /*  425 */  "popCountL_mem",
  /*  426 */  "loadB",
  /*  427 */  "loadB2L",
  /*  428 */  "loadUB",
  /*  429 */  "loadUB2L",
  /*  430 */  "loadUB2L_immI",
  /*  431 */  "loadS",
  /*  432 */  "loadS2B",
  /*  433 */  "loadS2L",
  /*  434 */  "loadUS",
  /*  435 */  "loadUS2B",
  /*  436 */  "loadUS2L",
  /*  437 */  "loadUS2L_immI_255",
  /*  438 */  "loadUS2L_immI",
  /*  439 */  "loadI",
  /*  440 */  "loadI2B",
  /*  441 */  "loadI2UB",
  /*  442 */  "loadI2S",
  /*  443 */  "loadI2US",
  /*  444 */  "loadI2L",
  /*  445 */  "loadI2L_immI_255",
  /*  446 */  "loadI2L_immI_65535",
  /*  447 */  "loadI2L_immU31",
  /*  448 */  "loadUI2L",
  /*  449 */  "loadL",
  /*  450 */  "loadL_volatile",
  /*  451 */  "loadLX_volatile",
  /*  452 */  "loadLX_reg_volatile",
  /*  453 */  "loadRange",
  /*  454 */  "loadP",
  /*  455 */  "loadKlass",
  /*  456 */  "loadDPR",
  /*  457 */  "loadD",
  /*  458 */  "loadD_partial",
  /*  459 */  "loadF",
  /*  460 */  "loadFPR",
  /*  461 */  "prefetchAlloc0",
  /*  462 */  "prefetchAlloc",
  /*  463 */  "prefetchAllocNTA",
  /*  464 */  "prefetchAllocT0",
  /*  465 */  "prefetchAllocT2",
  /*  466 */  "storeB",
  /*  467 */  "storeC",
  /*  468 */  "storeI",
  /*  469 */  "storeL",
  /*  470 */  "storeL2I",
  /*  471 */  "storeL_volatile",
  /*  472 */  "storeLX_volatile",
  /*  473 */  "storeLX_reg_volatile",
  /*  474 */  "storeP",
  /*  475 */  "storeImmI",
  /*  476 */  "storeImmI16",
  /*  477 */  "storeImmP",
  /*  478 */  "storeImmB",
  /*  479 */  "storeImmCM",
  /*  480 */  "storeDPR",
  /*  481 */  "storeDPR_rounded",
  /*  482 */  "storeD",
  /*  483 */  "storeF",
  /*  484 */  "storeFPR",
  /*  485 */  "storeFPR_rounded",
  /*  486 */  "storeFPR_Drounded",
  /*  487 */  "storeFPR_imm",
  /*  488 */  "storeF_imm",
  /*  489 */  "membar_acquire",
  /*  490 */  "membar_acquire_0",
  /*  491 */  "membar_acquire_lock",
  /*  492 */  "membar_release",
  /*  493 */  "membar_release_0",
  /*  494 */  "membar_release_lock",
  /*  495 */  "unnecessary_membar_volatile",
  /*  496 */  "membar_storestore",
  /*  497 */  "jmovI_reg",
  /*  498 */  "jmovI_regU",
  /*  499 */  "cmovI_reg",
  /*  500 */  "cmovI_regU",
  /*  501 */  "cmovI_regUCF",
  /*  502 */  "cmovI_mem",
  /*  503 */  "cmovI_memU",
  /*  504 */  "cmovI_memUCF",
  /*  505 */  "cmovP_reg",
  /*  506 */  "cmovP_reg_nonP6",
  /*  507 */  "cmovP_regU",
  /*  508 */  "cmovP_regUCF",
  /*  509 */  "fcmovDPR_regU",
  /*  510 */  "fcmovFPR_regU",
  /*  511 */  "fcmovDPR_regS",
  /*  512 */  "fcmovFPR_regS",
  /*  513 */  "fcmovF_regS",
  /*  514 */  "fcmovD_regS",
  /*  515 */  "fcmovF_regU",
  /*  516 */  "fcmovF_regUCF",
  /*  517 */  "fcmovD_regU",
  /*  518 */  "fcmovD_regUCF",
  /*  519 */  "cmovL_reg",
  /*  520 */  "cmovL_regU",
  /*  521 */  "cmovL_regUCF",
  /*  522 */  "addI_eReg",
  /*  523 */  "addI_eReg_imm",
  /*  524 */  "incI_eReg",
  /*  525 */  "leaI_eReg_immI",
  /*  526 */  "leaP_eReg_immI",
  /*  527 */  "decI_eReg",
  /*  528 */  "addP_eReg",
  /*  529 */  "addP_eReg_imm",
  /*  530 */  "addI_eReg_mem",
  /*  531 */  "addI_eReg_mem_0",
  /*  532 */  "addI_mem_eReg",
  /*  533 */  "addI_mem_eReg_0",
  /*  534 */  "addI_mem_imm",
  /*  535 */  "incI_mem",
  /*  536 */  "decI_mem",
  /*  537 */  "loadPLocked",
  /*  538 */  "compareAndSwapL",
  /*  539 */  "compareAndSwapL_0",
  /*  540 */  "compareAndSwapP",
  /*  541 */  "compareAndSwapP_0",
  /*  542 */  "compareAndSwapB",
  /*  543 */  "compareAndSwapB_0",
  /*  544 */  "compareAndSwapS",
  /*  545 */  "compareAndSwapS_0",
  /*  546 */  "compareAndSwapI",
  /*  547 */  "compareAndSwapI_0",
  /*  548 */  "compareAndExchangeL",
  /*  549 */  "compareAndExchangeP",
  /*  550 */  "compareAndExchangeB",
  /*  551 */  "compareAndExchangeS",
  /*  552 */  "compareAndExchangeI",
  /*  553 */  "xaddB_no_res",
  /*  554 */  "xaddB",
  /*  555 */  "xaddS_no_res",
  /*  556 */  "xaddS",
  /*  557 */  "xaddI_no_res",
  /*  558 */  "xaddI",
  /*  559 */  "xchgB",
  /*  560 */  "xchgS",
  /*  561 */  "xchgI",
  /*  562 */  "xchgP",
  /*  563 */  "subI_eReg",
  /*  564 */  "subI_eReg_imm",
  /*  565 */  "subI_eReg_mem",
  /*  566 */  "subI_mem_eReg",
  /*  567 */  "subP_eReg",
  /*  568 */  "negI_eReg",
  /*  569 */  "mulI_eReg",
  /*  570 */  "mulI_eReg_imm",
  /*  571 */  "mulI_imm_high",
  /*  572 */  "mulI_imm_high_0",
  /*  573 */  "mulI_imm_RShift_high",
  /*  574 */  "mulI_imm_RShift_high_0",
  /*  575 */  "mulI_mem_imm",
  /*  576 */  "mulI",
  /*  577 */  "mulI_0",
  /*  578 */  "mulAddS2I_rReg",
  /*  579 */  "mulI2L",
  /*  580 */  "mulI2L_0",
  /*  581 */  "mulIS_eReg",
  /*  582 */  "mulIS_eReg_0",
  /*  583 */  "mulL_eReg",
  /*  584 */  "mulL_eReg_lhi0",
  /*  585 */  "mulL_eReg_rhi0",
  /*  586 */  "mulL_eReg_hi0",
  /*  587 */  "mulL_eReg_con",
  /*  588 */  "divI_eReg",
  /*  589 */  "divL_eReg",
  /*  590 */  "divModI_eReg_divmod",
  /*  591 */  "modI_eReg",
  /*  592 */  "modL_eReg",
  /*  593 */  "divL_eReg_imm32",
  /*  594 */  "modL_eReg_imm32",
  /*  595 */  "shlI_eReg_1",
  /*  596 */  "salI_eReg_imm",
  /*  597 */  "salI_eReg_CL",
  /*  598 */  "sarI_eReg_1",
  /*  599 */  "sarI_mem_1",
  /*  600 */  "sarI_eReg_imm",
  /*  601 */  "sarI_mem_imm",
  /*  602 */  "sarI_eReg_CL",
  /*  603 */  "shrI_eReg_1",
  /*  604 */  "shrI_eReg_imm",
  /*  605 */  "i2b",
  /*  606 */  "i2s",
  /*  607 */  "shrI_eReg_CL",
  /*  608 */  "andI_eReg",
  /*  609 */  "andI_eReg_imm",
  /*  610 */  "andI_eReg_mem",
  /*  611 */  "andI_eReg_mem_0",
  /*  612 */  "andI_mem_eReg",
  /*  613 */  "andI_mem_eReg_0",
  /*  614 */  "andI_mem_imm",
  /*  615 */  "andnI_rReg_rReg_rReg",
  /*  616 */  "andnI_rReg_rReg_rReg_0",
  /*  617 */  "andnI_rReg_rReg_mem",
  /*  618 */  "andnI_rReg_rReg_mem_0",
  /*  619 */  "blsiI_rReg_rReg",
  /*  620 */  "blsiI_rReg_rReg_0",
  /*  621 */  "blsiI_rReg_mem",
  /*  622 */  "blsiI_rReg_mem_0",
  /*  623 */  "blsmskI_rReg_rReg",
  /*  624 */  "blsmskI_rReg_rReg_0",
  /*  625 */  "blsmskI_rReg_mem",
  /*  626 */  "blsmskI_rReg_mem_0",
  /*  627 */  "blsrI_rReg_rReg",
  /*  628 */  "blsrI_rReg_rReg_0",
  /*  629 */  "blsrI_rReg_mem",
  /*  630 */  "blsrI_rReg_mem_0",
  /*  631 */  "orI_eReg",
  /*  632 */  "orI_eReg_castP2X",
  /*  633 */  "orI_eReg_castP2X_0",
  /*  634 */  "orI_eReg_imm",
  /*  635 */  "orI_eReg_mem",
  /*  636 */  "orI_eReg_mem_0",
  /*  637 */  "orI_mem_eReg",
  /*  638 */  "orI_mem_eReg_0",
  /*  639 */  "orI_mem_imm",
  /*  640 */  "rolI_eReg_imm1",
  /*  641 */  "rolI_eReg_imm8",
  /*  642 */  "rolI_eReg_CL",
  /*  643 */  "rolI_eReg_i1",
  /*  644 */  "rolI_eReg_i1_0",
  /*  645 */  "rolI_eReg_i8",
  /*  646 */  "rolI_eReg_i8_0",
  /*  647 */  "rolI_eReg_Var_C0",
  /*  648 */  "rolI_eReg_Var_C0_0",
  /*  649 */  "rolI_eReg_Var_C32",
  /*  650 */  "rolI_eReg_Var_C32_0",
  /*  651 */  "rorI_eReg_imm1",
  /*  652 */  "rorI_eReg_imm8",
  /*  653 */  "rorI_eReg_CL",
  /*  654 */  "rorI_eReg_i1",
  /*  655 */  "rorI_eReg_i1_0",
  /*  656 */  "rorI_eReg_i8",
  /*  657 */  "rorI_eReg_i8_0",
  /*  658 */  "rorI_eReg_Var_C0",
  /*  659 */  "rorI_eReg_Var_C0_0",
  /*  660 */  "rorI_eReg_Var_C32",
  /*  661 */  "rorI_eReg_Var_C32_0",
  /*  662 */  "xorI_eReg",
  /*  663 */  "xorI_eReg_im1",
  /*  664 */  "xorI_eReg_imm",
  /*  665 */  "xorI_eReg_mem",
  /*  666 */  "xorI_eReg_mem_0",
  /*  667 */  "xorI_mem_eReg",
  /*  668 */  "xorI_mem_eReg_0",
  /*  669 */  "xorI_mem_imm",
  /*  670 */  "ci2b",
  /*  671 */  "convI2B",
  /*  672 */  "cp2b",
  /*  673 */  "convP2B",
  /*  674 */  "cmpLTMask",
  /*  675 */  "cmpLTMask0",
  /*  676 */  "cadd_cmpLTMask",
  /*  677 */  "cadd_cmpLTMask_1",
  /*  678 */  "cadd_cmpLTMask_0",
  /*  679 */  "cadd_cmpLTMask_2",
  /*  680 */  "and_cmpLTMask",
  /*  681 */  "and_cmpLTMask_0",
  /*  682 */  "absI_rReg",
  /*  683 */  "addL_eReg",
  /*  684 */  "addL_eReg_imm",
  /*  685 */  "addL_eReg_mem",
  /*  686 */  "addL_eReg_mem_0",
  /*  687 */  "subL_eReg",
  /*  688 */  "subL_eReg_imm",
  /*  689 */  "subL_eReg_mem",
  /*  690 */  "negL_eReg",
  /*  691 */  "andL_eReg",
  /*  692 */  "andL_eReg_imm",
  /*  693 */  "andL_eReg_mem",
  /*  694 */  "andL_eReg_mem_0",
  /*  695 */  "andnL_eReg_eReg_eReg",
  /*  696 */  "andnL_eReg_eReg_eReg_0",
  /*  697 */  "andnL_eReg_eReg_mem",
  /*  698 */  "andnL_eReg_eReg_mem_0",
  /*  699 */  "blsiL_eReg_eReg",
  /*  700 */  "blsiL_eReg_eReg_0",
  /*  701 */  "blsiL_eReg_mem",
  /*  702 */  "blsiL_eReg_mem_0",
  /*  703 */  "blsmskL_eReg_eReg",
  /*  704 */  "blsmskL_eReg_eReg_0",
  /*  705 */  "blsmskL_eReg_mem",
  /*  706 */  "blsmskL_eReg_mem_0",
  /*  707 */  "blsrL_eReg_eReg",
  /*  708 */  "blsrL_eReg_eReg_0",
  /*  709 */  "blsrL_eReg_mem",
  /*  710 */  "blsrL_eReg_mem_0",
  /*  711 */  "orl_eReg",
  /*  712 */  "orl_eReg_imm",
  /*  713 */  "orl_eReg_mem",
  /*  714 */  "orl_eReg_mem_0",
  /*  715 */  "xorl_eReg",
  /*  716 */  "xorl_eReg_im1",
  /*  717 */  "xorl_eReg_imm",
  /*  718 */  "xorl_eReg_mem",
  /*  719 */  "xorl_eReg_mem_0",
  /*  720 */  "shlL_eReg_1",
  /*  721 */  "shlL_eReg_2",
  /*  722 */  "shlL_eReg_3",
  /*  723 */  "shlL_eReg_1_31",
  /*  724 */  "shlL_eReg_32_63",
  /*  725 */  "salL_eReg_CL",
  /*  726 */  "shrL_eReg_1_31",
  /*  727 */  "shrL_eReg_32_63",
  /*  728 */  "shrL_eReg_CL",
  /*  729 */  "sarL_eReg_1_31",
  /*  730 */  "sarL_eReg_32_63",
  /*  731 */  "sarL_eReg_CL",
  /*  732 */  "cmpDPR_0",
  /*  733 */  "cmpDPR_reg",
  /*  734 */  "cmpD_reg",
  /*  735 */  "cmpD_regmem",
  /*  736 */  "subDPR_reg",
  /*  737 */  "subDPR_reg_round",
  /*  738 */  "subDPR_reg_mem",
  /*  739 */  "addDPR_reg",
  /*  740 */  "addDPR_reg_round",
  /*  741 */  "addDPR_reg_mem",
  /*  742 */  "addDPR_reg_mem_0",
  /*  743 */  "addDPR_mem_reg",
  /*  744 */  "addDPR_mem_reg_0",
  /*  745 */  "addDPR_reg_imm1",
  /*  746 */  "addDPR_reg_imm",
  /*  747 */  "addDPR_reg_imm_round",
  /*  748 */  "mulDPR_reg",
  /*  749 */  "strictfp_mulDPR_reg",
  /*  750 */  "mulDPR_reg_imm",
  /*  751 */  "mulDPR_reg_mem",
  /*  752 */  "mulDPR_reg_mem_0",
  /*  753 */  "mulDPR_reg_mem_cisc",
  /*  754 */  "mulDPR_reg_mem_cisc_0",
  /*  755 */  "addDPR_mulDPR_reg",
  /*  756 */  "addDPR_mulDPR_reg_0",
  /*  757 */  "subDPR_mulDPR_reg",
  /*  758 */  "divDPR_reg",
  /*  759 */  "strictfp_divDPR_reg",
  /*  760 */  "divDPR_reg_round",
  /*  761 */  "modDPR_reg",
  /*  762 */  "modD_reg",
  /*  763 */  "atanDPR_reg",
  /*  764 */  "atanD_reg",
  /*  765 */  "sqrtDPR_reg",
  /*  766 */  "cmpFPR_0",
  /*  767 */  "cmpFPR_reg",
  /*  768 */  "cmpF_reg",
  /*  769 */  "cmpF_regmem",
  /*  770 */  "subFPR24_reg",
  /*  771 */  "subFPR_reg",
  /*  772 */  "addFPR24_reg",
  /*  773 */  "addFPR_reg",
  /*  774 */  "addFPR24_reg_mem",
  /*  775 */  "addFPR24_reg_mem_0",
  /*  776 */  "addFPR_reg_mem",
  /*  777 */  "addFPR_reg_mem_0",
  /*  778 */  "addFPR24_mem_reg",
  /*  779 */  "addFPR24_mem_cisc",
  /*  780 */  "addFPR24_mem_cisc_0",
  /*  781 */  "addFPR24_mem_mem",
  /*  782 */  "addFPR24_reg_imm",
  /*  783 */  "addFPR_reg_imm",
  /*  784 */  "mulFPR24_reg",
  /*  785 */  "mulFPR_reg",
  /*  786 */  "mulFPR24_reg_mem",
  /*  787 */  "mulFPR24_reg_mem_0",
  /*  788 */  "mulFPR_reg_mem",
  /*  789 */  "mulFPR_reg_mem_0",
  /*  790 */  "mulFPR24_mem_mem",
  /*  791 */  "mulFPR24_reg_imm",
  /*  792 */  "mulFPR_reg_imm",
  /*  793 */  "mulFPR_reg_load1",
  /*  794 */  "mulFPR_reg_load1_0",
  /*  795 */  "addFPR_mulFPR_reg_load1",
  /*  796 */  "addFPR_mulFPR_reg_load1_1",
  /*  797 */  "addFPR_mulFPR_reg_load1_0",
  /*  798 */  "addFPR_mulFPR_reg_load1_2",
  /*  799 */  "addFPR_mulFPR_reg",
  /*  800 */  "addFPR_mulFPR_reg_0",
  /*  801 */  "subFPR_divFPR_reg",
  /*  802 */  "divFPR24_reg",
  /*  803 */  "divFPR_reg",
  /*  804 */  "modFPR24_reg",
  /*  805 */  "modFPR_reg",
  /*  806 */  "modF_reg",
  /*  807 */  "roundFloat_mem_reg",
  /*  808 */  "roundDouble_mem_reg",
  /*  809 */  "convDPR2FPR_reg",
  /*  810 */  "convDPR2F_reg",
  /*  811 */  "convD2F_reg",
  /*  812 */  "convFPR2DPR_reg_reg",
  /*  813 */  "convFPR2D_reg",
  /*  814 */  "convF2DPR_reg",
  /*  815 */  "convF2D_reg",
  /*  816 */  "convDPR2I_reg_reg",
  /*  817 */  "convD2I_reg_reg",
  /*  818 */  "convDPR2L_reg_reg",
  /*  819 */  "convD2L_reg_reg",
  /*  820 */  "convFPR2I_reg_reg",
  /*  821 */  "convF2I_reg",
  /*  822 */  "convFPR2L_reg_reg",
  /*  823 */  "convF2L_reg_reg",
  /*  824 */  "convI2DPR_reg",
  /*  825 */  "convI2D_reg",
  /*  826 */  "convI2D_mem",
  /*  827 */  "convXI2D_reg",
  /*  828 */  "convI2DPR_mem",
  /*  829 */  "conv24I2FPR_reg",
  /*  830 */  "convI2FPR_SSF",
  /*  831 */  "convI2FPR_SSF_mem",
  /*  832 */  "convI2FPR_reg",
  /*  833 */  "convI2FPR_mem",
  /*  834 */  "convI2F_reg",
  /*  835 */  "convXI2F_reg",
  /*  836 */  "convI2L_reg",
  /*  837 */  "convI2L_reg_zex",
  /*  838 */  "zerox_long",
  /*  839 */  "convL2DPR_reg",
  /*  840 */  "convL2D_reg",
  /*  841 */  "convL2F_reg",
  /*  842 */  "convL2FPR_reg",
  /*  843 */  "convL2I_reg",
  /*  844 */  "MoveFPR2I_reg_stack",
  /*  845 */  "MoveF2I_reg_stack_sse",
  /*  846 */  "MoveI2F_reg_stack",
  /*  847 */  "MoveDPR2L_reg_stack",
  /*  848 */  "MoveD2L_reg_stack_sse",
  /*  849 */  "MoveD2L_reg_reg_sse",
  /*  850 */  "MoveL2D_reg_stack",
  /*  851 */  "MoveL2D_reg_reg_sse",
  /*  852 */  "rep_stos",
  /*  853 */  "rep_stos_large",
  /*  854 */  "string_compareL",
  /*  855 */  "string_compareU",
  /*  856 */  "string_compareLU",
  /*  857 */  "string_compareUL",
  /*  858 */  "string_equals",
  /*  859 */  "string_indexof_conL",
  /*  860 */  "string_indexof_conU",
  /*  861 */  "string_indexof_conUL",
  /*  862 */  "string_indexofL",
  /*  863 */  "string_indexofU",
  /*  864 */  "string_indexofUL",
  /*  865 */  "string_indexofU_char",
  /*  866 */  "array_equalsB",
  /*  867 */  "array_equalsC",
  /*  868 */  "has_negatives",
  /*  869 */  "string_compress",
  /*  870 */  "string_inflate",
  /*  871 */  "encode_iso_array",
  /*  872 */  "minI_eReg",
  /*  873 */  "maxI_eReg",
  /*  874 */  "loopLimit_eReg",
  /*  875 */  "jumpXtnd",
  /*  876 */  "jmpDir",
  /*  877 */  "jmpCon",
  /*  878 */  "jmpLoopEnd",
  /*  879 */  "jmpLoopEndU",
  /*  880 */  "jmpLoopEndUCF",
  /*  881 */  "jmpLoopEnd_and_restoreMask",
  /*  882 */  "jmpLoopEndU_and_restoreMask",
  /*  883 */  "jmpLoopEndUCF_and_restoreMask",
  /*  884 */  "jmpConU",
  /*  885 */  "jmpConUCF",
  /*  886 */  "jmpConUCF2",
  /*  887 */  "partialSubtypeCheck",
  /*  888 */  "jmpDir_short",
  /*  889 */  "jmpCon_short",
  /*  890 */  "jmpLoopEnd_short",
  /*  891 */  "jmpLoopEndU_short",
  /*  892 */  "jmpLoopEndUCF_short",
  /*  893 */  "jmpConU_short",
  /*  894 */  "jmpConUCF_short",
  /*  895 */  "jmpConUCF2_short",
  /*  896 */  "cmpL3_reg_reg",
  /*  897 */  "cmpL_LTGE",
  /*  898 */  "cmpUL_LTGE",
  /*  899 */  "cmovLL_reg_LTGE",
  /*  900 */  "cmovLL_mem_LTGE",
  /*  901 */  "cmovII_reg_LTGE",
  /*  902 */  "cmovII_mem_LTGE",
  /*  903 */  "cmovPP_reg_LTGE",
  /*  904 */  "cmovDDPR_reg_LTGE",
  /*  905 */  "cmovDD_reg_LTGE",
  /*  906 */  "cmovFFPR_reg_LTGE",
  /*  907 */  "cmovFF_reg_LTGE",
  /*  908 */  "cmpL_EQNE",
  /*  909 */  "cmpUL_EQNE",
  /*  910 */  "cmovLL_reg_EQNE",
  /*  911 */  "cmovLL_mem_EQNE",
  /*  912 */  "cmovII_reg_EQNE",
  /*  913 */  "cmovII_mem_EQNE",
  /*  914 */  "cmovPP_reg_EQNE",
  /*  915 */  "cmovDDPR_reg_EQNE",
  /*  916 */  "cmovDD_reg_EQNE",
  /*  917 */  "cmovFFPR_reg_EQNE",
  /*  918 */  "cmovFF_reg_EQNE",
  /*  919 */  "cmpL_LEGT",
  /*  920 */  "cmpUL_LEGT",
  /*  921 */  "cmovLL_reg_LEGT",
  /*  922 */  "cmovLL_mem_LEGT",
  /*  923 */  "cmovII_reg_LEGT",
  /*  924 */  "cmovII_mem_LEGT",
  /*  925 */  "cmovPP_reg_LEGT",
  /*  926 */  "cmovDDPR_reg_LEGT",
  /*  927 */  "cmovDD_reg_LEGT",
  /*  928 */  "cmovFFPR_reg_LEGT",
  /*  929 */  "cmovFF_reg_LEGT",
  /*  930 */  "CallStaticJavaDirect",
  /*  931 */  "CallDynamicJavaDirect",
  /*  932 */  "CallRuntimeDirect",
  /*  933 */  "CallLeafDirect",
  /*  934 */  "CallLeafNoFPDirect",
  /*  935 */  "Ret",
  /*  936 */  "TailCalljmpInd",
  /*  937 */  "tailjmpInd",
  /*  938 */  "CreateException",
  /*  939 */  "RethrowException",
  /*  940 */  "safePoint_poll_tls",
  /*  941 */  "tlsLoadP",
  /*  942 */  "ShouldNotReachHere",
  /*  943 */  "setMask",
  /*  944 */  "addF_reg",
  /*  945 */  "addF_mem",
  /*  946 */  "addF_mem_0",
  /*  947 */  "addF_imm",
  /*  948 */  "addF_reg_reg",
  /*  949 */  "addF_reg_mem",
  /*  950 */  "addF_reg_mem_0",
  /*  951 */  "addF_reg_imm",
  /*  952 */  "addD_reg",
  /*  953 */  "addD_mem",
  /*  954 */  "addD_mem_0",
  /*  955 */  "addD_imm",
  /*  956 */  "addD_reg_reg",
  /*  957 */  "addD_reg_mem",
  /*  958 */  "addD_reg_mem_0",
  /*  959 */  "addD_reg_imm",
  /*  960 */  "subF_reg",
  /*  961 */  "subF_mem",
  /*  962 */  "subF_imm",
  /*  963 */  "subF_reg_reg",
  /*  964 */  "subF_reg_mem",
  /*  965 */  "subF_reg_imm",
  /*  966 */  "subD_reg",
  /*  967 */  "subD_mem",
  /*  968 */  "subD_imm",
  /*  969 */  "subD_reg_reg",
  /*  970 */  "subD_reg_mem",
  /*  971 */  "subD_reg_imm",
  /*  972 */  "mulF_reg",
  /*  973 */  "mulF_mem",
  /*  974 */  "mulF_mem_0",
  /*  975 */  "mulF_imm",
  /*  976 */  "mulF_reg_reg",
  /*  977 */  "mulF_reg_mem",
  /*  978 */  "mulF_reg_mem_0",
  /*  979 */  "mulF_reg_imm",
  /*  980 */  "mulD_reg",
  /*  981 */  "mulD_mem",
  /*  982 */  "mulD_mem_0",
  /*  983 */  "mulD_imm",
  /*  984 */  "mulD_reg_reg",
  /*  985 */  "mulD_reg_mem",
  /*  986 */  "mulD_reg_mem_0",
  /*  987 */  "mulD_reg_imm",
  /*  988 */  "divF_reg",
  /*  989 */  "divF_mem",
  /*  990 */  "divF_imm",
  /*  991 */  "divF_reg_reg",
  /*  992 */  "divF_reg_mem",
  /*  993 */  "divF_reg_imm",
  /*  994 */  "divD_reg",
  /*  995 */  "divD_mem",
  /*  996 */  "divD_imm",
  /*  997 */  "divD_reg_reg",
  /*  998 */  "divD_reg_mem",
  /*  999 */  "divD_reg_imm",
  /* 1000 */  "sqrtF_reg",
  /* 1001 */  "sqrtF_mem",
  /* 1002 */  "sqrtF_imm",
  /* 1003 */  "sqrtD_reg",
  /* 1004 */  "sqrtD_mem",
  /* 1005 */  "sqrtD_imm",
  /* 1006 */  "onspinwait",
  /* 1007 */  "fmaD_reg",
  /* 1008 */  "fmaF_reg",
  /* 1009 */  "loadV",
  /* 1010 */  "storeV",
  /* 1011 */  "ReplB_reg",
  /* 1012 */  "ReplB_mem",
  /* 1013 */  "ReplB_imm",
  /* 1014 */  "ReplB_zero",
  /* 1015 */  "ReplS_reg",
  /* 1016 */  "ReplS_mem",
  /* 1017 */  "ReplS_imm",
  /* 1018 */  "ReplS_zero",
  /* 1019 */  "ReplI_reg",
  /* 1020 */  "ReplI_mem",
  /* 1021 */  "ReplI_imm",
  /* 1022 */  "ReplI_zero",
  /* 1023 */  "ReplI_M1",
  /* 1024 */  "ReplI_M1_0",
  /* 1025 */  "ReplI_M1_1",
  /* 1026 */  "ReplL_reg",
  /* 1027 */  "ReplL_reg_leg",
  /* 1028 */  "ReplL_mem",
  /* 1029 */  "ReplL_imm",
  /* 1030 */  "ReplL_zero",
  /* 1031 */  "ReplL_M1",
  /* 1032 */  "ReplF_reg",
  /* 1033 */  "ReplF_mem",
  /* 1034 */  "ReplF_zero",
  /* 1035 */  "ReplD_reg",
  /* 1036 */  "ReplD_mem",
  /* 1037 */  "ReplD_zero",
  /* 1038 */  "reductionI",
  /* 1039 */  "reductionI_0",
  /* 1040 */  "reductionI_1",
  /* 1041 */  "reductionI_2",
  /* 1042 */  "reductionI_3",
  /* 1043 */  "reduction16I",
  /* 1044 */  "reduction16I_0",
  /* 1045 */  "reduction16I_1",
  /* 1046 */  "reduction16I_2",
  /* 1047 */  "reduction16I_3",
  /* 1048 */  "reductionF128",
  /* 1049 */  "reductionF128_0",
  /* 1050 */  "reduction8F",
  /* 1051 */  "reduction8F_0",
  /* 1052 */  "reduction16F",
  /* 1053 */  "reduction16F_0",
  /* 1054 */  "reduction2D",
  /* 1055 */  "reduction2D_0",
  /* 1056 */  "reduction4D",
  /* 1057 */  "reduction4D_0",
  /* 1058 */  "reduction8D",
  /* 1059 */  "reduction8D_0",
  /* 1060 */  "vaddB",
  /* 1061 */  "vaddB_reg",
  /* 1062 */  "vaddB_mem",
  /* 1063 */  "vaddB_mem_0",
  /* 1064 */  "vaddS",
  /* 1065 */  "vaddS_reg",
  /* 1066 */  "vaddS_mem",
  /* 1067 */  "vaddS_mem_0",
  /* 1068 */  "vaddI",
  /* 1069 */  "vaddI_reg",
  /* 1070 */  "vaddI_mem",
  /* 1071 */  "vaddI_mem_0",
  /* 1072 */  "vaddL",
  /* 1073 */  "vaddL_reg",
  /* 1074 */  "vaddL_mem",
  /* 1075 */  "vaddL_mem_0",
  /* 1076 */  "vaddF",
  /* 1077 */  "vaddF_reg",
  /* 1078 */  "vaddF_mem",
  /* 1079 */  "vaddF_mem_0",
  /* 1080 */  "vaddD",
  /* 1081 */  "vaddD_reg",
  /* 1082 */  "vaddD_mem",
  /* 1083 */  "vaddD_mem_0",
  /* 1084 */  "vsubB",
  /* 1085 */  "vsubB_reg",
  /* 1086 */  "vsubB_mem",
  /* 1087 */  "vsubS",
  /* 1088 */  "vsubS_reg",
  /* 1089 */  "vsubS_mem",
  /* 1090 */  "vsubI",
  /* 1091 */  "vsubI_reg",
  /* 1092 */  "vsubI_mem",
  /* 1093 */  "vsubL",
  /* 1094 */  "vsubL_reg",
  /* 1095 */  "vsubL_mem",
  /* 1096 */  "vsubF",
  /* 1097 */  "vsubF_reg",
  /* 1098 */  "vsubF_mem",
  /* 1099 */  "vsubD",
  /* 1100 */  "vsubD_reg",
  /* 1101 */  "vsubD_mem",
  /* 1102 */  "mulB_reg",
  /* 1103 */  "mul16B_reg",
  /* 1104 */  "vmul16B_reg_avx",
  /* 1105 */  "vmul32B_reg_avx",
  /* 1106 */  "vmul64B_reg_avx",
  /* 1107 */  "vmulS",
  /* 1108 */  "vmulS_reg",
  /* 1109 */  "vmulS_mem",
  /* 1110 */  "vmulS_mem_0",
  /* 1111 */  "vmulI",
  /* 1112 */  "vmulI_reg",
  /* 1113 */  "vmulI_mem",
  /* 1114 */  "vmulI_mem_0",
  /* 1115 */  "vmulL_reg",
  /* 1116 */  "vmulL_mem",
  /* 1117 */  "vmulL_mem_0",
  /* 1118 */  "vmulF",
  /* 1119 */  "vmulF_reg",
  /* 1120 */  "vmulF_mem",
  /* 1121 */  "vmulF_mem_0",
  /* 1122 */  "vmulD",
  /* 1123 */  "vmulD_reg",
  /* 1124 */  "vmulD_mem",
  /* 1125 */  "vmulD_mem_0",
  /* 1126 */  "vcmov8F_reg",
  /* 1127 */  "vcmov4D_reg",
  /* 1128 */  "vdivF",
  /* 1129 */  "vdivF_reg",
  /* 1130 */  "vdivF_mem",
  /* 1131 */  "vdivD",
  /* 1132 */  "vdivD_reg",
  /* 1133 */  "vdivD_mem",
  /* 1134 */  "vshiftB",
  /* 1135 */  "vshiftB_0",
  /* 1136 */  "vshiftB_1",
  /* 1137 */  "vshift16B",
  /* 1138 */  "vshift16B_0",
  /* 1139 */  "vshift16B_1",
  /* 1140 */  "vshift16B_avx",
  /* 1141 */  "vshift16B_avx_0",
  /* 1142 */  "vshift16B_avx_1",
  /* 1143 */  "vshift32B_avx",
  /* 1144 */  "vshift32B_avx_0",
  /* 1145 */  "vshift32B_avx_1",
  /* 1146 */  "vshift64B_avx",
  /* 1147 */  "vshift64B_avx_0",
  /* 1148 */  "vshift64B_avx_1",
  /* 1149 */  "vshiftS",
  /* 1150 */  "vshiftS_0",
  /* 1151 */  "vshiftS_1",
  /* 1152 */  "vshiftI",
  /* 1153 */  "vshiftI_0",
  /* 1154 */  "vshiftI_1",
  /* 1155 */  "vshiftL",
  /* 1156 */  "vshiftL_0",
  /* 1157 */  "vshiftL_arith_reg",
  /* 1158 */  "vshiftL_arith_reg_evex",
  /* 1159 */  "vand",
  /* 1160 */  "vand_reg",
  /* 1161 */  "vand_mem",
  /* 1162 */  "vand_mem_0",
  /* 1163 */  "vor",
  /* 1164 */  "vor_reg",
  /* 1165 */  "vor_mem",
  /* 1166 */  "vor_mem_0",
  /* 1167 */  "vxor",
  /* 1168 */  "vxor_reg",
  /* 1169 */  "vxor_mem",
  /* 1170 */  "vxor_mem_0",
  /* 1171 */  "vabsnegF",
  /* 1172 */  "vabsnegF_0",
  /* 1173 */  "vabsneg4F",
  /* 1174 */  "vabsneg4F_0",
  /* 1175 */  "vabsnegD",
  /* 1176 */  "vabsnegD_0",
  /* 1177 */  "vfmaF_reg",
  /* 1178 */  "vfmaF_mem",
  /* 1179 */  "vfmaD_reg",
  /* 1180 */  "vfmaD_mem",
  /* 1181 */  "vmuladdS2I_reg_sse",
  /* 1182 */  "vmuladdS2I_reg_avx",
  /* 1183 */  "vmuladdaddS2I_reg",
  /* 1184 */  "vmuladdaddS2I_reg_0",
  /* 1185 */  "vpternlog",
  /* 1186 */  "vpternlog_mem",
  /* 1187 */  "compareAndSwapP_shenandoah",
  /* 1188 */  "compareAndSwapP_shenandoah_0",
  /* 1189 */  "compareAndExchangeP_shenandoah",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  true,
  /*   45 */  false,
  /*   46 */  false,
  /*   47 */  false,
  /*   48 */  false,
  /*   49 */  false,
  /*   50 */  false,
  /*   51 */  false,
  /*   52 */  false,
  /*   53 */  false,
  /*   54 */  false,
  /*   55 */  false,
  /*   56 */  false,
  /*   57 */  false,
  /*   58 */  false,
  /*   59 */  false,
  /*   60 */  false,
  /*   61 */  false,
  /*   62 */  false,
  /*   63 */  false,
  /*   64 */  false,
  /*   65 */  false,
  /*   66 */  false,
  /*   67 */  false,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  false,
  /*   91 */  true,
  /*   92 */  false,
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  false,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  /*  105 */  false,
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  true,
  /*  115 */  true,
  /*  116 */  true,
  /*  117 */  true,
  /*  118 */  true,
  /*  119 */  true,
  /*  120 */  true,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  true,
  // last operand
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  // last operand class
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  // last internally defined operand
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  /*  628 */  false,
  /*  629 */  false,
  /*  630 */  false,
  /*  631 */  false,
  /*  632 */  false,
  /*  633 */  false,
  /*  634 */  false,
  /*  635 */  false,
  /*  636 */  false,
  /*  637 */  false,
  /*  638 */  false,
  /*  639 */  false,
  /*  640 */  false,
  /*  641 */  false,
  /*  642 */  false,
  /*  643 */  false,
  /*  644 */  false,
  /*  645 */  false,
  /*  646 */  false,
  /*  647 */  false,
  /*  648 */  false,
  /*  649 */  false,
  /*  650 */  false,
  /*  651 */  false,
  /*  652 */  false,
  /*  653 */  false,
  /*  654 */  false,
  /*  655 */  false,
  /*  656 */  false,
  /*  657 */  false,
  /*  658 */  false,
  /*  659 */  false,
  /*  660 */  false,
  /*  661 */  false,
  /*  662 */  false,
  /*  663 */  false,
  /*  664 */  false,
  /*  665 */  false,
  /*  666 */  false,
  /*  667 */  false,
  /*  668 */  false,
  /*  669 */  false,
  /*  670 */  false,
  /*  671 */  false,
  /*  672 */  false,
  /*  673 */  false,
  /*  674 */  false,
  /*  675 */  false,
  /*  676 */  false,
  /*  677 */  false,
  /*  678 */  false,
  /*  679 */  false,
  /*  680 */  false,
  /*  681 */  false,
  /*  682 */  false,
  /*  683 */  false,
  /*  684 */  false,
  /*  685 */  false,
  /*  686 */  false,
  /*  687 */  false,
  /*  688 */  false,
  /*  689 */  false,
  /*  690 */  false,
  /*  691 */  false,
  /*  692 */  false,
  /*  693 */  false,
  /*  694 */  false,
  /*  695 */  false,
  /*  696 */  false,
  /*  697 */  false,
  /*  698 */  false,
  /*  699 */  false,
  /*  700 */  false,
  /*  701 */  false,
  /*  702 */  false,
  /*  703 */  false,
  /*  704 */  false,
  /*  705 */  false,
  /*  706 */  false,
  /*  707 */  false,
  /*  708 */  false,
  /*  709 */  false,
  /*  710 */  false,
  /*  711 */  false,
  /*  712 */  false,
  /*  713 */  false,
  /*  714 */  false,
  /*  715 */  false,
  /*  716 */  false,
  /*  717 */  false,
  /*  718 */  false,
  /*  719 */  false,
  /*  720 */  false,
  /*  721 */  false,
  /*  722 */  false,
  /*  723 */  false,
  /*  724 */  false,
  /*  725 */  false,
  /*  726 */  false,
  /*  727 */  false,
  /*  728 */  false,
  /*  729 */  false,
  /*  730 */  false,
  /*  731 */  false,
  /*  732 */  false,
  /*  733 */  false,
  /*  734 */  false,
  /*  735 */  false,
  /*  736 */  false,
  /*  737 */  false,
  /*  738 */  false,
  /*  739 */  false,
  /*  740 */  false,
  /*  741 */  false,
  /*  742 */  false,
  /*  743 */  false,
  /*  744 */  false,
  /*  745 */  false,
  /*  746 */  false,
  /*  747 */  false,
  /*  748 */  false,
  /*  749 */  false,
  /*  750 */  false,
  /*  751 */  false,
  /*  752 */  false,
  /*  753 */  false,
  /*  754 */  false,
  /*  755 */  false,
  /*  756 */  false,
  /*  757 */  false,
  /*  758 */  false,
  /*  759 */  false,
  /*  760 */  false,
  /*  761 */  false,
  /*  762 */  false,
  /*  763 */  false,
  /*  764 */  false,
  /*  765 */  false,
  /*  766 */  false,
  /*  767 */  false,
  /*  768 */  false,
  /*  769 */  false,
  /*  770 */  false,
  /*  771 */  false,
  /*  772 */  false,
  /*  773 */  false,
  /*  774 */  false,
  /*  775 */  false,
  /*  776 */  false,
  /*  777 */  false,
  /*  778 */  false,
  /*  779 */  false,
  /*  780 */  false,
  /*  781 */  false,
  /*  782 */  false,
  /*  783 */  false,
  /*  784 */  false,
  /*  785 */  false,
  /*  786 */  false,
  /*  787 */  false,
  /*  788 */  false,
  /*  789 */  false,
  /*  790 */  false,
  /*  791 */  false,
  /*  792 */  false,
  /*  793 */  false,
  /*  794 */  false,
  /*  795 */  false,
  /*  796 */  false,
  /*  797 */  false,
  /*  798 */  false,
  /*  799 */  false,
  /*  800 */  false,
  /*  801 */  false,
  /*  802 */  false,
  /*  803 */  false,
  /*  804 */  false,
  /*  805 */  false,
  /*  806 */  false,
  /*  807 */  false,
  /*  808 */  false,
  /*  809 */  false,
  /*  810 */  false,
  /*  811 */  false,
  /*  812 */  false,
  /*  813 */  false,
  /*  814 */  false,
  /*  815 */  false,
  /*  816 */  false,
  /*  817 */  false,
  /*  818 */  false,
  /*  819 */  false,
  /*  820 */  false,
  /*  821 */  false,
  /*  822 */  false,
  /*  823 */  false,
  /*  824 */  false,
  /*  825 */  false,
  /*  826 */  false,
  /*  827 */  false,
  /*  828 */  false,
  /*  829 */  false,
  /*  830 */  false,
  /*  831 */  false,
  /*  832 */  false,
  /*  833 */  false,
  /*  834 */  false,
  /*  835 */  false,
  /*  836 */  false,
  /*  837 */  false,
  /*  838 */  false,
  /*  839 */  false,
  /*  840 */  false,
  /*  841 */  false,
  /*  842 */  false,
  /*  843 */  false,
  /*  844 */  false,
  /*  845 */  false,
  /*  846 */  false,
  /*  847 */  false,
  /*  848 */  false,
  /*  849 */  false,
  /*  850 */  false,
  /*  851 */  false,
  /*  852 */  false,
  /*  853 */  false,
  /*  854 */  false,
  /*  855 */  false,
  /*  856 */  false,
  /*  857 */  false,
  /*  858 */  false,
  /*  859 */  false,
  /*  860 */  false,
  /*  861 */  false,
  /*  862 */  false,
  /*  863 */  false,
  /*  864 */  false,
  /*  865 */  false,
  /*  866 */  false,
  /*  867 */  false,
  /*  868 */  false,
  /*  869 */  false,
  /*  870 */  false,
  /*  871 */  false,
  /*  872 */  false,
  /*  873 */  false,
  /*  874 */  false,
  /*  875 */  false,
  /*  876 */  false,
  /*  877 */  false,
  /*  878 */  false,
  /*  879 */  false,
  /*  880 */  false,
  /*  881 */  false,
  /*  882 */  false,
  /*  883 */  false,
  /*  884 */  false,
  /*  885 */  false,
  /*  886 */  false,
  /*  887 */  false,
  /*  888 */  false,
  /*  889 */  false,
  /*  890 */  false,
  /*  891 */  false,
  /*  892 */  false,
  /*  893 */  false,
  /*  894 */  false,
  /*  895 */  false,
  /*  896 */  false,
  /*  897 */  false,
  /*  898 */  false,
  /*  899 */  false,
  /*  900 */  false,
  /*  901 */  false,
  /*  902 */  false,
  /*  903 */  false,
  /*  904 */  false,
  /*  905 */  false,
  /*  906 */  false,
  /*  907 */  false,
  /*  908 */  false,
  /*  909 */  false,
  /*  910 */  false,
  /*  911 */  false,
  /*  912 */  false,
  /*  913 */  false,
  /*  914 */  false,
  /*  915 */  false,
  /*  916 */  false,
  /*  917 */  false,
  /*  918 */  false,
  /*  919 */  false,
  /*  920 */  false,
  /*  921 */  false,
  /*  922 */  false,
  /*  923 */  false,
  /*  924 */  false,
  /*  925 */  false,
  /*  926 */  false,
  /*  927 */  false,
  /*  928 */  false,
  /*  929 */  false,
  /*  930 */  false,
  /*  931 */  false,
  /*  932 */  false,
  /*  933 */  false,
  /*  934 */  false,
  /*  935 */  false,
  /*  936 */  false,
  /*  937 */  false,
  /*  938 */  false,
  /*  939 */  false,
  /*  940 */  false,
  /*  941 */  false,
  /*  942 */  false,
  /*  943 */  false,
  /*  944 */  false,
  /*  945 */  false,
  /*  946 */  false,
  /*  947 */  false,
  /*  948 */  false,
  /*  949 */  false,
  /*  950 */  false,
  /*  951 */  false,
  /*  952 */  false,
  /*  953 */  false,
  /*  954 */  false,
  /*  955 */  false,
  /*  956 */  false,
  /*  957 */  false,
  /*  958 */  false,
  /*  959 */  false,
  /*  960 */  false,
  /*  961 */  false,
  /*  962 */  false,
  /*  963 */  false,
  /*  964 */  false,
  /*  965 */  false,
  /*  966 */  false,
  /*  967 */  false,
  /*  968 */  false,
  /*  969 */  false,
  /*  970 */  false,
  /*  971 */  false,
  /*  972 */  false,
  /*  973 */  false,
  /*  974 */  false,
  /*  975 */  false,
  /*  976 */  false,
  /*  977 */  false,
  /*  978 */  false,
  /*  979 */  false,
  /*  980 */  false,
  /*  981 */  false,
  /*  982 */  false,
  /*  983 */  false,
  /*  984 */  false,
  /*  985 */  false,
  /*  986 */  false,
  /*  987 */  false,
  /*  988 */  false,
  /*  989 */  false,
  /*  990 */  false,
  /*  991 */  false,
  /*  992 */  false,
  /*  993 */  false,
  /*  994 */  false,
  /*  995 */  false,
  /*  996 */  false,
  /*  997 */  false,
  /*  998 */  false,
  /*  999 */  false,
  /* 1000 */  false,
  /* 1001 */  false,
  /* 1002 */  false,
  /* 1003 */  false,
  /* 1004 */  false,
  /* 1005 */  false,
  /* 1006 */  false,
  /* 1007 */  false,
  /* 1008 */  false,
  /* 1009 */  false,
  /* 1010 */  false,
  /* 1011 */  false,
  /* 1012 */  false,
  /* 1013 */  false,
  /* 1014 */  false,
  /* 1015 */  false,
  /* 1016 */  false,
  /* 1017 */  false,
  /* 1018 */  false,
  /* 1019 */  false,
  /* 1020 */  false,
  /* 1021 */  false,
  /* 1022 */  false,
  /* 1023 */  false,
  /* 1024 */  false,
  /* 1025 */  false,
  /* 1026 */  false,
  /* 1027 */  false,
  /* 1028 */  false,
  /* 1029 */  false,
  /* 1030 */  false,
  /* 1031 */  false,
  /* 1032 */  false,
  /* 1033 */  false,
  /* 1034 */  false,
  /* 1035 */  false,
  /* 1036 */  false,
  /* 1037 */  false,
  /* 1038 */  false,
  /* 1039 */  false,
  /* 1040 */  false,
  /* 1041 */  false,
  /* 1042 */  false,
  /* 1043 */  false,
  /* 1044 */  false,
  /* 1045 */  false,
  /* 1046 */  false,
  /* 1047 */  false,
  /* 1048 */  false,
  /* 1049 */  false,
  /* 1050 */  false,
  /* 1051 */  false,
  /* 1052 */  false,
  /* 1053 */  false,
  /* 1054 */  false,
  /* 1055 */  false,
  /* 1056 */  false,
  /* 1057 */  false,
  /* 1058 */  false,
  /* 1059 */  false,
  /* 1060 */  false,
  /* 1061 */  false,
  /* 1062 */  false,
  /* 1063 */  false,
  /* 1064 */  false,
  /* 1065 */  false,
  /* 1066 */  false,
  /* 1067 */  false,
  /* 1068 */  false,
  /* 1069 */  false,
  /* 1070 */  false,
  /* 1071 */  false,
  /* 1072 */  false,
  /* 1073 */  false,
  /* 1074 */  false,
  /* 1075 */  false,
  /* 1076 */  false,
  /* 1077 */  false,
  /* 1078 */  false,
  /* 1079 */  false,
  /* 1080 */  false,
  /* 1081 */  false,
  /* 1082 */  false,
  /* 1083 */  false,
  /* 1084 */  false,
  /* 1085 */  false,
  /* 1086 */  false,
  /* 1087 */  false,
  /* 1088 */  false,
  /* 1089 */  false,
  /* 1090 */  false,
  /* 1091 */  false,
  /* 1092 */  false,
  /* 1093 */  false,
  /* 1094 */  false,
  /* 1095 */  false,
  /* 1096 */  false,
  /* 1097 */  false,
  /* 1098 */  false,
  /* 1099 */  false,
  /* 1100 */  false,
  /* 1101 */  false,
  /* 1102 */  false,
  /* 1103 */  false,
  /* 1104 */  false,
  /* 1105 */  false,
  /* 1106 */  false,
  /* 1107 */  false,
  /* 1108 */  false,
  /* 1109 */  false,
  /* 1110 */  false,
  /* 1111 */  false,
  /* 1112 */  false,
  /* 1113 */  false,
  /* 1114 */  false,
  /* 1115 */  false,
  /* 1116 */  false,
  /* 1117 */  false,
  /* 1118 */  false,
  /* 1119 */  false,
  /* 1120 */  false,
  /* 1121 */  false,
  /* 1122 */  false,
  /* 1123 */  false,
  /* 1124 */  false,
  /* 1125 */  false,
  /* 1126 */  false,
  /* 1127 */  false,
  /* 1128 */  false,
  /* 1129 */  false,
  /* 1130 */  false,
  /* 1131 */  false,
  /* 1132 */  false,
  /* 1133 */  false,
  /* 1134 */  false,
  /* 1135 */  false,
  /* 1136 */  false,
  /* 1137 */  false,
  /* 1138 */  false,
  /* 1139 */  false,
  /* 1140 */  false,
  /* 1141 */  false,
  /* 1142 */  false,
  /* 1143 */  false,
  /* 1144 */  false,
  /* 1145 */  false,
  /* 1146 */  false,
  /* 1147 */  false,
  /* 1148 */  false,
  /* 1149 */  false,
  /* 1150 */  false,
  /* 1151 */  false,
  /* 1152 */  false,
  /* 1153 */  false,
  /* 1154 */  false,
  /* 1155 */  false,
  /* 1156 */  false,
  /* 1157 */  false,
  /* 1158 */  false,
  /* 1159 */  false,
  /* 1160 */  false,
  /* 1161 */  false,
  /* 1162 */  false,
  /* 1163 */  false,
  /* 1164 */  false,
  /* 1165 */  false,
  /* 1166 */  false,
  /* 1167 */  false,
  /* 1168 */  false,
  /* 1169 */  false,
  /* 1170 */  false,
  /* 1171 */  false,
  /* 1172 */  false,
  /* 1173 */  false,
  /* 1174 */  false,
  /* 1175 */  false,
  /* 1176 */  false,
  /* 1177 */  false,
  /* 1178 */  false,
  /* 1179 */  false,
  /* 1180 */  false,
  /* 1181 */  false,
  /* 1182 */  false,
  /* 1183 */  false,
  /* 1184 */  false,
  /* 1185 */  false,
  /* 1186 */  false,
  /* 1187 */  false,
  /* 1188 */  false,
  /* 1189 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // VecZ: 13
  0, // _last_machine_leaf: 14
  0, // AbsD: 15
  0, // AbsF: 16
  0, // AbsI: 17
  0, // AbsL: 18
  0, // AddD: 19
  0, // AddF: 20
  0, // AddI: 21
  0, // AddL: 22
  0, // AddP: 23
  0, // Allocate: 24
  0, // AllocateArray: 25
  0, // AndI: 26
  0, // AndL: 27
  0, // ArrayCopy: 28
  0, // AryEq: 29
  0, // AtanD: 30
  1, // Binary: 31
  1, // Bool: 32
  0, // BoxLock: 33
  0, // ReverseBytesI: 34
  0, // ReverseBytesL: 35
  0, // ReverseBytesUS: 36
  0, // ReverseBytesS: 37
  0, // CProj: 38
  0, // CacheWB: 39
  0, // CacheWBPreSync: 40
  0, // CacheWBPostSync: 41
  0, // CallDynamicJava: 42
  0, // CallJava: 43
  0, // CallLeaf: 44
  0, // CallLeafNoFP: 45
  0, // CallRuntime: 46
  0, // CallStaticJava: 47
  0, // CastII: 48
  0, // CastX2P: 49
  0, // CastP2X: 50
  0, // CastPP: 51
  0, // Catch: 52
  0, // CatchProj: 53
  0, // CheckCastPP: 54
  0, // ClearArray: 55
  0, // ConstraintCast: 56
  0, // CMoveD: 57
  0, // CMoveVD: 58
  0, // CMoveF: 59
  0, // CMoveVF: 60
  0, // CMoveI: 61
  0, // CMoveL: 62
  0, // CMoveP: 63
  0, // CMoveN: 64
  1, // CmpN: 65
  1, // CmpD: 66
  0, // CmpD3: 67
  1, // CmpF: 68
  0, // CmpF3: 69
  1, // CmpI: 70
  1, // CmpL: 71
  0, // CmpL3: 72
  0, // CmpLTMask: 73
  1, // CmpP: 74
  1, // CmpU: 75
  1, // CmpUL: 76
  0, // CompareAndSwapB: 77
  0, // CompareAndSwapS: 78
  0, // CompareAndSwapI: 79
  0, // CompareAndSwapL: 80
  0, // CompareAndSwapP: 81
  0, // CompareAndSwapN: 82
  0, // WeakCompareAndSwapB: 83
  0, // WeakCompareAndSwapS: 84
  0, // WeakCompareAndSwapI: 85
  0, // WeakCompareAndSwapL: 86
  0, // WeakCompareAndSwapP: 87
  0, // WeakCompareAndSwapN: 88
  0, // CompareAndExchangeB: 89
  0, // CompareAndExchangeS: 90
  0, // CompareAndExchangeI: 91
  0, // CompareAndExchangeL: 92
  0, // CompareAndExchangeP: 93
  0, // CompareAndExchangeN: 94
  0, // GetAndAddB: 95
  0, // GetAndAddS: 96
  0, // GetAndAddI: 97
  0, // GetAndAddL: 98
  0, // GetAndSetB: 99
  0, // GetAndSetS: 100
  0, // GetAndSetI: 101
  0, // GetAndSetL: 102
  0, // GetAndSetP: 103
  0, // GetAndSetN: 104
  0, // Con: 105
  0, // ConN: 106
  0, // ConNKlass: 107
  0, // ConD: 108
  0, // ConF: 109
  0, // ConI: 110
  0, // ConL: 111
  0, // ConP: 112
  0, // Conv2B: 113
  0, // ConvD2F: 114
  0, // ConvD2I: 115
  0, // ConvD2L: 116
  0, // ConvF2D: 117
  0, // ConvF2I: 118
  0, // ConvF2L: 119
  0, // ConvI2D: 120
  0, // ConvI2F: 121
  0, // ConvI2L: 122
  0, // ConvL2D: 123
  0, // ConvL2F: 124
  0, // ConvL2I: 125
  0, // CountedLoop: 126
  0, // CountedLoopEnd: 127
  0, // OuterStripMinedLoop: 128
  0, // OuterStripMinedLoopEnd: 129
  0, // CountLeadingZerosI: 130
  0, // CountLeadingZerosL: 131
  0, // CountTrailingZerosI: 132
  0, // CountTrailingZerosL: 133
  0, // CreateEx: 134
  0, // DecodeN: 135
  0, // DecodeNKlass: 136
  0, // DivD: 137
  0, // DivF: 138
  0, // DivI: 139
  0, // DivL: 140
  0, // DivMod: 141
  0, // DivModI: 142
  0, // DivModL: 143
  0, // EncodeISOArray: 144
  0, // EncodeP: 145
  0, // EncodePKlass: 146
  1, // FastLock: 147
  1, // FastUnlock: 148
  0, // FmaD: 149
  0, // FmaF: 150
  0, // Goto: 151
  0, // Halt: 152
  0, // HasNegatives: 153
  0, // If: 154
  0, // RangeCheck: 155
  0, // IfFalse: 156
  0, // IfTrue: 157
  0, // Initialize: 158
  0, // JProj: 159
  0, // Jump: 160
  0, // JumpProj: 161
  0, // LShiftI: 162
  0, // LShiftL: 163
  0, // LoadB: 164
  0, // LoadUB: 165
  0, // LoadUS: 166
  0, // LoadD: 167
  0, // LoadD_unaligned: 168
  0, // LoadF: 169
  0, // LoadI: 170
  0, // LoadKlass: 171
  0, // LoadNKlass: 172
  0, // LoadL: 173
  0, // LoadL_unaligned: 174
  0, // LoadPLocked: 175
  0, // LoadP: 176
  0, // LoadN: 177
  0, // LoadRange: 178
  0, // LoadS: 179
  0, // Lock: 180
  0, // Loop: 181
  0, // LoopLimit: 182
  0, // Mach: 183
  0, // MachProj: 184
  0, // MulAddS2I: 185
  0, // MaxD: 186
  0, // MaxF: 187
  0, // MaxI: 188
  0, // MemBarAcquire: 189
  0, // LoadFence: 190
  0, // SetVectMaskI: 191
  0, // MemBarAcquireLock: 192
  0, // MemBarCPUOrder: 193
  0, // MemBarRelease: 194
  0, // StoreFence: 195
  0, // MemBarReleaseLock: 196
  0, // MemBarVolatile: 197
  0, // MemBarStoreStore: 198
  0, // MergeMem: 199
  0, // MinD: 200
  0, // MinF: 201
  0, // MinI: 202
  0, // ModD: 203
  0, // ModF: 204
  0, // ModI: 205
  0, // ModL: 206
  0, // MoveI2F: 207
  0, // MoveF2I: 208
  0, // MoveL2D: 209
  0, // MoveD2L: 210
  0, // MulD: 211
  0, // MulF: 212
  0, // MulHiL: 213
  0, // MulI: 214
  0, // MulL: 215
  0, // Multi: 216
  0, // NegD: 217
  0, // NegF: 218
  0, // NeverBranch: 219
  0, // OnSpinWait: 220
  0, // Opaque1: 221
  0, // OpaqueLoopInit: 222
  0, // OpaqueLoopStride: 223
  0, // Opaque2: 224
  0, // Opaque3: 225
  0, // Opaque4: 226
  0, // ProfileBoolean: 227
  0, // OrI: 228
  0, // OrL: 229
  1, // OverflowAddI: 230
  1, // OverflowSubI: 231
  1, // OverflowMulI: 232
  1, // OverflowAddL: 233
  1, // OverflowSubL: 234
  1, // OverflowMulL: 235
  0, // PCTable: 236
  0, // Parm: 237
  0, // PartialSubtypeCheck: 238
  0, // SubTypeCheck: 239
  0, // Phi: 240
  0, // PopCountI: 241
  0, // PopCountL: 242
  0, // PopCountVI: 243
  0, // PrefetchAllocation: 244
  0, // Proj: 245
  0, // RShiftI: 246
  0, // RShiftL: 247
  0, // Region: 248
  0, // Rethrow: 249
  0, // Return: 250
  0, // Root: 251
  0, // RoundDouble: 252
  0, // RoundDoubleMode: 253
  0, // RoundDoubleModeV: 254
  0, // RoundFloat: 255
  0, // SafePoint: 256
  0, // SafePointScalarObject: 257
  0, // ShenandoahCompareAndExchangeP: 258
  0, // ShenandoahCompareAndExchangeN: 259
  0, // ShenandoahCompareAndSwapN: 260
  0, // ShenandoahCompareAndSwapP: 261
  0, // ShenandoahWeakCompareAndSwapN: 262
  0, // ShenandoahWeakCompareAndSwapP: 263
  0, // ShenandoahEnqueueBarrier: 264
  0, // ShenandoahLoadReferenceBarrier: 265
  0, // SCMemProj: 266
  0, // SqrtD: 267
  0, // SqrtF: 268
  0, // Start: 269
  0, // StartOSR: 270
  0, // StoreB: 271
  0, // StoreC: 272
  0, // StoreCM: 273
  0, // StorePConditional: 274
  0, // StoreIConditional: 275
  0, // StoreLConditional: 276
  0, // StoreD: 277
  0, // StoreF: 278
  0, // StoreI: 279
  0, // StoreL: 280
  0, // StoreP: 281
  0, // StoreN: 282
  0, // StoreNKlass: 283
  0, // StrComp: 284
  0, // StrCompressedCopy: 285
  0, // StrEquals: 286
  0, // StrIndexOf: 287
  0, // StrIndexOfChar: 288
  0, // StrInflatedCopy: 289
  0, // SubD: 290
  0, // SubF: 291
  0, // SubI: 292
  0, // SubL: 293
  0, // TailCall: 294
  0, // TailJump: 295
  0, // MacroLogicV: 296
  0, // ThreadLocal: 297
  0, // Unlock: 298
  0, // URShiftI: 299
  0, // URShiftL: 300
  0, // XorI: 301
  0, // XorL: 302
  0, // Vector: 303
  0, // AddVB: 304
  0, // AddVS: 305
  0, // AddVI: 306
  0, // AddReductionVI: 307
  0, // AddVL: 308
  0, // AddReductionVL: 309
  0, // AddVF: 310
  0, // AddReductionVF: 311
  0, // AddVD: 312
  0, // AddReductionVD: 313
  0, // SubVB: 314
  0, // SubVS: 315
  0, // SubVI: 316
  0, // SubVL: 317
  0, // SubVF: 318
  0, // SubVD: 319
  0, // MulVB: 320
  0, // MulVS: 321
  0, // MulVI: 322
  0, // MulReductionVI: 323
  0, // MulVL: 324
  0, // MulReductionVL: 325
  0, // MulVF: 326
  0, // MulReductionVF: 327
  0, // MulVD: 328
  0, // MulReductionVD: 329
  0, // MulAddVS2VI: 330
  0, // FmaVD: 331
  0, // FmaVF: 332
  0, // DivVF: 333
  0, // DivVD: 334
  0, // AbsVB: 335
  0, // AbsVS: 336
  0, // AbsVI: 337
  0, // AbsVL: 338
  0, // AbsVF: 339
  0, // AbsVD: 340
  0, // NegVF: 341
  0, // NegVD: 342
  0, // SqrtVD: 343
  0, // SqrtVF: 344
  0, // LShiftCntV: 345
  0, // RShiftCntV: 346
  0, // LShiftVB: 347
  0, // LShiftVS: 348
  0, // LShiftVI: 349
  0, // LShiftVL: 350
  0, // RShiftVB: 351
  0, // RShiftVS: 352
  0, // RShiftVI: 353
  0, // RShiftVL: 354
  0, // URShiftVB: 355
  0, // URShiftVS: 356
  0, // URShiftVI: 357
  0, // URShiftVL: 358
  0, // AndV: 359
  0, // AndReductionV: 360
  0, // OrV: 361
  0, // OrReductionV: 362
  0, // XorV: 363
  0, // XorReductionV: 364
  0, // MinV: 365
  0, // MaxV: 366
  0, // MinReductionV: 367
  0, // MaxReductionV: 368
  0, // LoadVector: 369
  0, // StoreVector: 370
  0, // Pack: 371
  0, // PackB: 372
  0, // PackS: 373
  0, // PackI: 374
  0, // PackL: 375
  0, // PackF: 376
  0, // PackD: 377
  0, // Pack2L: 378
  0, // Pack2D: 379
  0, // ReplicateB: 380
  0, // ReplicateS: 381
  0, // ReplicateI: 382
  0, // ReplicateL: 383
  0, // ReplicateF: 384
  0, // ReplicateD: 385
  0, // Extract: 386
  0, // ExtractB: 387
  0, // ExtractUB: 388
  0, // ExtractC: 389
  0, // ExtractS: 390
  0, // ExtractI: 391
  0, // ExtractL: 392
  0, // ExtractF: 393
  0, // ExtractD: 394
  0, // Digit: 395
  0, // LowerCase: 396
  0, // UpperCase: 397
  0 // no trailing comma // Whitespace: 398
};
//  The following instructions can cisc-spill
//  popCountI can cisc-spill operand 1 to popCountI_mem
//  popCountL can cisc-spill operand 1 to popCountL_mem
//  cmovI_reg can cisc-spill operand 4 to cmovI_mem
//  cmovI_regU can cisc-spill operand 4 to cmovI_memU
//  cmovI_regUCF can cisc-spill operand 4 to cmovI_memUCF
//  addI_eReg can cisc-spill operand 2 to addI_eReg_mem
//  subI_eReg can cisc-spill operand 2 to subI_eReg_mem
//  mulI_eReg can cisc-spill operand 2 to mulI
//  mulI_eReg_imm can cisc-spill operand 1 to mulI_mem_imm
//  i2b can cisc-spill operand 1 to loadI2B
//  i2s can cisc-spill operand 1 to loadI2S
//  andI_eReg can cisc-spill operand 2 to andI_eReg_mem
//  andnI_rReg_rReg_rReg can cisc-spill operand 3 to andnI_rReg_rReg_mem
//  andnI_rReg_rReg_rReg_0 can cisc-spill operand 1 to andnI_rReg_rReg_mem_0
//  orI_eReg can cisc-spill operand 2 to orI_eReg_mem
//  xorI_eReg can cisc-spill operand 2 to xorI_eReg_mem
//  addL_eReg can cisc-spill operand 2 to addL_eReg_mem
//  subL_eReg can cisc-spill operand 2 to subL_eReg_mem
//  andL_eReg can cisc-spill operand 2 to andL_eReg_mem
//  andnL_eReg_eReg_eReg can cisc-spill operand 3 to andnL_eReg_eReg_mem
//  andnL_eReg_eReg_eReg_0 can cisc-spill operand 1 to andnL_eReg_eReg_mem_0
//  orl_eReg can cisc-spill operand 2 to orl_eReg_mem
//  xorl_eReg can cisc-spill operand 2 to xorl_eReg_mem
//  cmpD_cc can cisc-spill operand 2 to cmpD_ccmem
//  cmpD_ccCF can cisc-spill operand 2 to cmpD_ccmemCF
//  cmpD_reg can cisc-spill operand 2 to cmpD_regmem
//  subDPR_reg can cisc-spill operand 2 to subDPR_reg_mem
//  addDPR_reg can cisc-spill operand 2 to addDPR_reg_mem
//  mulDPR_reg can cisc-spill operand 2 to mulDPR_reg_mem
//  cmpF_cc can cisc-spill operand 2 to cmpF_ccmem
//  cmpF_ccCF can cisc-spill operand 2 to cmpF_ccmemCF
//  cmpF_reg can cisc-spill operand 2 to cmpF_regmem
//  addFPR24_reg can cisc-spill operand 2 to addFPR24_reg_mem
//  addFPR_reg can cisc-spill operand 2 to addFPR_reg_mem
//  addFPR24_reg_mem can cisc-spill operand 1 to addFPR24_mem_cisc
//  addFPR24_reg_mem_0 can cisc-spill operand 2 to addFPR24_mem_cisc_0
//  addFPR24_mem_reg can cisc-spill operand 2 to addFPR24_mem_cisc
//  mulFPR24_reg can cisc-spill operand 2 to mulFPR24_reg_mem
//  mulFPR_reg can cisc-spill operand 2 to mulFPR_reg_mem
//  addFPR_mulFPR_reg can cisc-spill operand 1 to addFPR_mulFPR_reg_load1
//  addFPR_mulFPR_reg_0 can cisc-spill operand 2 to addFPR_mulFPR_reg_load1_1
//  convI2L_reg can cisc-spill operand 1 to loadI2L
//  convI2L_reg_zex can cisc-spill operand 1 to loadUI2L
//  compI_eReg can cisc-spill operand 2 to compI_eReg_mem
//  compU_eReg can cisc-spill operand 2 to compU_eReg_mem
//  compP_eReg can cisc-spill operand 2 to compP_eReg_mem
//  cmovLL_reg_LTGE can cisc-spill operand 4 to cmovLL_mem_LTGE
//  cmovII_reg_LTGE can cisc-spill operand 4 to cmovII_mem_LTGE
//  cmovLL_reg_EQNE can cisc-spill operand 4 to cmovLL_mem_EQNE
//  cmovII_reg_EQNE can cisc-spill operand 4 to cmovII_mem_EQNE
//  cmovLL_reg_LEGT can cisc-spill operand 4 to cmovLL_mem_LEGT
//  cmovII_reg_LEGT can cisc-spill operand 4 to cmovII_mem_LEGT
//  addF_reg can cisc-spill operand 2 to addF_mem
//  addF_reg_reg can cisc-spill operand 2 to addF_reg_mem
//  addD_reg can cisc-spill operand 2 to addD_mem
//  addD_reg_reg can cisc-spill operand 2 to addD_reg_mem
//  subF_reg can cisc-spill operand 2 to subF_mem
//  subF_reg_reg can cisc-spill operand 2 to subF_reg_mem
//  subD_reg can cisc-spill operand 2 to subD_mem
//  subD_reg_reg can cisc-spill operand 2 to subD_reg_mem
//  mulF_reg can cisc-spill operand 2 to mulF_mem
//  mulF_reg_reg can cisc-spill operand 2 to mulF_reg_mem
//  mulD_reg can cisc-spill operand 2 to mulD_mem
//  mulD_reg_reg can cisc-spill operand 2 to mulD_reg_mem
//  divF_reg can cisc-spill operand 2 to divF_mem
//  divF_reg_reg can cisc-spill operand 2 to divF_reg_mem
//  divD_reg can cisc-spill operand 2 to divD_mem
//  divD_reg_reg can cisc-spill operand 2 to divD_reg_mem
//  sqrtF_reg can cisc-spill operand 1 to sqrtF_mem
//  sqrtD_reg can cisc-spill operand 1 to sqrtD_mem
//  ReplI_reg can cisc-spill operand 1 to ReplI_mem
//  ReplF_reg can cisc-spill operand 1 to ReplF_mem
//  ReplD_reg can cisc-spill operand 1 to ReplD_mem



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "ECX",
  "EBX",
  "EBP",
  "EDI",
  "EAX",
  "EDX",
  "ESI",
  "ESP",
  "FPR0L",
  "FPR0H",
  "FPR1L",
  "FPR1H",
  "FPR2L",
  "FPR2H",
  "FPR3L",
  "FPR3H",
  "FPR4L",
  "FPR4H",
  "FPR5L",
  "FPR5H",
  "FPR6L",
  "FPR6H",
  "FPR7L",
  "FPR7H",
  "FILL0",
  "FILL1",
  "FILL2",
  "FILL3",
  "FILL4",
  "FILL5",
  "FILL6",
  "FILL7",
  "XMM0",
  "XMM0b",
  "XMM0c",
  "XMM0d",
  "XMM0e",
  "XMM0f",
  "XMM0g",
  "XMM0h",
  "XMM0i",
  "XMM0j",
  "XMM0k",
  "XMM0l",
  "XMM0m",
  "XMM0n",
  "XMM0o",
  "XMM0p",
  "XMM1",
  "XMM1b",
  "XMM1c",
  "XMM1d",
  "XMM1e",
  "XMM1f",
  "XMM1g",
  "XMM1h",
  "XMM1i",
  "XMM1j",
  "XMM1k",
  "XMM1l",
  "XMM1m",
  "XMM1n",
  "XMM1o",
  "XMM1p",
  "XMM2",
  "XMM2b",
  "XMM2c",
  "XMM2d",
  "XMM2e",
  "XMM2f",
  "XMM2g",
  "XMM2h",
  "XMM2i",
  "XMM2j",
  "XMM2k",
  "XMM2l",
  "XMM2m",
  "XMM2n",
  "XMM2o",
  "XMM2p",
  "XMM3",
  "XMM3b",
  "XMM3c",
  "XMM3d",
  "XMM3e",
  "XMM3f",
  "XMM3g",
  "XMM3h",
  "XMM3i",
  "XMM3j",
  "XMM3k",
  "XMM3l",
  "XMM3m",
  "XMM3n",
  "XMM3o",
  "XMM3p",
  "XMM4",
  "XMM4b",
  "XMM4c",
  "XMM4d",
  "XMM4e",
  "XMM4f",
  "XMM4g",
  "XMM4h",
  "XMM4i",
  "XMM4j",
  "XMM4k",
  "XMM4l",
  "XMM4m",
  "XMM4n",
  "XMM4o",
  "XMM4p",
  "XMM5",
  "XMM5b",
  "XMM5c",
  "XMM5d",
  "XMM5e",
  "XMM5f",
  "XMM5g",
  "XMM5h",
  "XMM5i",
  "XMM5j",
  "XMM5k",
  "XMM5l",
  "XMM5m",
  "XMM5n",
  "XMM5o",
  "XMM5p",
  "XMM6",
  "XMM6b",
  "XMM6c",
  "XMM6d",
  "XMM6e",
  "XMM6f",
  "XMM6g",
  "XMM6h",
  "XMM6i",
  "XMM6j",
  "XMM6k",
  "XMM6l",
  "XMM6m",
  "XMM6n",
  "XMM6o",
  "XMM6p",
  "XMM7",
  "XMM7b",
  "XMM7c",
  "XMM7d",
  "XMM7e",
  "XMM7f",
  "XMM7g",
  "XMM7h",
  "XMM7i",
  "XMM7j",
  "XMM7k",
  "XMM7l",
  "XMM7m",
  "XMM7n",
  "XMM7o",
  "XMM7p",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	rcx->as_VMReg(),
	rbx->as_VMReg(),
	rbp->as_VMReg(),
	rdi->as_VMReg(),
	rax->as_VMReg(),
	rdx->as_VMReg(),
	rsi->as_VMReg(),
	rsp->as_VMReg(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	as_FloatRegister(0)->as_VMReg(),
	as_FloatRegister(0)->as_VMReg()->next(),
	as_FloatRegister(1)->as_VMReg(),
	as_FloatRegister(1)->as_VMReg()->next(),
	as_FloatRegister(2)->as_VMReg(),
	as_FloatRegister(2)->as_VMReg()->next(),
	as_FloatRegister(3)->as_VMReg(),
	as_FloatRegister(3)->as_VMReg()->next(),
	as_FloatRegister(4)->as_VMReg(),
	as_FloatRegister(4)->as_VMReg()->next(),
	as_FloatRegister(5)->as_VMReg(),
	as_FloatRegister(5)->as_VMReg()->next(),
	as_FloatRegister(6)->as_VMReg(),
	as_FloatRegister(6)->as_VMReg()->next(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	xmm0->as_VMReg(),
	xmm0->as_VMReg()->next(1),
	xmm0->as_VMReg()->next(2),
	xmm0->as_VMReg()->next(3),
	xmm0->as_VMReg()->next(4),
	xmm0->as_VMReg()->next(5),
	xmm0->as_VMReg()->next(6),
	xmm0->as_VMReg()->next(7),
	xmm0->as_VMReg()->next(8),
	xmm0->as_VMReg()->next(9),
	xmm0->as_VMReg()->next(10),
	xmm0->as_VMReg()->next(11),
	xmm0->as_VMReg()->next(12),
	xmm0->as_VMReg()->next(13),
	xmm0->as_VMReg()->next(14),
	xmm0->as_VMReg()->next(15),
	xmm1->as_VMReg(),
	xmm1->as_VMReg()->next(1),
	xmm1->as_VMReg()->next(2),
	xmm1->as_VMReg()->next(3),
	xmm1->as_VMReg()->next(4),
	xmm1->as_VMReg()->next(5),
	xmm1->as_VMReg()->next(6),
	xmm1->as_VMReg()->next(7),
	xmm1->as_VMReg()->next(8),
	xmm1->as_VMReg()->next(9),
	xmm1->as_VMReg()->next(10),
	xmm1->as_VMReg()->next(11),
	xmm1->as_VMReg()->next(12),
	xmm1->as_VMReg()->next(13),
	xmm1->as_VMReg()->next(14),
	xmm1->as_VMReg()->next(15),
	xmm2->as_VMReg(),
	xmm2->as_VMReg()->next(1),
	xmm2->as_VMReg()->next(2),
	xmm2->as_VMReg()->next(3),
	xmm2->as_VMReg()->next(4),
	xmm2->as_VMReg()->next(5),
	xmm2->as_VMReg()->next(6),
	xmm2->as_VMReg()->next(7),
	xmm2->as_VMReg()->next(8),
	xmm2->as_VMReg()->next(9),
	xmm2->as_VMReg()->next(10),
	xmm2->as_VMReg()->next(11),
	xmm2->as_VMReg()->next(12),
	xmm2->as_VMReg()->next(13),
	xmm2->as_VMReg()->next(14),
	xmm2->as_VMReg()->next(15),
	xmm3->as_VMReg(),
	xmm3->as_VMReg()->next(1),
	xmm3->as_VMReg()->next(2),
	xmm3->as_VMReg()->next(3),
	xmm3->as_VMReg()->next(4),
	xmm3->as_VMReg()->next(5),
	xmm3->as_VMReg()->next(6),
	xmm3->as_VMReg()->next(7),
	xmm3->as_VMReg()->next(8),
	xmm3->as_VMReg()->next(9),
	xmm3->as_VMReg()->next(10),
	xmm3->as_VMReg()->next(11),
	xmm3->as_VMReg()->next(12),
	xmm3->as_VMReg()->next(13),
	xmm3->as_VMReg()->next(14),
	xmm3->as_VMReg()->next(15),
	xmm4->as_VMReg(),
	xmm4->as_VMReg()->next(1),
	xmm4->as_VMReg()->next(2),
	xmm4->as_VMReg()->next(3),
	xmm4->as_VMReg()->next(4),
	xmm4->as_VMReg()->next(5),
	xmm4->as_VMReg()->next(6),
	xmm4->as_VMReg()->next(7),
	xmm4->as_VMReg()->next(8),
	xmm4->as_VMReg()->next(9),
	xmm4->as_VMReg()->next(10),
	xmm4->as_VMReg()->next(11),
	xmm4->as_VMReg()->next(12),
	xmm4->as_VMReg()->next(13),
	xmm4->as_VMReg()->next(14),
	xmm4->as_VMReg()->next(15),
	xmm5->as_VMReg(),
	xmm5->as_VMReg()->next(1),
	xmm5->as_VMReg()->next(2),
	xmm5->as_VMReg()->next(3),
	xmm5->as_VMReg()->next(4),
	xmm5->as_VMReg()->next(5),
	xmm5->as_VMReg()->next(6),
	xmm5->as_VMReg()->next(7),
	xmm5->as_VMReg()->next(8),
	xmm5->as_VMReg()->next(9),
	xmm5->as_VMReg()->next(10),
	xmm5->as_VMReg()->next(11),
	xmm5->as_VMReg()->next(12),
	xmm5->as_VMReg()->next(13),
	xmm5->as_VMReg()->next(14),
	xmm5->as_VMReg()->next(15),
	xmm6->as_VMReg(),
	xmm6->as_VMReg()->next(1),
	xmm6->as_VMReg()->next(2),
	xmm6->as_VMReg()->next(3),
	xmm6->as_VMReg()->next(4),
	xmm6->as_VMReg()->next(5),
	xmm6->as_VMReg()->next(6),
	xmm6->as_VMReg()->next(7),
	xmm6->as_VMReg()->next(8),
	xmm6->as_VMReg()->next(9),
	xmm6->as_VMReg()->next(10),
	xmm6->as_VMReg()->next(11),
	xmm6->as_VMReg()->next(12),
	xmm6->as_VMReg()->next(13),
	xmm6->as_VMReg()->next(14),
	xmm6->as_VMReg()->next(15),
	xmm7->as_VMReg(),
	xmm7->as_VMReg()->next(1),
	xmm7->as_VMReg()->next(2),
	xmm7->as_VMReg()->next(3),
	xmm7->as_VMReg()->next(4),
	xmm7->as_VMReg()->next(5),
	xmm7->as_VMReg()->next(6),
	xmm7->as_VMReg()->next(7),
	xmm7->as_VMReg()->next(8),
	xmm7->as_VMReg()->next(9),
	xmm7->as_VMReg()->next(10),
	xmm7->as_VMReg()->next(11),
	xmm7->as_VMReg()->next(12),
	xmm7->as_VMReg()->next(13),
	xmm7->as_VMReg()->next(14),
	xmm7->as_VMReg()->next(15),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\x1',  // ECX
  (unsigned char)'\x3',  // EBX
  (unsigned char)'\x5',  // EBP
  (unsigned char)'\x7',  // EDI
  (unsigned char)'\x0',  // EAX
  (unsigned char)'\x2',  // EDX
  (unsigned char)'\x6',  // ESI
  (unsigned char)'\x4',  // ESP
  (unsigned char)'\x0',  // FPR0L
  (unsigned char)'\x0',  // FPR0H
  (unsigned char)'\x1',  // FPR1L
  (unsigned char)'\x1',  // FPR1H
  (unsigned char)'\x2',  // FPR2L
  (unsigned char)'\x2',  // FPR2H
  (unsigned char)'\x3',  // FPR3L
  (unsigned char)'\x3',  // FPR3H
  (unsigned char)'\x4',  // FPR4L
  (unsigned char)'\x4',  // FPR4H
  (unsigned char)'\x5',  // FPR5L
  (unsigned char)'\x5',  // FPR5H
  (unsigned char)'\x6',  // FPR6L
  (unsigned char)'\x6',  // FPR6H
  (unsigned char)'\x7',  // FPR7L
  (unsigned char)'\x7',  // FPR7H
  (unsigned char)'\x8',  // FILL0
  (unsigned char)'\x9',  // FILL1
  (unsigned char)'\xA',  // FILL2
  (unsigned char)'\xB',  // FILL3
  (unsigned char)'\xC',  // FILL4
  (unsigned char)'\xD',  // FILL5
  (unsigned char)'\xE',  // FILL6
  (unsigned char)'\xF',  // FILL7
  (unsigned char)'\x0',  // XMM0
  (unsigned char)'\x0',  // XMM0b
  (unsigned char)'\x0',  // XMM0c
  (unsigned char)'\x0',  // XMM0d
  (unsigned char)'\x0',  // XMM0e
  (unsigned char)'\x0',  // XMM0f
  (unsigned char)'\x0',  // XMM0g
  (unsigned char)'\x0',  // XMM0h
  (unsigned char)'\x0',  // XMM0i
  (unsigned char)'\x0',  // XMM0j
  (unsigned char)'\x0',  // XMM0k
  (unsigned char)'\x0',  // XMM0l
  (unsigned char)'\x0',  // XMM0m
  (unsigned char)'\x0',  // XMM0n
  (unsigned char)'\x0',  // XMM0o
  (unsigned char)'\x0',  // XMM0p
  (unsigned char)'\x1',  // XMM1
  (unsigned char)'\x1',  // XMM1b
  (unsigned char)'\x1',  // XMM1c
  (unsigned char)'\x1',  // XMM1d
  (unsigned char)'\x1',  // XMM1e
  (unsigned char)'\x1',  // XMM1f
  (unsigned char)'\x1',  // XMM1g
  (unsigned char)'\x1',  // XMM1h
  (unsigned char)'\x1',  // XMM1i
  (unsigned char)'\x1',  // XMM1j
  (unsigned char)'\x1',  // XMM1k
  (unsigned char)'\x1',  // XMM1l
  (unsigned char)'\x1',  // XMM1m
  (unsigned char)'\x1',  // XMM1n
  (unsigned char)'\x1',  // XMM1o
  (unsigned char)'\x1',  // XMM1p
  (unsigned char)'\x2',  // XMM2
  (unsigned char)'\x2',  // XMM2b
  (unsigned char)'\x2',  // XMM2c
  (unsigned char)'\x2',  // XMM2d
  (unsigned char)'\x2',  // XMM2e
  (unsigned char)'\x2',  // XMM2f
  (unsigned char)'\x2',  // XMM2g
  (unsigned char)'\x2',  // XMM2h
  (unsigned char)'\x2',  // XMM2i
  (unsigned char)'\x2',  // XMM2j
  (unsigned char)'\x2',  // XMM2k
  (unsigned char)'\x2',  // XMM2l
  (unsigned char)'\x2',  // XMM2m
  (unsigned char)'\x2',  // XMM2n
  (unsigned char)'\x2',  // XMM2o
  (unsigned char)'\x2',  // XMM2p
  (unsigned char)'\x3',  // XMM3
  (unsigned char)'\x3',  // XMM3b
  (unsigned char)'\x3',  // XMM3c
  (unsigned char)'\x3',  // XMM3d
  (unsigned char)'\x3',  // XMM3e
  (unsigned char)'\x3',  // XMM3f
  (unsigned char)'\x3',  // XMM3g
  (unsigned char)'\x3',  // XMM3h
  (unsigned char)'\x3',  // XMM3i
  (unsigned char)'\x3',  // XMM3j
  (unsigned char)'\x3',  // XMM3k
  (unsigned char)'\x3',  // XMM3l
  (unsigned char)'\x3',  // XMM3m
  (unsigned char)'\x3',  // XMM3n
  (unsigned char)'\x3',  // XMM3o
  (unsigned char)'\x3',  // XMM3p
  (unsigned char)'\x4',  // XMM4
  (unsigned char)'\x4',  // XMM4b
  (unsigned char)'\x4',  // XMM4c
  (unsigned char)'\x4',  // XMM4d
  (unsigned char)'\x4',  // XMM4e
  (unsigned char)'\x4',  // XMM4f
  (unsigned char)'\x4',  // XMM4g
  (unsigned char)'\x4',  // XMM4h
  (unsigned char)'\x4',  // XMM4i
  (unsigned char)'\x4',  // XMM4j
  (unsigned char)'\x4',  // XMM4k
  (unsigned char)'\x4',  // XMM4l
  (unsigned char)'\x4',  // XMM4m
  (unsigned char)'\x4',  // XMM4n
  (unsigned char)'\x4',  // XMM4o
  (unsigned char)'\x4',  // XMM4p
  (unsigned char)'\x5',  // XMM5
  (unsigned char)'\x5',  // XMM5b
  (unsigned char)'\x5',  // XMM5c
  (unsigned char)'\x5',  // XMM5d
  (unsigned char)'\x5',  // XMM5e
  (unsigned char)'\x5',  // XMM5f
  (unsigned char)'\x5',  // XMM5g
  (unsigned char)'\x5',  // XMM5h
  (unsigned char)'\x5',  // XMM5i
  (unsigned char)'\x5',  // XMM5j
  (unsigned char)'\x5',  // XMM5k
  (unsigned char)'\x5',  // XMM5l
  (unsigned char)'\x5',  // XMM5m
  (unsigned char)'\x5',  // XMM5n
  (unsigned char)'\x5',  // XMM5o
  (unsigned char)'\x5',  // XMM5p
  (unsigned char)'\x6',  // XMM6
  (unsigned char)'\x6',  // XMM6b
  (unsigned char)'\x6',  // XMM6c
  (unsigned char)'\x6',  // XMM6d
  (unsigned char)'\x6',  // XMM6e
  (unsigned char)'\x6',  // XMM6f
  (unsigned char)'\x6',  // XMM6g
  (unsigned char)'\x6',  // XMM6h
  (unsigned char)'\x6',  // XMM6i
  (unsigned char)'\x6',  // XMM6j
  (unsigned char)'\x6',  // XMM6k
  (unsigned char)'\x6',  // XMM6l
  (unsigned char)'\x6',  // XMM6m
  (unsigned char)'\x6',  // XMM6n
  (unsigned char)'\x6',  // XMM6o
  (unsigned char)'\x6',  // XMM6p
  (unsigned char)'\x7',  // XMM7
  (unsigned char)'\x7',  // XMM7b
  (unsigned char)'\x7',  // XMM7c
  (unsigned char)'\x7',  // XMM7d
  (unsigned char)'\x7',  // XMM7e
  (unsigned char)'\x7',  // XMM7f
  (unsigned char)'\x7',  // XMM7g
  (unsigned char)'\x7',  // XMM7h
  (unsigned char)'\x7',  // XMM7i
  (unsigned char)'\x7',  // XMM7j
  (unsigned char)'\x7',  // XMM7k
  (unsigned char)'\x7',  // XMM7l
  (unsigned char)'\x7',  // XMM7m
  (unsigned char)'\x7',  // XMM7n
  (unsigned char)'\x7',  // XMM7o
  (unsigned char)'\x7',  // XMM7p
  (unsigned char)'\x8' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone() const {
  return  new labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone() const {
  return  new methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *rRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *xRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_X_REG_mask();
}

const RegMask *eAXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask();
}

const RegMask *eBXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask();
}

const RegMask *eCXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask();
}

const RegMask *eDXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDX_REG_mask();
}

const RegMask *eDIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask();
}

const RegMask *naxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask();
}

const RegMask *nadxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NADX_REG_mask();
}

const RegMask *ncxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NCX_REG_mask();
}

const RegMask *eSIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *anyRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *eRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *eRegP_no_EBPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_NO_EBP_mask();
}

const RegMask *naxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask();
}

const RegMask *nabxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NABX_REG_mask();
}

const RegMask *pRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &P_REG_mask();
}

const RegMask *eAXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask();
}

const RegMask *eBXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask();
}

const RegMask *eCXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask();
}

const RegMask *eDXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDX_REG_mask();
}

const RegMask *eSIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *eDIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask();
}

const RegMask *eRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask();
}

const RegMask *eADXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask();
}

const RegMask *eBCXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBCX_REG_mask();
}

const RegMask *eADXRegL_low_onlyOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask();
}

const RegMask *eFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegUCFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *regDPROper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG_mask();
}

const RegMask *regDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG0_mask();
}

const RegMask *regDPR2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG1_mask();
}

const RegMask *regnotDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_NOTREG0_mask();
}

const RegMask *regFPROper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_FLT_REG_mask();
}

const RegMask *regFPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_FLT_REG0_mask();
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_LEGACY_mask();
}

const RegMask *vlRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_VL_mask();
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_LEGACY_mask();
}

const RegMask *vlRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_VL_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset32XOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexScaleOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *load_long_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *load_long_indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *load_long_indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *indirect_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset8_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_NO_EBP_mask();
}

const RegMask *indOffset32_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_NO_EBP_mask();
}

const RegMask *indIndexOffset_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_NO_EBP_mask();
  case 1: return &INT_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScale_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_NO_EBP_mask();
  case 1: return &INT_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOffset_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_NO_EBP_mask();
  case 1: return &INT_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *vecOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RegMask::Empty;
}

const RegMask *legVecOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RegMask::Empty;
}

const RegMask *vecSOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORS_REG_VLBWDQ_mask();
}

const RegMask *legVecSOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORS_REG_LEGACY_mask();
}

const RegMask *vecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_VLBWDQ_mask();
}

const RegMask *legVecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_LEGACY_mask();
}

const RegMask *vecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_VLBWDQ_mask();
}

const RegMask *legVecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_LEGACY_mask();
}

const RegMask *vecYOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORY_REG_VLBWDQ_mask();
}

const RegMask *legVecYOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORY_REG_LEGACY_mask();
}

const RegMask *vecZOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORZ_REG_mask();
}

const RegMask *legVecZOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORZ_REG_LEGACY_mask();
}

//------------------Define members for classes derived from MachNode----------

void popCountINode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountINode::cisc_version(int offset) {
  popCountI_memNode *node = new popCountI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void popCountLNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountLNode::cisc_version(int offset) {
  popCountL_memNode *node = new popCountL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regNode::cisc_version(int offset) {
  cmovI_memNode *node = new cmovI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUNode::cisc_version(int offset) {
  cmovI_memUNode *node = new cmovI_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUCFNode::cisc_version(int offset) {
  cmovI_memUCFNode *node = new cmovI_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addI_eRegNode::cisc_version(int offset) {
  addI_eReg_memNode *node = new addI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *subI_eRegNode::cisc_version(int offset) {
  subI_eReg_memNode *node = new subI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_eRegNode::cisc_version(int offset) {
  mulINode *node = new mulINode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulI_eReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_eReg_immNode::cisc_version(int offset) {
  mulI_mem_immNode *node = new mulI_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void i2bNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_X_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2bNode::cisc_version(int offset) {
  loadI2BNode *node = new loadI2BNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void i2sNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_X_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2sNode::cisc_version(int offset) {
  loadI2SNode *node = new loadI2SNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andI_eRegNode::cisc_version(int offset) {
  andI_eReg_memNode *node = new andI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnI_rReg_rReg_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnI_rReg_rReg_rRegNode::cisc_version(int offset) {
  andnI_rReg_rReg_memNode *node = new andnI_rReg_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnI_rReg_rReg_rReg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnI_rReg_rReg_rReg_0Node::cisc_version(int offset) {
  andnI_rReg_rReg_mem_0Node *node = new andnI_rReg_rReg_mem_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void orI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *orI_eRegNode::cisc_version(int offset) {
  orI_eReg_memNode *node = new orI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void xorI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorI_eRegNode::cisc_version(int offset) {
  xorI_eReg_memNode *node = new xorI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *addL_eRegNode::cisc_version(int offset) {
  addL_eReg_memNode *node = new addL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *subL_eRegNode::cisc_version(int offset) {
  subL_eReg_memNode *node = new subL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andL_eRegNode::cisc_version(int offset) {
  andL_eReg_memNode *node = new andL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnL_eReg_eReg_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnL_eReg_eReg_eRegNode::cisc_version(int offset) {
  andnL_eReg_eReg_memNode *node = new andnL_eReg_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnL_eReg_eReg_eReg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnL_eReg_eReg_eReg_0Node::cisc_version(int offset) {
  andnL_eReg_eReg_mem_0Node *node = new andnL_eReg_eReg_mem_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void orl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *orl_eRegNode::cisc_version(int offset) {
  orl_eReg_memNode *node = new orl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void xorl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorl_eRegNode::cisc_version(int offset) {
  xorl_eReg_memNode *node = new xorl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_ccNode::cisc_version(int offset) {
  cmpD_ccmemNode *node = new cmpD_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_ccCFNode::cisc_version(int offset) {
  cmpD_ccmemCFNode *node = new cmpD_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_regNode::cisc_version(int offset) {
  cmpD_regmemNode *node = new cmpD_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *subDPR_regNode::cisc_version(int offset) {
  subDPR_reg_memNode *node = new subDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *addDPR_regNode::cisc_version(int offset) {
  addDPR_reg_memNode *node = new addDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulDPR_regNode::cisc_version(int offset) {
  mulDPR_reg_memNode *node = new mulDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_ccNode::cisc_version(int offset) {
  cmpF_ccmemNode *node = new cmpF_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_ccCFNode::cisc_version(int offset) {
  cmpF_ccmemCFNode *node = new cmpF_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_regNode::cisc_version(int offset) {
  cmpF_regmemNode *node = new cmpF_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_regNode::cisc_version(int offset) {
  addFPR24_reg_memNode *node = new addFPR24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_regNode::cisc_version(int offset) {
  addFPR_reg_memNode *node = new addFPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_reg_memNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_reg_memNode::cisc_version(int offset) {
  addFPR24_mem_ciscNode *node = new addFPR24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_reg_mem_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_reg_mem_0Node::cisc_version(int offset) {
  addFPR24_mem_cisc_0Node *node = new addFPR24_mem_cisc_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_mem_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_mem_regNode::cisc_version(int offset) {
  addFPR24_mem_ciscNode *node = new addFPR24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulFPR24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulFPR24_regNode::cisc_version(int offset) {
  mulFPR24_reg_memNode *node = new mulFPR24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulFPR_regNode::cisc_version(int offset) {
  mulFPR_reg_memNode *node = new mulFPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_mulFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_mulFPR_regNode::cisc_version(int offset) {
  addFPR_mulFPR_reg_load1Node *node = new addFPR_mulFPR_reg_load1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_mulFPR_reg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_mulFPR_reg_0Node::cisc_version(int offset) {
  addFPR_mulFPR_reg_load1_1Node *node = new addFPR_mulFPR_reg_load1_1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void convI2L_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_regNode::cisc_version(int offset) {
  loadI2LNode *node = new loadI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void convI2L_reg_zexNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_reg_zexNode::cisc_version(int offset) {
  loadUI2LNode *node = new loadUI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compI_eRegNode::cisc_version(int offset) {
  compI_eReg_memNode *node = new compI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compU_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compU_eRegNode::cisc_version(int offset) {
  compU_eReg_memNode *node = new compU_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compP_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compP_eRegNode::cisc_version(int offset) {
  compP_eReg_memNode *node = new compP_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}

// Build short branch version of this instruction
MachNode *jmpDirNode::short_branch_version() {
  jmpDir_shortNode *node = new jmpDir_shortNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConNode::short_branch_version() {
  jmpCon_shortNode *node = new jmpCon_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndNode::short_branch_version() {
  jmpLoopEnd_shortNode *node = new jmpLoopEnd_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUNode::short_branch_version() {
  jmpLoopEndU_shortNode *node = new jmpLoopEndU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUCFNode::short_branch_version() {
  jmpLoopEndUCF_shortNode *node = new jmpLoopEndUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUNode::short_branch_version() {
  jmpConU_shortNode *node = new jmpConU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCFNode::short_branch_version() {
  jmpConUCF_shortNode *node = new jmpConUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCF2Node::short_branch_version() {
  jmpConUCF2_shortNode *node = new jmpConUCF2_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}


void cmovLL_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LTGENode::cisc_version(int offset) {
  cmovLL_mem_LTGENode *node = new cmovLL_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LTGENode::cisc_version(int offset) {
  cmovII_mem_LTGENode *node = new cmovII_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_EQNENode::cisc_version(int offset) {
  cmovLL_mem_EQNENode *node = new cmovLL_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_EQNENode::cisc_version(int offset) {
  cmovII_mem_EQNENode *node = new cmovII_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LEGTNode::cisc_version(int offset) {
  cmovLL_mem_LEGTNode *node = new cmovLL_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LEGTNode::cisc_version(int offset) {
  cmovII_mem_LEGTNode *node = new cmovII_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addF_regNode::cisc_version(int offset) {
  addF_memNode *node = new addF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addF_reg_regNode::cisc_version(int offset) {
  addF_reg_memNode *node = new addF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addD_regNode::cisc_version(int offset) {
  addD_memNode *node = new addD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addD_reg_regNode::cisc_version(int offset) {
  addD_reg_memNode *node = new addD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subF_regNode::cisc_version(int offset) {
  subF_memNode *node = new subF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subF_reg_regNode::cisc_version(int offset) {
  subF_reg_memNode *node = new subF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subD_regNode::cisc_version(int offset) {
  subD_memNode *node = new subD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subD_reg_regNode::cisc_version(int offset) {
  subD_reg_memNode *node = new subD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulF_regNode::cisc_version(int offset) {
  mulF_memNode *node = new mulF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulF_reg_regNode::cisc_version(int offset) {
  mulF_reg_memNode *node = new mulF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulD_regNode::cisc_version(int offset) {
  mulD_memNode *node = new mulD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulD_reg_regNode::cisc_version(int offset) {
  mulD_reg_memNode *node = new mulD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divF_regNode::cisc_version(int offset) {
  divF_memNode *node = new divF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divF_reg_regNode::cisc_version(int offset) {
  divF_reg_memNode *node = new divF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divD_regNode::cisc_version(int offset) {
  divD_memNode *node = new divD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divD_reg_regNode::cisc_version(int offset) {
  divD_reg_memNode *node = new divD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void sqrtF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *sqrtF_regNode::cisc_version(int offset) {
  sqrtF_memNode *node = new sqrtF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void sqrtD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *sqrtD_regNode::cisc_version(int offset) {
  sqrtD_memNode *node = new sqrtD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void ReplI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *ReplI_regNode::cisc_version(int offset) {
  ReplI_memNode *node = new ReplI_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void ReplF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_VL_mask();
}

// Build CISC version of this instruction
MachNode *ReplF_regNode::cisc_version(int offset) {
  ReplF_memNode *node = new ReplF_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void ReplD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_VL_mask();
}

// Build CISC version of this instruction
MachNode *ReplD_regNode::cisc_version(int offset) {
  ReplD_memNode *node = new ReplD_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode(MachNode* node) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone();
  }
}

void jmpDirNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDirNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCFNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCFNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEnd_and_restoreMaskNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_and_restoreMaskNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndU_and_restoreMaskNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_and_restoreMaskNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCF_and_restoreMaskNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCF_and_restoreMaskNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCFNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCFNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2Node::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2Node::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpDir_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDir_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpCon_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpCon_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEnd_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndU_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCF_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCF_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConU_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConU_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_LTGENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_LTGENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_LTGENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_LTGENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_EQNENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_EQNENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_EQNENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_EQNENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_LEGTNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_LEGTNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_LEGTNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_LEGTNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int popCountI_memNode::reloc() const {
  return 1;
}

int popCountL_memNode::reloc() const {
  return 1;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadUB2L_immINode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2BNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2BNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadUS2L_immI_255Node::reloc() const {
  return 1;
}

int loadUS2L_immINode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2BNode::reloc() const {
  return 1;
}

int loadI2UBNode::reloc() const {
  return 1;
}

int loadI2SNode::reloc() const {
  return 1;
}

int loadI2USNode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadI2L_immI_255Node::reloc() const {
  return 1;
}

int loadI2L_immI_65535Node::reloc() const {
  return 1;
}

int loadI2L_immU31Node::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadL_volatileNode::reloc() const {
  return 1;
}

int loadLX_volatileNode::reloc() const {
  return 1;
}

int loadLX_reg_volatileNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadDPRNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadD_partialNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadFPRNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConFPRNode::reloc() const {
  return 2;
}

int loadConFPR0Node::reloc() const {
  return 2;
}

int loadConFPR1Node::reloc() const {
  return 2;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConF0Node::reloc() const {
  return 2;
}

int loadConDPRNode::reloc() const {
  return 2;
}

int loadConDPR0Node::reloc() const {
  return 2;
}

int loadConDPR1Node::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int loadConD0Node::reloc() const {
  return 2;
}

int prefetchAlloc0Node::reloc() const {
  return 1;
}

int prefetchAllocNode::reloc() const {
  return 1;
}

int prefetchAllocNTANode::reloc() const {
  return 1;
}

int prefetchAllocT0Node::reloc() const {
  return 1;
}

int prefetchAllocT2Node::reloc() const {
  return 1;
}

int storeBNode::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeL2INode::reloc() const {
  return 2;
}

int storeL_volatileNode::reloc() const {
  return 2;
}

int storeLX_volatileNode::reloc() const {
  return 2;
}

int storeLX_reg_volatileNode::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeImmINode::reloc() const {
  return 2;
}

int storeImmI16Node::reloc() const {
  return 2;
}

int storeImmPNode::reloc() const {
  return 3;
}

int storeImmBNode::reloc() const {
  return 2;
}

int storeImmCMNode::reloc() const {
  return 2;
}

int storeDPRNode::reloc() const {
  return 2;
}

int storeDPR_roundedNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeFPRNode::reloc() const {
  return 2;
}

int storeFPR_roundedNode::reloc() const {
  return 2;
}

int storeFPR_DroundedNode::reloc() const {
  return 2;
}

int storeFPR_immNode::reloc() const {
  return 3;
}

int storeF_immNode::reloc() const {
  return 3;
}

int cmovI_memNode::reloc() const {
  return 1;
}

int cmovI_memUNode::reloc() const {
  return 1;
}

int cmovI_memUCFNode::reloc() const {
  return 1;
}

int addI_eReg_memNode::reloc() const {
  return 1;
}

int addI_eReg_mem_0Node::reloc() const {
  return 1;
}

int addI_mem_eRegNode::reloc() const {
  return 3;
}

int addI_mem_eReg_0Node::reloc() const {
  return 3;
}

int addI_mem_immNode::reloc() const {
  return 3;
}

int incI_memNode::reloc() const {
  return 3;
}

int decI_memNode::reloc() const {
  return 3;
}

int loadPLockedNode::reloc() const {
  return 1;
}

int storePConditionalNode::reloc() const {
  return 1;
}

int storeIConditionalNode::reloc() const {
  return 1;
}

int storeLConditionalNode::reloc() const {
  return 1;
}

int xaddB_no_resNode::reloc() const {
  return 1;
}

int xaddBNode::reloc() const {
  return 1;
}

int xaddS_no_resNode::reloc() const {
  return 1;
}

int xaddSNode::reloc() const {
  return 1;
}

int xaddI_no_resNode::reloc() const {
  return 1;
}

int xaddINode::reloc() const {
  return 1;
}

int xchgBNode::reloc() const {
  return 1;
}

int xchgSNode::reloc() const {
  return 1;
}

int xchgINode::reloc() const {
  return 1;
}

int xchgPNode::reloc() const {
  return 1;
}

int subI_eReg_memNode::reloc() const {
  return 1;
}

int subI_mem_eRegNode::reloc() const {
  return 3;
}

int mulI_mem_immNode::reloc() const {
  return 1;
}

int mulINode::reloc() const {
  return 1;
}

int mulI_0Node::reloc() const {
  return 1;
}

int sarI_mem_1Node::reloc() const {
  return 3;
}

int sarI_mem_immNode::reloc() const {
  return 3;
}

int andI_eReg_memNode::reloc() const {
  return 1;
}

int andI_eReg_mem_0Node::reloc() const {
  return 1;
}

int andI_mem_eRegNode::reloc() const {
  return 3;
}

int andI_mem_eReg_0Node::reloc() const {
  return 3;
}

int andI_mem_immNode::reloc() const {
  return 3;
}

int andnI_rReg_rReg_memNode::reloc() const {
  return 1;
}

int andnI_rReg_rReg_mem_0Node::reloc() const {
  return 1;
}

int blsiI_rReg_memNode::reloc() const {
  return 2;
}

int blsiI_rReg_mem_0Node::reloc() const {
  return 2;
}

int blsmskI_rReg_memNode::reloc() const {
  return 2;
}

int blsmskI_rReg_mem_0Node::reloc() const {
  return 2;
}

int blsrI_rReg_memNode::reloc() const {
  return 2;
}

int blsrI_rReg_mem_0Node::reloc() const {
  return 2;
}

int orI_eReg_memNode::reloc() const {
  return 1;
}

int orI_eReg_mem_0Node::reloc() const {
  return 1;
}

int orI_mem_eRegNode::reloc() const {
  return 3;
}

int orI_mem_eReg_0Node::reloc() const {
  return 3;
}

int orI_mem_immNode::reloc() const {
  return 3;
}

int xorI_eReg_memNode::reloc() const {
  return 1;
}

int xorI_eReg_mem_0Node::reloc() const {
  return 1;
}

int xorI_mem_eRegNode::reloc() const {
  return 3;
}

int xorI_mem_eReg_0Node::reloc() const {
  return 3;
}

int xorI_mem_immNode::reloc() const {
  return 3;
}

int addL_eReg_memNode::reloc() const {
  return 1;
}

int addL_eReg_mem_0Node::reloc() const {
  return 1;
}

int subL_eReg_memNode::reloc() const {
  return 1;
}

int andL_eReg_memNode::reloc() const {
  return 1;
}

int andL_eReg_mem_0Node::reloc() const {
  return 1;
}

int andnL_eReg_eReg_memNode::reloc() const {
  return 1;
}

int andnL_eReg_eReg_mem_0Node::reloc() const {
  return 1;
}

int blsiL_eReg_memNode::reloc() const {
  return 2;
}

int blsiL_eReg_mem_0Node::reloc() const {
  return 2;
}

int blsmskL_eReg_memNode::reloc() const {
  return 2;
}

int blsmskL_eReg_mem_0Node::reloc() const {
  return 2;
}

int blsrL_eReg_memNode::reloc() const {
  return 2;
}

int blsrL_eReg_mem_0Node::reloc() const {
  return 2;
}

int orl_eReg_memNode::reloc() const {
  return 1;
}

int orl_eReg_mem_0Node::reloc() const {
  return 1;
}

int xorl_eReg_memNode::reloc() const {
  return 1;
}

int xorl_eReg_mem_0Node::reloc() const {
  return 1;
}

int cmpDPR_0Node::reloc() const {
  return 1;
}

int cmpD_ccmemNode::reloc() const {
  return 1;
}

int cmpD_ccmemCFNode::reloc() const {
  return 1;
}

int cmpD_regmemNode::reloc() const {
  return 1;
}

int subDPR_reg_memNode::reloc() const {
  return 1;
}

int addDPR_reg_memNode::reloc() const {
  return 1;
}

int addDPR_reg_mem_0Node::reloc() const {
  return 1;
}

int addDPR_mem_regNode::reloc() const {
  return 3;
}

int addDPR_mem_reg_0Node::reloc() const {
  return 3;
}

int addDPR_reg_imm1Node::reloc() const {
  return 1;
}

int addDPR_reg_immNode::reloc() const {
  return 1;
}

int addDPR_reg_imm_roundNode::reloc() const {
  return 1;
}

int mulDPR_reg_immNode::reloc() const {
  return 1;
}

int mulDPR_reg_memNode::reloc() const {
  return 1;
}

int mulDPR_reg_mem_0Node::reloc() const {
  return 1;
}

int mulDPR_reg_mem_ciscNode::reloc() const {
  return 1;
}

int mulDPR_reg_mem_cisc_0Node::reloc() const {
  return 1;
}

int cmpFPR_0Node::reloc() const {
  return 1;
}

int cmpF_ccmemNode::reloc() const {
  return 1;
}

int cmpF_ccmemCFNode::reloc() const {
  return 1;
}

int cmpF_regmemNode::reloc() const {
  return 1;
}

int addFPR24_reg_memNode::reloc() const {
  return 1;
}

int addFPR24_reg_mem_0Node::reloc() const {
  return 1;
}

int addFPR_reg_memNode::reloc() const {
  return 1;
}

int addFPR_reg_mem_0Node::reloc() const {
  return 1;
}

int addFPR24_mem_regNode::reloc() const {
  return 1;
}

int addFPR24_mem_ciscNode::reloc() const {
  return 2;
}

int addFPR24_mem_cisc_0Node::reloc() const {
  return 2;
}

int addFPR24_mem_memNode::reloc() const {
  return 2;
}

int addFPR24_reg_immNode::reloc() const {
  return 1;
}

int addFPR_reg_immNode::reloc() const {
  return 1;
}

int mulFPR24_reg_memNode::reloc() const {
  return 1;
}

int mulFPR24_reg_mem_0Node::reloc() const {
  return 1;
}

int mulFPR_reg_memNode::reloc() const {
  return 1;
}

int mulFPR_reg_mem_0Node::reloc() const {
  return 1;
}

int mulFPR24_mem_memNode::reloc() const {
  return 2;
}

int mulFPR24_reg_immNode::reloc() const {
  return 1;
}

int mulFPR_reg_immNode::reloc() const {
  return 1;
}

int mulFPR_reg_load1Node::reloc() const {
  return 1;
}

int mulFPR_reg_load1_0Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_1Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_0Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_2Node::reloc() const {
  return 1;
}

int convI2D_memNode::reloc() const {
  return 1;
}

int convI2DPR_memNode::reloc() const {
  return 1;
}

int convI2FPR_SSF_memNode::reloc() const {
  return 1;
}

int convI2FPR_memNode::reloc() const {
  return 1;
}

int compI_eReg_memNode::reloc() const {
  return 1;
}

int testI_reg_memNode::reloc() const {
  return 1;
}

int compU_eReg_memNode::reloc() const {
  return 1;
}

int compP_eReg_immNode::reloc() const {
  return 1;
}

int compP_eReg_memNode::reloc() const {
  return 1;
}

int compP_mem_eRegNode::reloc() const {
  return 1;
}

int testP_regNode::reloc() const {
  return 1;
}

int testP_Reg_memNode::reloc() const {
  return 1;
}

int partialSubtypeCheck_vs_ZeroNode::reloc() const {
  return 1;
}

int cmovLL_mem_LTGENode::reloc() const {
  return 1;
}

int cmovII_mem_LTGENode::reloc() const {
  return 1;
}

int cmovLL_mem_EQNENode::reloc() const {
  return 1;
}

int cmovII_mem_EQNENode::reloc() const {
  return 1;
}

int cmovLL_mem_LEGTNode::reloc() const {
  return 1;
}

int cmovII_mem_LEGTNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int tailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int safePoint_poll_tlsNode::reloc() const {
  return 1;
}

int addF_memNode::reloc() const {
  return 1;
}

int addF_mem_0Node::reloc() const {
  return 1;
}

int addF_immNode::reloc() const {
  return 1;
}

int addF_reg_memNode::reloc() const {
  return 1;
}

int addF_reg_mem_0Node::reloc() const {
  return 1;
}

int addF_reg_immNode::reloc() const {
  return 1;
}

int addD_memNode::reloc() const {
  return 1;
}

int addD_mem_0Node::reloc() const {
  return 1;
}

int addD_immNode::reloc() const {
  return 1;
}

int addD_reg_memNode::reloc() const {
  return 1;
}

int addD_reg_mem_0Node::reloc() const {
  return 1;
}

int addD_reg_immNode::reloc() const {
  return 1;
}

int subF_memNode::reloc() const {
  return 1;
}

int subF_immNode::reloc() const {
  return 1;
}

int subF_reg_memNode::reloc() const {
  return 1;
}

int subF_reg_immNode::reloc() const {
  return 1;
}

int subD_memNode::reloc() const {
  return 1;
}

int subD_immNode::reloc() const {
  return 1;
}

int subD_reg_memNode::reloc() const {
  return 1;
}

int subD_reg_immNode::reloc() const {
  return 1;
}

int mulF_memNode::reloc() const {
  return 1;
}

int mulF_mem_0Node::reloc() const {
  return 1;
}

int mulF_immNode::reloc() const {
  return 1;
}

int mulF_reg_memNode::reloc() const {
  return 1;
}

int mulF_reg_mem_0Node::reloc() const {
  return 1;
}

int mulF_reg_immNode::reloc() const {
  return 1;
}

int mulD_memNode::reloc() const {
  return 1;
}

int mulD_mem_0Node::reloc() const {
  return 1;
}

int mulD_immNode::reloc() const {
  return 1;
}

int mulD_reg_memNode::reloc() const {
  return 1;
}

int mulD_reg_mem_0Node::reloc() const {
  return 1;
}

int mulD_reg_immNode::reloc() const {
  return 1;
}

int divF_memNode::reloc() const {
  return 1;
}

int divF_immNode::reloc() const {
  return 1;
}

int divF_reg_memNode::reloc() const {
  return 1;
}

int divF_reg_immNode::reloc() const {
  return 1;
}

int divD_memNode::reloc() const {
  return 1;
}

int divD_immNode::reloc() const {
  return 1;
}

int divD_reg_memNode::reloc() const {
  return 1;
}

int divD_reg_immNode::reloc() const {
  return 1;
}

int sqrtF_memNode::reloc() const {
  return 1;
}

int sqrtF_immNode::reloc() const {
  return 1;
}

int sqrtD_memNode::reloc() const {
  return 1;
}

int sqrtD_immNode::reloc() const {
  return 1;
}

int loadVNode::reloc() const {
  return 1;
}

int storeVNode::reloc() const {
  return 2;
}

int ReplB_memNode::reloc() const {
  return 1;
}

int ReplS_memNode::reloc() const {
  return 1;
}

int ReplI_memNode::reloc() const {
  return 1;
}

int ReplL_memNode::reloc() const {
  return 1;
}

int ReplF_memNode::reloc() const {
  return 1;
}

int ReplF_zeroNode::reloc() const {
  return 1;
}

int ReplD_memNode::reloc() const {
  return 1;
}

int ReplD_zeroNode::reloc() const {
  return 1;
}

int vaddB_memNode::reloc() const {
  return 1;
}

int vaddB_mem_0Node::reloc() const {
  return 1;
}

int vaddS_memNode::reloc() const {
  return 1;
}

int vaddS_mem_0Node::reloc() const {
  return 1;
}

int vaddI_memNode::reloc() const {
  return 1;
}

int vaddI_mem_0Node::reloc() const {
  return 1;
}

int vaddL_memNode::reloc() const {
  return 1;
}

int vaddL_mem_0Node::reloc() const {
  return 1;
}

int vaddF_memNode::reloc() const {
  return 1;
}

int vaddF_mem_0Node::reloc() const {
  return 1;
}

int vaddD_memNode::reloc() const {
  return 1;
}

int vaddD_mem_0Node::reloc() const {
  return 1;
}

int vsubB_memNode::reloc() const {
  return 1;
}

int vsubS_memNode::reloc() const {
  return 1;
}

int vsubI_memNode::reloc() const {
  return 1;
}

int vsubL_memNode::reloc() const {
  return 1;
}

int vsubF_memNode::reloc() const {
  return 1;
}

int vsubD_memNode::reloc() const {
  return 1;
}

int vmulS_memNode::reloc() const {
  return 1;
}

int vmulS_mem_0Node::reloc() const {
  return 1;
}

int vmulI_memNode::reloc() const {
  return 1;
}

int vmulI_mem_0Node::reloc() const {
  return 1;
}

int vmulL_memNode::reloc() const {
  return 1;
}

int vmulL_mem_0Node::reloc() const {
  return 1;
}

int vmulF_memNode::reloc() const {
  return 1;
}

int vmulF_mem_0Node::reloc() const {
  return 1;
}

int vmulD_memNode::reloc() const {
  return 1;
}

int vmulD_mem_0Node::reloc() const {
  return 1;
}

int vdivF_memNode::reloc() const {
  return 1;
}

int vdivD_memNode::reloc() const {
  return 1;
}

int vsqrtF_memNode::reloc() const {
  return 1;
}

int vsqrtD_memNode::reloc() const {
  return 1;
}

int vand_memNode::reloc() const {
  return 1;
}

int vand_mem_0Node::reloc() const {
  return 1;
}

int vor_memNode::reloc() const {
  return 1;
}

int vor_mem_0Node::reloc() const {
  return 1;
}

int vxor_memNode::reloc() const {
  return 1;
}

int vxor_mem_0Node::reloc() const {
  return 1;
}

int vfmaF_memNode::reloc() const {
  return 1;
}

int vfmaD_memNode::reloc() const {
  return 1;
}

int vpternlog_memNode::reloc() const {
  return 1;
}

int compareAndSwapP_shenandoahNode::reloc() const {
  return 1;
}

int compareAndSwapP_shenandoah_0Node::reloc() const {
  return 1;
}

int compareAndExchangeP_shenandoahNode::reloc() const {
  return 1;
}


void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 13173 "ad_x86.cpp"
  }
  {

#line 1747 "src/hotspot/cpu/x86/x86_32.ad"

    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 13181 "ad_x86.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1751 "src/hotspot/cpu/x86/x86_32.ad"

    int destlo = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int desthi = HIGH_FROM_LOW(destlo);
    // bswap lo
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, destlo);
    // bswap hi
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, desthi);
    // xchg lo and hi
    emit_opcode(cbuf, 0x87);
    emit_rm(cbuf, 0x3, destlo, desthi);
  
#line 13206 "ad_x86.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5139 "src/hotspot/cpu/x86/x86_32.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ shrl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 13223 "ad_x86.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5152 "src/hotspot/cpu/x86/x86_32.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 13240 "ad_x86.cpp"
  }
}

void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5168 "src/hotspot/cpu/x86/x86_32.ad"

    __ lzcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 13256 "ad_x86.cpp"
  }
}

void countLeadingZerosI_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5185 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label skip;
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, skip);
    __ movl(Rdst, -1);
    __ bind(skip);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerInt - 1);
  
#line 13280 "ad_x86.cpp"
  }
}

void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5209 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label done;
    __ lzcntl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::carryClear, done);
    __ lzcntl(Rdst, Rsrc);
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 13304 "ad_x86.cpp"
  }
}

void countLeadingZerosL_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5238 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label msw_is_zero;
    Label not_zero;
    __ bsrl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::zero, msw_is_zero);
    __ addl(Rdst, BitsPerInt);
    __ jmpb(not_zero);
    __ bind(msw_is_zero);
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, not_zero);
    __ movl(Rdst, -1);
    __ bind(not_zero);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerLong - 1);
  
#line 13335 "ad_x86.cpp"
  }
}

void countTrailingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5264 "src/hotspot/cpu/x86/x86_32.ad"

    __ tzcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 13351 "ad_x86.cpp"
  }
}

void countTrailingZerosI_bsfNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5279 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ bsfl(Rdst, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ jccb(Assembler::notZero, done);
    __ movl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 13372 "ad_x86.cpp"
  }
}

void countTrailingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5300 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label done;
    __ tzcntl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ tzcntl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 13396 "ad_x86.cpp"
  }
}

void countTrailingZerosL_bsfNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5326 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label msw_not_zero;
    Label done;
    __ bsfl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ bsfl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::notZero, msw_not_zero);
    __ movl(Rdst, BitsPerInt);
    __ bind(msw_not_zero);
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 13424 "ad_x86.cpp"
  }
}

void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5352 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 13440 "ad_x86.cpp"
  }
}

void popCountI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5364 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13456 "ad_x86.cpp"
  }
}

void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5379 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 13476 "ad_x86.cpp"
  }
}

void popCountL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5396 "src/hotspot/cpu/x86/x86_32.ad"

    //__ popcntl($dst$$Register, $mem$$Address$$first);
    //__ popcntl($tmp$$Register, $mem$$Address$$second);
    __ popcntl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), relocInfo::none));
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none));
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 13498 "ad_x86.cpp"
  }
}

void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5416 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13514 "ad_x86.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5433 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 7); // 24+1 MSB are already signed extended.
  
#line 13532 "ad_x86.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5449 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13548 "ad_x86.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5465 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 13566 "ad_x86.cpp"
  }
}

void loadUB2L_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5482 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant()& right_n_bits(8));
  
#line 13586 "ad_x86.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5498 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13602 "ad_x86.cpp"
  }
}

void loadS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5511 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13619 "ad_x86.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5527 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 15); // 16+1 MSB are already signed extended.
  
#line 13637 "ad_x86.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5543 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13653 "ad_x86.cpp"
  }
}

void loadUS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5556 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13670 "ad_x86.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5571 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */));
  
#line 13687 "ad_x86.cpp"
  }
}

void loadUS2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5586 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 13706 "ad_x86.cpp"
  }
}

void loadUS2L_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5602 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzwl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant()& right_n_bits(16));
  
#line 13726 "ad_x86.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5618 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13742 "ad_x86.cpp"
  }
}

void loadI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5631 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13759 "ad_x86.cpp"
  }
}

void loadI2UBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5643 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13776 "ad_x86.cpp"
  }
}

void loadI2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5655 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13793 "ad_x86.cpp"
  }
}

void loadI2USNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5667 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 13810 "ad_x86.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5683 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 31);
  
#line 13828 "ad_x86.cpp"
  }
}

void loadI2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5699 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 13847 "ad_x86.cpp"
  }
}

void loadI2L_immI_65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5714 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzwl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 13866 "ad_x86.cpp"
  }
}

void loadI2L_immU31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5730 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 13886 "ad_x86.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5748 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */));
  
#line 13904 "ad_x86.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5766 "src/hotspot/cpu/x86/x86_32.ad"

    Address Amemlo = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), relocInfo::none);
    Address Amemhi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Amemlo);
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), Amemhi);
  
#line 13923 "ad_x86.cpp"
  }
}

void loadL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3107 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x05;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
    store_to_stackslot( cbuf, 0x0DF, 0x07, opnd_array(0)->disp(ra_,this,0) );
  
#line 13946 "ad_x86.cpp"
  }
}

void loadLX_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5797 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 13964 "ad_x86.cpp"
  }
}

void loadLX_reg_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5813 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
    __ psrlq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, 32);
    __ movdl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 13984 "ad_x86.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 13999 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14013 "ad_x86.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14028 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14042 "ad_x86.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14057 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14071 "ad_x86.cpp"
  }
}

void loadDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 14086 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 14100 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 14109 "ad_x86.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5876 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14125 "ad_x86.cpp"
  }
}

void loadD_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5887 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14141 "ad_x86.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5900 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14157 "ad_x86.cpp"
  }
}

void loadFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 14172 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 14186 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 14195 "ad_x86.cpp"
  }
}

void leaP8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 14210 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14224 "ad_x86.cpp"
  }
}

void leaP32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 14239 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14253 "ad_x86.cpp"
  }
}

void leaPIdxOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 14268 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14282 "ad_x86.cpp"
  }
}

void leaPIdxScaleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 14297 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14311 "ad_x86.cpp"
  }
}

void leaPIdxScaleOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 14326 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14340 "ad_x86.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32(cbuf, opnd_array(1)->constant());
  
#line 14358 "ad_x86.cpp"
  }
}

void loadConI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 14373 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14381 "ad_x86.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1966 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, (0xB8 /*primary()*/) + opnd_array(0)->reg(ra_,this)/* dst */);
    
if ( opnd_array(1)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(1)->constant(), opnd_array(1)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(1)->constant());
};
  
#line 14404 "ad_x86.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1973 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 14430 "ad_x86.cpp"
  }
  {

#line 1988 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */ + 2;
    int src_con = ((julong)(opnd_array(1)->constantL())) >> 32;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 14449 "ad_x86.cpp"
  }
}

void loadConL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2017 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14465 "ad_x86.cpp"
  }
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 14474 "ad_x86.cpp"
  }
}

void loadConFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6029 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14491 "ad_x86.cpp"
  }
}

void loadConFPRNode::eval_constant(Compile* C) {
  {

#line 6030 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 14500 "ad_x86.cpp"
  }
}
void loadConFPR0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6042 "src/hotspot/cpu/x86/x86_32.ad"

    __ fldz();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14516 "ad_x86.cpp"
  }
}

void loadConFPR1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6055 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14533 "ad_x86.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6067 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 14549 "ad_x86.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 6068 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 14558 "ad_x86.cpp"
  }
}
void loadConF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6078 "src/hotspot/cpu/x86/x86_32.ad"

    __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 14573 "ad_x86.cpp"
  }
}

void loadConDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6091 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14590 "ad_x86.cpp"
  }
}

void loadConDPRNode::eval_constant(Compile* C) {
  {

#line 6092 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 14599 "ad_x86.cpp"
  }
}
void loadConDPR0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6105 "src/hotspot/cpu/x86/x86_32.ad"

    __ fldz();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14615 "ad_x86.cpp"
  }
}

void loadConDPR1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6119 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 14632 "ad_x86.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6131 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 14648 "ad_x86.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 6132 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 14657 "ad_x86.cpp"
  }
}
void loadConD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6142 "src/hotspot/cpu/x86/x86_32.ad"

    __ xorpd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 14672 "ad_x86.cpp"
  }
}

void loadSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14687 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14701 "ad_x86.cpp"
  }
}

void loadSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14716 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14730 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*secondary()*/));
  
#line 14738 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 14752 "ad_x86.cpp"
  }
}

void loadSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14767 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14781 "ad_x86.cpp"
  }
}

void loadSSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 14796 "ad_x86.cpp"
  }
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 14810 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 14819 "ad_x86.cpp"
  }
}

void loadSSDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 14834 "ad_x86.cpp"
  }
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 14848 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 14857 "ad_x86.cpp"
  }
}

void prefetchAlloc0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  // User did not define which encode class to use.
}

uint prefetchAlloc0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void prefetchAllocNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6226 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetchw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14886 "ad_x86.cpp"
  }
}

void prefetchAllocNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6238 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetchnta(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14902 "ad_x86.cpp"
  }
}

void prefetchAllocT0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6250 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetcht0(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14918 "ad_x86.cpp"
  }
}

void prefetchAllocT2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6262 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetcht2(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 14934 "ad_x86.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x88 /*primary()*/));
  
#line 14950 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 14964 "ad_x86.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x66 /*secondary()*/));
  
#line 14980 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 14988 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15002 "ad_x86.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 15018 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15032 "ad_x86.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 15048 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15062 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 15070 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 15084 "ad_x86.cpp"
  }
}

void storeL2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6321 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 15101 "ad_x86.cpp"
  }
}

void storeL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 15117 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = (EAX_enc);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15131 "ad_x86.cpp"
  }
  {

#line 3123 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0x0DF, 0x05, opnd_array(2)->disp(ra_,this,idx2) );
    cbuf.set_insts_mark();            // Mark start of FIST in case $mem has an oop
    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x07;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15148 "ad_x86.cpp"
  }
}

void storeLX_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6352 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(rax, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, Address(rsp, opnd_array(2)->disp(ra_,this,idx2)));
    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 15168 "ad_x86.cpp"
  }
}

void storeLX_reg_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6370 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(rax, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdl(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp2 */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src */));
    __ punpckldq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp2 */);
    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
  
#line 15191 "ad_x86.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 15207 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15221 "ad_x86.cpp"
  }
}

void storeImmINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15237 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15251 "ad_x86.cpp"
  }
  {

#line 2039 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 15260 "ad_x86.cpp"
  }
}

void storeImmI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1626 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0x66);
  
#line 15276 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15284 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15298 "ad_x86.cpp"
  }
  {

#line 2058 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d16(cbuf, opnd_array(2)->constant());
  
#line 15307 "ad_x86.cpp"
  }
}

void storeImmPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15323 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15337 "ad_x86.cpp"
  }
  {

#line 2039 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    
if ( opnd_array(2)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), opnd_array(2)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
  
#line 15351 "ad_x86.cpp"
  }
}

void storeImmBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 15367 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15381 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 15396 "ad_x86.cpp"
  }
}

void storeImmCMNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 15412 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15426 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 15441 "ad_x86.cpp"
  }
}

void storeDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2284 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15472 "ad_x86.cpp"
  }
}

void storeDPR_roundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2284 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15503 "ad_x86.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6479 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 15520 "ad_x86.cpp"
  }
}

void MoveD2VLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6489 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15536 "ad_x86.cpp"
  }
}

void MoveVL2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6499 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15552 "ad_x86.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6512 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 15569 "ad_x86.cpp"
  }
}

void MoveF2VLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6522 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15585 "ad_x86.cpp"
  }
}

void MoveVL2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6532 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15601 "ad_x86.cpp"
  }
}

void storeFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2284 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15632 "ad_x86.cpp"
  }
}

void storeFPR_roundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2284 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15663 "ad_x86.cpp"
  }
}

void storeFPR_DroundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2284 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15694 "ad_x86.cpp"
  }
}

void storeFPR_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15710 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15724 "ad_x86.cpp"
  }
  {

#line 2044 "src/hotspot/cpu/x86/x86_32.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  
#line 15735 "ad_x86.cpp"
  }
}

void storeF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15751 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 15765 "ad_x86.cpp"
  }
  {

#line 2051 "src/hotspot/cpu/x86/x86_32.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  
#line 15776 "ad_x86.cpp"
  }
}

void storeSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2448 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(0)->disp(ra_,this,0) );
  
#line 15791 "ad_x86.cpp"
  }
}

void storeSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2448 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(0)->disp(ra_,this,0) );
  
#line 15806 "ad_x86.cpp"
  }
}

void storeSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 15821 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    relocInfo::relocType disp_reloc = opnd_array(0)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 15835 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 15843 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( opnd_array(0)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 15857 "ad_x86.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquireNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_releaseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_release_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6686 "src/hotspot/cpu/x86/x86_32.ad"

    __ membar(Assembler::StoreLoad);
  
#line 15951 "ad_x86.cpp"
  }
}

void unnecessary_membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_storestoreNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // User did not define which encode class to use.
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2005 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 16000 "ad_x86.cpp"
  }
}

void jmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6739 "src/hotspot/cpu/x86/x86_32.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 16023 "ad_x86.cpp"
  }
}

void jmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6756 "src/hotspot/cpu/x86/x86_32.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 16046 "ad_x86.cpp"
  }
}

void cmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16065 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 16073 "ad_x86.cpp"
  }
}

void cmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16092 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 16100 "ad_x86.cpp"
  }
}

void cmovI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16119 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16133 "ad_x86.cpp"
  }
}

void cmovI_memUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16152 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16166 "ad_x86.cpp"
  }
}

void cmovP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16185 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 16193 "ad_x86.cpp"
  }
}

void cmovP_reg_nonP6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1781 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x2) );
  
#line 16213 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8b /*primary()*/));
  
#line 16221 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 16229 "ad_x86.cpp"
  }
}

void cmovP_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16248 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 16256 "ad_x86.cpp"
  }
}

void fcmovDPR_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1774 "src/hotspot/cpu/x86/x86_32.ad"

    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)/* src */-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  
#line 16276 "ad_x86.cpp"
  }
}

void fcmovFPR_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1774 "src/hotspot/cpu/x86/x86_32.ad"

    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)/* src */-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  
#line 16296 "ad_x86.cpp"
  }
}

void fcmovDPR_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1781 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  
#line 16316 "ad_x86.cpp"
  }
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4)/* src */ );   // FLD ST(i-1)
  
#line 16325 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xdd /*primary()*/));
  
#line 16333 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3)/* dst */ );
  
#line 16341 "ad_x86.cpp"
  }
}

void fcmovFPR_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1781 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  
#line 16361 "ad_x86.cpp"
  }
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4)/* src */ );
  
#line 16370 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xdd /*primary()*/));
  
#line 16378 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3)/* dst */ );
  
#line 16386 "ad_x86.cpp"
  }
}

void fcmovF_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6955 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 16409 "ad_x86.cpp"
  }
}

void fcmovD_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6973 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 16432 "ad_x86.cpp"
  }
}

void fcmovF_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6991 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 16455 "ad_x86.cpp"
  }
}

void fcmovD_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7018 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 16478 "ad_x86.cpp"
  }
}

void cmovL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16497 "ad_x86.cpp"
  }
  {

#line 2027 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 16505 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16514 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 16522 "ad_x86.cpp"
  }
}

void cmovL_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16541 "ad_x86.cpp"
  }
  {

#line 2027 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 16549 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16558 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 16566 "ad_x86.cpp"
  }
}

void addI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 16582 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 16590 "ad_x86.cpp"
  }
}

uint addI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void addI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 16619 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 16634 "ad_x86.cpp"
  }
}

void incI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1686 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x40 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 16650 "ad_x86.cpp"
  }
}

uint incI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 1);
}

void leaI_eReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 16671 "ad_x86.cpp"
  }
  {

#line 2250 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    relocInfo::relocType disp_reloc = relocInfo::none;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16685 "ad_x86.cpp"
  }
}

void leaP_eReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 16701 "ad_x86.cpp"
  }
  {

#line 2250 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    relocInfo::relocType disp_reloc = relocInfo::none;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16715 "ad_x86.cpp"
  }
}

void decI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1686 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x48 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 16731 "ad_x86.cpp"
  }
}

uint decI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 1);
}

void addP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 16752 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 16760 "ad_x86.cpp"
  }
}

uint addP_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void addP_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 16789 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 16804 "ad_x86.cpp"
  }
}

void addI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 16820 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16834 "ad_x86.cpp"
  }
}

void addI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 16850 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16864 "ad_x86.cpp"
  }
}

void addI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 16880 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16894 "ad_x86.cpp"
  }
}

void addI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 16910 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16924 "ad_x86.cpp"
  }
}

void addI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1692 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 16946 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 16960 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 16975 "ad_x86.cpp"
  }
}

void incI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 16991 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 17005 "ad_x86.cpp"
  }
}

void decI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 17021 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x01);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 17035 "ad_x86.cpp"
  }
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 17079 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17093 "ad_x86.cpp"
  }
}

void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// heap_top_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2073 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xF0);         // [Lock]
  
#line 17110 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 17118 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xB1));
  
#line 17126 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17140 "ad_x86.cpp"
  }
}

void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2073 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xF0);         // [Lock]
  
#line 17157 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 17165 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xB1));
  
#line 17173 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17187 "ad_x86.cpp"
  }
}

void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7282 "src/hotspot/cpu/x86/x86_32.ad"

    // Note: we need to swap rbx, and rcx before and after the
    //       cmpxchg8 instruction because the instruction uses
    //       rcx as the high order word of the new value to store but
    //       our register encoding uses rbx.
    __ xchgl(as_Register(EBX_enc), as_Register(ECX_enc));
    __ lock();
    __ cmpxchg8(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xchgl(as_Register(EBX_enc), as_Register(ECX_enc));
  
#line 17212 "ad_x86.cpp"
  }
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2082 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 17241 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17260 "ad_x86.cpp"
  }
}

void compareAndSwapL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2082 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 17289 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17308 "ad_x86.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2098 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17331 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17350 "ad_x86.cpp"
  }
}

void compareAndSwapP_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2098 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17373 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17392 "ad_x86.cpp"
  }
}

void compareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2108 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17415 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17434 "ad_x86.cpp"
  }
}

void compareAndSwapB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2108 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17457 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17476 "ad_x86.cpp"
  }
}

void compareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2118 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17502 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17521 "ad_x86.cpp"
  }
}

void compareAndSwapS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2118 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17547 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17566 "ad_x86.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2098 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17589 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17608 "ad_x86.cpp"
  }
}

void compareAndSwapI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2098 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17631 "ad_x86.cpp"
  }
  {

#line 2131 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 17650 "ad_x86.cpp"
  }
}

void compareAndExchangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2082 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 17679 "ad_x86.cpp"
  }
}

void compareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2098 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17702 "ad_x86.cpp"
  }
}

void compareAndExchangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2108 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17725 "ad_x86.cpp"
  }
}

void compareAndExchangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2118 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17751 "ad_x86.cpp"
  }
}

void compareAndExchangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2098 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 17774 "ad_x86.cpp"
  }
}

void xaddB_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7412 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 17792 "ad_x86.cpp"
  }
}

void xaddBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7424 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 17810 "ad_x86.cpp"
  }
}

void xaddS_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7436 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 17828 "ad_x86.cpp"
  }
}

void xaddSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7447 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 17846 "ad_x86.cpp"
  }
}

void xaddI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7459 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 17864 "ad_x86.cpp"
  }
}

void xaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7470 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 17882 "ad_x86.cpp"
  }
}

void xchgBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7481 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgb(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17899 "ad_x86.cpp"
  }
}

void xchgSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7490 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgw(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17916 "ad_x86.cpp"
  }
}

void xchgINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7499 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17933 "ad_x86.cpp"
  }
}

void xchgPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7508 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17950 "ad_x86.cpp"
  }
}

void subI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 17966 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 17974 "ad_x86.cpp"
  }
}

uint subI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void subI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 18003 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18018 "ad_x86.cpp"
  }
}

void subI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 18034 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18048 "ad_x86.cpp"
  }
}

void subI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x29 /*primary()*/));
  
#line 18064 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18078 "ad_x86.cpp"
  }
}

void subP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 18095 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(3)->reg(ra_,this,idx3)/* src */);
  
#line 18103 "ad_x86.cpp"
  }
}

uint subP_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void negI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18124 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 18132 "ad_x86.cpp"
  }
}

uint negI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void mulI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 18153 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 18161 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 18169 "ad_x86.cpp"
  }
}

uint mulI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void mulI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 1692 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 18196 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 18204 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18219 "ad_x86.cpp"
  }
}

void loadConL_low_onlyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1973 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 18245 "ad_x86.cpp"
  }
}

void mulI_imm_highNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2772 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 18271 "ad_x86.cpp"
  }
}

void mulI_imm_high_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2772 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 18297 "ad_x86.cpp"
  }
}

void mulI_imm_RShift_highNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2772 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 18323 "ad_x86.cpp"
  }
}

void mulI_imm_RShift_high_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2772 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 18349 "ad_x86.cpp"
  }
}

void mulI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 1692 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 18371 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18385 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18400 "ad_x86.cpp"
  }
}

void mulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 18416 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 18424 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18438 "ad_x86.cpp"
  }
}

void mulI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 18454 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 18462 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18476 "ad_x86.cpp"
  }
}

void mulI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2798 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 18495 "ad_x86.cpp"
  }
}

void mulI2L_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2798 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 18514 "ad_x86.cpp"
  }
}

void mulIS_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2805 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 18534 "ad_x86.cpp"
  }
}

void mulIS_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  {

#line 2805 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 18554 "ad_x86.cpp"
  }
}

void mulL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2812 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: lo(result) = lo(x_lo * y_lo)
    //             hi(result) = hi(x_lo * y_lo) + lo(x_hi * y_lo) + lo(x_lo * y_hi)
    // MOV    $tmp,$src.lo
    encode_Copy( cbuf, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    // IMUL   $tmp,EDX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // MOV    EDX,$src.hi
    encode_Copy( cbuf, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */) );
    // IMUL   EDX,EAX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    // ADD    $tmp,EDX
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // MUL   EDX:EAX,$src.lo
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(3)->reg(ra_,this,idx3)/* tmp */ );
  
#line 18593 "ad_x86.cpp"
  }
}

void mulL_eReg_lhi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7744 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src */));
    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, rax);
    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ addl(rdx, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 18614 "ad_x86.cpp"
  }
}

void mulL_eReg_rhi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7765 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, rdx);
    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ addl(rdx, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 18635 "ad_x86.cpp"
  }
}

void mulL_eReg_hi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7783 "src/hotspot/cpu/x86/x86_32.ad"

    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 18652 "ad_x86.cpp"
  }
}

void mulL_eReg_conNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2838 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: lo(result) = lo(src * y_lo)
    //             hi(result) = hi(src * y_lo) + lo(src * y_hi)
    // IMUL   $tmp,EDX,$src
    emit_opcode( cbuf, 0x6B );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    emit_d8( cbuf, (int)opnd_array(2)->constantL() );
    // MOV    EDX,$src
    emit_opcode(cbuf, 0xB8 + EDX_enc);
    emit_d32( cbuf, (int)opnd_array(2)->constantL() );
    // MUL   EDX:EAX,EDX
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, EDX_enc );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, EDX_enc, opnd_array(3)->reg(ra_,this,idx3)/* tmp */ );
  
#line 18683 "ad_x86.cpp"
  }
}

uint mulL_eReg_conNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void divI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1644 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 18741 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18749 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 18757 "ad_x86.cpp"
  }
}

uint divI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void divL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2856 "src/hotspot/cpu/x86/x86_32.ad"

    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */  );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::ldiv) - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  
#line 18793 "ad_x86.cpp"
  }
}

void divModI_eReg_divmodNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// div
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rdx
  {

#line 1644 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 18846 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18854 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 18862 "ad_x86.cpp"
  }
}

uint divModI_eReg_divmodNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void modI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1644 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 18920 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18928 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 18936 "ad_x86.cpp"
  }
}

uint modI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void modL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2875 "src/hotspot/cpu/x86/x86_32.ad"

    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */  );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::lrem ) - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  
#line 18972 "ad_x86.cpp"
  }
}

void divL_eReg_imm32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7914 "src/hotspot/cpu/x86/x86_32.ad"

    int con = (int)opnd_array(2)->constantL();
    assert(con != 0 && con != -1 && con != min_jint, "wrong divisor");
    int pcon = (con > 0) ? con : -con;
    Label Lfast, Lpos, Ldone;

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, pcon);
    __ xorl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */,opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jccb(Assembler::above, Lfast); // result fits into 32 bit

    __ movl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */); // save
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),0); // preserve flags
    __ jccb(Assembler::lessEqual, Lpos); // result is positive

    // Negative dividend.
    // convert value to positive to use unsigned division
    __ lneg(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ xchgl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    // revert result back to negative
    __ lneg(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ jmpb(Ldone);

    __ bind(Lpos);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */); // Use unsigned division
    __ xchgl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    // Fallthrow for final divide, tmp2 has 32 bit hi result

    __ bind(Lfast);
    // fast path: src is positive
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */); // Use unsigned division

    __ bind(Ldone);
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    if (con < 0) {
      __ lneg(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */), opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    }
  
#line 19029 "ad_x86.cpp"
  }
}

void modL_eReg_imm32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7984 "src/hotspot/cpu/x86/x86_32.ad"

    int con = (int)opnd_array(2)->constantL();
    assert(con != 0 && con != -1 && con != min_jint, "wrong divisor");
    int pcon = (con > 0) ? con : -con;
    Label  Lfast, Lpos, Ldone;

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, pcon);
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jccb(Assembler::above, Lfast); // src is positive and result fits into 32 bit

    __ movl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */); // save
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),0); // preserve flags
    __ jccb(Assembler::lessEqual, Lpos); // result is positive

    // Negative dividend.
    // convert value to positive to use unsigned division
    __ lneg(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    // revert remainder back to negative
    __ negl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jmpb(Ldone);

    __ bind(Lpos);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);

    __ bind(Lfast);
    // fast path: src is positive
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);

    __ bind(Ldone);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ sarl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */), 31); // result sign

  
#line 19083 "ad_x86.cpp"
  }
}

void shlI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 19099 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 19107 "ad_x86.cpp"
  }
}

uint shlI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void salI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1953 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 19130 "ad_x86.cpp"
  }
}

uint salI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void salI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 19151 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 19159 "ad_x86.cpp"
  }
}

uint salI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void sarI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 19180 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 19188 "ad_x86.cpp"
  }
}

uint sarI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void sarI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 19209 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 19223 "ad_x86.cpp"
  }
}

void sarI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1953 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 19241 "ad_x86.cpp"
  }
}

uint sarI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void sarI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 19262 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 19276 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19291 "ad_x86.cpp"
  }
}

void sarI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 19307 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 19315 "ad_x86.cpp"
  }
}

uint sarI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void shrI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 19336 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 19344 "ad_x86.cpp"
  }
}

uint shrI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void shrI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1953 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 19367 "ad_x86.cpp"
  }
}

uint shrI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void i2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8151 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19389 "ad_x86.cpp"
  }
}

uint i2bNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void i2sNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8164 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19411 "ad_x86.cpp"
  }
}

uint i2sNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void shrI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 19432 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 19440 "ad_x86.cpp"
  }
}

uint shrI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void andI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 19461 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 19469 "ad_x86.cpp"
  }
}

uint andI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void andI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 19498 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19513 "ad_x86.cpp"
  }
}

void andI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 19529 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19543 "ad_x86.cpp"
  }
}

void andI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 19559 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19573 "ad_x86.cpp"
  }
}

void andI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 19589 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19603 "ad_x86.cpp"
  }
}

void andI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 19619 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19633 "ad_x86.cpp"
  }
}

void andI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1692 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 19655 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 19669 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19684 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8256 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* src2 */);
  
#line 19702 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8256 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */, opnd_array(1)->as_Register(ra_,this,idx1)/* src2 */);
  
#line 19720 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8270 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()));
  
#line 19738 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8270 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 19756 "ad_x86.cpp"
  }
}

void blsiI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8283 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 19773 "ad_x86.cpp"
  }
}

void blsiI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8283 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19790 "ad_x86.cpp"
  }
}

void blsiI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8297 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 19807 "ad_x86.cpp"
  }
}

void blsiI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8297 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 19824 "ad_x86.cpp"
  }
}

void blsmskI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8311 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19841 "ad_x86.cpp"
  }
}

void blsmskI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8311 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19858 "ad_x86.cpp"
  }
}

void blsmskI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8327 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 19875 "ad_x86.cpp"
  }
}

void blsmskI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8327 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 19892 "ad_x86.cpp"
  }
}

void blsrI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8342 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19909 "ad_x86.cpp"
  }
}

void blsrI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8342 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19926 "ad_x86.cpp"
  }
}

void blsrI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8358 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 19943 "ad_x86.cpp"
  }
}

void blsrI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8358 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 19960 "ad_x86.cpp"
  }
}

void orI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 19976 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 19984 "ad_x86.cpp"
  }
}

uint orI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 20005 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 20013 "ad_x86.cpp"
  }
}

uint orI_eReg_castP2XNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_castP2X_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 20034 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 20042 "ad_x86.cpp"
  }
}

uint orI_eReg_castP2X_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 20071 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20086 "ad_x86.cpp"
  }
}

void orI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 20102 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20116 "ad_x86.cpp"
  }
}

void orI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 20132 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20146 "ad_x86.cpp"
  }
}

void orI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 20162 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20176 "ad_x86.cpp"
  }
}

void orI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 20192 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20206 "ad_x86.cpp"
  }
}

void orI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1692 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 20228 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 20242 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20257 "ad_x86.cpp"
  }
}

void rolI_eReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20273 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20281 "ad_x86.cpp"
  }
}

void rolI_eReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1953 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20299 "ad_x86.cpp"
  }
}

void rolI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20315 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20323 "ad_x86.cpp"
  }
}

void rorI_eReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20339 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20347 "ad_x86.cpp"
  }
}

void rorI_eReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1953 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20365 "ad_x86.cpp"
  }
}

void rorI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20381 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20389 "ad_x86.cpp"
  }
}

void xorI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 20405 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 20413 "ad_x86.cpp"
  }
}

uint xorI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void xorI_eReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8591 "src/hotspot/cpu/x86/x86_32.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
  
#line 20435 "ad_x86.cpp"
  }
}

uint xorI_eReg_im1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void xorI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 20464 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20479 "ad_x86.cpp"
  }
}

void xorI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 20495 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20509 "ad_x86.cpp"
  }
}

void xorI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 20525 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20539 "ad_x86.cpp"
  }
}

void xorI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 20555 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20569 "ad_x86.cpp"
  }
}

void xorI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 20585 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20599 "ad_x86.cpp"
  }
}

void xorI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1692 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 20621 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x6 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 20635 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20650 "ad_x86.cpp"
  }
}

void movI_nocopyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2005 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 20665 "ad_x86.cpp"
  }
}

void ci2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2303 "src/hotspot/cpu/x86/x86_32.ad"

    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20683 "ad_x86.cpp"
  }
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 20692 "ad_x86.cpp"
  }
}

uint ci2bNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void movP_nocopyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2005 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 20712 "ad_x86.cpp"
  }
}

void cp2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2303 "src/hotspot/cpu/x86/x86_32.ad"

    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 20730 "ad_x86.cpp"
  }
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 20739 "ad_x86.cpp"
  }
}

void cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8709 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Rd = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ xorl(Rd, Rd);
    __ cmpl(Rp, Rq);
    __ setb(Assembler::less, Rd);
    __ negl(Rd);
  
#line 20763 "ad_x86.cpp"
  }
}

void cmpLTMask0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8729 "src/hotspot/cpu/x86/x86_32.ad"

  __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 31);
  
#line 20780 "ad_x86.cpp"
  }
}

void cadd_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8744 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 20805 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8744 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 20830 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8744 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 20855 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8744 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 20880 "ad_x86.cpp"
  }
}

void and_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8769 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 20905 "ad_x86.cpp"
  }
}

void and_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8769 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 20930 "ad_x86.cpp"
  }
}

void overflowAddI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8796 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 20947 "ad_x86.cpp"
  }
}

void overflowAddI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8809 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 20964 "ad_x86.cpp"
  }
}

void overflowSubI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8820 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 20981 "ad_x86.cpp"
  }
}

void overflowSubI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8831 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 20998 "ad_x86.cpp"
  }
}

void overflowNegI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8843 "src/hotspot/cpu/x86/x86_32.ad"

    __ negl(opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 21015 "ad_x86.cpp"
  }
}

void overflowMulI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8855 "src/hotspot/cpu/x86/x86_32.ad"

    __ imull(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 21032 "ad_x86.cpp"
  }
}

void overflowMulI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8867 "src/hotspot/cpu/x86/x86_32.ad"

    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 21050 "ad_x86.cpp"
  }
}

void absI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8884 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ sarl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, 31);
    __ movl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ xorl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ subl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 21072 "ad_x86.cpp"
  }
}

void addL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2017 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x03 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21089 "ad_x86.cpp"
  }
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x13 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 21098 "ad_x86.cpp"
  }
}

void addL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1725 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 21121 "ad_x86.cpp"
  }
  {

#line 1736 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x02 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 21136 "ad_x86.cpp"
  }
}

void addL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 21152 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21166 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x13 /*secondary()*/));
  
#line 21174 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 21188 "ad_x86.cpp"
  }
}

void addL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 21204 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21218 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x13 /*secondary()*/));
  
#line 21226 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 21240 "ad_x86.cpp"
  }
}

void subL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2017 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x2B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21257 "ad_x86.cpp"
  }
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x1B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 21266 "ad_x86.cpp"
  }
}

void subL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1725 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 21289 "ad_x86.cpp"
  }
  {

#line 1736 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x03 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 21304 "ad_x86.cpp"
  }
}

void subL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 21320 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21334 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x1B /*secondary()*/));
  
#line 21342 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 21356 "ad_x86.cpp"
  }
}

void negL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2940 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xF7);    // NEG hi
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */));
    emit_opcode(cbuf,0xF7);    // NEG lo
    emit_rm    (cbuf,0x3, 0x3,               opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
    emit_opcode(cbuf,0x83);    // SBB hi,0
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */));
    emit_d8    (cbuf,0 );
  
#line 21378 "ad_x86.cpp"
  }
}

void andL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2017 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x23 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21395 "ad_x86.cpp"
  }
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x23 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 21404 "ad_x86.cpp"
  }
}

void andL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1725 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 21427 "ad_x86.cpp"
  }
  {

#line 1736 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 21442 "ad_x86.cpp"
  }
}

void andL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 21458 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21472 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*secondary()*/));
  
#line 21480 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 21494 "ad_x86.cpp"
  }
}

void andL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 21510 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21524 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*secondary()*/));
  
#line 21532 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 21546 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9019 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */;
    Register Rsrc2 = opnd_array(3)->as_Register(ra_,this,idx3)/* src2 */;
    __ andnl(Rdst, Rsrc1, Rsrc2);
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), HIGH_FROM_LOW(Rsrc2));
  
#line 21569 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9019 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */;
    Register Rsrc2 = opnd_array(1)->as_Register(ra_,this,idx1)/* src2 */;
    __ andnl(Rdst, Rsrc1, Rsrc2);
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), HIGH_FROM_LOW(Rsrc2));
  
#line 21592 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9039 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */;
    Address src2_hi = Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3)+ 4, relocInfo::none);

    __ andnl(Rdst, Rsrc1, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()));
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), src2_hi);
  
#line 21616 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9039 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */;
    Address src2_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ andnl(Rdst, Rsrc1, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), src2_hi);
  
#line 21640 "ad_x86.cpp"
  }
}

void blsiL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9062 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(2)->as_Register(ra_,this,idx2)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 21665 "ad_x86.cpp"
  }
}

void blsiL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9062 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 21690 "ad_x86.cpp"
  }
}

void blsiL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9088 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 21716 "ad_x86.cpp"
  }
}

void blsiL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9088 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 21742 "ad_x86.cpp"
  }
}

void blsmskL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9115 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 21767 "ad_x86.cpp"
  }
}

void blsmskL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9115 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 21792 "ad_x86.cpp"
  }
}

void blsmskL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9143 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 21818 "ad_x86.cpp"
  }
}

void blsmskL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9143 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 21844 "ad_x86.cpp"
  }
}

void blsrL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9171 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ blsrl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 21869 "ad_x86.cpp"
  }
}

void blsrL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9171 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ blsrl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 21894 "ad_x86.cpp"
  }
}

void blsrL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9199 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(HIGH_FROM_LOW(Rdst), src_hi);
    __ blsrl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 21919 "ad_x86.cpp"
  }
}

void blsrL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9199 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(HIGH_FROM_LOW(Rdst), src_hi);
    __ blsrl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 21944 "ad_x86.cpp"
  }
}

void orl_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2017 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 21961 "ad_x86.cpp"
  }
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 21970 "ad_x86.cpp"
  }
}

void orl_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1725 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 21993 "ad_x86.cpp"
  }
  {

#line 1736 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22008 "ad_x86.cpp"
  }
}

void orl_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22024 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22038 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*secondary()*/));
  
#line 22046 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22060 "ad_x86.cpp"
  }
}

void orl_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22076 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22090 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*secondary()*/));
  
#line 22098 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22112 "ad_x86.cpp"
  }
}

void xorl_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2017 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 22129 "ad_x86.cpp"
  }
  {

#line 2022 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 22138 "ad_x86.cpp"
  }
}

void xorl_eReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9263 "src/hotspot/cpu/x86/x86_32.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
     __ notl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 22156 "ad_x86.cpp"
  }
}

void xorl_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1725 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22179 "ad_x86.cpp"
  }
  {

#line 1736 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 22194 "ad_x86.cpp"
  }
}

void xorl_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 22210 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22224 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*secondary()*/));
  
#line 22232 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22246 "ad_x86.cpp"
  }
}

void xorl_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 22262 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22276 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*secondary()*/));
  
#line 22284 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 22298 "ad_x86.cpp"
  }
}

void shlL_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9301 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 22316 "ad_x86.cpp"
  }
}

void shlL_eReg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9318 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 22336 "ad_x86.cpp"
  }
}

void shlL_eReg_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9339 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 22358 "ad_x86.cpp"
  }
}

void shlL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2170 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xA4 /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xA4 /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 22383 "ad_x86.cpp"
  }
}

void shlL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2196 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0x4 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  
#line 22411 "ad_x86.cpp"
  }
}

void salL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2333 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.hi,$dst.lo
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    // CLR    $dst.lo
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* dst */);
// small:
    // SHLD   $dst.hi,$dst.lo,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xA5);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    // SHL    $dst.lo,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 22447 "ad_x86.cpp"
  }
}

uint salL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 17);
}

void shrL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2170 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 22477 "ad_x86.cpp"
  }
}

void shrL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2196 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0x5 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  
#line 22505 "ad_x86.cpp"
  }
}

void shrL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2357 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // CLR    $dst.hi
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    // SHR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x5, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
  
#line 22541 "ad_x86.cpp"
  }
}

uint shrL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 17);
}

void sarL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2170 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 22571 "ad_x86.cpp"
  }
}

void sarL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2183 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0x8B ); // Move
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_d8(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    emit_d8(cbuf,31);
  
#line 22596 "ad_x86.cpp"
  }
}

void sarL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2381 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // SAR    $dst.hi,31
    emit_opcode(cbuf, 0xC1);
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    emit_d8(cbuf, 0x1F );
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    // SAR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x7, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
  
#line 22633 "ad_x86.cpp"
  }
}

uint sarL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 18, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 18);
}

void cmpDPR_cc_P6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 22655 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 22663 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 22671 "ad_x86.cpp"
  }
  {

#line 2668 "src/hotspot/cpu/x86/x86_32.ad"

    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  
#line 22690 "ad_x86.cpp"
  }
}

void cmpDPR_cc_P6CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 22707 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 22715 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 22723 "ad_x86.cpp"
  }
}

void cmpDPR_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 22740 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 22748 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 22756 "ad_x86.cpp"
  }
  {

#line 2645 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  
#line 22783 "ad_x86.cpp"
  }
}

void cmpDPR_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 22800 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 22808 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE4 /*primary()*/));
  
#line 22816 "ad_x86.cpp"
  }
  {

#line 2415 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 22825 "ad_x86.cpp"
  }
  {

#line 2698 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 22858 "ad_x86.cpp"
  }
}

void cmpDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 22875 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 22883 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 22891 "ad_x86.cpp"
  }
  {

#line 2698 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 22924 "ad_x86.cpp"
  }
}

void cmpD_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9569 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 22942 "ad_x86.cpp"
  }
}

void cmpD_ccCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9581 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 22959 "ad_x86.cpp"
  }
}

void cmpD_ccmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9598 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp_fixup(_masm);
  
#line 22977 "ad_x86.cpp"
  }
}

void cmpD_ccmemCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9610 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 22994 "ad_x86.cpp"
  }
}

void cmpD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9629 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 23012 "ad_x86.cpp"
  }
}

void cmpD_regmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9649 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 23030 "ad_x86.cpp"
  }
}

void subDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 23047 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23055 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23063 "ad_x86.cpp"
  }
}

void subDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );   // FLD ST(i-1)
  
#line 23080 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 23088 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 23096 "ad_x86.cpp"
  }
  {

#line 2472 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 23104 "ad_x86.cpp"
  }
}

void subDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 23120 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23134 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23142 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23150 "ad_x86.cpp"
  }
}

void absDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 23165 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE1 /*primary()*/));
  
#line 23173 "ad_x86.cpp"
  }
}

void negDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 23188 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0 /*primary()*/));
  
#line 23196 "ad_x86.cpp"
  }
}

void addDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 23213 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23221 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23229 "ad_x86.cpp"
  }
}

uint addDPR_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );   // FLD ST(i-1)
  
#line 23251 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 23259 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 23267 "ad_x86.cpp"
  }
  {

#line 2472 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 23275 "ad_x86.cpp"
  }
}

void addDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 23291 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23305 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23313 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23321 "ad_x86.cpp"
  }
}

void addDPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 23337 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23351 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23359 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 23367 "ad_x86.cpp"
  }
}

void addDPR_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 23383 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23397 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 23405 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 23413 "ad_x86.cpp"
  }
  {

#line 2146 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 23421 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 23429 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23443 "ad_x86.cpp"
  }
}

void addDPR_mem_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 23459 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23473 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 23481 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 23489 "ad_x86.cpp"
  }
  {

#line 2146 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 23497 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 23505 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23519 "ad_x86.cpp"
  }
}

void addDPR_reg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9783 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ faddp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 23537 "ad_x86.cpp"
  }
}

void addDPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9796 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ faddp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 23555 "ad_x86.cpp"
  }
}

void addDPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 9797 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 23564 "ad_x86.cpp"
  }
}
void addDPR_reg_imm_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9810 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fadd(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fstp_d(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 23582 "ad_x86.cpp"
  }
}

void addDPR_reg_imm_roundNode::eval_constant(Compile* C) {
  {

#line 9811 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 23591 "ad_x86.cpp"
  }
}
void mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 23607 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23615 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23623 "ad_x86.cpp"
  }
}

void strictfp_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2431 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23643 "ad_x86.cpp"
  }
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 23652 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23660 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23668 "ad_x86.cpp"
  }
  {

#line 2439 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23680 "ad_x86.cpp"
  }
}

void mulDPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9863 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fmulp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 23698 "ad_x86.cpp"
  }
}

void mulDPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 9864 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 23707 "ad_x86.cpp"
  }
}
void mulDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 23722 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23736 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23744 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23752 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 23768 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23782 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 23790 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 23798 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_ciscNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 23814 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23828 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 23837 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 23846 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_cisc_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 23862 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23876 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 23885 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 23894 "ad_x86.cpp"
  }
}

void addDPR_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 23912 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 23923 "ad_x86.cpp"
  }
  {

#line 3066 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 23933 "ad_x86.cpp"
  }
}

void addDPR_mulDPR_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src0
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src0 */ );
  
#line 23951 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 23962 "ad_x86.cpp"
  }
  {

#line 3066 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
  
#line 23972 "ad_x86.cpp"
  }
}

void subDPR_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 23990 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 24001 "ad_x86.cpp"
  }
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE));
  
#line 24009 "ad_x86.cpp"
  }
  {

#line 1686 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0) + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 24017 "ad_x86.cpp"
  }
}

void divDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 24034 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24042 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24050 "ad_x86.cpp"
  }
}

void strictfp_divDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2431 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24070 "ad_x86.cpp"
  }
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 24079 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24087 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24095 "ad_x86.cpp"
  }
  {

#line 2439 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24107 "ad_x86.cpp"
  }
}

void divDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 24124 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 24132 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x6 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 24140 "ad_x86.cpp"
  }
  {

#line 2472 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 24148 "ad_x86.cpp"
  }
}

void modDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2521 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 24178 "ad_x86.cpp"
  }
  {

#line 2623 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 24204 "ad_x86.cpp"
  }
  {

#line 2594 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 24226 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24235 "ad_x86.cpp"
  }
}

void modD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2539 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
    __ fld_d(Address(rsp, 0));
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src0 */);
    __ fld_d(Address(rsp, 0));
  
#line 24256 "ad_x86.cpp"
  }
  {

#line 2623 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 24282 "ad_x86.cpp"
  }
  {

#line 2557 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 24293 "ad_x86.cpp"
  }
  {

#line 2415 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 24302 "ad_x86.cpp"
  }
}

void atanDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 24319 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 24327 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF3 /*secondary()*/));
  
#line 24335 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0xF3 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24343 "ad_x86.cpp"
  }
}

void atanD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2571 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
    __ fld_d(Address(rsp, 0));
  
#line 24362 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 24370 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF3 /*secondary()*/));
  
#line 24378 "ad_x86.cpp"
  }
  {

#line 2557 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 24389 "ad_x86.cpp"
  }
}

void sqrtDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 24405 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 24413 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFA /*primary()*/));
  
#line 24421 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 24430 "ad_x86.cpp"
  }
}

void cmpFPR_cc_P6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 24447 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 24455 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 24463 "ad_x86.cpp"
  }
  {

#line 2668 "src/hotspot/cpu/x86/x86_32.ad"

    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  
#line 24482 "ad_x86.cpp"
  }
}

void cmpFPR_cc_P6CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 24499 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 24507 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 24515 "ad_x86.cpp"
  }
}

void cmpFPR_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 24532 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 24540 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 24548 "ad_x86.cpp"
  }
  {

#line 2645 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  
#line 24575 "ad_x86.cpp"
  }
}

void cmpFPR_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 24592 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 24600 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE4 /*primary()*/));
  
#line 24608 "ad_x86.cpp"
  }
  {

#line 2415 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 24617 "ad_x86.cpp"
  }
  {

#line 2698 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 24650 "ad_x86.cpp"
  }
}

void cmpFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 24667 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 24675 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 24683 "ad_x86.cpp"
  }
  {

#line 2698 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 24716 "ad_x86.cpp"
  }
}

void cmpF_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10166 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 24734 "ad_x86.cpp"
  }
}

void cmpF_ccCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10178 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 24751 "ad_x86.cpp"
  }
}

void cmpF_ccmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10195 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp_fixup(_masm);
  
#line 24769 "ad_x86.cpp"
  }
}

void cmpF_ccmemCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10207 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 24786 "ad_x86.cpp"
  }
}

void cmpF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10226 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 24804 "ad_x86.cpp"
  }
}

void cmpF_regmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10246 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 24822 "ad_x86.cpp"
  }
}

void subFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 24839 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 24848 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 24856 "ad_x86.cpp"
  }
}

void subFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 24873 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24881 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24889 "ad_x86.cpp"
  }
}

void addFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 24906 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 24915 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 24923 "ad_x86.cpp"
  }
}

void addFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 24940 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 24948 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 24956 "ad_x86.cpp"
  }
}

void absFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 24971 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE1 /*primary()*/));
  
#line 24979 "ad_x86.cpp"
  }
}

void negFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 24994 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0 /*primary()*/));
  
#line 25002 "ad_x86.cpp"
  }
}

void addFPR24_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25018 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25032 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25041 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25049 "ad_x86.cpp"
  }
}

void addFPR24_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25065 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25079 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 25088 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25096 "ad_x86.cpp"
  }
}

void addFPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25112 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25126 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25134 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25142 "ad_x86.cpp"
  }
}

void addFPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25158 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25172 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25180 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 25188 "ad_x86.cpp"
  }
}

void addFPR24_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25204 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25218 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25227 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25235 "ad_x86.cpp"
  }
}

void addFPR24_mem_ciscNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25251 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25265 "ad_x86.cpp"
  }
  {

#line 2146 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 25273 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25281 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25295 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25303 "ad_x86.cpp"
  }
}

void addFPR24_mem_cisc_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25319 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25333 "ad_x86.cpp"
  }
  {

#line 2146 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 25341 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25349 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25363 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25371 "ad_x86.cpp"
  }
}

void addFPR24_mem_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25387 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25401 "ad_x86.cpp"
  }
  {

#line 2146 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 25409 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25417 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25431 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25439 "ad_x86.cpp"
  }
}

void addFPR24_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10404 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fadd_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_s(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 25458 "ad_x86.cpp"
  }
}

void addFPR24_reg_immNode::eval_constant(Compile* C) {
  {

#line 10406 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 25467 "ad_x86.cpp"
  }
}
void addFPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10419 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fadd_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 25485 "ad_x86.cpp"
  }
}

void addFPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 10421 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 25494 "ad_x86.cpp"
  }
}
void mulFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25510 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25519 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25527 "ad_x86.cpp"
  }
}

void mulFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25544 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25553 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 25562 "ad_x86.cpp"
  }
}

void mulFPR24_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25578 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25592 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25601 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25609 "ad_x86.cpp"
  }
}

void mulFPR24_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25625 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25639 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 25648 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25656 "ad_x86.cpp"
  }
}

void mulFPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25672 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25686 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25695 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 25704 "ad_x86.cpp"
  }
}

void mulFPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25720 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25734 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 25743 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 25752 "ad_x86.cpp"
  }
}

void mulFPR24_mem_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25768 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25782 "ad_x86.cpp"
  }
  {

#line 2146 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 25790 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25798 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25812 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25820 "ad_x86.cpp"
  }
}

void mulFPR24_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10510 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fmul_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_s(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 25839 "ad_x86.cpp"
  }
}

void mulFPR24_reg_immNode::eval_constant(Compile* C) {
  {

#line 10512 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 25848 "ad_x86.cpp"
  }
}
void mulFPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10526 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fmul_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 25866 "ad_x86.cpp"
  }
}

void mulFPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 10528 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 25875 "ad_x86.cpp"
  }
}
void mulFPR_reg_load1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25890 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25904 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 25913 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 25922 "ad_x86.cpp"
  }
}

void mulFPR_reg_load1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  {

#line 1622 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 25938 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25952 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 25961 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 25970 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 25987 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26001 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 26012 "ad_x86.cpp"
  }
  {

#line 3059 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 26023 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26032 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 26049 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26063 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 26074 "ad_x86.cpp"
  }
  {

#line 3059 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 26085 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26094 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 26111 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26125 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 26136 "ad_x86.cpp"
  }
  {

#line 3059 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 26147 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26156 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mem1
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 26173 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(3)->base(ra_,this,idx3);
    int index    = opnd_array(3)->index(ra_,this,idx3);
    int scale    = opnd_array(3)->scale();
    int displace = opnd_array(3)->disp(ra_,this,idx3);
    relocInfo::relocType disp_reloc = opnd_array(3)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26187 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 26198 "ad_x86.cpp"
  }
  {

#line 3059 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 26209 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26218 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 26236 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 26247 "ad_x86.cpp"
  }
  {

#line 3066 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 26257 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src0
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src0 */ );
  
#line 26275 "ad_x86.cpp"
  }
  {

#line 3052 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 26286 "ad_x86.cpp"
  }
  {

#line 3066 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
  
#line 26296 "ad_x86.cpp"
  }
}

void subFPR_divFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src2 */ );
  
#line 26314 "ad_x86.cpp"
  }
  {

#line 3072 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand has been loaded into fp ST (stack top)
      // FSUB   ST,$src1
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xE0 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);

      // FDIV
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xF0 + opnd_array(3)->reg(ra_,this,idx3)/* src3 */);
  
#line 26329 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26338 "ad_x86.cpp"
  }
}

void divFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 26355 "ad_x86.cpp"
  }
  {

#line 2409 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x6 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 26364 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26372 "ad_x86.cpp"
  }
}

void divFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 26389 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26397 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26405 "ad_x86.cpp"
  }
}

void modFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2521 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src2 */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 26435 "ad_x86.cpp"
  }
  {

#line 2623 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 26461 "ad_x86.cpp"
  }
  {

#line 2594 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src2 */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 26483 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26491 "ad_x86.cpp"
  }
}

void modFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2521 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 26521 "ad_x86.cpp"
  }
  {

#line 2623 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 26547 "ad_x86.cpp"
  }
  {

#line 2594 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 26569 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26578 "ad_x86.cpp"
  }
}

void modF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2548 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
    __ fld_s(Address(rsp, 0));
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src0 */);
    __ fld_s(Address(rsp, 0));
  
#line 26599 "ad_x86.cpp"
  }
  {

#line 2623 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 26625 "ad_x86.cpp"
  }
  {

#line 2564 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_s(Address(rsp, 0));
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, (0x4));
  
#line 26636 "ad_x86.cpp"
  }
  {

#line 2415 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 26645 "ad_x86.cpp"
  }
}

void roundFloat_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2487 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  
#line 26666 "ad_x86.cpp"
  }
}

void roundDouble_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2498 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  
#line 26687 "ad_x86.cpp"
  }
}

void convDPR2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10726 "src/hotspot/cpu/x86/x86_32.ad"

    __ subptr(rsp, 4);
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */!= FPR1L_enc) {
      __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */-1);
      __ fstp_s(Address(rsp, 0));
    } else {
      __ fst_s(Address(rsp, 0));
    }
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 4);
  
#line 26711 "ad_x86.cpp"
  }
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10745 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsd2ss (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 26727 "ad_x86.cpp"
  }
}

void convFPR2DPR_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2509 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0xD0 - 1; // -1 since we skip FLD
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(src-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0xD8;
    }
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, pop+opnd_array(0)->reg(ra_,this)/* dst */ );      // FST<P> ST(i)
  
#line 26749 "ad_x86.cpp"
  }
}

void convF2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10777 "src/hotspot/cpu/x86/x86_32.ad"

    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 26769 "ad_x86.cpp"
  }
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10791 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtss2sd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 26785 "ad_x86.cpp"
  }
}

void convDPR2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 26801 "ad_x86.cpp"
  }
  {

#line 2967 "src/hotspot/cpu/x86/x86_32.ad"

    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 26848 "ad_x86.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10831 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ cvttsd2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ addptr(rsp, 8);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2i_wrapper())));
    __ bind(fast);
  
#line 26873 "ad_x86.cpp"
  }
}

void convDPR2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2426 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 26889 "ad_x86.cpp"
  }
  {

#line 3010 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 26935 "ad_x86.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10891 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_trunc()));
    __ fistp_d(Address(rsp, 0));
    // Restore the rounding mode, mask the exception
    if (Compile::current()->in_24_bit_fp_mode()) {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    } else {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Load the converted long, adjust CPU stack
    __ pop(rax);
    __ pop(rdx);
    __ cmpl(rdx, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ testl(rax, rax);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ addptr(rsp, 8);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2l_wrapper())));
    __ bind(fast);
  
#line 26975 "ad_x86.cpp"
  }
}

void convFPR2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 26991 "ad_x86.cpp"
  }
  {

#line 2967 "src/hotspot/cpu/x86/x86_32.ad"

    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 27038 "ad_x86.cpp"
  }
}

void convF2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10961 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ cvttss2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2i_wrapper())));
    __ bind(fast);
  
#line 27063 "ad_x86.cpp"
  }
}

void convFPR2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2481 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 27079 "ad_x86.cpp"
  }
  {

#line 3010 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  
#line 27125 "ad_x86.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11022 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ subptr(rsp, 8);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_trunc()));
    __ fistp_d(Address(rsp, 0));
    // Restore the rounding mode, mask the exception
    if (Compile::current()->in_24_bit_fp_mode()) {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    } else {
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Load the converted long, adjust CPU stack
    __ pop(rax);
    __ pop(rdx);
    __ cmpl(rdx, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ testl(rax, rax);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::d2l_wrapper())));
    __ bind(fast);
  
#line 27165 "ad_x86.cpp"
  }
}

void convI2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2461 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 27180 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27189 "ad_x86.cpp"
  }
}

void convI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11066 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 27205 "ad_x86.cpp"
  }
}

void convI2D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11076 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 27221 "ad_x86.cpp"
  }
}

void convXI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11089 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 27238 "ad_x86.cpp"
  }
}

void convI2DPR_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 27253 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27267 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27276 "ad_x86.cpp"
  }
}

void conv24I2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2461 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 27291 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27300 "ad_x86.cpp"
  }
}

void convI2FPR_SSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2461 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 27315 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27323 "ad_x86.cpp"
  }
}

void convI2FPR_SSF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 27338 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27352 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27360 "ad_x86.cpp"
  }
}

void convI2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2461 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 27375 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27384 "ad_x86.cpp"
  }
}

void convI2FPR_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 27399 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27413 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27422 "ad_x86.cpp"
  }
}

void convI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11174 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2ssl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 27438 "ad_x86.cpp"
  }
}

void convXI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11187 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 27455 "ad_x86.cpp"
  }
}

void convI2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2743 "src/hotspot/cpu/x86/x86_32.ad"

    // mov $dst.lo,$src
    int dst_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    encode_Copy( cbuf, dst_encoding  , src_encoding );
    // mov $dst.hi,$src
    encode_Copy( cbuf, HIGH_FROM_LOW(dst_encoding), src_encoding );
    // sar $dst.hi,31
    emit_opcode( cbuf, 0xC1 );
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW(dst_encoding) );
    emit_d8(cbuf, 0x1F );
  
#line 27479 "ad_x86.cpp"
  }
}

void convI2L_reg_zexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 2005 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 27495 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 27503 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 27511 "ad_x86.cpp"
  }
}

void zerox_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 2005 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 27527 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 27535 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 27543 "ad_x86.cpp"
  }
}

void convL2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2756 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  
#line 27570 "ad_x86.cpp"
  }
  {

#line 2472 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27578 "ad_x86.cpp"
  }
}

void convL2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2786 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  
#line 27601 "ad_x86.cpp"
  }
  {

#line 2557 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 27612 "ad_x86.cpp"
  }
}

void convL2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2786 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  
#line 27635 "ad_x86.cpp"
  }
  {

#line 2564 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_s(Address(rsp, 0));
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, (0x8));
  
#line 27646 "ad_x86.cpp"
  }
}

void convL2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2756 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  
#line 27673 "ad_x86.cpp"
  }
  {

#line 2466 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27681 "ad_x86.cpp"
  }
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2009 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 27696 "ad_x86.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11299 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 27712 "ad_x86.cpp"
  }
}

void MoveFPR2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2487 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  
#line 27733 "ad_x86.cpp"
  }
}

void MoveF2I_reg_stack_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11323 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 27749 "ad_x86.cpp"
  }
}

void MoveF2I_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11335 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 27765 "ad_x86.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11347 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 27781 "ad_x86.cpp"
  }
}

void MoveI2FPR_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 27796 "ad_x86.cpp"
  }
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 27810 "ad_x86.cpp"
  }
  {

#line 2476 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27819 "ad_x86.cpp"
  }
}

void MoveI2F_stack_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11375 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 27835 "ad_x86.cpp"
  }
}

void MoveI2F_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11388 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 27851 "ad_x86.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 27866 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 27880 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*secondary()*/));
  
#line 27888 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 27902 "ad_x86.cpp"
  }
}

void MoveDPR2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2498 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  
#line 27923 "ad_x86.cpp"
  }
}

void MoveD2L_reg_stack_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11423 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 27939 "ad_x86.cpp"
  }
}

void MoveD2L_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11437 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x4e);
    __ movdl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 27958 "ad_x86.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 27973 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    relocInfo::relocType disp_reloc = opnd_array(0)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 27987 "ad_x86.cpp"
  }
  {

#line 1617 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 27995 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( opnd_array(0)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 28009 "ad_x86.cpp"
  }
}

void MoveL2DPR_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 28024 "ad_x86.cpp"
  }
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 28038 "ad_x86.cpp"
  }
  {

#line 2421 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28047 "ad_x86.cpp"
  }
}

void MoveL2D_stack_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11480 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 28063 "ad_x86.cpp"
  }
}

void MoveL2D_stack_reg_sse_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11493 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 28079 "ad_x86.cpp"
  }
}

void MoveL2D_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11507 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
    __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 28099 "ad_x86.cpp"
  }
}

void rep_stosNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11570 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, false);
  
#line 28118 "ad_x86.cpp"
  }
}

void rep_stos_largeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11619 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, true);
  
#line 28137 "ad_x86.cpp"
  }
}

void string_compareLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11633 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LL);
  
#line 28159 "ad_x86.cpp"
  }
}

void string_compareUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11648 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UU);
  
#line 28181 "ad_x86.cpp"
  }
}

void string_compareLUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11663 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LU);
  
#line 28203 "ad_x86.cpp"
  }
}

void string_compareULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11678 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */,
                      opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UL);
  
#line 28225 "ad_x86.cpp"
  }
}

void string_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11693 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(false, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, false /* char */);
  
#line 28247 "ad_x86.cpp"
  }
}

void string_indexof_conLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11710 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 16) {
      // IndexOf for constant substrings with size >= 16 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
    }
  
#line 28281 "ad_x86.cpp"
  }
}

void string_indexof_conUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11738 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 8) {
      // IndexOf for constant substrings with size >= 8 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
    }
  
#line 28315 "ad_x86.cpp"
  }
}

void string_indexof_conULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11766 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 8) {
      // IndexOf for constant substrings with size >= 8 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
    }
  
#line 28349 "ad_x86.cpp"
  }
}

void string_indexofLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11793 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
  
#line 28372 "ad_x86.cpp"
  }
}

void string_indexofUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11809 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
  
#line 28395 "ad_x86.cpp"
  }
}

void string_indexofULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11825 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
  
#line 28418 "ad_x86.cpp"
  }
}

void string_indexofU_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vec1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vec3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11840 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */, opnd_array(0)->as_Register(ra_,this)/* result */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vec1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vec3 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp */);
  
#line 28441 "ad_x86.cpp"
  }
}

void array_equalsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11857 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, false /* char */);
  
#line 28462 "ad_x86.cpp"
  }
}

void array_equalsCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11874 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, true /* char */);
  
#line 28483 "ad_x86.cpp"
  }
}

void has_negativesNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// len
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11889 "src/hotspot/cpu/x86/x86_32.ad"

    __ has_negatives(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* len */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */);
  
#line 28504 "ad_x86.cpp"
  }
}

void string_compressNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11904 "src/hotspot/cpu/x86/x86_32.ad"

    __ char_array_compress(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                           opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */);
  
#line 28528 "ad_x86.cpp"
  }
}

void string_inflateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11919 "src/hotspot/cpu/x86/x86_32.ad"

    __ byte_array_inflate(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 28549 "ad_x86.cpp"
  }
}

void encode_iso_arrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11934 "src/hotspot/cpu/x86/x86_32.ad"

    __ encode_iso_array(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                        opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                        opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */);
  
#line 28573 "ad_x86.cpp"
  }
}

void compI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 28589 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 28597 "ad_x86.cpp"
  }
}

void compI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 28621 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 28636 "ad_x86.cpp"
  }
}

void compI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 28652 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 28666 "ad_x86.cpp"
  }
}

void testI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 28682 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 28690 "ad_x86.cpp"
  }
}

void testI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 28707 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 28715 "ad_x86.cpp"
  }
  {

#line 2039 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 28724 "ad_x86.cpp"
  }
}

void testI_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 28741 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 28755 "ad_x86.cpp"
  }
}

void compU_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 28771 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 28779 "ad_x86.cpp"
  }
}

void compU_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 28803 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 28818 "ad_x86.cpp"
  }
}

void compU_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 28834 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 28848 "ad_x86.cpp"
  }
}

void testU_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 28864 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 28872 "ad_x86.cpp"
  }
}

void compP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 28888 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 28896 "ad_x86.cpp"
  }
}

void compP_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1702 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 28920 "ad_x86.cpp"
  }
  {

#line 1714 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      
if ( opnd_array(2)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), opnd_array(2)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
    }
  
#line 28940 "ad_x86.cpp"
  }
}

void compP_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 28956 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 28970 "ad_x86.cpp"
  }
}

void compP_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 28986 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 29000 "ad_x86.cpp"
  }
}

void testP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29016 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 29024 "ad_x86.cpp"
  }
}

void testP_Reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 29040 "ad_x86.cpp"
  }
  {

#line 2240 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 29054 "ad_x86.cpp"
  }
  {

#line 2063 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d32(cbuf,(0xFFFFFFFF));
  
#line 29062 "ad_x86.cpp"
  }
}

void minI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2260 "src/hotspot/cpu/x86/x86_32.ad"

    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // jmp dst < src around move
    emit_opcode(cbuf,0x7C);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 29086 "ad_x86.cpp"
  }
}

void maxI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2272 "src/hotspot/cpu/x86/x86_32.ad"

    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // jmp dst > src around move
    emit_opcode(cbuf,0x7F);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 29110 "ad_x86.cpp"
  }
}

void loopLimit_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// init
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// limit
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// stride
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// limit_hi
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12218 "src/hotspot/cpu/x86/x86_32.ad"

    int strd = (int)opnd_array(3)->constant();
    assert(strd != 1 && strd != -1, "sanity");
    int m1 = (strd > 0) ? 1 : -1;
    // Convert limit to long (EAX:EDX)
    __ cdql();
    // Convert init to long (init:tmp)
    __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
    __ sarl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, 31);
    // $limit - $init
    __ subl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
    __ sbbl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    // + ($stride - 1)
    if (strd > 0) {
      __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, (strd - 1));
      __ adcl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, 0);
      __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, strd);
    } else {
      __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, (strd + 1));
      __ adcl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, -1);
      __ lneg(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, opnd_array(2)->as_Register(ra_,this,idx2)/* limit */);
      __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, -strd);
    }
    // signed devision: (EAX:EDX) / pos_stride
    __ idivl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    if (strd < 0) {
      // restore sign
      __ negl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    }
    // (EAX) * stride
    __ mull(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    // + init (ignore upper bits)
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
  
#line 29161 "ad_x86.cpp"
  }
}

void jumpXtndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->output()->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12262 "src/hotspot/cpu/x86/x86_32.ad"

    // Jump to Address(table_base + switch_reg)
    Address index(noreg, opnd_array(1)->as_Register(ra_,this,idx1)/* switch_val */, Address::times_1);
    __ jump(ArrayAddress(InternalAddress(__ code()->consts()->start() + constant_offset()), index));
  
#line 29180 "ad_x86.cpp"
  }
}

void jumpXtndNode::eval_constant(Compile* C) {
  _constant = C->output()->constant_table().add_jump_table(this);
  // User did not define an encode section.
}
void jmpDirNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12278 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(1)->label();
    __ jmp(*L, false); // Always long jump
  
#line 29201 "ad_x86.cpp"
  }
}

uint jmpDirNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 5, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 5);
}

void jmpConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12293 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 29225 "ad_x86.cpp"
  }
}

uint jmpConNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12309 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 29249 "ad_x86.cpp"
  }
}

uint jmpLoopEndNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12325 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 29273 "ad_x86.cpp"
  }
}

uint jmpLoopEndUNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12340 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 29297 "ad_x86.cpp"
  }
}

uint jmpLoopEndUCFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEnd_and_restoreMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12358 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
    __ restorevectmask();
  
#line 29322 "ad_x86.cpp"
  }
}

uint jmpLoopEnd_and_restoreMaskNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 10, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 10);
}

void jmpLoopEndU_and_restoreMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12376 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
    __ restorevectmask();
  
#line 29347 "ad_x86.cpp"
  }
}

uint jmpLoopEndU_and_restoreMaskNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 10, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 10);
}

void jmpLoopEndUCF_and_restoreMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12393 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
    __ restorevectmask();
  
#line 29372 "ad_x86.cpp"
  }
}

uint jmpLoopEndUCF_and_restoreMaskNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 10, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 10);
}

void jmpConUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12409 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 29396 "ad_x86.cpp"
  }
}

uint jmpConUNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12423 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 29420 "ad_x86.cpp"
  }
}

uint jmpConUCFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCF2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12445 "src/hotspot/cpu/x86/x86_32.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jcc(Assembler::parity, *l, false);
      __ jcc(Assembler::notEqual, *l, false);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jcc(Assembler::equal, *l, false);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 29454 "ad_x86.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 1787 "src/hotspot/cpu/x86/x86_32.ad"

    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label miss;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Resi, Reax, Recx, Redi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x1 /*primary()*/)) {
      __ xorptr(Redi, Redi);
    }
    __ bind(miss);
  
#line 29483 "ad_x86.cpp"
  }
}

void partialSubtypeCheck_vs_ZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1787 "src/hotspot/cpu/x86/x86_32.ad"

    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label miss;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Resi, Reax, Recx, Redi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x0 /*primary()*/)) {
      __ xorptr(Redi, Redi);
    }
    __ bind(miss);
  
#line 29513 "ad_x86.cpp"
  }
}

void jmpDir_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12524 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(1)->label();
    __ jmpb(*L);
  
#line 29530 "ad_x86.cpp"
  }
}

uint jmpDir_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpCon_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12540 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 29554 "ad_x86.cpp"
  }
}

uint jmpCon_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEnd_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12556 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 29578 "ad_x86.cpp"
  }
}

uint jmpLoopEnd_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEndU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12572 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 29602 "ad_x86.cpp"
  }
}

uint jmpLoopEndU_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEndUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12587 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 29626 "ad_x86.cpp"
  }
}

uint jmpLoopEndUCF_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12603 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 29650 "ad_x86.cpp"
  }
}

uint jmpConU_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12618 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 29674 "ad_x86.cpp"
  }
}

uint jmpConUCF_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF2_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12642 "src/hotspot/cpu/x86/x86_32.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jccb(Assembler::parity, *l);
      __ jccb(Assembler::notEqual, *l);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jccb(Assembler::equal, *l);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 29708 "ad_x86.cpp"
  }
}

uint jmpConUCF2_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12698 "src/hotspot/cpu/x86/x86_32.ad"

    Label p_one, m_one, done;
    __ xorptr(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ cmpl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */));
    __ jccb(Assembler::less,    m_one);
    __ jccb(Assembler::greater, p_one);
    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */);
    __ jccb(Assembler::below,   m_one);
    __ jccb(Assembler::equal,   done);
    __ bind(p_one);
    __ incrementl(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ jmpb(done);
    __ bind(m_one);
    __ decrementl(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ bind(done);
  
#line 29743 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29759 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 29767 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2915 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 29792 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29808 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 29816 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2915 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 29841 "ad_x86.cpp"
  }
}

void cmovLL_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 29860 "ad_x86.cpp"
  }
  {

#line 2027 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 29868 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 29877 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 29885 "ad_x86.cpp"
  }
}

void cmovLL_mem_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 29904 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 29918 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 29927 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 29941 "ad_x86.cpp"
  }
}

void cmovII_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 29960 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 29968 "ad_x86.cpp"
  }
}

void cmovII_mem_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 29987 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30001 "ad_x86.cpp"
  }
}

void cmovPP_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30020 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30028 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2894 "src/hotspot/cpu/x86/x86_32.ad"

    // MOV   $tmp,$src.lo
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
    // OR    $tmp,$src.hi
    emit_opcode(cbuf, 0x0B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 30050 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2903 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // JNE,s  skip
    emit_cc(cbuf, 0x70, 0x5);
    emit_d8(cbuf,2);
    // CMP    $src1.hi,$src2.hi
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 30074 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2894 "src/hotspot/cpu/x86/x86_32.ad"

    // MOV   $tmp,$src.lo
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
    // OR    $tmp,$src.hi
    emit_opcode(cbuf, 0x0B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 30096 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2903 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // JNE,s  skip
    emit_cc(cbuf, 0x70, 0x5);
    emit_d8(cbuf,2);
    // CMP    $src1.hi,$src2.hi
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 30120 "ad_x86.cpp"
  }
}

void cmovLL_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30139 "ad_x86.cpp"
  }
  {

#line 2027 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30147 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30156 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 30164 "ad_x86.cpp"
  }
}

void cmovLL_mem_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30183 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30197 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30206 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 30220 "ad_x86.cpp"
  }
}

void cmovII_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30239 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30247 "ad_x86.cpp"
  }
}

void cmovII_mem_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30266 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30280 "ad_x86.cpp"
  }
}

void cmovPP_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30299 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30307 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2927 "src/hotspot/cpu/x86/x86_32.ad"

    // XOR    $tmp,$tmp
    emit_opcode(cbuf,0x33);  // XOR
    emit_rm(cbuf,0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    // CMP    $tmp,$src.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // SBB    $tmp,$src.hi
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */) );
  
#line 30332 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2915 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* src2 */, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
  
#line 30357 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2927 "src/hotspot/cpu/x86/x86_32.ad"

    // XOR    $tmp,$tmp
    emit_opcode(cbuf,0x33);  // XOR
    emit_rm(cbuf,0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    // CMP    $tmp,$src.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // SBB    $tmp,$src.hi
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src */) );
  
#line 30382 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2915 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* src2 */, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
  
#line 30407 "ad_x86.cpp"
  }
}

void cmovLL_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30426 "ad_x86.cpp"
  }
  {

#line 2027 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30434 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30443 "ad_x86.cpp"
  }
  {

#line 2031 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 30451 "ad_x86.cpp"
  }
}

void cmovLL_mem_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30470 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30484 "ad_x86.cpp"
  }
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30493 "ad_x86.cpp"
  }
  {

#line 2160 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 30507 "ad_x86.cpp"
  }
}

void cmovII_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30526 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30534 "ad_x86.cpp"
  }
}

void cmovII_mem_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30553 "ad_x86.cpp"
  }
  {

#line 2150 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30567 "ad_x86.cpp"
  }
}

void cmovPP_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1769 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 30586 "ad_x86.cpp"
  }
  {

#line 2013 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 30594 "ad_x86.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1862 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 30620 "ad_x86.cpp"
  }
  {

#line 1885 "src/hotspot/cpu/x86/x86_32.ad"

    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_IMM32);
    } else {
      int method_index = resolved_method_index(cbuf);
      RelocationHolder rspec = _optimized_virtual ? opt_virtual_call_Relocation::spec(method_index)
                                                  : static_call_Relocation::spec(method_index);
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                     rspec, RELOC_DISP32);
      // Emit stubs for static call.
      address stub = CompiledStaticCall::emit_to_interp_stub(cbuf);
      if (stub == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    }
  
#line 30649 "ad_x86.cpp"
  }
  {

#line 2168 "src/hotspot/cpu/x86/x86.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      C2_MacroAssembler _masm(&cbuf);
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 30667 "ad_x86.cpp"
  }
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 30679 "ad_x86.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1862 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 30705 "ad_x86.cpp"
  }
  {

#line 1910 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ ic_call((address)opnd_array(1)->method(), resolved_method_index(cbuf));
  
#line 30714 "ad_x86.cpp"
  }
  {

#line 2168 "src/hotspot/cpu/x86/x86.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      C2_MacroAssembler _masm(&cbuf);
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 30732 "ad_x86.cpp"
  }
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 30744 "ad_x86.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1862 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 30770 "ad_x86.cpp"
  }
  {

#line 1804 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  
#line 30793 "ad_x86.cpp"
  }
  {

#line 1830 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 30829 "ad_x86.cpp"
  }
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 30841 "ad_x86.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1862 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 30867 "ad_x86.cpp"
  }
  {

#line 1804 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  
#line 30890 "ad_x86.cpp"
  }
  {

#line 1830 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 30926 "ad_x86.cpp"
  }
  {

#line 1823 "src/hotspot/cpu/x86/x86_32.ad"

    if( VerifyFPU ) {
      MacroAssembler masm(&cbuf);
      masm.verify_FPU( -3, "Returning from Runtime Leaf call");
    }
  
#line 30937 "ad_x86.cpp"
  }
  {

#line 1877 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  
#line 30949 "ad_x86.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1862 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 30975 "ad_x86.cpp"
  }
  {

#line 1830 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 31011 "ad_x86.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC3 /*primary()*/));
  
#line 31026 "ad_x86.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 31042 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ );
  
#line 31050 "ad_x86.cpp"
  }
}

void tailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2950 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0x5A);
  
#line 31066 "ad_x86.cpp"
  }
  {

#line 1612 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 31074 "ad_x86.cpp"
  }
  {

#line 1765 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ );
  
#line 31082 "ad_x86.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2954 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE9);        // jmp    entry
    emit_d32_reloc(cbuf, (int)OptoRuntime::rethrow_stub() - ((int)cbuf.insts_end())-4,
                   runtime_call_Relocation::spec(), RELOC_IMM32 );
  
#line 31113 "ad_x86.cpp"
  }
}

void cmpFastLockRTMNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scr
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// cx1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// cx2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13379 "src/hotspot/cpu/x86/x86_32.ad"

    __ fast_lock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */,
                 opnd_array(4)->as_Register(ra_,this,idx4)/* scr */, opnd_array(5)->as_Register(ra_,this,idx5)/* cx1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* cx2 */,
                 _counters, _rtm_counters, _stack_rtm_counters,
                 ((Method*)(ra_->C->method()->constant_encoding()))->method_data(),
                 true, ra_->C->profile_rtm());
  
#line 31138 "ad_x86.cpp"
  }
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13395 "src/hotspot/cpu/x86/x86_32.ad"

    __ fast_lock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */,
                 opnd_array(4)->as_Register(ra_,this,idx4)/* scr */, noreg, noreg, _counters, NULL, NULL, NULL, false, false);
  
#line 31158 "ad_x86.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13407 "src/hotspot/cpu/x86/x86_32.ad"

    __ fast_unlock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, ra_->C->use_rtm());
  
#line 31176 "ad_x86.cpp"
  }
}

void safePoint_poll_tlsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13425 "src/hotspot/cpu/x86/x86_32.ad"

    __ relocate(relocInfo::poll_type);
    address pre_pc = __ pc();
    __ testl(rax, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* poll */, 0));
    address post_pc = __ pc();
    guarantee(pre_pc[0] == 0x85, "must emit test-ax [reg]");
  
#line 31196 "ad_x86.cpp"
  }
}

uint safePoint_poll_tlsNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13445 "src/hotspot/cpu/x86/x86_32.ad"

    Register dstReg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ get_thread(dstReg);
  
#line 31218 "ad_x86.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2342 "src/hotspot/cpu/x86/x86.ad"

    if (is_reachable()) {
      __ stop(_halt_reason);
    }
  
#line 31236 "ad_x86.cpp"
  }
}

void setMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2357 "src/hotspot/cpu/x86/x86.ad"

    __ setvectmask(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 31253 "ad_x86.cpp"
  }
}

void addF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2371 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 31270 "ad_x86.cpp"
  }
}

void addF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2383 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31287 "ad_x86.cpp"
  }
}

void addF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2383 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 31304 "ad_x86.cpp"
  }
}

void addF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2394 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31321 "ad_x86.cpp"
  }
}

void addF_immNode::eval_constant(Compile* C) {
  {

#line 2395 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31330 "ad_x86.cpp"
  }
}
void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2406 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 31346 "ad_x86.cpp"
  }
}

void addF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2418 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31363 "ad_x86.cpp"
  }
}

void addF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2418 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 31380 "ad_x86.cpp"
  }
}

void addF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2430 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31397 "ad_x86.cpp"
  }
}

void addF_reg_immNode::eval_constant(Compile* C) {
  {

#line 2431 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31406 "ad_x86.cpp"
  }
}
void addD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2442 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 31422 "ad_x86.cpp"
  }
}

void addD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2454 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31439 "ad_x86.cpp"
  }
}

void addD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2454 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 31456 "ad_x86.cpp"
  }
}

void addD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2465 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31473 "ad_x86.cpp"
  }
}

void addD_immNode::eval_constant(Compile* C) {
  {

#line 2466 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31482 "ad_x86.cpp"
  }
}
void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2477 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 31498 "ad_x86.cpp"
  }
}

void addD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2489 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31515 "ad_x86.cpp"
  }
}

void addD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2489 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 31532 "ad_x86.cpp"
  }
}

void addD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2501 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31549 "ad_x86.cpp"
  }
}

void addD_reg_immNode::eval_constant(Compile* C) {
  {

#line 2502 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31558 "ad_x86.cpp"
  }
}
void subF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2513 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 31574 "ad_x86.cpp"
  }
}

void subF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2525 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31591 "ad_x86.cpp"
  }
}

void subF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2536 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31608 "ad_x86.cpp"
  }
}

void subF_immNode::eval_constant(Compile* C) {
  {

#line 2537 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31617 "ad_x86.cpp"
  }
}
void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2548 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 31633 "ad_x86.cpp"
  }
}

void subF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2560 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31650 "ad_x86.cpp"
  }
}

void subF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2572 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31667 "ad_x86.cpp"
  }
}

void subF_reg_immNode::eval_constant(Compile* C) {
  {

#line 2573 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31676 "ad_x86.cpp"
  }
}
void subD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2584 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 31692 "ad_x86.cpp"
  }
}

void subD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2596 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31709 "ad_x86.cpp"
  }
}

void subD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2607 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31726 "ad_x86.cpp"
  }
}

void subD_immNode::eval_constant(Compile* C) {
  {

#line 2608 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31735 "ad_x86.cpp"
  }
}
void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2619 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 31751 "ad_x86.cpp"
  }
}

void subD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2631 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31768 "ad_x86.cpp"
  }
}

void subD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2643 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31785 "ad_x86.cpp"
  }
}

void subD_reg_immNode::eval_constant(Compile* C) {
  {

#line 2644 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31794 "ad_x86.cpp"
  }
}
void mulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2655 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 31810 "ad_x86.cpp"
  }
}

void mulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2667 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31827 "ad_x86.cpp"
  }
}

void mulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2667 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 31844 "ad_x86.cpp"
  }
}

void mulF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2678 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31861 "ad_x86.cpp"
  }
}

void mulF_immNode::eval_constant(Compile* C) {
  {

#line 2679 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31870 "ad_x86.cpp"
  }
}
void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2690 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 31886 "ad_x86.cpp"
  }
}

void mulF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2702 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31903 "ad_x86.cpp"
  }
}

void mulF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2702 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 31920 "ad_x86.cpp"
  }
}

void mulF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2714 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 31937 "ad_x86.cpp"
  }
}

void mulF_reg_immNode::eval_constant(Compile* C) {
  {

#line 2715 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 31946 "ad_x86.cpp"
  }
}
void mulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2726 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 31962 "ad_x86.cpp"
  }
}

void mulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2738 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 31979 "ad_x86.cpp"
  }
}

void mulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2738 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 31996 "ad_x86.cpp"
  }
}

void mulD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2749 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32013 "ad_x86.cpp"
  }
}

void mulD_immNode::eval_constant(Compile* C) {
  {

#line 2750 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32022 "ad_x86.cpp"
  }
}
void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2761 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32038 "ad_x86.cpp"
  }
}

void mulD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2773 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32055 "ad_x86.cpp"
  }
}

void mulD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2773 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 32072 "ad_x86.cpp"
  }
}

void mulD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2785 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32089 "ad_x86.cpp"
  }
}

void mulD_reg_immNode::eval_constant(Compile* C) {
  {

#line 2786 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32098 "ad_x86.cpp"
  }
}
void divF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2797 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32114 "ad_x86.cpp"
  }
}

void divF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2809 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32131 "ad_x86.cpp"
  }
}

void divF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2820 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32148 "ad_x86.cpp"
  }
}

void divF_immNode::eval_constant(Compile* C) {
  {

#line 2821 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32157 "ad_x86.cpp"
  }
}
void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2832 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32173 "ad_x86.cpp"
  }
}

void divF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2844 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32190 "ad_x86.cpp"
  }
}

void divF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2856 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32207 "ad_x86.cpp"
  }
}

void divF_reg_immNode::eval_constant(Compile* C) {
  {

#line 2857 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32216 "ad_x86.cpp"
  }
}
void divD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2868 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32232 "ad_x86.cpp"
  }
}

void divD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2880 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32249 "ad_x86.cpp"
  }
}

void divD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2891 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32266 "ad_x86.cpp"
  }
}

void divD_immNode::eval_constant(Compile* C) {
  {

#line 2892 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32275 "ad_x86.cpp"
  }
}
void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2903 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32291 "ad_x86.cpp"
  }
}

void divD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2915 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 32308 "ad_x86.cpp"
  }
}

void divD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2927 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32325 "ad_x86.cpp"
  }
}

void divD_reg_immNode::eval_constant(Compile* C) {
  {

#line 2928 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 32334 "ad_x86.cpp"
  }
}
void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2938 "src/hotspot/cpu/x86/x86.ad"

    __ andps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signmask()));
  
#line 32349 "ad_x86.cpp"
  }
}

void absF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2949 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = 0;
    __ vandps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(float_signmask()), vector_len);
  
#line 32367 "ad_x86.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2963 "src/hotspot/cpu/x86/x86.ad"

    __ andpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signmask()));
  
#line 32383 "ad_x86.cpp"
  }
}

void absD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2975 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = 0;
    __ vandpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(double_signmask()), vector_len);
  
#line 32401 "ad_x86.cpp"
  }
}

void negF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2988 "src/hotspot/cpu/x86/x86.ad"

    __ xorps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signflip()));
  
#line 32417 "ad_x86.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2999 "src/hotspot/cpu/x86/x86.ad"

    __ vnegatess(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
                 ExternalAddress(float_signflip()));
  
#line 32434 "ad_x86.cpp"
  }
}

void negD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3012 "src/hotspot/cpu/x86/x86.ad"

    __ xorpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signflip()));
  
#line 32450 "ad_x86.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3024 "src/hotspot/cpu/x86/x86.ad"

    __ vnegatesd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
                 ExternalAddress(double_signflip()));
  
#line 32467 "ad_x86.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3037 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 32483 "ad_x86.cpp"
  }
}

void sqrtF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3049 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 32499 "ad_x86.cpp"
  }
}

void sqrtF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3061 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32515 "ad_x86.cpp"
  }
}

void sqrtF_immNode::eval_constant(Compile* C) {
  {

#line 3062 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 32524 "ad_x86.cpp"
  }
}
void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3073 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 32539 "ad_x86.cpp"
  }
}

void sqrtD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3085 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 32555 "ad_x86.cpp"
  }
}

void sqrtD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3096 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32571 "ad_x86.cpp"
  }
}

void sqrtD_immNode::eval_constant(Compile* C) {
  {

#line 3097 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 32580 "ad_x86.cpp"
  }
}
void onspinwaitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3193 "src/hotspot/cpu/x86/x86.ad"

    __ pause();
  
#line 32595 "ad_x86.cpp"
  }
}

void fmaD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3205 "src/hotspot/cpu/x86/x86.ad"

    __ fmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */);
  
#line 32613 "ad_x86.cpp"
  }
}

void fmaF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3217 "src/hotspot/cpu/x86/x86.ad"

    __ fmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */);
  
#line 32631 "ad_x86.cpp"
  }
}

void MoveVec2LegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3229 "src/hotspot/cpu/x86/x86.ad"

    ShouldNotReachHere();
  
#line 32647 "ad_x86.cpp"
  }
}

void MoveLeg2VecNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3238 "src/hotspot/cpu/x86/x86.ad"

    ShouldNotReachHere();
  
#line 32663 "ad_x86.cpp"
  }
}

void loadVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3251 "src/hotspot/cpu/x86/x86.ad"

    switch (vector_length_in_bytes(this)) {
      case  4: __ movdl    (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case  8: __ movq     (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case 16: __ movdqu   (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case 32: __ vmovdqu  (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc())); break;
      case 64: __ evmovdqul(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), Assembler::AVX_512bit); break;
      default: ShouldNotReachHere();
    }
  
#line 32686 "ad_x86.cpp"
  }
}

void storeVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3269 "src/hotspot/cpu/x86/x86.ad"

    switch (vector_length_in_bytes(this, opnd_array(2))) {
      case  4: __ movdl    (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case  8: __ movq     (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 16: __ movdqu   (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 32: __ vmovdqu  (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 64: __ evmovdqul(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Assembler::AVX_512bit); break;
      default: ShouldNotReachHere();
    }
  
#line 32710 "ad_x86.cpp"
  }
}

void ReplB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3288 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 64 || VM_Version::supports_avx512vlbw()) { // AVX512VL for <512bit operands
      assert(VM_Version::supports_avx512bw(), "required"); // 512-bit byte vectors assume AVX512BW
      int vlen_enc = vector_length_encoding(this);
      __ evpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ punpcklbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
      __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
      if (vlen >= 16) {
        __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        if (vlen >= 32) {
          assert(vlen == 32, "sanity");
          __ vinserti128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 32742 "ad_x86.cpp"
  }
}

void ReplB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3314 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 32759 "ad_x86.cpp"
  }
}

void ReplB_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3324 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen == 4) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
    } else {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      if (vlen >= 16) {
        if (VM_Version::supports_avx2()) {
          int vlen_enc = vector_length_encoding(this);
          __ vpbroadcastq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        } else {
          assert(vlen == 16, "sanity");
          __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 32790 "ad_x86.cpp"
  }
}

void ReplB_immNode::eval_constant(Compile* C) {
  {

#line 3326 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 1));
#line 32799 "ad_x86.cpp"
  }
}
void ReplB_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3349 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 16) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      // Use vpxor since AVX512F does not have 512bit vxorpd (requires AVX512DQ).
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 32821 "ad_x86.cpp"
  }
}

void ReplS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3367 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 32 || VM_Version::supports_avx512vlbw()) { // AVX512VL for <512bit operands
      assert(VM_Version::supports_avx512bw(), "required"); // 512-bit short vectors assume AVX512BW
      int vlen_enc = vector_length_encoding(this);
      __ evpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
      if (vlen >= 8) {
        __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        if (vlen >= 16) {
          assert(vlen == 16, "sanity");
          __ vinserti128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 32852 "ad_x86.cpp"
  }
}

void ReplS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3392 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 32869 "ad_x86.cpp"
  }
}

void ReplS_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3402 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen == 2) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
    } else {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      if (vlen >= 8) {
        if (VM_Version::supports_avx2()) {
          int vlen_enc = vector_length_encoding(this);
          __ vpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        } else {
          assert(vlen == 8, "sanity");
          __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
        }
      }
    }
  
#line 32900 "ad_x86.cpp"
  }
}

void ReplS_immNode::eval_constant(Compile* C) {
  {

#line 3404 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 2));
#line 32909 "ad_x86.cpp"
  }
}
void ReplS_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3426 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 8) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 32930 "ad_x86.cpp"
  }
}

void ReplI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3443 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 16 || VM_Version::supports_avx512vl()) { // AVX512VL for <512bit operands
      int vlen_enc = vector_length_encoding(this);
      __ evpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
      if (vlen >= 8) {
        assert(vlen == 8, "sanity");
        __ vinserti128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
      }
    }
  
#line 32957 "ad_x86.cpp"
  }
}

void ReplI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3463 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vector_len = vector_length_encoding(this);
      __ vpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
    }
  
#line 32981 "ad_x86.cpp"
  }
}

void ReplI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3480 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen <= 4) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      if (vlen == 4) {
        __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
      }
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vector_len = vector_length_encoding(this);
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      __ vpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector_len);
    }
  
#line 33009 "ad_x86.cpp"
  }
}

void ReplI_immNode::eval_constant(Compile* C) {
  {

#line 3482 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 4));
#line 33018 "ad_x86.cpp"
  }
}
void ReplI_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3502 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 33039 "ad_x86.cpp"
  }
}

void ReplI_M1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3521 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 33057 "ad_x86.cpp"
  }
}

void ReplI_M1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3521 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 33075 "ad_x86.cpp"
  }
}

void ReplI_M1_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3521 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 33093 "ad_x86.cpp"
  }
}

void ReplL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3559 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
    } else if (VM_Version::supports_avx512vl()) { // AVX512VL for <512bit operands
      int vector_len = Assembler::AVX_256bit;
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ vpbroadcastq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
    } else {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti128_high(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
    }
  
#line 33129 "ad_x86.cpp"
  }
}

void ReplL_reg_legNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3588 "src/hotspot/cpu/x86/x86.ad"

    if (VM_Version::supports_avx512vl()) {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti128_high(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti64x4(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 0x1);
    } else {
      int vector_len = Assembler::AVX_512bit;
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ vpbroadcastq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
    }
  
#line 33160 "ad_x86.cpp"
  }
}

void ReplL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3611 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ vpbroadcastq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    }
  
#line 33184 "ad_x86.cpp"
  }
}

void ReplL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3629 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    InternalAddress const_addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    if (vlen == 2) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      assert(VM_Version::supports_avx2(), "sanity");
      int vlen_enc = vector_length_encoding(this);
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, const_addr);
      __ vpbroadcastq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 33210 "ad_x86.cpp"
  }
}

void ReplL_immNode::eval_constant(Compile* C) {
  {

#line 3631 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 33219 "ad_x86.cpp"
  }
}
void ReplL_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3648 "src/hotspot/cpu/x86/x86.ad"

    int vlen = vector_length(this);
    if (vlen == 2) {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 33240 "ad_x86.cpp"
  }
}

void ReplL_M1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3665 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vector_len);
  
#line 33258 "ad_x86.cpp"
  }
}

void ReplF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3677 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
   } else if (VM_Version::supports_avx2()) {
      int vector_len = vector_length_encoding(this);
      __ vbroadcastss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len); // reg-to-reg variant requires AVX2
    } else {
      assert(vlen == 8, "sanity");
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
      __ vinsertf128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 33284 "ad_x86.cpp"
  }
}

void ReplF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3696 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    } else {
      assert(VM_Version::supports_avx(), "sanity");
      int vector_len = vector_length_encoding(this);
      __ vbroadcastss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
    }
  
#line 33308 "ad_x86.cpp"
  }
}

void ReplF_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3713 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */,opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc); // 512bit vxorps requires AVX512DQ
    }
  
#line 33330 "ad_x86.cpp"
  }
}

void ReplD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3731 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x44);
    } else if (VM_Version::supports_avx2()) {
      int vector_len = vector_length_encoding(this);
      __ vbroadcastsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len); // reg-to-reg variant requires AVX2
    } else {
      assert(vlen == 4, "sanity");
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x44);
      __ vinsertf128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 33356 "ad_x86.cpp"
  }
}

void ReplD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3750 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x44);
    } else {
      assert(VM_Version::supports_avx(), "sanity");
      int vector_len = vector_length_encoding(this);
      __ vbroadcastsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
    }
  
#line 33380 "ad_x86.cpp"
  }
}

void ReplD_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3767 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      __ xorpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc); // 512bit vxorps requires AVX512DQ
    }
  
#line 33402 "ad_x86.cpp"
  }
}

void reductionINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3792 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33423 "ad_x86.cpp"
  }
}

void reductionI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3792 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33444 "ad_x86.cpp"
  }
}

void reductionI_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3792 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33465 "ad_x86.cpp"
  }
}

void reductionI_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3792 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33486 "ad_x86.cpp"
  }
}

void reductionI_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3792 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33507 "ad_x86.cpp"
  }
}

void reduction16INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3810 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33528 "ad_x86.cpp"
  }
}

void reduction16I_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3810 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33549 "ad_x86.cpp"
  }
}

void reduction16I_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3810 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33570 "ad_x86.cpp"
  }
}

void reduction16I_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3810 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33591 "ad_x86.cpp"
  }
}

void reduction16I_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3810 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33612 "ad_x86.cpp"
  }
}

void reductionF128Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3866 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 33632 "ad_x86.cpp"
  }
}

void reductionF128_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3866 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 33652 "ad_x86.cpp"
  }
}

void reduction8FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3880 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33673 "ad_x86.cpp"
  }
}

void reduction8F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3880 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33694 "ad_x86.cpp"
  }
}

void reduction16FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3894 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33715 "ad_x86.cpp"
  }
}

void reduction16F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3894 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33736 "ad_x86.cpp"
  }
}

void reduction2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3910 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 33756 "ad_x86.cpp"
  }
}

void reduction2D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3910 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 33776 "ad_x86.cpp"
  }
}

void reduction4DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3924 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33797 "ad_x86.cpp"
  }
}

void reduction4D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3924 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33818 "ad_x86.cpp"
  }
}

void reduction8DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3938 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33839 "ad_x86.cpp"
  }
}

void reduction8D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3938 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 33860 "ad_x86.cpp"
  }
}

void vaddBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3955 "src/hotspot/cpu/x86/x86.ad"

    __ paddb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33877 "ad_x86.cpp"
  }
}

void vaddB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3965 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 33895 "ad_x86.cpp"
  }
}

void vaddB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3977 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 33913 "ad_x86.cpp"
  }
}

void vaddB_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3977 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 33931 "ad_x86.cpp"
  }
}

void vaddSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3989 "src/hotspot/cpu/x86/x86.ad"

    __ paddw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33948 "ad_x86.cpp"
  }
}

void vaddS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3999 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 33966 "ad_x86.cpp"
  }
}

void vaddS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4011 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 33984 "ad_x86.cpp"
  }
}

void vaddS_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4011 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34002 "ad_x86.cpp"
  }
}

void vaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4023 "src/hotspot/cpu/x86/x86.ad"

    __ paddd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34019 "ad_x86.cpp"
  }
}

void vaddI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4033 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34037 "ad_x86.cpp"
  }
}

void vaddI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4046 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34055 "ad_x86.cpp"
  }
}

void vaddI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4046 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34073 "ad_x86.cpp"
  }
}

void vaddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4058 "src/hotspot/cpu/x86/x86.ad"

    __ paddq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34090 "ad_x86.cpp"
  }
}

void vaddL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4068 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34108 "ad_x86.cpp"
  }
}

void vaddL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4080 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34126 "ad_x86.cpp"
  }
}

void vaddL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4080 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34144 "ad_x86.cpp"
  }
}

void vaddFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4092 "src/hotspot/cpu/x86/x86.ad"

    __ addps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34161 "ad_x86.cpp"
  }
}

void vaddF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4102 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34179 "ad_x86.cpp"
  }
}

void vaddF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4114 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34197 "ad_x86.cpp"
  }
}

void vaddF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4114 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34215 "ad_x86.cpp"
  }
}

void vaddDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4126 "src/hotspot/cpu/x86/x86.ad"

    __ addpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34232 "ad_x86.cpp"
  }
}

void vaddD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4136 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34250 "ad_x86.cpp"
  }
}

void vaddD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4148 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34268 "ad_x86.cpp"
  }
}

void vaddD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4148 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34286 "ad_x86.cpp"
  }
}

void vsubBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4162 "src/hotspot/cpu/x86/x86.ad"

    __ psubb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34303 "ad_x86.cpp"
  }
}

void vsubB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4172 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34321 "ad_x86.cpp"
  }
}

void vsubB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4184 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34339 "ad_x86.cpp"
  }
}

void vsubSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4196 "src/hotspot/cpu/x86/x86.ad"

    __ psubw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34356 "ad_x86.cpp"
  }
}

void vsubS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4207 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34374 "ad_x86.cpp"
  }
}

void vsubS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4219 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34392 "ad_x86.cpp"
  }
}

void vsubINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4231 "src/hotspot/cpu/x86/x86.ad"

    __ psubd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34409 "ad_x86.cpp"
  }
}

void vsubI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4241 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34427 "ad_x86.cpp"
  }
}

void vsubI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4253 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34445 "ad_x86.cpp"
  }
}

void vsubLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4265 "src/hotspot/cpu/x86/x86.ad"

    __ psubq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34462 "ad_x86.cpp"
  }
}

void vsubL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4275 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34480 "ad_x86.cpp"
  }
}

void vsubL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4288 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34498 "ad_x86.cpp"
  }
}

void vsubFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4300 "src/hotspot/cpu/x86/x86.ad"

    __ subps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34515 "ad_x86.cpp"
  }
}

void vsubF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4310 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34533 "ad_x86.cpp"
  }
}

void vsubF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4322 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34551 "ad_x86.cpp"
  }
}

void vsubDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4334 "src/hotspot/cpu/x86/x86.ad"

    __ subpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34568 "ad_x86.cpp"
  }
}

void vsubD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4344 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34586 "ad_x86.cpp"
  }
}

void vsubD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4356 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34604 "ad_x86.cpp"
  }
}

void mulB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4372 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ pmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 34630 "ad_x86.cpp"
  }
}

void mul16B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4389 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ pmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, 0xEE);
    __ pshufd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, 0xEE);
    __ pmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ pmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pmullw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 34663 "ad_x86.cpp"
  }
}

void vmul16B_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4412 "src/hotspot/cpu/x86/x86.ad"

  int vector_len = Assembler::AVX_256bit;
    __ vpmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, vector_len);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
    __ vpmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, 0);
  
#line 34690 "ad_x86.cpp"
  }
}

void vmul32B_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4430 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int vector_len = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ vpmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vector_len);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, vector_len);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
    __ vpmullw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vector_len);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vector_len);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vector_len);
  
#line 34726 "ad_x86.cpp"
  }
}

void vmul64B_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4456 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vector_len = Assembler::AVX_512bit;
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ vextracti64x4_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ vpmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vector_len);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpmovsxbw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, vector_len);
    __ vpmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
    __ vpmullw(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vector_len);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vector_len, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
  
#line 34763 "ad_x86.cpp"
  }
}

void vmulSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4483 "src/hotspot/cpu/x86/x86.ad"

    __ pmullw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34780 "ad_x86.cpp"
  }
}

void vmulS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4493 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34798 "ad_x86.cpp"
  }
}

void vmulS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4505 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34816 "ad_x86.cpp"
  }
}

void vmulS_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4505 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34834 "ad_x86.cpp"
  }
}

void vmulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4517 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmulld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34852 "ad_x86.cpp"
  }
}

void vmulI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4528 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34870 "ad_x86.cpp"
  }
}

void vmulI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4540 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34888 "ad_x86.cpp"
  }
}

void vmulI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4540 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34906 "ad_x86.cpp"
  }
}

void vmulL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4551 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vector_len = vector_length_encoding(this);
    __ vpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34925 "ad_x86.cpp"
  }
}

void vmulL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4563 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vector_len = vector_length_encoding(this);
    __ vpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 34944 "ad_x86.cpp"
  }
}

void vmulL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4563 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vector_len = vector_length_encoding(this);
    __ vpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 34963 "ad_x86.cpp"
  }
}

void vmulFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4576 "src/hotspot/cpu/x86/x86.ad"

    __ mulps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34980 "ad_x86.cpp"
  }
}

void vmulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4586 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 34998 "ad_x86.cpp"
  }
}

void vmulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4598 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 35016 "ad_x86.cpp"
  }
}

void vmulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4598 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 35034 "ad_x86.cpp"
  }
}

void vmulDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4610 "src/hotspot/cpu/x86/x86.ad"

    __ mulpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35051 "ad_x86.cpp"
  }
}

void vmulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4620 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 35069 "ad_x86.cpp"
  }
}

void vmulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4632 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 35087 "ad_x86.cpp"
  }
}

void vmulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4632 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 35105 "ad_x86.cpp"
  }
}

void vcmov8F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// copnd
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cop
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4646 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = 1;
    int cond = (Assembler::Condition)(opnd_array(1)->ccode());
    __ cmpps(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, cond, vector_len);
    __ blendvps(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, vector_len);
  
#line 35128 "ad_x86.cpp"
  }
}

void vcmov4D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// copnd
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cop
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4662 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = 1;
    int cond = (Assembler::Condition)(opnd_array(1)->ccode());
    __ cmppd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, cond, vector_len);
    __ blendvpd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* dst */, vector_len);
  
#line 35151 "ad_x86.cpp"
  }
}

void vdivFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4678 "src/hotspot/cpu/x86/x86.ad"

    __ divps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35168 "ad_x86.cpp"
  }
}

void vdivF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4688 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 35186 "ad_x86.cpp"
  }
}

void vdivF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4700 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 35204 "ad_x86.cpp"
  }
}

void vdivDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4712 "src/hotspot/cpu/x86/x86.ad"

    __ divpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35221 "ad_x86.cpp"
  }
}

void vdivD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4722 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 35239 "ad_x86.cpp"
  }
}

void vdivD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4734 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 35257 "ad_x86.cpp"
  }
}

void vsqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4746 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vector_len = vector_length_encoding(this);
    __ vsqrtps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
  
#line 35275 "ad_x86.cpp"
  }
}

void vsqrtF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4758 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vector_len = vector_length_encoding(this);
    __ vsqrtps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 35293 "ad_x86.cpp"
  }
}

void vsqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4770 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vector_len = vector_length_encoding(this);
    __ vsqrtpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
  
#line 35311 "ad_x86.cpp"
  }
}

void vsqrtD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4782 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vector_len = vector_length_encoding(this);
    __ vsqrtpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 35329 "ad_x86.cpp"
  }
}

void vshiftcntNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4798 "src/hotspot/cpu/x86/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 35345 "ad_x86.cpp"
  }
}

void vshiftcnt_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4798 "src/hotspot/cpu/x86/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 35361 "ad_x86.cpp"
  }
}

void vshiftBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4812 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 35387 "ad_x86.cpp"
  }
}

void vshiftB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4812 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 35413 "ad_x86.cpp"
  }
}

void vshiftB_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4812 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 35439 "ad_x86.cpp"
  }
}

void vshift16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4831 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();

    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 35471 "ad_x86.cpp"
  }
}

void vshift16B_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4831 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();

    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 35503 "ad_x86.cpp"
  }
}

void vshift16B_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4831 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();

    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 35535 "ad_x86.cpp"
  }
}

void vshift16B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4855 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_256bit;
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 35561 "ad_x86.cpp"
  }
}

void vshift16B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4855 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_256bit;
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 35587 "ad_x86.cpp"
  }
}

void vshift16B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4855 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_256bit;
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 35613 "ad_x86.cpp"
  }
}

void vshift32B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4874 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    __ vextendbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vector_len);
  
#line 35644 "ad_x86.cpp"
  }
}

void vshift32B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4874 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    __ vextendbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vector_len);
  
#line 35675 "ad_x86.cpp"
  }
}

void vshift32B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4874 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    __ vextendbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vector_len, opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vector_len);
  
#line 35706 "ad_x86.cpp"
  }
}

void vshift64B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4898 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vector_len);
    __ vextendbw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vector_len);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vector_len, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
  
#line 35741 "ad_x86.cpp"
  }
}

void vshift64B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4898 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vector_len);
    __ vextendbw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vector_len);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vector_len, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
  
#line 35776 "ad_x86.cpp"
  }
}

void vshift64B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4898 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    int vector_len = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vector_len);
    __ vextendbw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vector_len);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vector_len, opnd_array(6)->as_Register(ra_,this,idx6)/* scratch */);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vector_len);
  
#line 35811 "ad_x86.cpp"
  }
}

void vshiftSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4929 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 35846 "ad_x86.cpp"
  }
}

void vshiftS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4929 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 35881 "ad_x86.cpp"
  }
}

void vshiftS_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4929 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 35916 "ad_x86.cpp"
  }
}

void vshiftINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4959 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 35948 "ad_x86.cpp"
  }
}

void vshiftI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4959 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 35980 "ad_x86.cpp"
  }
}

void vshiftI_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4959 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    } else {
      int vlen = vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 36012 "ad_x86.cpp"
  }
}

void vshiftLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4985 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    } else {
      assert(vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    }
  
#line 36038 "ad_x86.cpp"
  }
}

void vshiftL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4985 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
    } else {
      assert(vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    }
  
#line 36064 "ad_x86.cpp"
  }
}

void vshiftL_arith_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5006 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ psrlq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_long_sign_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
      __ psrlq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      __ pxor(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
      __ psubq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    } else {
      assert(vlen == 4, "sanity");
      assert(UseAVX > 1, "required");
      int vector_len = Assembler::AVX_256bit;
      __ vpsrlq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
      __ vmovdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_long_sign_mask()), opnd_array(5)->as_Register(ra_,this,idx5)/* scratch */);
      __ vpsrlq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
      __ vpxor(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
      __ vpsubq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vector_len);
    }
  
#line 36102 "ad_x86.cpp"
  }
}

void vshiftL_arith_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5034 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ evpsraq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
  
#line 36120 "ad_x86.cpp"
  }
}

void vandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5047 "src/hotspot/cpu/x86/x86.ad"

    __ pand(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36137 "ad_x86.cpp"
  }
}

void vand_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5057 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 36155 "ad_x86.cpp"
  }
}

void vand_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5069 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 36173 "ad_x86.cpp"
  }
}

void vand_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5069 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 36191 "ad_x86.cpp"
  }
}

void vorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5082 "src/hotspot/cpu/x86/x86.ad"

    __ por(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36208 "ad_x86.cpp"
  }
}

void vor_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5092 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 36226 "ad_x86.cpp"
  }
}

void vor_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5104 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 36244 "ad_x86.cpp"
  }
}

void vor_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5104 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 36262 "ad_x86.cpp"
  }
}

void vxorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5117 "src/hotspot/cpu/x86/x86.ad"

    __ pxor(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36279 "ad_x86.cpp"
  }
}

void vxor_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5127 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 36297 "ad_x86.cpp"
  }
}

void vxor_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5139 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vector_len);
  
#line 36315 "ad_x86.cpp"
  }
}

void vxor_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5139 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vector_len);
  
#line 36333 "ad_x86.cpp"
  }
}

void vabsB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5151 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 16) {
      __ pabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 36355 "ad_x86.cpp"
  }
}

void vabsS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5166 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 8) {
      __ pabsw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 36377 "ad_x86.cpp"
  }
}

void vabsI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5181 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = vector_length(this);
    if (vlen <= 4) {
      __ pabsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 36399 "ad_x86.cpp"
  }
}

void vabsL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5196 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vector_len = vector_length_encoding(this);
    __ evpabsq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
  
#line 36417 "ad_x86.cpp"
  }
}

void vabsnegFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5213 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this);
    if (vlen == 2) {
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      assert(vlen == 8 || vlen == 16, "required");
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 36442 "ad_x86.cpp"
  }
}

void vabsnegF_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5213 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = vector_length(this);
    if (vlen == 2) {
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      assert(vlen == 8 || vlen == 16, "required");
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 36467 "ad_x86.cpp"
  }
}

void vabsneg4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5234 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ vabsnegf(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
  
#line 36485 "ad_x86.cpp"
  }
}

void vabsneg4F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5234 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ vabsnegf(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
  
#line 36503 "ad_x86.cpp"
  }
}

void vabsnegDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5246 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    uint vlen = vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 36528 "ad_x86.cpp"
  }
}

void vabsnegD_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5246 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    uint vlen = vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, opnd_array(2)->as_Register(ra_,this,idx2)/* scratch */);
    }
  
#line 36553 "ad_x86.cpp"
  }
}

void vfmaF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5267 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vector_len = vector_length_encoding(this);
    __ vfmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vector_len);
  
#line 36573 "ad_x86.cpp"
  }
}

void vfmaF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5280 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vector_len = vector_length_encoding(this);
    __ vfmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vector_len);
  
#line 36593 "ad_x86.cpp"
  }
}

void vfmaD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5292 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vector_len = vector_length_encoding(this);
    __ vfmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vector_len);
  
#line 36613 "ad_x86.cpp"
  }
}

void vfmaD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5305 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vector_len = vector_length_encoding(this);
    __ vfmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vector_len);
  
#line 36633 "ad_x86.cpp"
  }
}

void vmuladdS2I_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5319 "src/hotspot/cpu/x86/x86.ad"

    __ pmaddwd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
  
#line 36650 "ad_x86.cpp"
  }
}

void vmuladdS2I_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5329 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpmaddwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 36668 "ad_x86.cpp"
  }
}

void vmuladdaddS2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5342 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vector_len = vector_length_encoding(this);
    __ evpdpwssd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector_len);
  
#line 36688 "ad_x86.cpp"
  }
}

void vmuladdaddS2I_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5342 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vector_len = vector_length_encoding(this);
    __ evpdpwssd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src2 */, vector_len);
  
#line 36708 "ad_x86.cpp"
  }
}

void vpopcountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5356 "src/hotspot/cpu/x86/x86.ad"

    assert(UsePopCountInstruction, "not enabled");

    int vector_len = vector_length_encoding(this);
    __ vpopcntd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vector_len);
  
#line 36727 "ad_x86.cpp"
  }
}

void vpternlogNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5371 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpternlogd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src3 */, vector_len);
  
#line 36747 "ad_x86.cpp"
  }
}

void vpternlog_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5383 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpternlogd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), vector_len);
  
#line 36767 "ad_x86.cpp"
  }
}

void compareAndSwapP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 42 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   opnd_array(0)->as_Register(ra_,this)/* res */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   false, // swap
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 36790 "ad_x86.cpp"
  }
}

void compareAndSwapP_shenandoah_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 42 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   opnd_array(0)->as_Register(ra_,this)/* res */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   false, // swap
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 36813 "ad_x86.cpp"
  }
}

void compareAndExchangeP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 63 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   NULL, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   true,  // exchange
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 36836 "ad_x86.cpp"
  }
}

const MachOper* popCountI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_immINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2UBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2USNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_65535Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immU31Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_partialNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAlloc0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmPNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCMNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDPR_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_DroundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmovI_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* addI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddB_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddBNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddS_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddSNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddINode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgBNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgSNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgINode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgPNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulINode::memory_operand() const { return _opnds[2]; }
const MachOper* mulI_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andnI_rReg_rReg_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* andnI_rReg_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsiI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* blsiI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskI_rReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsrI_rReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsrI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andnL_eReg_eReg_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* andnL_eReg_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsiL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* blsiL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskL_eReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsrL_eReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsrL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpD_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* subDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addDPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addDPR_mem_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* addDPR_mem_reg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulDPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulDPR_reg_mem_ciscNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulDPR_reg_mem_cisc_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpF_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR24_mem_regNode::memory_operand() const { return _opnds[1]; }
const TypePtr *addFPR24_mem_ciscNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_ciscNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addFPR24_mem_cisc_0Node::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_cisc_0Node::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addFPR24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulFPR24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulFPR24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulFPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulFPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const TypePtr *mulFPR24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* mulFPR24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulFPR_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulFPR_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR_mulFPR_reg_load1_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1_2Node::memory_operand() const { return _opnds[3]; }
const MachOper* convI2D_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2DPR_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2FPR_SSF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2FPR_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* compI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compU_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_mem_eRegNode::memory_operand() const { return _opnds[2]; }
const MachOper* testP_Reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmovLL_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_LEGTNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LEGTNode::memory_operand() const { return _opnds[4]; }
const MachOper* addF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* sqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* sqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadVNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeVNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplB_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplS_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vaddB_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddB_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddS_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vsubB_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulS_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vdivF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdivD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vsqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vand_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vand_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vor_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vor_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vxor_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vxor_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vfmaF_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* vfmaD_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* vpternlog_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* compareAndSwapP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapP_shenandoah_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeP_shenandoahNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // VecZ
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    true ,  // AbsI
    false,  // AbsL
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // ArrayCopy
    true ,  // AryEq
    true ,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // CProj
    false,  // CacheWB
    false,  // CacheWBPreSync
    false,  // CacheWBPostSync
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveVD
    true ,  // CMoveF
    true ,  // CMoveVF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    false,  // CMoveN
    false,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapB
    true ,  // CompareAndSwapS
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    false,  // CompareAndSwapN
    true ,  // WeakCompareAndSwapB
    true ,  // WeakCompareAndSwapS
    true ,  // WeakCompareAndSwapI
    true ,  // WeakCompareAndSwapL
    true ,  // WeakCompareAndSwapP
    false,  // WeakCompareAndSwapN
    true ,  // CompareAndExchangeB
    true ,  // CompareAndExchangeS
    true ,  // CompareAndExchangeI
    true ,  // CompareAndExchangeL
    true ,  // CompareAndExchangeP
    false,  // CompareAndExchangeN
    true ,  // GetAndAddB
    true ,  // GetAndAddS
    true ,  // GetAndAddI
    false,  // GetAndAddL
    true ,  // GetAndSetB
    true ,  // GetAndSetS
    true ,  // GetAndSetI
    false,  // GetAndSetL
    true ,  // GetAndSetP
    false,  // GetAndSetN
    false,  // Con
    false,  // ConN
    false,  // ConNKlass
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    false,  // OuterStripMinedLoop
    false,  // OuterStripMinedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    false,  // DecodeN
    false,  // DecodeNKlass
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    true ,  // DivModI
    false,  // DivModL
    true ,  // EncodeISOArray
    false,  // EncodeP
    false,  // EncodePKlass
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // FmaD
    true ,  // FmaF
    true ,  // Goto
    true ,  // Halt
    true ,  // HasNegatives
    true ,  // If
    false,  // RangeCheck
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    false,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    false,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // Loop
    true ,  // LoopLimit
    false,  // Mach
    false,  // MachProj
    true ,  // MulAddS2I
    false,  // MaxD
    false,  // MaxF
    true ,  // MaxI
    true ,  // MemBarAcquire
    true ,  // LoadFence
    true ,  // SetVectMaskI
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // StoreFence
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    false,  // MinD
    false,  // MinF
    true ,  // MinI
    true ,  // ModD
    true ,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    false,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    true ,  // OnSpinWait
    false,  // Opaque1
    false,  // OpaqueLoopInit
    false,  // OpaqueLoopStride
    false,  // Opaque2
    false,  // Opaque3
    false,  // Opaque4
    false,  // ProfileBoolean
    true ,  // OrI
    true ,  // OrL
    true ,  // OverflowAddI
    true ,  // OverflowSubI
    true ,  // OverflowMulI
    false,  // OverflowAddL
    false,  // OverflowSubL
    false,  // OverflowMulL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // SubTypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    true ,  // PopCountVI
    true ,  // PrefetchAllocation
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    false,  // RoundDoubleMode
    false,  // RoundDoubleModeV
    true ,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    true ,  // ShenandoahCompareAndExchangeP
    false,  // ShenandoahCompareAndExchangeN
    false,  // ShenandoahCompareAndSwapN
    true ,  // ShenandoahCompareAndSwapP
    false,  // ShenandoahWeakCompareAndSwapN
    true ,  // ShenandoahWeakCompareAndSwapP
    false,  // ShenandoahEnqueueBarrier
    false,  // ShenandoahLoadReferenceBarrier
    false,  // SCMemProj
    true ,  // SqrtD
    true ,  // SqrtF
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    false,  // StoreN
    false,  // StoreNKlass
    true ,  // StrComp
    true ,  // StrCompressedCopy
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // StrIndexOfChar
    true ,  // StrInflatedCopy
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // MacroLogicV
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    true ,  // AddVB
    true ,  // AddVS
    true ,  // AddVI
    true ,  // AddReductionVI
    true ,  // AddVL
    false,  // AddReductionVL
    true ,  // AddVF
    true ,  // AddReductionVF
    true ,  // AddVD
    true ,  // AddReductionVD
    true ,  // SubVB
    true ,  // SubVS
    true ,  // SubVI
    true ,  // SubVL
    true ,  // SubVF
    true ,  // SubVD
    true ,  // MulVB
    true ,  // MulVS
    true ,  // MulVI
    true ,  // MulReductionVI
    true ,  // MulVL
    false,  // MulReductionVL
    true ,  // MulVF
    true ,  // MulReductionVF
    true ,  // MulVD
    true ,  // MulReductionVD
    true ,  // MulAddVS2VI
    true ,  // FmaVD
    true ,  // FmaVF
    true ,  // DivVF
    true ,  // DivVD
    true ,  // AbsVB
    true ,  // AbsVS
    true ,  // AbsVI
    true ,  // AbsVL
    true ,  // AbsVF
    true ,  // AbsVD
    true ,  // NegVF
    true ,  // NegVD
    true ,  // SqrtVD
    true ,  // SqrtVF
    true ,  // LShiftCntV
    true ,  // RShiftCntV
    true ,  // LShiftVB
    true ,  // LShiftVS
    true ,  // LShiftVI
    true ,  // LShiftVL
    true ,  // RShiftVB
    true ,  // RShiftVS
    true ,  // RShiftVI
    true ,  // RShiftVL
    true ,  // URShiftVB
    true ,  // URShiftVS
    true ,  // URShiftVI
    true ,  // URShiftVL
    true ,  // AndV
    true ,  // AndReductionV
    true ,  // OrV
    true ,  // OrReductionV
    true ,  // XorV
    true ,  // XorReductionV
    false,  // MinV
    false,  // MaxV
    false,  // MinReductionV
    false,  // MaxReductionV
    true ,  // LoadVector
    true ,  // StoreVector
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    true ,  // ReplicateL
    true ,  // ReplicateF
    true ,  // ReplicateD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    false,  // ExtractS
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false,  // ExtractD
    false,  // Digit
    false,  // LowerCase
    false,  // UpperCase
    false   // Whitespace
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 1; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 1 +
              align_up((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::in_preserve_stack_slots() { return 2+VerifyStackAtCalls; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return 0; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

#line 3246 "src/hotspot/cpu/x86/x86_32.ad"

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
#line 37542 "ad_x86.cpp"

}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

#line 3258 "src/hotspot/cpu/x86/x86_32.ad"

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, /*regs2=*/NULL, length);
  
#line 37553 "ad_x86.cpp"

}

OptoRegPair Matcher::return_value(uint ideal_reg, bool is_outgoing) {

#line 3280 "src/hotspot/cpu/x86/x86_32.ad"

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0_num);
    if( ideal_reg == Op_RegF && UseSSE>=1 )
      return OptoRegPair(OptoReg::Bad,XMM0_num);
    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
#line 37570 "ad_x86.cpp"

}

OptoRegPair Matcher::c_return_value(uint ideal_reg, bool is_outgoing) {

#line 3264 "src/hotspot/cpu/x86/x86_32.ad"

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };

    // in SSE2+ mode we want to keep the FPU stack clean so pretend
    // that C functions return float and double results in XMM0.
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0_num);
    if( ideal_reg == Op_RegF && UseSSE>=2 )
      return OptoRegPair(OptoReg::Bad,XMM0_num);

    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
#line 37591 "ad_x86.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(EAX_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(EBX_num); }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(EBP_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(ESP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

bool Compile::needs_clone_jvms() { return false; }

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DIA32=1
#ifndef IA32
#  error "IA32 must be defined"
#endif // IA32
// Check adlc -U_LP64
#ifdef _LP64
#  error "_LP64 must not be defined"
#endif // _LP64
